/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************* fusion_floue.h  **********************************/
/*                                                                            */
/*      descripteurs et prototypes associes a l'operateur fusion_floue        */
/*                                                                            */
/*  Derniere modif : Lionel VALET, 19/11/99                                   */
/* ****************************************************************************/

/* pas de code avant cette ligne*/
#ifndef __ff_symb2
#define __ff_symb2


#define MAXI(i,j) ( (i) > (j) ? (i) : (j) )
#define MINI(i,j) ( (i) < (j) ? (i) : (j) )
#define SQR(i) ( (i) * (i) )

typedef int (*PFI)();		//pointer to function that returns int

typedef char chaine[100];
typedef unsigned char Byte;

typedef struct
{
    chaine nom;
    float gauche;
    float mode;
    float droite;
}   Symbol;

typedef struct
{
    unsigned char * liste_in;
    unsigned char nom_out;
    float ponderation;
}Une_regle;

typedef struct
{
    int nb_regle;
    Une_regle * regle;
}Les_regles;

typedef struct
{
    chaine nom;
    unsigned char nb_fct;
    Symbol *part;
}Partition;

typedef float degre_app;
typedef degre_app *ss_ens_flou_symb;

typedef struct  
{
    int nb_descripteur;
    ss_ens_flou_symb * descript;
}Descripteur;

typedef struct
{
    unsigned char nb_entree;
    Partition *partition;
}Fct_appartenance;

/** \ingroup ff_symb
    descripteur pour l'oprateur de fusion floue*/
typedef struct {
    int Tnorme;     /**< choix de l'operateur T-normes */
    int Tconorme;    /**< choix de l'operateur T-conormes*/
    PFI	tn;		//pointeur a la fonction de calcul de norme
    PFI	tcn;		//pointeur a la fonction de calcul de tconorme
    int defuzz;      /**< choix de la methode de dfuzzification */
    Fct_appartenance fonction; /**< ensemble des fonctions d'appartenace*/
    Les_regles rgl;            /**< base de regles */
    Descripteur des;           /**< les descripteurs floues*/
    char *nom;    /**< chaine de caractere represantant le fichier des parametres */
} fusion_floue_t;

param *ff_symb_lect(fusion_floue_t *des, param *ptp, char *debq);
int ff_symb_init(fusion_floue_t *des);     
int ff_symb_calc(fusion_floue_t *des, float *data, float *result);     
int ff_symb_ferm(fusion_floue_t *des);


/*pas de code aprs cette ligne */
#endif
