/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef __format_learning_set_from_imau1
#define __format_learning_set_from_imau1

#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <stdio.h>
#include "format_imau1_classifier_input.h"
#include "format_learning_set_from_imau1.h"



/** Pas d'erreur lors de l'excution des fonctions
de l'oprateur format_learning_set_from_imau1 (LSFI1) */
#define NO_ERROR_FORMAT_LSFI1 0

/** Lors de l'initialisation, une seule a t trouve
car un seul label existait dans le masque. */
#define UNE_SEULE_CLASSE_LSFI1 1 

/** Lors de l'initialisation le nombre de points
d'apprentissage tait infrieur  2 */ 
#define NB_PTS_INF_2_LSFI1 


/** La structure classe permet de stocker les
diffrents labels existant dans un masque ainsi
que le nombre de points par labels */
typedef struct{

/** La valeur du label. */
  int label;    

 /** Le nombre de points par label.  */ 
  int nb_pts;                               
                                                                  
}une_classe;


/** 
    \ingroup format_ouput
    Le pseudo oprateur a besoin de connatre la zone de donnes 
    (rectangulaire ou carre donc 2 points sont ncessaires)  
    slectionner sur les attributs pour remplir la structure output. */
typedef struct{
  
  /** Coordonnes du premier coin. Le premier point (haut gauche) 
      dfinissant le rectangle de slection des donnes de l'attribut image */
  point premier_point;      
  
  /** Coordonnes du deuxime coin. Le deuxime point (bas droite) 
      dfinissant le rectangle de slection des donnes de l'attribut image.*/
  point deuxieme_point;    
  
  /** Choix image ENTIERE ou image PARTIELLE. Renseigne sur la
      zone des donnes  prendre dans l'attribut d'entres : toutes les donnes 
      (image entire) ou une partie (image partielle). */
  int choix;       

}format_imau1_classifier_crisp_output_t;



/** \ingroup format_learn
    Le descripteur de l'oprateur format_learning_set_from_imau1 */
typedef struct{                     
  
  /** Stocke les labels et le nombre de points par label. */
  une_classe *classes;      
  
  /** Le label de la classe a rejeter (le fond) */  
  int c_max;   
  
  /** Le nombre d'attributs */
  int nb_att;          
       
  /** Pointeur sur les attributs imau1  */          
  imau1 *att_ima;             
  
  /** Le masque */                                                                       
  imau1 masque_ima;    
  
  /** Pour lecture du masque */ 
  read_ima_t masque_rea;          
  
  /** Pour lecture des attributs */  
  read_ima_t *att_rea;                    
  
  /** Pour le formatage des attributs */  
  format_imau1_classifier_input_t format_des_input;        
  
  /** Pour le formatage du masque */      
  format_imau1_classifier_crisp_output_t format_des_output;   
  
  /** Les donnes d'attributs stockes sous la forme data_input */
  data_input dataIattributs;  
  
  /** Les donnes du masque stockes sous la forme data_input */
  data_output dataOmasque;                                                      
            
}format_learning_set_from_imau1_t;




param* format_learning_set_from_imau1_lect(
					   format_learning_set_from_imau1_t *desc_learn, 
					   param *ptp,
					   char *debq 
					   );

int format_learning_set_from_imau1_init(
					format_learning_set_from_imau1_t *desc_learn, 
					const int LABEL_FOND,
					learning_set *learn
					);


int format_learning_set_from_imau1_calc(
					format_learning_set_from_imau1_t *desc_learn, 
					learning_set *learn
					);

int format_learning_set_from_imau1_ferm(
					format_learning_set_from_imau1_t *desc_learn,
					learning_set learn
					);




  

param *format_imau1_classifier_crisp_output_lect(
						 format_imau1_classifier_crisp_output_t *desc, 
						 param *ptp, 
						 char *debq);

int format_imau1_classifier_crisp_output_init(
					      format_imau1_classifier_crisp_output_t *desc, 
					      imau1 image_u1, 
					      data_output *data);


int format_imau1_classifier_crisp_output_calc(
					      format_imau1_classifier_crisp_output_t *desc, 
					      imau1 image_u1, 
					      data_output *data);


#endif		       
