/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file
    Kohonen CSOM Fusion Library
 */

/*------------------------------------------------------------*/
/*							      */
/* Kohonen Circular Self Organizing Map Neural Network header */
/*							      */
/* author: Petre Rodan <rodan@subdimension.com>		      */
/*							      */
/*------------------------------------------------------------*/

/**
   @defgroup fkoh Kohonen Fusion
   Functions used by the Kohonen Fusion
*/

#ifndef __csom_H__
#define __csom_H__

#ifdef __cplusplus
#  define BEGIN_C_DECLS         extern "C" {
#  define END_C_DECLS           }
#else
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif

#include "image.h"
#include "proto2D.h"

BEGIN_C_DECLS

typedef struct {
  int    input_neurons;     /* number of input level neurons*/
  int    output_neurons;    /* number of output level neurons */
  int    element;           /* number of elements in every attribute */
  int    max_neigh;         /* how many neurons share the winning (on each side of the winner) */
  int    max_epoch;         /* how many itterations to perform  */
  char   random_init;       /* use random init or not */
  char	 rescale_sw;        /* rescale the input space or not */
  float  max_learn_rate;    /* initial value of learning rate */	
  float  **x;               /* the input attributes */
  float  **w;               /* the coefficients for first element of input vector */
  float  **w_new;	    /* new coefficients */
  float  *learn_rate;       /* function of learning rate */
  float  *neigh_rate;       /* function of neighborhood */
  int    *csom_out;         /* output vector of csom  */
  /* internal data */
  int    element_current;   /* current vector */
  int    epoch;             /* current epoch  */
  float  *d;		    /* distances */
  /* debugging */
  int    *activations;      /* how many times was a neuron activated */
  float  evolution;         /* the changes of coefficients between 2 epochs */
} csom_t;


extern param *csom_lect(csom_t *csom, param *ptp, char *debq);
extern int csom_init(csom_t *csom);
extern int csom_calc(csom_t *csom);
extern int csom_destruct(csom_t *csom);

extern int imau1tocsomvect(float **data, int nb_attr, ...);
extern int csomvecttoimau1(csom_t *csom, int dimX, int dimY, imau1 *imres);

END_C_DECLS

#endif
