/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ********************** proto2D.h ************************/
/*                                                         */
/*  prototypes des fonctions de base de routines.c         */
/*  descripteurs et prototypes pour les operateurs 2D      */
/*                                                         */
/*  Derniere modif : E. TROUVE, 25/11/99                   */
/* *********************************************************/

/*pas de code avant cette ligne */
#ifndef __proto
#define __proto


#include <stdio.h>
#include "image.h"
#include <stdlib.h>

/* ******************************************************************/
/*     prototypes des fonctions comunnes de routines.c              */
/* ******************************************************************/

/* debut/fin de programme */
int param_debut(int argc, char **argv, param *ptp);
int param_fin(int argc, char **argv, param *ptp);

/* lecture de parametre */
void lec_param(char *chaine, param *ptp);
int flec_param(FILE *fp, param *ptp);
void alloc_param(param *ptp);

void lec_int(int *pt_i, char *chaine);
void lec_float(float *pt_f, char *chaine);
void lec_double(double *pt_d, char *chaine);
void lec_nom(char *pt_nom, char *chaine);
int nom_image_suivante( char *nom0, char *nomres);

/* allocation d'image */
int alloc_imau1(imau1 *im);
int alloc_imau1_as(imau1 *modele, imau1 *out);
int alloc_imarvb(imarvb *im);
int alloc_imau2(imau2 *im);
int alloc_imas2(imas2 *im);
int alloc_imau4(imau4 *im);
int alloc_imas4(imas4 *im);
int alloc_imafl(imafl *im);
int alloc_imadb(imadb *im);
int alloc_imacx4(imacx4 *im);
int alloc_imacx8(imacx8 *im);
int alloc_imacx16(imacx16 *im);

/* liberation memoire image */
int free_imau1(imau1 *im);
int free_imarvb(imarvb *im);
int free_imau2(imau2 *im);
int free_imas2(imas2 *im);
int free_imau4(imau4 *im);
int free_imas4(imas4 *im);
int free_imafl(imafl *im);
int free_imadb(imadb *im);
int free_imacx4(imacx4 *im);
int free_imacx8(imacx8 *im);
int free_imacx16(imacx16 *im);

/*conversion d'image*/
void conv_TLS_RVB(imarvb *ima_tls, imarvb *ima_rvb);
void conv_RVB_TLS(imarvb *ima_rvb, imarvb *ima_tls);


/* *************************************************************************/
/*  descripteurs et prototypes des fonctions associees aux operateurs 2D   */  
/* *************************************************************************/

/* **************************** lread_ima.c ******************************/
/* lecture d'une image complete,                                         */
/*      differents format .ras/.ima / .imw / .imf / .cxs / .cxf + .dim   */

/**\ingroup read_ima
   descripteur pour la lecture d'image*/
typedef struct {
   char nom[200];  /**< nom du fichier, sans suffix */
   char ext[10];     
}read_ima_t;

param *read_ima_lect(read_ima_t *des, param *ptp, char *debq);
int read_imau1_init(read_ima_t *des, imau1 *im);   /* .ras/.ima  */
int read_imau2_init(read_ima_t *des, imau2 *im);   /* .imw       */
int read_imau4_init(read_ima_t *des, imau4 *im);   /* .iml       */
int read_imafl_init(read_ima_t *des, imafl *im);   /* .imf       */
int read_imacx4_init(read_ima_t *des, imacx4 *im); /* .cxs       */
int read_imacx8_init(read_ima_t *des, imacx8 *im); /* .cxf       */
int read_imarvb_init(read_ima_t *des, imarvb *im); /* .ras       */
/* pas de read_ima_calc : lecture une fois pour toute! */


/* **************************** lwrite_ima.c ******************************/
/* ecriture d'une image complete,                                         */
/*      differents format       .ima / .imw / .imf / .cxs / .cxf + .dim   */

/** \ingroup write_ima
    descripteur pour l'criture d'image*/
typedef struct {
   FILE *pfd;       /**< pointeur fichier dimension */
   int  dfi;        /**< descripteur fichier image */
   char nom[200];   /**< nom du fichier, sans suffix */
   char ext[10];
} write_ima_t;

param *write_ima_lect(write_ima_t *des, param *ptp, char *debq);
int write_ima_init(write_ima_t *des);
/* pas de write_ima_calc : ecriture en une fois a la fin */
int write_imau1_ferm(write_ima_t *des, imau1 im);   /* .ima       */
int write_imau2_ferm(write_ima_t *des, imau2 im);   /* .imw       */
int write_imau4_ferm(write_ima_t *des, imau4 im);   /* .iml       */
int write_imafl_ferm(write_ima_t *des, imafl im);   /* .imf       */
int write_imacx4_ferm(write_ima_t *des, imacx4 im); /* .cxs       */
int write_imacx8_ferm(write_ima_t *des, imacx8 im); /* .cxf       */
int write_imarvb_ferm(write_ima_t *des, imarvb im); /* .ras       */

/* **************************** lselect_ima.c *****************************/
/* lecture d'une image bloc par bloc (1 ou plusieurs lignes)              */
/*      differents format .ras/.ima / .imw / .imf / .cxs / .cxf + .dim    */

/** \ingroup select_ima
    descripteur pour la lecture d'une image bloc par bloc*/
typedef struct {
   char nom[200];  /**< nom du fichier, sans suffix */
   char ext[10];     
   int dimX, dimY; /**< dimension initiales de l'image */
   int col0, lig0; /**< coin superieur gauche premier bloc */
   int nb_col, nb_lig; /**< nombre de colonne, nombre de ligne d'un bloc */
   int nb_bloc;    /**< nombre de bloc a lire */  
   int dfi;        /**< numero logique fichier image */
   int next_lig;   /**< indice premiere ligne bloc suivant */
   int nb_bloc_lu; /**< nombre de bloc lu */
   int header;     /**< nombre d'octets de l'entete */
   int lut_ng;     /**< presence de LUT de type "niveau de gris" */ 
} select_ima_t;

param *select_ima_lect(select_ima_t *des, param *ptp, char *debq);

int select_imau1_init(select_ima_t *des, imau1 *im);   /* .ima       */
int select_imau2_init(select_ima_t *des, imau2 *im);   /* .imw       */
int select_imau4_init(select_ima_t *des, imau4 *im);   /* .iml       */
int select_imafl_init(select_ima_t *des, imafl *im);   /* .imf       */
int select_imacx4_init(select_ima_t *des, imacx4 *im); /* .cxs       */
int select_imacx8_init(select_ima_t *des, imacx8 *im); /* .cxf       */

int select_imau1_calc(select_ima_t *des, imau1 *im);   /* .ima       */
int select_imau2_calc(select_ima_t *des, imau2 *im);   /* .imw       */
int select_imau4_calc(select_ima_t *des, imau4 *im);   /* .iml       */
int select_imafl_calc(select_ima_t *des, imafl *im);   /* .imf       */
int select_imacx4_calc(select_ima_t *des, imacx4 *im); /* .cxs       */
int select_imacx8_calc(select_ima_t *des, imacx8 *im); /* .cxf       */

/* **************************** lsave_ima.c ******************************/
/* ecriture d'une image complete,                                         */
/*      differents format       .ima / .imw / .imf / .cxs / .cxf + .dim   */

/** \ingroup save_ima
    descripteur pour l'ecriture d'une image complete*/
typedef struct { 
   FILE *pfd;       /**< pointeur fichier dimension ou donnees pour .ras */
   int  dfi;        /**< numero logique fichier image  */
   char nom[200];   /**< nom du fichier, sans suffix */
   char ext[10];    
   int  nb_lig_ecr;     /**< nombre de lignes ecrites */
   int  nb_col_ecr;     /**< nombre de colonnes ecrites */
   char  legende[200];   /**< commentaire pour .origima */
} save_ima_t;

/* lecture */
param *save_ima_lect(save_ima_t *des, param *ptp, char *debq);
/* init */
int save_imau1_init(save_ima_t *des, imau1 im); /* .ima | .ras */
int save_ima_init(save_ima_t *des);   /* autres types que u1 */
/* calcul */
int save_imau1_calc(save_ima_t *des, imau1 im);   /* .ima  */
int save_imau2_calc(save_ima_t *des, imau2 im);   /* .imw       */
int save_imau4_calc(save_ima_t *des, imau4 im);   /* .iml       */
int save_imafl_calc(save_ima_t *des, imafl im);   /* .imf       */
int save_imacx4_calc(save_ima_t *des, imacx4 im); /* .cxs       */
int save_imacx8_calc(save_ima_t *des, imacx8 im); /* .cxf       */
/* fermeture */
int save_ima_ferm(select_ima_t sel, save_ima_t des);  


/* **************************** lmoy2D.c *****************************/
/*  moyennage spatiale                                                */
/*    entree : u1, u2, fl, db                                        */
/*    sortie : u1, u2, fl, db (meme type que l'entree)               */
/* *******************************************************************/


/** \ingroup moy2D
 decripteur de l'oprateur moy2D*/
typedef struct { 
   int dimX, dimY;          /**< dimension de la fenetre de moyennage */
   pixu4 *tmpcolu4;         /**< colonne de travail pour u1, u2 */
   double *tmpcoldb;        /**< colonne de travail pour fl, db */
} moy2D_t;

param *moy2D_lect(moy2D_t *des, param *ptp, char *debq);
int moy2Du1_init(moy2D_t *des, imau1 im0, imau1 *imres);   
int moy2Du2_init(moy2D_t *des, imau2 im0, imau2 *imres);   
int moy2Dfl_init(moy2D_t *des, imafl im0, imafl *imres);   
int moy2Ddb_init(moy2D_t *des, imadb im0, imadb *imres);   
int moy2Du1_calc(moy2D_t *des, imau1 im0, imau1 *imres);   
int moy2Du2_calc(moy2D_t *des, imau2 im0, imau2 *imres);   
int moy2Dfl_calc(moy2D_t *des, imafl im0, imafl *imres);   
int moy2Ddb_calc(moy2D_t *des, imadb im0, imadb *imres);   

/* **************************** lmoycarre2D.c ************************************/
/*   moyenne des carres d'une image - calcul glissant, exact sur les bords       */
/* *******************************************************************************/
/**\ingroup moycarre2D
 descripteur de l'operateur moycarre2D*/
typedef struct {
   int dimX, dimY; /**< dimension de la fenetre de moyennage       */
   double *tab;    /**< tableau des carres pour les entiers */
   double *tmplig; /**< ligne de travail pour les carres des reels */
   double *tmpcol; /**< colonne de travail pour traitement colonne */
} moycarre2D_t;

param *moycarre2D_lect(moycarre2D_t *des, param *ptp, char *debq);
/* init */
int moycarre2Du1_init(moycarre2D_t *des, imau1 im0, imafl *imres);     
int moycarre2Du2_init(moycarre2D_t *des, imau2 im0, imafl *imres);     
int moycarre2Dfl_init(moycarre2D_t *des, imafl im0, imafl *imres);     
int moycarre2Ddb_init(moycarre2D_t *des, imadb im0, imadb *imres);     
/* calc */
int moycarre2Du1_calc(moycarre2D_t *des, imau1 im0, imafl *imres);   
int moycarre2Du2_calc(moycarre2D_t *des, imau2 im0, imafl *imres);   
int moycarre2Dfl_calc(moycarre2D_t *des, imafl im0, imafl *imres);   
int moycarre2Ddb_calc(moycarre2D_t *des, imadb im0, imadb *imres);   


/* *************************** add_color_map **************************/
/*  ajout d'une color map                                             */
/* *******************************************************************/
/** \ingroup add_colormap
 descripteur pour l'ajout d'une color map*/
typedef struct {
    char *nom; /**< nom du fichier*/
    pixu1 *lutr, *lutv, *lutb;   /**< luts de 256 valeurs */
} add_colormap_t;

param * add_colormap_lect(add_colormap_t *des, param * ptp, char * debq);
int add_colormap_init(add_colormap_t *des);
int add_colormap_calc(add_colormap_t *des, imau1 *ima);
int free_lut_u1(imau1 *im);
int alloc_lut_u1(imau1 *im);

short breshenham( imau1 *image,short x1, short y1, short x2,short y2,short intensity);

/*pas de code aprs cette ligne*/
#endif
