/* **************************** lessai.c ******************************/
/*                                                                    */
/* ********************************************************************/

/** \defgroup essai

operateur essai : exemple d'operateur type.

Illustration du decoupage d'un operateur en 3 fonctions (lect, init, calc).
Ici simple recopie de l'image entrante dans l'image sortant


Type entree : imau1 | ... (selon les types)\n
Type sortie : imau1 | ... (respectivement)

\author E. Trouve
\version 1.0 (19/10/99); Include : proto2D.h, essai.h
\see messai_XX,  XX = u1 | ... .
*/



#include "image.h"
#include "proto2D.h"
#include "essai.h"
#include <stdlib.h>

/* *************************  LECTURE  *******************************/
/** \ingroup essai

    lecture des parametres de l'operateur essai. \n
    dimX, dimY : exemple d'entiers : nombre de colonnes, de lignes d'un masque\n
    gain : exemple de reel\n
    forme : exemple de chaine de charactere : rectangle/triangle
    
    @param des pointeur descripteur     
    @param ptp pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h, essai.h
*/
param *essai_lect( essai_t *des, param *ptp, char *debq){
  char question[500];

  sprintf(question, "%s dimX (entier)", debq); 
  lec_param(question, ptp);
  des->dimX = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s dimY (entier)", debq); 
  lec_param(question, ptp);
  des->dimY = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s gain (reel)", debq); 
  lec_param(question, ptp);
  des->gain = (float)atof(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s forme (nom)", debq); 
  lec_param(question, ptp);
  strcpy(des->forme, ptp->rep);
  ptp = ptp->next;

  return(ptp);
}

/* *************************  INITIALISATION  ***************************/
/** \ingroup essai

    XX = u1 | etc : initialisation de l'operateur essai.

    allocation image resultat et images de travail, 
    calcul des coeff du masque
    
    @param des  pointeur descripteur
    @param im0  image initiale (taille connue)
    @param imres  pointeur image resultat
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h, essai.h
*/
int essaiu1_init(essai_t *des, imau1 im0, imau1 *imres){
  int i, j;
 
  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s_essai_%dx%d", im0.nom, des->dimX, des->dimY);
  alloc_imau1(imres);

  /* variables de l'operateur */
  des->msq.nc = des->dimX;
  des->msq.nr = des->dimY;
  alloc_imafl(&(des->msq));

  /* calcul du masque selon la forme */
  if( strcmp(des->forme, "rectangle") == 0 ){
      printf("\n>>essaiu1_init : masque %s %dx%d\n", 
	     des->forme, des->dimX, des->dimY); 
      for(j=0; j<des->dimY; j++)
	  for(i=0; i<des->dimX; i++){
	      des->msq.p[j][i] = des->gain;
	  }
  }
  else if( strcmp(des->forme, "triangle") == 0 ){
      printf("\n>>essaiu1_init : masque %s %dx%d\n", 
	     des->forme, des->dimX, des->dimY); 
      /* ... */
  }
  else{
      printf("\n>>ERREUR essaiu1_init : masque %s inconnu\n", 
	     des->forme, des->dimX, des->dimY); 
      exit(1);
  }    
}


/* *************************  CALCUL  ***************************/
/** \ingroup essai
    
    XX = u1 | etc : calcul de l'operateur essai.

    Calcul du resultat de l'operateur essai (ici simple recopie!)

    @param des  pointeur descripteur
    @param im0  image initiale 
    @param imres  pointeur image resultat
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h, essai.h
*/
int essaiu1_calc(essai_t *des, imau1 im0, imau1 *imres){
  int i, j;
      
  for(j=0; j<im0.nr; j++)
      for(i=0; i<im0.nc; i++)
	  imres->p[j][i] = im0.p[j][i];

}

