/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/


/*pas de code avant cette ligne*/
#ifndef __choquet
#define __choquet

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */




/* constantes */
#define TAILLE_NOM_FICHIER 20
#define AUCUNE_ERREUR 0
#define ERREUR_CHOQUET_FILE 1
#define ERREUR_CHOQUET_LIGNE_MATRICE 2
#define ERREUR_CHOQUET_COLONNE_MATRICE 3
#define ERREUR_CHOQUET_NU 4
#define ERREUR_CHOQUET_INTERACTION 5


/* definition des types */
typedef float *parametre;

/**
\ingroup choquet_interaction
descripteur pour l'operateur sur le calcul de l'intgrale de Choquet
*/
typedef struct	{
  char fichier[TAILLE_NOM_FICHIER]; /**< Nom du fichier contenant les interactions */
  int nbcoeff; /**< nombre de critres */
  int erreur; /**< valeur de retour en cas d'erreur */
  parametre *ptparam; /**< Matrice allouer en mmoire pour le stockage des interactions*/
}choquet_interaction_t;


/* declaration des fonctions : prototypes */

/*lecture du nom du fichier contenant les parametres*/
param *choquet_interaction_lect(choquet_interaction_t *desc, param *ptp, char *debq);

/*initialisation des valeurs des parametres*/
int choquet_interaction_init(choquet_interaction_t *choquet_interaction_desc);

/*calcul de la valeur de l integrale de Choquet pour une serie de valeurs donnee*/
int choquet_interaction_calc(choquet_interaction_t *choquet_interaction_desc,float *donnee,float *resultat);

/* liberation de l'espace memoire */
int choquet_interaction_ferm(choquet_interaction_t *desc4);


/*pas de code apres cette ligne*/
#endif
