/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ****************************************************************** */
/*                         classifier.h                               */
/* ****************************************************************** */
/*       Prototype de structure pour les entrees et sorties           */
/*                                                                    */
/*                                                                    */
/* Projet 120h :                                                      */
/* FORMALISATION des ETAPES de CLASSIFICATION en FUSION INFORMATIONS  */
/*                                                                    */
/*                                                                    */
/* ****************************************************************** */
/* ****************************************************************** */
/*                                                                    */
/* Nous dcrirons dans ce fichier les structures et les types qui     */
/* seront utiliss pour les donnes d'entre, de sortie,              */
/* d'apprentissage et d'valuation.                                   */
/*                                                                    */
/* ****************************************************************** */
/* ****************************************************************** */
/*                                                                    */
/* Creer le: 05/11/03                                                 */
/* Modifier le 17/11/03                                               */
/* Crer par RAMASSO Emmanuel & JULLIEN Sylvie                       */
/*                                                                    */
/* ****************************************************************** */

#ifndef __classifier
#define __classifier


#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <stdio.h>

/*! 
\ingroup base
\brief Donnes nettes.

Cette constante permet d'affirmer que les donnes traites sont CRISP 
c'est  dire nettes.  */
#define CRISP_CHOICE 0

/*!  
\ingroup base
\brief Donnes floues.

Cette constante permet d'affirmer que les donnes traites sont FUZZY 
c'est  dire floues.  */
#define FUZZY_CHOICE 1         


/*!  
\ingroup base
\brief Pointeur sur des flottants.

Ce type explicite le type de donnes traites,  savoir un ensemble de 
donnes flottantes mises sous la forme d'un vecteur. */
typedef float *vector;


/*!  
\ingroup base
\brief Structure de CHOIX entre donnes nettes et floues. 

Elle permet d'allouer en mmoire un espace qui peut contenir SOIT un 
pointeur d'entier (16 bits) SOIT un pointeur sur un pointeur (matrice) 
de flottants (32 bits). Ainsi l'application de l'utilisateur peut 
traiter soit des donnes nettes (entires) soit des donnes floues 
(flottantes). */
union result{
  int *crisp;    /*!< Le pointeur sur un vecteur de sortie de donnes 
		   nettes.*/
  vector *fuzzy; /*!< Le pointeur sur une matrice d'entre OU de sortie 
		   contenant des donnes floues. */
            };


/*!  
\ingroup base
\brief Structure des donnes d'entre du classifieur.

En fonction des applications, tous ses champs ou seulement une partie 
de ses champs peuvent tre utiliss. Certains peuvent tre allous 
par des procdures cres pour cela (suivre les liens). */
typedef struct {

    int nb_attr;             /*!< \brief Le nombre d'attributs d'entre.
			     Il n'est pas forcment connu a priori : il est 
			     dduit par la fonction de calcul de l'oprateur 
			     de formatage d'images "vecteur par vecteur", 
			     laquelle,  chaque appel, incrmente ce nombre. 
			     S'il est connu et fix, alors il faut utiliser 
			     la fonction de calcul de l'oprateur de formatage 
			     d'image "bloc par bloc". Si les attributs d'entres 
			     du classifieur ne sont pas des images ce champ doit 
			     tre renseign manuellement.*/

    int nb_pts;              /*!< \brief Le dbut d'une srie de points  traiter. 

			     Ce champ peut tre utilis pour distinguer les points 
			     d'apprentissage et les points d'valuation. Ce champ 
			     est  utiliser conjointement avec le champ "end".*/

    int begin;               /*!< \brief Le dbut d'une srie de points  traiter. 

			     Ce champ peut tre utilis pour distinguer les points 
			     d'apprentissage et les points d'valuation. Ce champ 
			     est  utiliser conjointement avec le champ "end".*/

    int end;                 /*!< \brief La fin d'une srie de points  traiter.
 
			     Ce champ peut tre utilis pour distinguer les points 
			     d'apprentissage et les points d'valuation. Ce champ 
			     est  utiliser conjointement avec le champ "begin".*/

    int equ_nb_rows;         /*!< \brief Nombre de lignes.

			     Dans le cas d'attributs de type image, c'est le nombre 
			     de lignes. */

    int equ_nb_cols;         /*!< \brief Nombre de colonnes.

			     Dans le cas d'attributs de type image, c'est le nombre 
			     de colonnes. */

    int equ_nb_depth;        /*!< \brief Nombre de pixels en profondeur.

			     Dans le cas d'attributs de type image  trois dimensions,
			     c'est le nombre de pixels en profondeur.*/
      
    vector *attributes;      /*!< \brief Le pointeur sur la matrice des donnes 
			     d'entres (ensemble d'attributs).

			     Il sera allou puis rempli par, respectivement, 
			     l'initialisation du formatage et le calcul du formatage. 
			     Si les attributs d'entre du classifieur ne sont pas 
			     des images alors ce champ doit tre renseign manuellement.*/

             }data_input;


/*! 
\ingroup base
\brief Structure des donnes de sortie du classifieur. 

L'oprateur utilis pour la classification doit se charger de mettre  jour le champ 
"classes" qui contient les donnes classifies sous une forme de matrices 2x2 avec 
les valeurs d'appartenance  chaque classe. Dans le cas flou, chaque case de cette 
matrice peut avoir une valeur non nulle. Dans le net, une seule des cases possde 
une valeur non nulle.*/

typedef struct {
   int nb_class;            /*!< \brief Le nombre de classes  rechercher. */
   int nb_pts;              /*!< \brief Le nombre de points traits. */
   int type;                /*!< \brief Les donnes floues ou nettes. */
   int equ_nb_rows;         /*!< \brief Nombre de lignes. */
   int equ_nb_cols;         /*!< \brief Nombre de colonnes. */
   int equ_nb_depth;        /*!< \brief Nombre de pixels en profondeur.*/
   union result classes;    /*!< \brief Le pointeur sur la matrice de donne. */
}data_output;


/*!  
\ingroup base
\brief Donnes d'apprentissage et d'valuation.

Cette structure de donnes permet de dfinir deux ensembles distincts : un pour les 
donnes d'apprentissage et un pour les donnes d'valuation. Lorsqu'un classifieur 
est supervis, il a besoin d'un ensemble (set) de donnes d'apprentissage qu'il utilise 
(input) pour crer un ensemble de donnes de rfrence (output) qui lui servira  
classifier les attributs. Quant  l'valuation des classifieurs, il est ncessaire 
d'avoir une structure qui contiendra les donnes  valuer et les rsultats de cette 
valuation.*/

struct reference_set{

   data_input input_ref;   /*!< \brief Entres de rfrence.

			   C'est une structure qui contiendra les donnes d'entres 
			   pour la phase d'apprentissage ou d'valuation du classifieur. */


   data_output output_ref; /*!< \brief Sorties de rfrence.

			   C'est une structure qui contiendra les donnes de sorties du 
			   classifieur aprs la phase d'apprentissage ou d'valuation. */
                   };


/*!  
\ingroup base
\brief Donnes d'apprentissage.

Dfinition du type "ensemble des donnes d'apprentissage E/S" */

typedef struct reference_set learning_set;

/*!  
\ingroup base
\brief Donnes d'valuation.

Dfinition du type "ensemble des donnes d'valuation E/S" */

typedef struct reference_set evaluation_set;



int alloc_classifier_data_input_one_by_one(data_input *data);
int alloc_classifier_data_input_completely(data_input *data);
int alloc_classifier_data_output(data_output *dataO);
int data_output_crisp_to_imageu1(data_output dataOC, imau1 *image_u1);
int data_output_fuzzy_to_imageu1(data_output dataOF,imau1 **image_u1);
int data_output_fuzzy_to_imagefl(data_output dataOF, imafl *image_fl);
int free_data_input(data_input *dataI);
int free_data_output(data_output *dataO);
int normaliser_data_input_sur_a_b(float a, float b, data_input in, data_input *out);
int normaliser_data_output_sur_a_b(float a, float b, data_output in, data_output *out);

#endif
