/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************************************************* */
/*                defuzzyfication_voisinage_3x3.h                */
/* ************************************************************* */
/*                                                               */
/*      Descripteurs et prototypes associs  l'oprateur        */
/*           de dfuzzification avec voisinage 3x3               */
/*                                                               */
/*  Dernire modif : Ramasso E. & Jullien S.  08/12/03           */
/*                                                               */
/* ************************************************************* */

#ifndef __defuzzyfication_voisinage_3x3
#define __defuzzyfication_voisinage_3x3

/* oprateur de dfuzzyfication avec voisinage 3x3 */
#include "image.h"
#include "proto2D.h"
#include <stdlib.h>





/**
  \ingroup defuzz  
  \brief descripteur_ de l'oprateur de dfuzzyfication

  Pour dfuzzyfier une structure en prenant en compte son voisinage 3x3, 
  nous avons besoin de plusieurs variables :
   - max_coeff_appartenance:  Le plus grand coefficient d'appartenance
   - somme_coeff_appartenance: La somme des coefficients d'appartenance
   - nb_data_incertaines_initiales: Le nombre de donnes incertaines avant 
   l'utilisation d'une fentre 3x3 
   - nb_data_incertaines_finales: Le nombre de donnes incertaines aprs 
   l'utilisation d'une fentre 3x3 
   - position_data_incertaines: La position d'une donnes incertaines
   - nb_classes: Le nombre de classes entrant en jeu dans la defuzzyfication

  */
typedef struct {
  float max_coeff_appartenance;      /*!< Le plus grand coefficient d'appartenance */

  float *somme_coeff_appartenance;   /*!< La somme des coefficients d'appartenance */

  int nb_data_incertaines_initiales; /*!< Le nombre de donnes incertaines 
				       (c'est  dire avec plusieurs max) avant fentre 3x3  */

  int nb_data_incertaines_finales;   /*!< Le nombre de donnes incertaines 
				       (c'est  dire avec plusieurs max) aprs fentre 3x3 */

  int *position_data_incertaines;    /*!< La position d'une donnes incertaines 
				       (1 si data incertaines et 0 sinon) pour post traitement */

  int nb_classes;                    /*!< Le nombre de classes entrant en jeu dans la 
				       defuzzyfication*/

}defuzzyfication_voisinage_3x3_t;







param *defuzzyfication_voisinage_3x3_lect(defuzzyfication_voisinage_3x3_t *desc, 
					  param *ptp, 
					  char *debq);
int defuzzyfication_voisinage_3x3_init(defuzzyfication_voisinage_3x3_t *desc, 
				       data_output dataOF , 
				       data_output *dataOC);
int defuzzyfication_voisinage_3x3_calc(defuzzyfication_voisinage_3x3_t *desc, 
				       data_output dataOF , 
				       data_output *dataOC);


#endif
