/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************************************************************* */
/*                format_imau1_classifier_input.h                            */
/* ************************************************************************* */
/*          Formatage d'une image de type U1 en structure d'entre           */
/*                     pour un classifieur                                   */
/*                                                                           */
/* ************************************************************************* */
/*                                                                           */
/*  Dernire modification: 08/12/03                                          */
/*  Crer par: Ramasso E. & Jullien S.                                       */
/*                                                                           */
/* ************************************************************************* */


#ifndef __format_imau1_classifier_input
#define __format_imau1_classifier_input

#include "classifier.h"
#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <stdio.h>





/** 
    \ingroup format
    \brief Formater toute l'image.
    Si l'utilisateur souhaite prendre l'attribut dans sa totalit. 
*/
#define IMAGE_ENTIERE 0






/** 
    \ingroup format
    \brief Formater une partie de l'image.

    Si l'utilisateur souhaite slectionner une partie de l'attribut.
*/
#define IMAGE_PARTIELLE 1






/**
   \ingroup format
   \brief Le type point.

   Dfinit le type point dans un espace 2D.  
*/
typedef struct {
  int Yrow; /** \brief Ordonne d'un point.

		On considre une ordonne comme tant sur l'axe 
		vertical de l'attribut.*/

  int Xcol; /** \brief Abscisse d'un point.

		On considre une abscisse comme tant sur l'axe
		horizontal de l'attribut.*/
}point;





/**
  \ingroup format
  \brief descripteur de l'oprateur de formatage.

  L'oprateur a besoin de connatre la zone de donnes 
  (rectangulaire ou carre donc 2 points sont ncessaires) 
   slectionner sur les attributs pour remplir la structure input. 
*/
typedef struct {
  point premier_point;     /*!< \brief Coordonnes du premier coin. 

			     Le premier point (haut gauche) dfinissant 
			     le rectangle de slection des donnes de 
			     l'attribut image. */
  
  point deuxieme_point;    /*!< \brief Coordonnes du deuxime coin.
 
			     Le deuxime point (bas droite) dfinissant
			     le rectangle de slection des donnes de 
			     l'attribut image.*/
  
  int choix;               /*!< \brief Choix image ENTIERE ou image PARTIELLE.

		           Renseigne sur la zone des donnes  prendre dans 
		           l'attribut d'entres : toutes les donnes (image 
		           entire) ou une partie (image partielle). */

}format_imau1_classifier_input_t;
  




param *format_imau1_classifier_input_lect(format_imau1_classifier_input_t *desc, 
					  param *ptp, 
					  char *debq);
int format_imau1_classifier_input_init(format_imau1_classifier_input_t *desc, 
				       imau1 image_u1, 
				       data_input *data);
int format_imau1_classifier_input_calc(format_imau1_classifier_input_t *desc, 
				       imau1 image_u1, 
				       data_input *data);

#endif
