/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************* fusion_floue.h  **********************************/
/*                                                                            */
/*      descripteurs et prototypes associes a l'operateur fusion_floue        */
/*                                                                            */
/*  Derniere modif : Lionel VALET, 19/11/99                                   */
/* ****************************************************************************/

/* pas de code avant cette ligne*/
#ifndef __ff_symb2
#define __ff_symb2


#define gr_MAXI_t(i,j) ( (i) > (j) ? (i) : (j) )
#define gr_MINI_t(i,j) ( (i) < (j) ? (i) : (j) )
#define gr_SQR_t(i) ( (i) * (i) )


typedef int (*gr_PFI_t)();		//pointer to function that returns int

typedef char gr_chaine_t[100];
typedef unsigned char gr_byte_t;

/**
       Func_type is, at this moment, a value between 0 and 5 which defines the form of the membership function and indicates if the values are necessary or not:

              0: a constant membership function, it will return 0 all along the universe of discourse
              1: similar to 0, but will return 1
                                                   ___
              2: linear function with the form ___/   , it streches from 0 to 1 and requires 2 values which define it

                                               ___
              3: linear function with the form    \___, it streches from 0 to 1 and requires 2 values which define it

                                                  ____
	      4: linear function with the form __|    |__, it streches from 0 to 1 and requires 2 values which define it

                                               __      __
	      5: linear function with the form   |____|   , it streches from 0 to 1 and requires 2 values which define it
       New membership function types may be added at any time, provided that the "cf_appartenance" function is modified in order to implement them in the case structure it uses
*/

typedef struct
{
    gr_chaine_t nom;
    int type;
    float gauche;
    float mode;
    float droite;
}   gr_symbol_t;

typedef struct
{
    unsigned char * liste_in;
    unsigned char nom_out;
    float ponderation;
}gr_uneRegle_t;

typedef struct
{
    int nb_regle;
    gr_uneRegle_t * regle;
}gr_lesRegles_t;

typedef struct
{
    gr_chaine_t nom;
    unsigned char nb_fct;
    gr_symbol_t *part;
}gr_partition_t;

typedef float gr_degreApp_t;
typedef gr_degreApp_t *gr_ssEnsFlouSymb_t;

typedef struct  
{
    int nb_descripteur;
    gr_ssEnsFlouSymb_t * descript;
}gr_descripteur_t;

typedef struct
{
    unsigned char nb_entree;
    gr_partition_t *partition;
}gr_fctAppartenance_t;

/** \ingroup ff_symb
    descripteur pour l'oprateur de fusion floue*/
typedef struct {
    int oper[3];     /**< choix des operateurs */
    gr_PFI_t	op[3];		//pointeur a la fonction de calcul des operateurs
    int defuzz;      /**< choix de la methode de dfuzzification */
    gr_fctAppartenance_t fonction; /**< ensemble des fonctions d'appartenace*/
    gr_lesRegles_t rgl;            /**< base de regles */
    gr_descripteur_t des;           /**< les descripteurs floues*/
    char *nom;    /**< chaine de caractere represantant le fichier des parametres */
} gr_fusionFloue_t;

param *grClassif_lect(gr_fusionFloue_t *des, param *ptp, char *debq);
int grClassif_init(gr_fusionFloue_t *des);     
int grClassif_calc(gr_fusionFloue_t *des, float *data, float *result);     
int grClassif_ferm(gr_fusionFloue_t *des);


/*pas de code aprs cette ligne */
#endif
