/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************************************************* */
/*              fonctions_de_base_classifieurs.c                 */
/* ************************************************************* */
/*                                                               */
/*      Fonction d'allocation et de transformation               */
/*               des donnes en image                            */
/*                                                               */
/*  Dernire modif : Ramasso E. & Jullien S.  08/12/03           */
/* ************************************************************* */


#include "classifier.h"
#include "macros.h"

/** 
    \defgroup classif Classification de donnes
    page d'accueil  faire
*/

/** 
    \ingroup classif
    \defgroup base Fonctions et types de base
    page d'accueil des fonctions de bases  faire
*/



/** 
\ingroup base 
@name Fonctions d'allocation des donnes d'entre
*/
//@{
/* **********  ALLOCATION DONNEES ENTREE EN PLUSIEURS FOIS ***************/
/*!
 *    \brief Fonction d'allocation de la structure d'entre avec un nombre d'attributs inconnu
 *
 *    Cette fonction alloue l'espace mmoire ncessaire pour une structure de donnes d'entre.\n
 *    Elle permet de rajouter un attribut  la structure quelque soit le nombre d'attributs.\n
 *
 *    L'utilisateur doit avoir renseign le champ:
 *    - nb_pts : le nombre de points (valeurs)  traiter
 *   
 *    Le champ nb_attr (nombre d'attributs) doit tre incrment  chaque fois que l'on veut rajouter un attribut.
 *    Ces deux oprations seront effectues dans format_XXXXX_classifier_input_init et dans format_XXXXX_classifier_input_calc.
 *
 *    \param data pointeur sur la structure contenant les donnes d'entre
 *    \return renvoie un entier qui indique si l'opration s'est bien droule
 *    \author E. Ramasso & S. Jullien
 *    \version 01/12/03
*/
int alloc_classifier_data_input_one_by_one(data_input *data)
{
  /* alloue le champ attributes de data */
  /* le champ nb_pts doit etre renseigne par format_XXXXX_classifier_input_init */

  if(data->nb_attr == 1)
    {
      if( (data->attributes = (vector *)malloc(sizeof(vector))) == NULL )
	{
	  printf ("\n allocation attributs impossible\n");  
	  exit(1);
	}
    }

  else 
    {   
      if( (data->attributes = (vector *)realloc(data->attributes,(data->nb_attr)*(sizeof(vector)))) == NULL )
	{
	  printf ("\n allocation attributs impossible\n");  
	  exit(1);
	}
    }

 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (data->attributes[data->nb_attr - 1] = (vector)malloc((data->nb_pts)*(sizeof(float)))) == NULL)
    {
      printf ("\n allocation image impossible\n");  
      exit(1);
    }

  return 0;
}



/* **********  ALLOCATION DONNEES ENTREE EN UNE FOIS ***************/
/*!
 *    \ingroup base 
 *    \brief Fonction d'alloction de la structure d'entre avec un nombre d'attributs connu
 *
 *    Cette fonction alloue l'espace mmoire ncessaire pour une structure de donnes d'entre.\n
 *    L'utilisateur doit avoir renseign les champs:
 *    - nb_pts : le nombre de points (valeurs)  traiter
 *    - nb_attr : le nombre d'attributs que possdent chaque valeur traite
 *
 *    Ces deux oprations seront effectues dans format_XXXXX_classifier_input_init
 *
 *    \param data pointeur sur la structure contenant les donnes d'entre
 *    \return renvoie un entier qui indique si l'opration s'est bien droule
 *    \author E. Ramasso & S. Jullien
 *    \version 01/12/03
*/
int alloc_classifier_data_input_completely(data_input *data)
{
  int i;

  /* alloue le champ attributes de data */
  /* le champ nb_pts doit tre renseign par format_XXXXX_classifier_input_init */

  if(data->nb_attr != 0)
    {
      if( (data->attributes = (vector *)malloc(data->nb_attr*sizeof(vector))) == NULL )
	{
	  printf ("\n allocation attributs impossible\n");  
	  exit(1);
	}
    }
  else 
    {
      printf("champ nombre d attribut non rempli");
      exit(1);
    }

  /* on alloue une place memoire pour les donnees de chaque attribut      */

  for(i=0;i<data->nb_attr;i++)
    if( (data->attributes[i] = (vector)malloc((data->nb_pts)*(sizeof(float)))) == NULL)
      {
	printf ("\n allocation points impossible\n");  
	exit(1);
      }

  return 0;
}

//@}




/** @name Fonction d'allocation des donnes de sortie
 * Cette fonction alloue un espace mmoire pour stocker les donnes de sortie qu'elles soient floues ou nettes
 */
//@{
/* *************************  ALLOCATION DONNEES SORTIE  ***************/
/*!
 *    \ingroup base 
 *    \brief Fonction d'alloction de la structure de sortie
 *
 *    Cette fonction alloue l'espace mmoire nessecaire pour une structure de donnes de sortie.\n
 *    En fonction du type entre dans dataO : 
 *    -  FUZZY_CHOICE donnes de sortie floues
 *    -  CRISP_CHOICE donnes de sortie nettes
 *
 *    \param dataO pointeur sur la structure contenant les donnes de sortie
 *    \return renvoie un entier qui indique si l'opration s'est bien droule
 *    \author E. Ramasso & S. Jullien
 *    \version 01/12/03
*/
int alloc_classifier_data_output(data_output *dataO)
{
  int i;

  if(dataO->type == FUZZY_CHOICE)
    {
      if( (dataO->classes.fuzzy = (vector *)malloc(dataO->nb_class*sizeof(vector))) == NULL )
	{
	  printf ("\n allocation classes fuzzy sortie impossible\n");  
	  exit(1);
	}

      for(i=0;i<dataO->nb_class;i++)
	if( (dataO->classes.fuzzy[i] = (vector)malloc((dataO->nb_pts)*(sizeof(float)))) == NULL)
	  {
	    printf ("\n allocation points sortie fuzzy impossible\n");  
	    exit(1);
	  }
    }
  
  if(dataO->type == CRISP_CHOICE)
    {
      
      if( (dataO->classes.crisp = (int*)malloc((dataO->nb_pts)*(sizeof(int)))) == NULL)
	{
	  printf ("\n allocation points sortie crisp impossible\n");  
	  exit(1);
	}
    }

  return 0;
}
//@}







/** @name Fonction de transformation des donnes de sortie en image U1
 */
//@{
/* **********  TRANSFORMATION DONNEES NETTES EN IMAGE U1 ***************/
/*!
 *    \ingroup base 
 *    \brief Fonction de transposition d'une structure de donnes de sortie nettes en image U1
 *
 *    Cette fonction transforme une structure de sortie en une image U1.\n
 *    L'utilisateur doit s'assurer que tous les champs de la structure de donnes nettes sont bien renseigns.\n
 *    Surtout le champ type qui doit tre gale  CRISP_CHOICE.
 *
 *    \param dataOC pointeur sur la structure contenant les donnes de sortie nettes
 *    \param image_u1 pointeur sur image U1
 *    \return renvoie un entier qui indique si l'opration s'est bien droule
 *    \author E. Ramasso & S. Jullien
 *    \version 01/12/03
*/
int data_output_crisp_to_imageu1(data_output dataOC, imau1 *image_u1)
{
  int i,j;

  /* allouer image de sortie */
  image_u1->nr = dataOC.equ_nb_rows;
  image_u1->nc = dataOC.equ_nb_cols;
  alloc_imau1(image_u1);
 

  if(dataOC.type == CRISP_CHOICE)
    {
      /* transferer les donnees */
      for(i=0;i<dataOC.equ_nb_rows;i++)
	for(j=0;j<dataOC.equ_nb_cols;j++)
	   image_u1->p[i][j] = (pixu1)dataOC.classes.crisp[i*dataOC.equ_nb_cols + j];
    }
  else
    {
      printf("data_output_crisp_to_imageu1 : Resultat du mauvais type");
      exit(1);
    }

  return 0;
}




/* **********  TRANSFORMATION DONNEES FLOUES EN IMAGE U1 ***************/
/*!
 *    \ingroup base 
 *    \brief Fonction de transposition d'une structure de donnes de sortie floues en image U1
 *
 *    Cette fonction transforme une structure de sortie en une image U1.\n
 *    L'utilisateur doit s'assurer que tous les champs de la structure de donnes nettes sont bien renseigns :
champ type qui doit tre gale  FUZZY_CHOICE, nb lignes et nb cols.
 *    
 *    Le pointeur sur les images de sortie est allou en interne : autant d'images que de classes.
 *
 *    \param dataOF pointeur sur la structure contenant les donnes de sortie floues
 *    \param image_u1 pointeur sur pointeur sur image U1
 *    \return renvoie un entier qui indique si l'opration s'est bien droule
 *    \author E. Ramasso & S. Jullien
 *    \version 01/12/03
*/

int data_output_fuzzy_to_imageu1(data_output dataOF, imau1 **image_u1)
{
  int i,j,k;

  /* allocations des images de sorties */
  if(((*image_u1) = (imau1*)malloc(dataOF.nb_class*sizeof(imau1)))==NULL)
    {
      printf("\nErreur allocation images 2 dans data_output_fuzzy_to_imageu1 impossible\n");
      exit(1);
    }


  for(i=0;i<dataOF.nb_class;i++)
    {
      (*image_u1)[i].nc = dataOF.equ_nb_cols;
      (*image_u1)[i].nr = dataOF.equ_nb_rows;
      alloc_imau1(&((*image_u1)[i]));
    }
  
  /* remplissage */
  if(dataOF.type == FUZZY_CHOICE)
    {
      for(k=0;k<dataOF.nb_class;k++)
	for(i=0;i<dataOF.equ_nb_rows;i++)
	  for(j=0;j<dataOF.equ_nb_cols;j++)
	    (*image_u1)[k].p[i][j] = (int)(dataOF.classes.fuzzy[k][i*dataOF.equ_nb_cols + j]*255.0);
      
    }
  else
    {
      printf("data_output_fuzzy_to_imageu1 : Resultat du mauvais type");
      exit(1);
    }

  return 0;
}
//@}





/** @name Fonction de transformation des donnes de sortie en images FL
 */
//@{

/* **********  TRANSFORMATION DONNEES FLOUES EN IMAGE FL ***************/
/*!
 *    \ingroup base 
 *    \brief Fonction de transposition d'une structure de donnes de sortie floues en image FL
 *
 *    Cette fonction transforme une structure de sortie en une image FL.\n
 *    L'utilisateur doit s'assurer que tous les champs de la structure de donnes nettes sont bien renseigns.\n
 *    Surtout le champ type qui doit tre gale  FUZZY_CHOICE.
 *    De plus le pointeur sur les images U1 doit tre allou au pralable. Il doit donc avoir suffisant de place mmoire pour stocker toutes les images reprsentant les diffrentes classes.\n
 *
 *    Nous obtiendrons donc autant d'images que de classes calcules.
 *
 *    \param dataOF pointeur sur la structure contenant les donnes de sortie floues
 *    \param image_fl pointeur sur image FL
 *    \return renvoie un entier qui indique si l'opration s'est bien droule
 *    \author E. Ramasso & S. Jullien
 *    \version 01/12/03
*/
int data_output_fuzzy_to_imagefl(data_output dataOF, imafl *image_fl)
{
  int i,j,k;

  
  if(dataOF.type == FUZZY_CHOICE)
    {
      for(k=0;k<dataOF.nb_class;k++)
	for(i=0;i<dataOF.equ_nb_rows;i++)
	  for(j=0;j<dataOF.equ_nb_cols;j++)
	    image_fl[k].p[i][j] = (pixfl)dataOF.classes.fuzzy[k][i*dataOF.equ_nb_cols + j];
      
    }
  else
    {
      printf("Resultat du mauvais type");
      exit(1);
    }

  return 0;
}

//@}

/** @name Fonction de libration
 */
//@{
/* **********************LIBERATION DONNEES D ENTREE************************ */
/*! 
 *    \ingroup base 
\brief Libration des donnes d'entre

Cette fonction a besoin d'une structure data_input comme donne d'entre. Il faut que le champ nb_attr soit bien renseign pour que la libration se passe sans problme.

     \param dataI le pointeur sur la structure contenant les donnes d'entre
     \author E. Ramasso & S. Jullien
     \version 18/12/03
*/
int free_data_input(data_input *dataI)
{
  int i;
  for(i=0;i<dataI->nb_attr;i++)
    free(dataI->attributes[i]);
      

  free(dataI->attributes);
  return(0);
}
/* **********************LIBERATION DONNEES DE SORTIE************************ */
/*! 

  \ingroup base 
  \brief Libration des donnes de sortie

Cette fonction a besoin d'une structure data_output comme donne d'entre. Les champs type et nb_class(dans le cas ou les donnes sont floues) soient bien renseigns.

     \param dataO le pointeur sur la structure contenant les donnes de sortie
     \author S. Jullien & E. Ramasso
     \version 18/12/03
*/

int free_data_output(data_output *dataO)
{
  int i;

 if(dataO->type == CRISP_CHOICE)
      free(dataO->classes.crisp);
     
 else 
   {
     if(dataO->type == FUZZY_CHOICE)
       {
	 for(i=0;i<dataO->nb_class;i++)
	   free(dataO->classes.fuzzy[i]);
	 
	 free(dataO->classes.fuzzy);
       }
     
     else 
       {
	 printf("\nERREUR : free_data_output : Type inconnu \n");
	 exit(1);
       }
   }
}

/* **********************NORMALISATION DE DATA INPUT************************ */
/*! 

  \ingroup base 
  \brief Normalisation des donnes de data_input dans un intervalle donn.

  Cette fonction a besoin d'une structure data_input renseigne et d'une structure data_input pour le rsultat
qui soit allou au pralable. La data_input de sortie peut etre la meme que seule d'entre.

\param a borne inferieure de l'intervalle
\param b borne superieure de l'intervalle
\param in data_input d'entre
\param out resultat
\author E. Ramasso
\version 14/05/04
*/


int normaliser_data_input_sur_a_b(float a, float b, data_input in, data_input *out)
{
  int i,j;
  float min,max;

  if(in.nb_pts != out->nb_pts)
    ERREUR("In et Out de taille differente\n");

  if(in.nb_attr != out->nb_attr)
    ERREUR("In et Out : nb attributs differents");
  
  if(a == b)
     ERREUR("a et b sont egaux => normalisation non logique\n");

  for(j=0;j<in.nb_attr;j++)
    {
      /* chercher le max et le min */
      min = in.attributes[j][0];
      max =in.attributes[j][0];
      
      for(i=1; i<in.nb_pts;i++)
	{
	  max = (( in.attributes[j][i]) > max ?  in.attributes[j][i] : max);
	  min  = (( in.attributes[j][i]) <  min  ?  in.attributes[j][i] : min);
	  if(min == a && max == b)
	    {
	      printf("Normalisation : fin -> deja normalise sur intervalle specifie [%1.2f,%1.2f]!\n",a,b);
	      return NO_ERROR;
	    }
	}

      /* normaliser */
      for(i=0; i<in.nb_pts;i++)
	{
	  /* normaliser sur [0 1] */
	  out->attributes[j][i] = ( in.attributes[j][i] - min ) / ( max - min );
	  
	  if((a != 0 && b != 1) || (a != 1 && b != 0))
	    {
	      /* normaliser sur [a b] */
	      if(a < b) out->attributes[j][i] = out->attributes[j][i] * (b - a) + a;
	      
	      else  out->attributes[j][i] = out->attributes[j][i] * (a - b) + b; 
	      
	    }
	}	
      
    }
  
  
  if(a<b)
    printf("\nNormalisation data input sur [%1.3f,%1.3f] faite.\n",a,b);
  
  if(a>b)
    printf("\nNormalisation data input sur [%1.3f,%1.3f] faite.\n",b,a);

  return NO_ERROR;
}





/* **********************NORMALISATION DE DATA OUTPUT************************ */
/*! 

  \ingroup base 
  \brief Normalisation des donnes de data_output dans un intervalle donn.

  Cette fonction a besoin d'une structure data_output renseigne et d'une structure data_output pour le rsultat
  qui soit alloue au pralable. La data_output de sortie peut etre la meme que seule d'entre.
  
  \param a borne inferieure de l'intervalle
  \param b borne superieure de l'intervalle
  \param in data_input d'entre
  \param out resultat
  \author E. Ramasso
  \version 14/05/04
*/


int normaliser_data_output_sur_a_b(float a, float b, data_output in, data_output *out)
{
  int i,j;
  float min,max;

  if(in.nb_pts != out->nb_pts)
    ERREUR("In et Out de taille differente\n");

  if(in.type != out->type)
    ERREUR("In et Out de type differents");
  
  if(a == b)
     ERREUR("a et b sont egaux => normalisation non logique\n");


  switch(in.type)
    {
    case CRISP_CHOICE :
      /* sur chaque classe faire */
      
      /* chercher le max et le min */
      min = in.classes.crisp[0];
      max = in.classes.crisp[0];
      
      for(i=1; i<in.nb_pts;i++)
	{
	  max = (( in.classes.crisp[i]) > max ?  in.classes.crisp[i] : max);
	  min  = (( in.classes.crisp[i]) <  min  ?  in.classes.crisp[i] : min);
	  if(min == a && max == b)
	    {
	      printf("Normalisation : fin car deja normalise sur intervalle specifie [%1.2f,%1.2f]!\n",a,b);
	      return NO_ERROR;
	    }
	}
      
      /* normaliser sur [0 1] puis sur [a b]*/
      for(i=0; i<in.nb_pts;i++)
	{
	  /* normaliser sur [0 1] */
	  out->classes.crisp[i] = ( in.classes.crisp[i] - min ) / ( max - min );
	  
	  if((a != 0 && b != 1) || (a != 1 && b != 0))
	    {
	      /* normaliser sur [a b] */
	      if(a < b) out->classes.crisp[i] = out->classes.crisp[i] * (b - a) + a;

	      else  out->classes.crisp[i] = out->classes.crisp[i] * (a - b) + b; 
	      
	    }
	}	
      
      break;

    case FUZZY_CHOICE :
      
      /* sur chaque classe faire */
      for(j=0;j<in.nb_class;j++)
	{
	  /* chercher le max et le min */
	  min = in.classes.fuzzy[j][0];
	  max = in.classes.fuzzy[j][0];
	  
	  for(i=1; i<in.nb_pts;i++)
	    {
	      max = (( in.classes.fuzzy[j][i]) > max ?  in.classes.fuzzy[j][i] : max);
	      min  = (( in.classes.fuzzy[j][i]) <  min  ?  in.classes.fuzzy[j][i] : min);
	    }
	  //printf("max %f min %f\n",max,min);
	  if(min == a && max == b)
	    {
	      printf("Normalisation : fin car deja normalise sur intervalle specifie [%1.2f,%1.2f]!\n",a,b);
	      return NO_ERROR;
	    }

	  /* normaliser sur [0 1] puis sur [a b]*/
	  for(i=0; i<in.nb_pts;i++)
	    {
	      /* normaliser sur [0 1] */
	      out->classes.fuzzy[j][i] = ( in.classes.fuzzy[j][i] - min ) / ( max - min );
	      
	      if((a != 0 && b != 1) || (a != 1 && b != 0))
		{
		  /* normaliser sur [a b] */
		  if(a < b)  out->classes.fuzzy[j][i] = out->classes.fuzzy[j][i] * (b - a) + a;
		  else out->classes.fuzzy[j][i] = out->classes.fuzzy[j][i] * (a - b) + b;
		}
	    }
	}
      
      break;
      
    default : ERREUR("Type problematique\n");
    }
  
  if(a<b)
    printf("\nNormalisation data output sur [%1.3f,%1.3f] faite.\n",a,b);
  if(a>b)
    printf("\nNormalisation data output sur [%1.3f,%1.3f] faite.\n",b,a);

  return NO_ERROR;
}
 

//@}
