/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* *******************************************************************************
**		lff_symb.c
*********************************************************************************/
/*
 ensemble de fonction pour l'operateur de fusion floue symbolique
*/

#include <stdio.h>
#include <stdarg.h>
#include "image.h"
#include "proto2D.h"
#include "ff_symb.h"


int
tn_zadeh(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	*z	=	MINI(*x, *y);
	return 0;
}

int tc_zadeh(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	*z	=	MAXI(*x, *y);
	return 0;
}

int tn_proba(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	*z	=	(*x)  *  (*y);
	return 0;
}

int tc_proba(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	(*z)	=	(*x) + (*y) - (*x)*(*y);
	return 0;
}

int tn_luk(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	degre_app	tmp;
	tmp	=	*x+*y-1;
	*z	=	MAXI(tmp,0);
}

int tc_luk(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	degre_app	tmp;
	tmp	=	*x+*y;
	*z	=	MINI(tmp,1);
}

int tn_weber(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	(*z)	=	(*x == 1) ? (*y) : ((*y == 1) ? (*x) : (0));
	return 0;
}

int tc_weber(
	const	degre_app*	x,
	const	degre_app*	y,
		degre_app*	z
){
	(*z)	=	(*x == 0) ? (*y) : ((*y == 0) ? (*x) : (1));
	return 0;
}


/*initialisation*/
init(char * filename, Fct_appartenance * fct, Les_regles * rules, Descripteur *dpteur)
{
    FILE * fin;
    int i, j, tmp;
    int nb_regle=1;
    chaine var;
    /*ouverture du fichier contenant les parametres*/
    if( (fin=fopen(filename, "r")) == NULL) {
	printf("ERREUR: impossible d'ouvrir le fichier %s\n", filename);
	exit(1);
    }
	
    /*lecture et allocation du nombre d'entre*/
    printf("lecture du nombre d'entree\n");
    fscanf(fin, "%d", &tmp);
    fct->nb_entree = (unsigned char)tmp;
    if( (fct->partition=(Partition *)malloc(fct->nb_entree*sizeof(Partition))) ==NULL){ 	printf("problme d'allocation mmoire.\n"); 	exit(1);     }

    /*Pour chaque entree, lecture de son nom et de son nombre de fonction d'appartenance*/
    for(i=0; i<fct->nb_entree; i++)
	{
	  //printf("lect des entrees %d\n",i);
	    fscanf(fin, "%s %d", fct->partition[i].nom, &tmp );
	    fct->partition[i].nb_fct = (unsigned char)tmp;
	    if( (fct->partition[i].part = (Symbol *)malloc(fct->partition[i].nb_fct*sizeof(Symbol))) == NULL){
		printf("problme d'allocation mmoire.\n");
		exit(1);
	    }

	    /*Pour chaque fonction d'appartenance, lecture de son nom et de ses parametres*/
	    for(j=0; j<fct->partition[i].nb_fct; j++)
	      	fscanf(fin, "%s %f %f %f", fct->partition[i].part[j].nom,  &fct->partition[i].part[j].gauche, &fct->partition[i].part[j].mode, &fct->partition[i].part[j].droite);
	}
	
    /*allocation de la table de rgles et initialisation*/
    //printf("lecture du nombre de regle\n");
    fscanf(fin, "%d", &rules->nb_regle);
    if( (rules->regle=(Une_regle *)malloc(rules->nb_regle * sizeof(Une_regle))) ==NULL){
	printf("problme d'allocation mmoire.\n");
	exit(1);
    }
    /*allocation du nombre d'entre par regle*/
    for(i=0; i<rules->nb_regle; i++)
	if( (rules->regle[i].liste_in = (unsigned char *)calloc( fct->nb_entree-1, sizeof(unsigned char)) ) == NULL){
	    printf("problme d'allocation mmoire.\n");
	    exit(1);
	}
	
    /*lecture des regles*/
    //printf("lecture des regles\n");
    for(j=0; j<rules->nb_regle; j++) {    
      for(i=0; i<fct->nb_entree-1; i++){
	fscanf(fin, "%s", var);
	tmp = 0;
	while(strcmp(var,fct->partition[i].part[tmp].nom)!=0)
	  tmp++;
	rules->regle[j].liste_in[i] = tmp;
      }
      fscanf(fin, "%s %f", var, &rules->regle[j].ponderation );
      tmp = 0;
      while(strcmp(var,fct->partition[fct->nb_entree-1].part[tmp].nom)!=0)
	tmp++;
      rules->regle[j].nom_out = tmp;

    }

    fclose(fin);

    /*allocation des descripteurs*/
    //printf("allocation des descripteurs\n");
    dpteur->nb_descripteur = fct->nb_entree;
    if( (dpteur->descript = (ss_ens_flou_symb *)malloc(dpteur->nb_descripteur*sizeof(ss_ens_flou_symb))) == NULL) {
	    printf("problme d'allocation mmoire.\n");
	    exit(1);
	}
    for(i=0; i<fct->nb_entree; i++)
	if( (dpteur->descript[i] = (degre_app *)malloc(fct->partition[i].nb_fct*sizeof(degre_app))) == NULL) {
	    printf("problme d'allocation mmoire.\n");
	    exit(1);
	}
 
    
}

int
ff_symb_init_set_pfi(
	fusion_floue_t*	ff
){
    switch(ff->Tnorme){
	case 0:
	    ff->tn	=	tn_zadeh;
	    break;
	case 1:
	    ff->tn	=	tn_proba;
	    break;
	case 2:
	    ff->tn	=	tn_luk;
	    break;
	case 3:
	    ff->tn	=	tn_weber;
	    break;
	default:
	    printf("erreur sur le choix du type de T-norme\n");
	    exit(1);
	    break;
    }
    switch(ff->Tconorme){
	case 0:
	    ff->tcn	=	tc_zadeh;
	    break;
	case 1:
	    ff->tcn	=	tc_proba;
	    break;
	case 2:
	    ff->tcn	=	tc_luk;
	    break;
	case 3:
	    ff->tcn	=	tc_weber;
	    break;
	default:
	    printf("erreur sur le choix du type de T-norme\n");
	    exit(1);
	    break;
    }
    return 0;
}
	

/* ************************************/
float
calcul_premisse(
	Fct_appartenance*	fct,
	Descripteur*		desc,
	unsigned char*		liste,
	int (*tnorme)()
){
    int l;
    float premisse=1.0;

    /*pour tous les mots de la liste de la regles selectionnee*/
    for(l=0; l<fct->nb_entree-1; l++)
      /*mise a jour de la premisse connaissant la position du degree pour le mot l*/
      (*tnorme)(&premisse, &desc->descript[l][liste[l]], &premisse);

    return(premisse);
}

/* ************ appartenance ***********/
degre_app appartenance (float x, Symbol s)     
{
    if (x <= s.gauche)     
	if(s.gauche==s.mode)
	    return(1.0);
	else
	    return (0.0);

    if (x <= s.mode)
	return ( ( x - s.gauche ) / ( s.mode - s.gauche ) );

    if (x <= s.droite)
	if ( s.droite != s.mode )
	    return ( ( s.droite - x ) / ( s.droite - s.mode ) );
	else
	    return (1.0);

    if ( x > s.droite)
	if(s.droite==s.mode)
	    return (1.0);
	else
	    return (0.0);

    printf("Erreur dans la fonction appartenance");
    return(0.0);
}


/* *************** fuzzification *********************/
fuzzification(float *val, Fct_appartenance *fct, Descripteur *desc)
{                              
    int i, j;
   
    /* pour toutes les entrees du systemes */
    for(i=0; i<fct->nb_entree-1; i++)
	/*pour toutes les fonctions d'appartenance dcrivant l'entre i*/
	for(j=0; j<fct->partition[i].nb_fct; j++)
	    /* calcul du degre d'appartenance*/
	    desc->descript[i][j] = appartenance(val[i],fct->partition[i].part[j]);
}




/* ************ defuzzification *******/
int defuzzification(float *result, Fct_appartenance *fct, Descripteur * desc, int t) 
{
  int i; 
  float max=0.0, num=0.0, denum=0.0;
  int mot=0;

  switch(t){
  case 0:
    /* la classe max est retourne et tous les  degres vont dans result*/
    for(i=0; i<fct->partition[fct->nb_entree-1].nb_fct; i++) {
      result[i]=desc->descript[fct->nb_entree-1][i];
      if( desc->descript[fct->nb_entree-1][i] > max ){
	max = desc->descript[fct->nb_entree-1][i];
	mot = i;
      }
    }
    return(mot);
    break;
  case 1:
    /* */
    for(i=0; i<fct->partition[fct->nb_entree-1].nb_fct; i++){
      num = num + desc->descript[fct->nb_entree-1][i] * fct->partition[fct->nb_entree-1].part[i].mode;
      denum = denum + desc->descript[fct->nb_entree-1][i];
    }
    result[0] = (float)(num/denum);
    break;
  default:
    printf("type defuzzification incorrect\n");
    exit(1);
  }
}



/***************** inference *********/
int
inference(
	Fct_appartenance*	fct,
	Les_regles*		rules,
	Descripteur*		desc,
	int (*tnorme) (),
	int (*tconorme) ()
){
    int s, r;
    float premisse, combinaison, projection;

    /*Pour tous les mots definisants la variable de  sorties*/
    for (s=0; s<fct->partition[fct->nb_entree-1].nb_fct; s++)
	{
	    projection = 0.0;
	    /*test de toutes les regles*/
	    for(r=0; r<rules->nb_regle; r++)
		{
		    if( rules->regle[r].nom_out == s)
			{
			    /*si regle OK alors calcul de la premisse correspondante*/
			    premisse = calcul_premisse(fct, desc, rules->regle[r].liste_in, (*tnorme));
			    /*combinaison*/
			    (*tnorme)( &premisse, &rules->regle[r].ponderation, &combinaison);
			    /*projection*/
			    (*tconorme)( &projection, &combinaison, &projection );
			}
		}
	    desc->descript[desc->nb_descripteur-1][s] = projection;
	    //printf("%.2f/%s ", projection, fct->partition[fct->nb_entree-1].part[s].nom );
	}
}

/**
 \defgroup ff_symb fusion floue symbolique
oprateur fus_flou_symb: fusion de donne scalaire  l'aide d'un systme floue symbolique.

Cet oprateur ralise l'aggrgation de plusieurs grandeurs d'entres. Chaque entre est dfinie par plusieurs mots. Il n'y a pas de restriction sur le nombre d'entres, ni sur le nombre de mots dcrivant chaque entre. 

Cet oprateur va lire tous les paramtres d'initialisation du systme dans un fichier. La constitution de ce fichier est explique dans la doc qui suit. Le nom de ce fichier est demand  l'utilisateur par la fonction ff_symb_lect. 

Cette fusion passe par une fuzzification des entres  l'aide des fonctions d'appartenance. Pour le moment, cet oprateur ne gre que les fonctions d'appartenance triangulaires et donc dfinies par trois paramtres:


- la valeur modal: valeur corresponadant  un degr d'appartenance gale  1.
-  la valeur gauche: premire valeur se trouvant a gauche de la valeur modale et correspondant  un degr d'appartenance nul.
-  la valeur droite: premire valeur se trouvant a droite de la valeur modale et correspondant  un degr d'appartenance nul.

Il est possible d'avoir par exemple une valeur modale et une valeur gauche identique signifiant ainsi que tous les degrs d'appartenance valent 1  gauche de cette valeur modale. Ceci est valable aussi pour le cot droit. exemple de dclaration: {\bf grand 0 50 100}

Aprs la fuzzification, l'tape d'infrence permet de fusionner les reprsentations floues symboliques des entres,  l'aide d'un jeux de rgles et d'oprateurs d'agrgation (t-normes, t-conormes). La rgle d'infrence utilise ici est la rgle du {\bf modus ponens gnralise}. Le jeux de rgle doit tre dcrit dans le fichier de configuration du systme de fusion, de la manire suivante:

- Une rgle est compose d'une mot pour chaque entre, suivait d'un mot caractrisant l'tat de la sortie puis d'un coefficient de pondration compris entre 0 et 1. Il n'est pas obligatoire de rentrer la totalit des rgles. 

- Une rgle non prsente est considrer comme ayant une pondration nulle. C'est pourquoi l'utilisateur doit prciser le nombre de rgle prsent dans la base afin d'allouer l'espace ncessaire. 

- exemple de rgle: {\bf grand petit moyen 0.8} (si entre1 est grand et entre2 est petit alors sortie est moyen avec un coef de pondration gal  0.8). 
- Il faut absolument respecter l'ordre d'apparition des entres dans la rgles.

Choix des oprateurs d'agrgations:

\begin{center}
\begin{tabular}{cccc}
num & nom & T-normes & T-conorme \\
0 & Zadeh & min(x,y) & max(x,y) \\
1 & probabiliste & x*y & x+y-xy \\
2 & Lukasiewicz & max(x+y-1,0) & min(x+y,1)\\
3 & Weber & & \\
\end{tabular}
\end{center}
\\
Voici les diffrentes mthodes de dfuzzification utilisables pour le moment:
\begin{itemize}
\item {\bf 0:} Recherche du mot de sortie ayant le maximum de degr d'appartenance. C'est une classification. Le numro de la classe est retourn par la fonction ff_symb_calc(). Dans la variable sortie se trouve les degrs d'appartenances pour chacunes des classes.
\item {\bf 1:} Calculate the output value by means of the weighted mean of the maximums of the output fuzzy sets. The weights are the corresponding membership degrees. The result is given in the parameter sortie[0] of the function ff_symb_calc().
\end{itemize}

Voici la composition exact du fichier d'initialisation de l'oprateur:\\
{\small
Nombre_de_variable_floue(Nb entre + 1 sortie)\\
\\
Nom_entre_1 Nb_mot\\
mot1 g m d\\
mot2 g m d\\
... \\
\\
Nb_rgle\\
mot_entree1 mot_entree2 ... mot_sortie pondration\\
...
}\\ 
\\
Descripteur :
\begin{itemize}
    \item int Tnorme:         choix de l'operateur T-normes 
    \item  int Tconorme:      choix de l'operateur T-conormes 
    \item int defuzz:         choix de la methode de dfuzzification 
    \item Fct_appartenance    fonction:  ensemble des fonctions d'appartenace 
    \item Les_regles rgl:     base de regles 
    \item Descripteur des:    les descripteurs floues 
    \item float *data:        les valeurs des variables d'entres 
    \item char *nom:          chaine de caractere represantant le fichier des paramtres 
\end{itemize}

@name lfusion_floue_symb
@author Lionel VALET
@version 1.0 (02/12/99); Include : ff_symb.h
@see mfus_flou_symb
*/





/** \ingroup ff_symb
    lecture des parametres de l'operateur ff_symb. 
    - nom: chaine de caractere represantant le fichier des paramtres
    - Tnorme: choix de l'operateur T-normes
    - Tconorme: choix de l'operateur T-conormes
    - defuzz: choix de la methode de dfuzzification
    @param d  pointeur descripteur     
    @param ptp  pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @author L. VALET
    @version 1.0 (02/12/99); Include : ff_symb.h
  */
param *ff_symb_lect( fusion_floue_t *d, param *ptp, char *debq)
{
    char question[500];

    sprintf(question, "%s fichier d'initialisation", debq); 
    lec_param(question, ptp);
    d->nom = ptp->rep;
    ptp = ptp->next;

    sprintf(question, "%s type de Tnorme (0-3)", debq);
    lec_param(question, ptp);
    d->Tnorme = atoi(ptp->rep);
    ptp = ptp->next;

    sprintf(question, "%s type de Tconorme (0-3)", debq);
    lec_param(question, ptp);
    d->Tconorme = atoi(ptp->rep);
    ptp = ptp->next;

    sprintf(question, "%s type de dfuzzification (0-1)", debq);
    lec_param(question, ptp);
    d->defuzz = atoi(ptp->rep);
    ptp = ptp->next;

    return(ptp);
}



/** \ingroup ff_symb
    initialisation de l'operateur ff_symb.
    lecture du fichier de configuration.
    @param des  pointeur descripteur
    @author L. VALET
    @version 1.0 (19/10/99); Include : ff_sym.h
*/
int ff_symb_init(fusion_floue_t *d){
    init(d->nom, &d->fonction, &d->rgl, &d->des);
    ff_symb_init_set_pfi( d);
}



/** \ingroup ff_symb
    Fusion floue symbolique .
    @param des  pointeur descripteur
    @param data  pointeur sur un tableau de float contenant les donnes d'entre
    @param result pointeur sur le resultat, celui-ci depend de la methode de deffuzzification utilise. Il est allour par l'operateur.
    @author L. VALET
    @version 1.0 (19/10/99); Include : distance_ima.h
*/
int
ff_symb_calc(
		fusion_floue_t*	d,
		degre_app*	data,
		degre_app*	result
){
    fuzzification(data, &d->fonction, &d->des);
    inference(	&d->fonction,
		&d->rgl,
		&d->des,
		d->tn,
		d->tcn
		);
    return( defuzzification(result, &d->fonction, &d->des, d->defuzz) );
}


/** \ingroup ff_symb
    Liberation de la memoire de l'operateur lff_symb.
    @param d  pointeur descripteur
    @author L. VALET
    @version 1.0 (19/10/99); Include : distance_ima.h
*/
int
ff_symb_ferm(
	fusion_floue_t *d
){
	int	i;				//iterator
	
	free( d->fonction.partition);
	for ( i = 0; i < d->fonction.nb_entree; i++) {
		free( d->fonction.partition[i].part );
	}
	free( d->rgl.regle );
	free( d->des.descript );
	for ( i = 0; i < d->fonction.nb_entree; i++) {
		free(d->des.descript[i]);
	}
	return 0;
}
