/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************************************************************* */
/*               lformat_imau1_classifier_input.c                            */
/* ************************************************************************* */
/*          Formatage d'une image de type U1 en structure d'entre           */
/*                     pour un classifieur                                   */
/*                                                                           */
/* ************************************************************************* */
/*                                                                           */
/*  Dernire modification: 08/12/03                                          */
/*  Crer par: Ramasso E. & Jullien S.                                       */
/*                                                                           */
/* ************************************************************************* */


/**
\ingroup classif 
\defgroup format formatage de donnes d'entres  partir d'images
*/


#include "format_imau1_classifier_input.h"
#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <stdio.h>



/** 
    \ingroup format
    @name Oprateur
 * \n
 * Cet oprateur permet de transformer une image de type imau1 en 
 * donne d'entre de classifieur de type input_data.\n
 * Le nombre d'attributs (images d'entre de classifieurs) tant suprieur 
 *  1, cet oprateur est  appeler de manire itrative. \n
Cet oprateur autorise la slection d'une zone rectangulaire de l'attribut 
 * (au lieu de classifier toute l'image). Il faut alors spcifier les 
 * coordonnes des deux points opposs du rectangle de slection. Dans 
 * le cas ou les coordonnes sont incorrectes, l'attribut est pris dans 
 * sa totalit. 
 */
//@{

/* *************************  LECTURE  *******************************/
/*! \brief Fonction lisant les paramtres de l'oprateur format_imau1_classifier_input 
 *
 *    Lecture des paramtres de l'oprateur format_imau1_classifier_input:
 *   - premier_point: coordonnes du premier point dfinissant le rectangle de slection
 *   - deuxieme_point: coordonnes du deuxime point dfinissant le rectangle de slection
 *   - choix: dfinit si l'utilisateur utilise pour les attributs l'image entire ou une partie de l'image
 * 
 *  
 *  \param desc pointeur descripteur     
 *  \param ptp pointeur sur paramtre courant dj allou (a lire),     
 *  \param debq pointeur de chane de caractres place en debut de question
 *  \return renvoie nouveau pointeur de paramtres dj allou (pour la suite) 
 *  \author E. Ramasso & S. Jullien
 *  \version 08/12/03 
*/
param *format_imau1_classifier_input_lect(format_imau1_classifier_input_t *desc, param *ptp, char *debq)
{
  char question[500];
  int choix;

  sprintf(question, "%s Image entiere (0) ou partielle (1)", debq);
  lec_param(question, ptp);
  desc->choix = atoi(ptp->rep);
  ptp = ptp->next;


  switch(desc->choix)
    {
      /* si une partie des donnees */
    case IMAGE_PARTIELLE :

      sprintf(question, "%s Coordonnes X du premier point (colonne : 0 a nb colonnes - 1)", debq);
      lec_param(question, ptp);
      desc->premier_point.Xcol = atoi(ptp->rep);
      ptp = ptp->next;
      
      sprintf(question, "%s Coordonnes Y du premier point (ligne : 0 a nb lignes - 1)", debq);
      lec_param(question, ptp);
      desc->premier_point.Yrow = atoi(ptp->rep);
      ptp = ptp->next;
      
      sprintf(question, "%s Coordonnes X deuxieme point (colonne : 0 a nb colonnes - 1)", debq);
      lec_param(question, ptp);
      desc->deuxieme_point.Xcol = atoi(ptp->rep);
      ptp = ptp->next;
      
      sprintf(question, "%s Coordonnes Y deuxieme point (ligne : 0 a nb lignes - 1) ", debq);
      lec_param(question, ptp);
      desc->deuxieme_point.Yrow = atoi(ptp->rep);
      ptp = ptp->next;
      
      break;


      /* mise a jour dans init */
    case IMAGE_ENTIERE : break;

      /* sinon erreur */
    default : 
      printf("\nMauvaise reponse\n"); 
      exit(1);

    }

  return ptp;
}


/* *************************  INITIALISATION  ***************************/
/*! \brief Fonction initialisant l'oprateur format_imau1_classifier_input
 *
 *    Vrification de la validit de l'espace slectionn par l'utilisateur.\n
 *    Initialisation dans la structure de sortie:
 *     - du nombre de lignes
 *     - du nombre de colonnes
 *     - du nombre de points
 *     - du nombre d'attributs
 *     - du pointeur sur les attributs
 *
 *    Allocation de la matrice contenant les centodes
 *
 *    \param desc pointeur descripteur
 *    \param image_u1 une image de type U1 qui contient les donnes d'un attribut
 *    \param data pointeur sur la structure contenant les donnes d'entre
 *    \return renvoie un entier qui indique si l'operation s'est bien droule 
 *    \author E. Ramasso & S. Jullien
 *    \version 01/12/03
*/
int format_imau1_classifier_input_init(format_imau1_classifier_input_t *desc, imau1 image_u1, data_input *data)
{
  printf("\nInit format...");
  switch(desc->choix)
    {
      /* toute l image */
    case IMAGE_ENTIERE :
      desc->premier_point.Xcol = 0;
      desc->premier_point.Yrow = 0;
      desc->deuxieme_point.Xcol = image_u1.nc - 1;
      desc->deuxieme_point.Yrow = image_u1.nr - 1;
      break;

      /* une partie de l image => verif dim */
    case IMAGE_PARTIELLE :
      if(!(desc->premier_point.Xcol >= 0 && desc->premier_point.Xcol < image_u1.nc &&
	   desc->premier_point.Yrow >= 0 && desc->premier_point.Yrow < image_u1.nr &&
	   desc->deuxieme_point.Xcol >= 0 && desc->deuxieme_point.Xcol < image_u1.nc &&
	   desc->deuxieme_point.Yrow >= 0 && desc->deuxieme_point.Yrow < image_u1.nr &&
	   desc->deuxieme_point.Xcol - desc->premier_point.Xcol >= 0 &&
	   desc->deuxieme_point.Yrow - desc->premier_point.Yrow >= 0))
	{
	  printf("\nErreur dimension - coordonnees points invalides\n");
	  exit(1);
	}
      
      break;

    default :   
      printf("\nErreur dans init : choix non valide\n"); 
      exit(1);
    }

  
  /* taille de la nouvelle image */
  data->equ_nb_cols = desc->deuxieme_point.Xcol - desc->premier_point.Xcol + 1;
  data->equ_nb_rows = desc->deuxieme_point.Yrow - desc->premier_point.Yrow + 1;

  /* l'image image_u1 a ete prealablement lue et initialisee => nc et nr connus */
  data->nb_pts = data->equ_nb_cols*data->equ_nb_rows;

  /* mise a jour des champs de data_input */
  data->begin = 0;
  data->end = data->nb_pts;

  /* init pointeur attributes */
  data->attributes = NULL;
  
  /* init nb attributs */
  data->nb_attr = 0;
  
  printf("OK.");
  return 0;
}

/* *************************  CALCUL  ***************************/
/*!
    \brief Fonction transformant une image de type U1 en structure de sortie pour le classifieur

    A chaque fois que cette fonction est lance, on alloue un vecteur qui contiendra les donnes du nouvel attribut.\n
    Les donnes du nouvel attribut sont contenus dans l'image U1. Donc on transfert ces donnes dans la structure.\n
    Donc si l'utilisateur veut entrer trois attributs il appellera trois fois cette fonction.
  
    \param desc pointeur descripteur
    \param image_u1 une image de type U1 qui contient les donnes d'un attribut
    \param data pointeur sur la structure contenant les donnes d'entre
    \return renvoie un entier qui indique si l'opration s'est bien droule 
    \author E. Ramasso & S. Jullien
    \version 01/12/03
*/
int format_imau1_classifier_input_calc(format_imau1_classifier_input_t *desc, imau1 image_u1, data_input *data)
{
  int i,j,k;
   printf("Calcul format...");

  /* incrementation nb attributs */
  data->nb_attr += 1;

  /* allocation prealablement effectuee de data->attributes */
  alloc_classifier_data_input_one_by_one(data);

  /* transfert des donnees de l image dans la structure d entree */
  k=0;
   for(i=0;i<data->equ_nb_rows;i++)
     for(j=0;j<data->equ_nb_cols;j++)
       {
	 data->attributes[data->nb_attr - 1][k] =(float)(image_u1.p[desc->premier_point.Yrow+i][desc->premier_point.Xcol+j]);
	 k++;
       }
   printf("OK\n");
 
   return 0;
}

//@}

