/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************************************************************* */
/*               lformat_learning_set_from_imau1.c                            */
/* ************************************************************************* */
/*          Oprateur crant une structure de donnes de type */
/* learning_set  partir d'images et d'un masque de type imau1 */
/*                                                                           */
/* ************************************************************************* */
/*                                                                           */
/*  Derniere modification: 02/04/2004                                         */
/*  Creer par: Ramasso E.                                        */
/*                                                                           */
/* ************************************************************************* */

/**
\ingroup classif 
\defgroup format_learn formatage de donnes d'apprentissage  partir d'images
*/


#include "format_learning_set_from_imau1.h"
#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <stdio.h>




/** 
    \ingroup format_learn
    @name Oprateur
    \n
    L'oprateur format_learning_set_from_imau1 permet de crer
    une structure de donnes de type learning_set  partir d'attributs
    de type imau1 et d'un masque de type imau1.\n
    
    Lors de la lecture, il est possible que les attributs entre eux et 
    le masque n'aient pas les memes dimensions. C'est pourquoi, l'oprateur 
    format_imau1_classifier_input est appel afin de slectionner des 
    zones de pixels (optionnel mais utile dans le cas ou les images n'ont pas 
    les memes dimensions). Lors de cette slection, les zones doivent avoir 
    les memes dimensions.\n

    L'initialisation dtermine automatiquement les labels et le nombre de points par
    label. C'est dans l'init qu'il est demand la classe  sparer (automatiquement
    il est propos le label avec le plus grand nombre points mais il est possible de
    modifier ce choix).

    Le descripteur stocke temporairement les attributs sous leur forme
    initiale (image imau1) ainsi que les structures associes (read_ima_t).
    Mais elles sont libres dans l'init. 
    \n
*/
//@{


  


/* *************************  LECTURE  *******************************/
/**
* Lors de la lecture, le nombre d'attributs, le chemin des attributs et le chemin
* du masque sont demands. De plus, il est ncessaire de savoir si les attributs
* et le masque ont les memes dimensions afin de choisir les images dans leur
* totalit (choix 0) ou la slection d'une zone de pixels (choix 1) lors de l'appel de 
* format_imau1_classifier_input_lect.\n
*
*
*  \param desc_format pointeur sur le descripteur     
*  \param ptp pointeur sur paramtre courant dj allou (a lire),     
*  \param debq pointeur de chane de caractres place en debut de question
*  \return renvoie nouveau pointeur de paramtres dj allou (pour la suite) 
*  \author E. Ramasso 
*  \version 1.0 - 02/04/2004 
*/


  param* format_learning_set_from_imau1_lect(
				  format_learning_set_from_imau1_t *desc_format, 
				  param *ptp,
				  char *debq					   
				  )
  {
    char question[500];
    int i;
    
    /* nb attributs */
    sprintf(question, "%s Nombre d'attributs ", debq);
    lec_param(question, ptp);
    desc_format->nb_att = atoi(ptp->rep);
    ptp = ptp->next;
    
    if(desc_format->nb_att == 0)
      {
	printf("Nb Att = 0!!\n\n");
	exit(1);
      }
    
    /* lire attributs */
    if((desc_format->att_rea = (read_ima_t*)malloc(desc_format->nb_att*sizeof(read_ima_t)))==NULL) 
      {
	printf("Alloc structures %d images imp.",desc_format->nb_att);
	exit(1);
      }
    
    for(i=0;i<desc_format->nb_att;i++)
      {
	printf("\nChemin de l'attributs %d/%d :",i+1,desc_format->nb_att);	
	ptp = read_ima_lect(&desc_format->att_rea[i], ptp, ">> Chemin de l'attribut (.ras) ");
      }
    
    /* formatage attributs entree en data input*/
    ptp = format_imau1_classifier_input_lect(&desc_format->format_des_input, ptp, ">> Format attributs->data input lecture : ");
    
    /* le masque : data output */
    ptp = read_ima_lect(&desc_format->masque_rea, ptp, ">> Format learning set : chemin du masque (.ras) ");
    
    /* formatage du masque */
    ptp = format_imau1_classifier_crisp_output_lect(&desc_format->format_des_output, ptp, ">> Format data output  lecture : ");
    
  return ptp;
  }
  






/* *************************  INITIALISATION  ***************************/
/** Tout d'abord les attributs sont traits : avec read_ima_init,  
   format_imau1_classifier_input_init, format_imau1_classifier_crisp_output. 
   Une fois que les donnes images ont t recopies
   dans les structures data_input, elles sont libres.\n

   Puis les diffrents labels sont dtermins automatiquement ainsi que 
   le nombre de points par label. Ces informations sont stockes dans le champ 
   classes du descripteur. C'est alors que la classe (avec un certain label) 
   est propos comme tant probablement le label du fond ( supprimer). Ce choix 
   est fait en prenant le label avec le plus grand nombre de points. Si cette classe
   correspond au label pass en paramtre alors tout se passe bien. Sinon une confirmation
   est demande (question sur terminal) pour choisir le label.  Il est possible de modifier 
   ce choix et les diffrentes informations sur les labels sont affiches pour faciliter 
   ce choix. \n
 
   Attention : le champ nb_class de la structure output_ref de l'ensemble 
   d'apprentissage ne comprend pas le fond. 

 *
 *    \param desc_format pointeur sur le descripteur
 *    \param learn de type learning_set pour stocker les donnes d'apprentissage
 *    \param LABEL_FOND pemet de connaitre le label du fond a separer des donnes.
 *    \return renvoie 0 si OK et exit sinon
 *    \author E. Ramasso 
 *    \version 1.0 - 02/04/2004
*/

  int format_learning_set_from_imau1_init(
			       format_learning_set_from_imau1_t *desc_format,
			       const int LABEL_FOND,
			       learning_set *learn
			       )
  {

    int i,j,k;   //indices de boucle
    int stop;   //pour while
    int l;   //reponse utilisateur
    int indice_c_max; //contient l'indice de la classe a rejeter (pas le label)

    printf("\nInitialisation de learning set...\n\n");
  
    
  /* --------------------------------------------
     les attributs
     -------------------------------------------*/
    
  printf("Traitement des attributs...\n");
  
  /* alloc des attributs */
  if((desc_format->att_ima = (imau1*)malloc(desc_format->nb_att*sizeof(imau1))) == NULL)
    {
      printf("\nAlloc images entres impossible\n");
      exit(1);
    }

  /* init des attributs */
  for(i=0;i<desc_format->nb_att;i++)
    read_imau1_init(desc_format->att_rea+i,desc_format->att_ima+i);
  
  /* verifier si bonne dimension pour attributs */
  for(i=0;i<desc_format->nb_att;i++)
    format_imau1_classifier_input_init(&desc_format->format_des_input, desc_format->att_ima[i], &desc_format->dataIattributs); 
  
  /* formatage du masque*/
  read_imau1_init(&desc_format->masque_rea,&desc_format->masque_ima);
  format_imau1_classifier_crisp_output_init(&desc_format->format_des_output,desc_format->masque_ima,&desc_format->dataOmasque);
  
  /* le remplissage de data input */
  for(i=0;i<desc_format->nb_att;i++)
    format_imau1_classifier_input_calc(&desc_format->format_des_input, desc_format->att_ima[i], &desc_format->dataIattributs);

  /* le remplissage de data input */
  format_imau1_classifier_crisp_output_calc(&desc_format->format_des_output, desc_format->masque_ima, &desc_format->dataOmasque);
  
  /* liberation des images */
  printf("\nLiberation des images et des structures associees...");fflush(stdout);
  for(i=0;i<desc_format->nb_att;i++) 
    free_imau1(&desc_format->att_ima[i]); 
  free(desc_format->att_ima); 
  free(desc_format->att_rea); 

  printf("Ok\n"); 



  /*-------------------------------------------
    dimension de l'ensemble
    --------------------------------------------*/

  /* verifier nb pts >=2 */
  if(desc_format->dataIattributs.nb_pts < 2 || desc_format->dataOmasque.nb_pts < 2)
    {
      printf("\n\nERREUR learning set : Pas assez de points attributs = %d et masque = %d  (<2?)\n",
	     desc_format->dataIattributs.nb_pts,desc_format->dataOmasque.nb_pts);
      exit(1);
    }

  /* si pas meme dimension exit */
  if(desc_format->dataIattributs.nb_pts != desc_format->dataOmasque.nb_pts)
    {
      printf("\n\nERREUR : le masque et les attributs doivent comporter le meme nombre de points.\n\n");
      printf("desc_format->dataIattributs.nb_pts = %d - desc_format->dataOmasque.nb_pts = %d\n",desc_format->dataIattributs.nb_pts,desc_format->dataOmasque.nb_pts);
      printf("desc_format->dataIattributs.equ_nb_rows = %d - desc_format->dataOmasque.equ_nb_rows = %d\n",desc_format->dataIattributs.equ_nb_rows,desc_format->dataOmasque.equ_nb_rows);
      printf("desc_format->dataIattributs.equ_nb_cols = %d - desc_format->dataOmasque.equ_nb_cols = %d\n",desc_format->dataIattributs.equ_nb_cols,desc_format->dataOmasque.equ_nb_cols);
      exit(1);
    }


  /*------------------------------------------------
    les classes : label et nb pts/label
    -----------------------------------------------*/

  /* au moins 1 classe dans le masque : le fond ou l'objet */
  learn->output_ref.nb_class = 1;

  /* allocation champ contenant info sur les classes */
  if((desc_format->classes = (une_classe *)malloc(1*sizeof(une_classe))) == NULL)
    {
      printf ("\n\nERREUR learning set : allocation classes learn impossible\n");  
      exit(1);
    }

  /* le premier point des donnees est le premier label */
  desc_format->classes[0].label = desc_format->dataOmasque.classes.crisp[0];
  desc_format->classes->nb_pts  = 1;

  /* calcul nombre de classes et nombre de points pour allocation */
  j = 1;

  //printf("nb pts att = %d\n",desc_format->dataIattributs.nb_pts);
  //printf("nb att = %d\n",desc_format->nb_att);
  /* pas prendre le premier point (deja fait) => i=1 */
  for(i=1;i<desc_format->dataIattributs.nb_pts;i++)
    {
      //printf("masque[%d] = %d\n",i,desc_format->dataOmasque.classes.crisp[i] );
      /* verif */
      k=0;
      while(k< learn->output_ref.nb_class)
	{
	  /* label deja vu */
	  if(desc_format->classes[k].label == desc_format->dataOmasque.classes.crisp[i])
	    {
	      desc_format->classes[k].nb_pts++;
	     
	      /* sort */
	      break;
	    }
	  else
	    {
	      k++;
	      /* si on a tout parcouru alors nouvelle classe */
	      if(k == learn->output_ref.nb_class)
		{
		  /* recopier les structures */
		  learn->output_ref.nb_class++;
		  if((desc_format->classes = 
		      (une_classe *)realloc(desc_format->classes, (learn->output_ref.nb_class)*(sizeof(une_classe)))) == NULL)
		    {
		      printf ("\n\nERREUR learning set : reallocation classes impossible\n");  
		      exit(1);
		    }

		  desc_format->classes[k].nb_pts = 1;
		  desc_format->classes[k].label = desc_format->dataOmasque.classes.crisp[i];
		  break;
		}
	    }
	}
    }


  /* si une seule classe alors pas d'interet de continuer */
  if(learn->output_ref.nb_class == 1)
    {
      printf("\n\nERREUR learning set : Il n'y a visiblement qu'une seule classe, \n");
      printf("donc tous les points sont dans la meme classe. Pas d'interet.\n");
      exit(1);
    }
 
  /* cherche la classe avec le plus grand nombre de points */

  /* init */
  indice_c_max = 0;
  desc_format->c_max = desc_format->classes[indice_c_max].label;
  
  /* recherche */
  for(i=1;i<learn->output_ref.nb_class;i++)
    {
      if(desc_format->classes[i].nb_pts > desc_format->classes[indice_c_max].nb_pts) 
	{
	  desc_format->c_max = desc_format->classes[i].label;
	  indice_c_max = i;
	}
    }

  
  if(desc_format->c_max != LABEL_FOND)
    {
      printf("\n---------------------------------------------------------------------\n");
      printf("\nATTENTION \nLe label du fond specifie en parametre de la fonction init \n");
      printf("de l'operateur format_learning_set_from_imau1 est different\n");
      printf("de celui detecte par la fonction.");
      
      /*-----------------------------------------------------
	quel est le label du fond ?
	-----------------------------------------------------*/
      
      /* Prendre le label a exclure pour le fond */
      /* par DEFAUT le grand nombre de points */
      printf("\n\nINFO : La classe avec le label '%d' est prise comme le fond et ne sera pas\n",
	     desc_format->classes[indice_c_max].label);
      printf("prise dans les donnees d'apprentissage vu qu'elle est celle \n");
      printf("avec le plus grand nombre de points (%d points sur un total de %d). \n",
	     desc_format->classes[indice_c_max].nb_pts,desc_format->dataIattributs.nb_pts);
      printf("Seulement les autres classes seront incluses dans le learning set.\n");
      printf("\nAcceptez (1 + <enter>) ou changez le label a exclure (0 + <enter>) ? : ");
      scanf("%d",&l);
      
      stop = 0;
      while(stop == 0)
	{
	  switch(l)
	    {	    
	case 0 : /* changer le label a exclure */
	  while(stop == 0)
	    {
	      printf("\nLes differents labels sont : \n");
	      /* affichage et reponse */
	      for(i=0;i<learn->output_ref.nb_class;i++)
		printf("                               label %d (%d points)  \n",desc_format->classes[i].label,desc_format->classes[i].nb_pts);
	      
	      printf("\nNumero du label a exclure ?   :  ");
	      scanf("%d",&desc_format->c_max);
	      
	      /* verification que le label est juste */
	      i=0;
	      while(i<learn->output_ref.nb_class)
		{
		  if(desc_format->classes[i].label == desc_format->c_max)
		    {	  	      
		      stop = 1;
		      indice_c_max = 0;
		      while(desc_format->classes[indice_c_max].label != desc_format->c_max) 		       
			indice_c_max ++;	   
		      break;
		    }
		  
		  if(i+1==learn->output_ref.nb_class)
		    {
		      printf("\nCe label n'existe pas.\n");
		      break;
		    }
		  
		  i++;
		}
	      break;
	      
	    case 1: /* ok */
	      stop = 1;
	      break;
	      
	    default : /* erreur */
	      printf("\nTapez 0 (pour modifier le label de la classe de fond) ou 1 (pour continuer) : ");
	      scanf("%d",&l);
	      break;
	    }
	}
	}
    }
  


  /* ------------------------------------------------------------------
  init champs des structures dataI et dataO avant allocation  
  ------------------------------------------------------------------*/

  printf("\nInit champs de la structure..."); fflush(stdout);
  
  /* les input et output ont le meme nombre de points */
  /* egal au nombre de points total (desc_format->rows*col) - le nombre de points */
  /* dans le label a rejeter (fond) */
  /* indice_c_max contient l'indice de la classe a rejeter (pas le label) */
  learn->input_ref.nb_pts = desc_format->dataIattributs.nb_pts - desc_format->classes[indice_c_max].nb_pts;
  learn->output_ref.nb_pts = learn->input_ref.nb_pts;

  /* le nombre d'attributs est le meme qu'au depart */
  learn->input_ref.nb_attr = desc_format->dataIattributs.nb_attr;

  /* le nombre de classes pour l'allocation de output */
  desc_format->dataOmasque.nb_class = learn->output_ref.nb_class;
  
  /* -1 pour enlever le fond */
  learn->output_ref.nb_class -= 1;
  
  /* le type de donnees pour l'allocation */
  learn->output_ref.type = CRISP_CHOICE;
  
  printf("Ok\n");

 /* ------------------------------------------------------------------
  allocation 
 ------------------------------------------------------------------*/

  /* init structure learning set */
  printf("Allocations datainput et dataoutput...");fflush(stdout);

  /* allocation de la structure input : donnees */
  alloc_classifier_data_input_completely(&learn->input_ref);
  
  /* allocation de la structure output : classes */
  alloc_classifier_data_output(&learn->output_ref);

  
  printf("\n\n---------------    RAPPORT  DES LEARNING DATA  ------------------- \n");

  /* nombre de points total */
  printf("\nDonnees initiales : %d points et %d attributs\n",
	 desc_format->dataIattributs.nb_pts,desc_format->dataIattributs.nb_attr);

  /* nombre de classes */
  printf("\n%d classes ont ete detectees :\n",learn->output_ref.nb_class);

  /* affichage nombre de points par label */
  /* fond compris => +1 */
  for(i=0;i<learn->output_ref.nb_class + 1;i++)
    printf("Le label \t %d \t contient \t %d points.\n",
	   desc_format->classes[i].label,desc_format->classes[i].nb_pts);

  /* la classe fond */
  printf("\nLa classe avec le label %d a ete supprimee des donnees (fond).\n",
	 desc_format->classes[indice_c_max].label );

  /* affichage % de points d'apprentissage par rapport au nb de points total */
  printf("\nIl y a %d points d'apprentissage soit %2.2f/100 des points initiaux\n\n", 
	 learn->input_ref.nb_pts,
	 100.0*(float)learn->input_ref.nb_pts/ (float)(desc_format->dataIattributs.nb_pts));

  printf("------------------------------------------------------------------------------------- \n");
  
  return NO_ERROR_FORMAT_LSFI1;
}





/* *************************  CALCUL  ***************************/
/** Remplissage de la structure learning set avec les attributs.\n
*
 *    \param desc_format pointeur sur le descripteur
 *    \param learn de type learning_set pour stocker les donnes d'apprentissage
 *    \return renvoie 0 si OK et exit sinon
 *    \author E. Ramasso 
 *    \version 1.0 - 02/04/2004
*/


int format_learning_set_from_imau1_calc(
					format_learning_set_from_imau1_t *desc_format,
					learning_set *learn
					)
{
  
  int i,j,k;
  

 /* ------------------------------------------------------------------
  remplir les points
 ------------------------------------------------------------------*/

  /* remplir la structure en enlevant les points appartenant */
  /* a la classe fond */
  printf("\nRemplissage de la structure learning set avec les donnees...");fflush(stdout);
 
  /* Parcourir tous les points du masque */
  /* desc_format->c_max est l'indice permettant d'avoir le label a exclure */
  /* j indice le data du learning set */
  j=0;
  for(i=0;i<desc_format->dataIattributs.nb_pts;i++)
    {
      if(desc_format->dataOmasque.classes.crisp[i] != desc_format->c_max)
	{
	  /* alors recopier output et input dans learning set */
	  /* j est l'indice dans la nouvelle structure et i dans la nouvelle */
	  learn->output_ref.classes.crisp[j] = desc_format->dataOmasque.classes.crisp[i];
	
	  for(k=0;k<learn->input_ref.nb_attr;k++)
	    /* j est l'indice dans la nouvelle structure et i dans la nouvelle */
	    learn->input_ref.attributes[k][j] = desc_format->dataIattributs.attributes[k][i];
  
	  j++;	  
	}
    }

  /* mise a jour des autres champs */
  learn->input_ref.equ_nb_rows = 0;
  learn->input_ref.equ_nb_cols = 0;
  learn->output_ref.equ_nb_rows = 0;
  learn->output_ref.equ_nb_cols = 0;
  learn->input_ref.begin = 0;
  learn->input_ref.end = 0;
  
  printf("ok\n");
  
  return NO_ERROR_FORMAT_LSFI1;

  }




int format_learning_set_from_imau1_ferm(
			     format_learning_set_from_imau1_t *desc_format,
			     learning_set learn
			     )
{
  printf("\nLiberation de la memoire allouee pour le descripteur de learning set...");
  free(desc_format->classes);
  free_data_input(&desc_format->dataIattributs);
  free_data_output(&desc_format->dataOmasque);
  printf("ok\n");

  printf("Liberation de la memoire allouee pour la structure de learning set...");
  free_data_input(&learn.input_ref);
  free_data_output(&learn.output_ref);
  printf("ok\n");

  return NO_ERROR_FORMAT_LSFI1;
}


//@}

















/**
   \ingroup classif 
   \defgroup format_ouput formatage d'un ensemble de sortie  partir d'images
*/


/** 
    \ingroup format_ouput
    @name formatage d'un ensemble de sortie  partir d'images
    \n

    Cet ensemble de fonctions permet de formater une structure de sortie 
    d'un classifieur data_output nette  partir d'une image de type imau1.
    Cette oprateur est notamment  utilis pour crer une structure de 
    donnes data_output. Typiquement, elles peuvent etre utilises pour formater 
    un masque (d'apprentissage ou de vrit).\n

    Elles ont t initialement cres pour etre appele par l'oprateur 
    format_learning_set_from_imau1 qui cre une structure d'apprentissage 
    learning_set par assemblage d'une data input et d'une data output nette.\n
    
    Cet ensemble de fonctions est format comme un oprateur et est bas
    sur l'oprateur format_imau1_classifier_input.\n
 
    Pour l'utiliser, il est ncessaire de dclarer une structure data_output. Celle
    ci sera alloue dans le calc.\n
*/
//@{




/* *************************  LECTURE  *******************************/
/*!  Lecture des paramtres du pseudo oprateur format_imau1_classifier_crisp_output 
 *
 *    Lecture des paramtres du pseudo oprateur format_imau1_classifier_crisp_output:
 *   - premier_point: coordonnes du premier point dfinissant le rectangle de slection
 *   - deuxieme_point: coordonnes du deuxime point dfinissant le rectangle de slection
 *   - choix: dfinit si l'utilisateur utilise pour les attributs l'image entire ou une partie de l'image
 *  
 *  
 *  \param desc pointeur descripteur     
 *  \param ptp pointeur sur paramtre courant dj allou (a lire),     
 *  \param debq pointeur de chane de caractres place en debut de question
 *  \return renvoie nouveau pointeur de paramtres dj allou (pour la suite) 
 *  \author E. Ramasso 
 *  \version 22/03/04
*/


param *format_imau1_classifier_crisp_output_lect(
					   format_imau1_classifier_crisp_output_t *desc, 
					   param *ptp, 
					   char *debq
					   )
{
  char question[500];
  int choix;

  sprintf(question, "%s format output : prendre image entiere (0) ou partielle (1)", debq);
  lec_param(question, ptp);
  desc->choix = atoi(ptp->rep);
  ptp = ptp->next;


  switch(desc->choix)
    {
      /* si une partie des donnees */
    case IMAGE_PARTIELLE :

      sprintf(question, "%s Coordonnes X du premier point (colonne : 0 a nb colonnes - 1)", debq);
      lec_param(question, ptp);
      desc->premier_point.Xcol = atoi(ptp->rep);
      ptp = ptp->next;
      
      sprintf(question, "%s Coordonnes Y du premier point (ligne : 0 a nb lignes - 1)", debq);
      lec_param(question, ptp);
      desc->premier_point.Yrow = atoi(ptp->rep);
      ptp = ptp->next;
      
      sprintf(question, "%s Coordonnes X deuxieme point (colonne : 0 a nb colonnes - 1)", debq);
      lec_param(question, ptp);
      desc->deuxieme_point.Xcol = atoi(ptp->rep);
      ptp = ptp->next;
      
      sprintf(question, "%s Coordonnes Y deuxieme point (ligne : 0 a nb lignes - 1) ", debq);
      lec_param(question, ptp);
      desc->deuxieme_point.Yrow = atoi(ptp->rep);
      ptp = ptp->next;
      
      break;


      /* mise a jour dans init */
    case IMAGE_ENTIERE : break;

      /* sinon erreur */
    default : 
      printf("\nMauvaise reponse\n"); 
      exit(1);

    }

  return ptp;
}




/* *************************  INITIALISATION  ***************************/
/*!  Fonction initialisant l'oprateur format_imau1_classifier_crisp_output
 *
 *    Vrification de la validit de l'espace slectionn par l'utilisateur.\n
 *    Initialisation dans la structure de sortie:
 *     - du nombre de lignes
 *     - du nombre de colonnes
 *     - du nombre de points
 *     - du nombre d'attributs
 *     - du pointeur sur les attributs
 *
 *    \param desc pointeur descripteur
 *    \param image_u1 une image de type U1 qui contient les donnes du masque
 *    \param dataO pointeur sur la structure qui contiendra les donnes rsultats
 *    \return renvoie un entier qui indique si l'operation s'est bien droule 
 *    \author E. Ramasso 
 *    \version 22/03/04
*/



int format_imau1_classifier_crisp_output_init(
					format_imau1_classifier_crisp_output_t *desc, 
					imau1 image_u1, 
					data_output *dataO
					)
{
  printf("\n\nInit format crisp output...");
  switch(desc->choix)
    {
      /* toute l image */
    case IMAGE_ENTIERE :
      desc->premier_point.Xcol = 0;
      desc->premier_point.Yrow = 0;
      desc->deuxieme_point.Xcol = image_u1.nc - 1;
      desc->deuxieme_point.Yrow = image_u1.nr - 1;
      break;

      /* une partie de l image => verif dim */
    case IMAGE_PARTIELLE :
      if(!(desc->premier_point.Xcol >= 0 && desc->premier_point.Xcol < image_u1.nc &&
	   desc->premier_point.Yrow >= 0 && desc->premier_point.Yrow < image_u1.nr &&
	   desc->deuxieme_point.Xcol >= 0 && desc->deuxieme_point.Xcol < image_u1.nc &&
	   desc->deuxieme_point.Yrow >= 0 && desc->deuxieme_point.Yrow < image_u1.nr &&
	   desc->deuxieme_point.Xcol - desc->premier_point.Xcol >= 0 &&
	   desc->deuxieme_point.Yrow - desc->premier_point.Yrow >= 0))
	{
	  printf("\nErreur dimension - coordonnees points invalides\n");
	  printf("desc->premier_point.Xcol =%d \n",desc->premier_point.Xcol);
	  printf("desc->premier_point.Yrow =%d\n",desc->premier_point.Yrow);
	  printf("image_u1.nr =%d\n",image_u1.nr);
	  printf("image_u1.nc =%d\n",image_u1.nc);
	  printf("desc->deuxieme_point.Xcol = %d\n",desc->deuxieme_point.Xcol);
	  printf("desc->deuxieme_point.Yrow =%d\n ",desc->deuxieme_point.Yrow );
	  exit(1);
	}
      
      break;

    default :   
      printf("\nErreur dans init : choix non valide\n"); 
      exit(1);
    }

  /* la structure de sortie est nette - crisp */
  dataO->type = CRISP_CHOICE;
  
  /* taille de la nouvelle image */
  dataO->equ_nb_cols = desc->deuxieme_point.Xcol - desc->premier_point.Xcol + 1;
  dataO->equ_nb_rows = desc->deuxieme_point.Yrow - desc->premier_point.Yrow + 1;
 

  /* l'image image_u1 a ete prealablement lue et initialisee => nc et nr connus */
  dataO->nb_pts = dataO->equ_nb_cols*dataO->equ_nb_rows;

  /* init pointeur sur donnes de sorties nettes */
  dataO->classes.crisp = NULL;
  
   printf("OK.\n\n");
  return 0;
}




/* *************************  CALCUL  ***************************/
/*!
    Fonction transformant une image de type imau1 en structure de sortie pour le classifieur

    Aprs l'allocation de la structure dataO en interne, le remplissage est effectu.\n

    \param desc pointeur descripteur
    \param image_u1 une image de type U1 qui contient les donnes du masque
    \param data pointeur sur la structure contenant les donnes d'entre
    \return renvoie un entier qui indique si l'opration s'est bien droule 
    \author E. Ramasso 
    \version 22/03/04
*/


int format_imau1_classifier_crisp_output_calc(
					format_imau1_classifier_crisp_output_t *desc,
					imau1 image_u1, 
					data_output *dataO
					)
{
  int i,j,k;
   printf("\nCalcul format crisp_output...");

  /* allocation de dataO-> */
    alloc_classifier_data_output(dataO);

  /* transfert des donnees de l image dans la structure de sortie */
  k=0;
  for(i=0;i<dataO->equ_nb_rows;i++)
    for(j=0;j<dataO->equ_nb_cols;j++)
      {
	dataO->classes.crisp[k] =
	  (int)(image_u1.p[desc->premier_point.Yrow+i][desc->premier_point.Xcol+j]);
	k++;
      }

  printf("OK\n");
  
  return 0;
}

//@}






