/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/*********************************************************************************
**		lff_symb.c
*********************************************************************************/
#include <stdio.h>
#include <stdarg.h>
#include "image.h"
#include "proto2D.h"
#include "gradual_rules_classif.h"

int gr_tn_zadeh
(
	 const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
	*z	=	gr_MINI_t(*x, *y);
	return 0;
}

int gr_tc_zadeh(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
  *z	=	gr_MAXI_t(*x, *y);
	return 0;
}

int gr_tn_proba(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
	*z	=	(*x)  *  (*y);
	return 0;
}

int gr_tc_proba(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
	(*z)	=	(*x) + (*y) - (*x)*(*y);
	return 0;
}

int gr_tn_luk(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
	gr_degreApp_t	tmp;
	tmp	=	*x+*y-1;
	*z	=	gr_MAXI_t(tmp,0);
}

int gr_tc_luk(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
	gr_degreApp_t	tmp;
	tmp	=	*x+*y;
	*z	=	gr_MINI_t(tmp,1);
}

int gr_tn_weber(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
	(*z)	=	(*x == 1) ? (*y) : ((*y == 1) ? (*x) : (0));
	return 0;
}

int gr_tc_weber(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
	(*z)	=	(*x == 0) ? (*y) : ((*y == 0) ? (*x) : (1));
	return 0;
}


int gr_op_Rescher_Gaines(
	const	gr_degreApp_t*	x,
	const	gr_degreApp_t*	y,
		gr_degreApp_t*	z
){
        (*z)	=	(*x > *y) ? (0) : (1);
}

/*initialisation*/
gr_init(char * filename, gr_fctAppartenance_t * fct, gr_lesRegles_t * rules, gr_descripteur_t *dpteur)
{
    FILE * fin;
    int i, j, tmp;
    int nb_regle=1;
    gr_chaine_t var;
    /*ouverture du fichier contenant les parametres*/
    if( (fin=fopen(filename, "r")) == NULL) {
	printf("ERREUR: impossible d'ouvrir le fichier %s\n", filename);
	exit(1);
    }
	
    /*lecture et allocation du nombre d'entre*/

    fscanf(fin, "%d", &tmp);
    fct->nb_entree = (unsigned char)tmp;
    if( (fct->partition=(gr_partition_t *)malloc(fct->nb_entree*sizeof(gr_partition_t))) ==NULL){ 
	printf("problme d'allocation mmoire.\n"); 	exit(1);     }

    /*Pour chaque entree, lecture de son nom et de son nombre de fonction d'appartenance*/
    for(i=0; i<fct->nb_entree; i++)
	{

	    fscanf(fin, "%s %d", fct->partition[i].nom, &tmp );
	    fct->partition[i].nb_fct = (unsigned char)tmp;
	    if( (fct->partition[i].part = (gr_symbol_t *)malloc(fct->partition[i].nb_fct*sizeof(gr_symbol_t))) == NULL){
		printf("problme d'allocation mmoire.\n");
		exit(1);
	    }


	    /*Pour chaque fonction d'appartenance, lecture de son nom et de ses parametres*/
	    for(j=0; j<fct->partition[i].nb_fct; j++)
	      {
	      	fscanf(fin, "%s %d", fct->partition[i].part[j].nom,  &fct->partition[i].part[j].type);
		if (fct->partition[i].part[j].type > 1 && fct->partition[i].part[j].type < 6)
		  fscanf(fin, "%f %f", &fct->partition[i].part[j].gauche, &fct->partition[i].part[j].droite);
	      }
	}


	
    /*allocation de la table de rgles et initialisation*/
 
    fscanf(fin, "%d", &rules->nb_regle);

    if( (rules->regle=(gr_uneRegle_t *)malloc(rules->nb_regle * sizeof(gr_uneRegle_t))) ==NULL){
	printf("problme d'allocation mmoire.\n");
	exit(1);
    }
    /*allocation du nombre d'entre par regle*/
    for(i=0; i<rules->nb_regle; i++)
	if( (rules->regle[i].liste_in = (unsigned char *)calloc( fct->nb_entree-1, sizeof(unsigned char)) ) == NULL){
	    printf("problme d'allocation mmoire.\n");
	    exit(1);
	}
	

    /*lecture des regles*/

    for(j=0; j<rules->nb_regle; j++) 
      {    
	for(i=0; i<fct->nb_entree-1; i++)
	  {
	    fscanf(fin, "%s", var);
	    tmp = 0;
	    while(strcmp(var,fct->partition[i].part[tmp].nom)!=0)
	      tmp++;
	    rules->regle[j].liste_in[i] = tmp;
	  }
	fscanf(fin, "%s %f", var, &rules->regle[j].ponderation );
	tmp = 0;
	while(strcmp(var,fct->partition[fct->nb_entree-1].part[tmp].nom)!=0)
	  tmp++;
	rules->regle[j].nom_out = tmp;

      }

    fclose(fin);

    /*allocation des descripteurs*/

    dpteur->nb_descripteur = fct->nb_entree;
    if( (dpteur->descript = (gr_ssEnsFlouSymb_t *)malloc(dpteur->nb_descripteur*sizeof(gr_ssEnsFlouSymb_t))) == NULL) {
	    printf("problme d'allocation mmoire.\n");
	    exit(1);
	}
    for(i=0; i<fct->nb_entree; i++)
	if( (dpteur->descript[i] = (gr_degreApp_t *)malloc(fct->partition[i].nb_fct*sizeof(gr_degreApp_t))) == NULL) {
	    printf("problme d'allocation mmoire.\n");
	    exit(1);
	}
 
    
}

int grClassif_init_set_pfi(
	gr_fusionFloue_t*	ff
){
  int i;
  for (i=0; i<3; i++)
    switch(ff->oper[i]){
	case 0:
	    ff->op[i]	=	gr_tn_zadeh;
	    break;
	case 1:
	    ff->op[i]	=	gr_tn_proba;
	    break;
	case 2:
	    ff->op[i]	=	gr_tn_luk;
	    break;
	case 3:
	    ff->op[i]	=	gr_tn_weber;
	    break;
        case 4:
	    ff->op[i]	=	gr_tc_zadeh;
	    break;
	case 5:
	    ff->op[i]	=	gr_tc_proba;
	    break;
	case 6:
	    ff->op[i]	=	gr_tc_luk;
	    break;
	case 7:
	    ff->op[i]	=	gr_tc_weber;
	    break;
        case 8:
	    ff->op[i]	=	gr_op_Rescher_Gaines;
	    break;
	default:
	    printf("Erreur sur le choix du type d'operateur\n");
	    exit(1);
	    break;
    }
    return 0;
}
	

/* ************************************/
float
gr_calcul_premisse(
	gr_fctAppartenance_t*       fct,
	gr_descripteur_t*		desc,
	unsigned char*		liste,
	int (*oper3)()
){
    int l;
    float premisse;
    (*oper3)(&desc->descript[0][liste[0]],&desc->descript[1][liste[1]], &premisse);
    /*pour tous les mots de la liste de la regles selectionnee*/
           for(l=2; l<fct->nb_entree-1; l++)
      /*mise a jour de la premisse connaissant la position du degree pour le mot l*/
	     (*oper3)(&premisse, &desc->descript[l][liste[l]], &premisse);
    return(premisse);
}


gr_degreApp_t gr_appartenance (float x, gr_symbol_t s)     
{

  switch(s.type){
  case 0:
    return(0.0);
  case 1:
    return(1.0);
  case 2:
    if (x <= s.gauche)    
	    return (0.0);
    else
      if (x >= s.droite)
	    return (1.0);
      else
	return (x / (s.droite - s.gauche) + s.gauche / (s.gauche - s.droite));
  case 3:
    if (x <= s.gauche)    
	    return (1.0);
    else
      if (x >= s.droite)
	    return (0.0);
      else
	return (x / (s.gauche - s.droite) + s.droite / (s.droite - s.gauche));
  case 4:
    if (x >= s.gauche && x <= s.droite)
      return (1.0);
    else
      return (0.0);
  case 5:
   if (x >= s.gauche && x <= s.droite)
      return (0.0);
    else
      return (1.0);
  default:
    printf("Erreur dans la fonction appartenance");
    break;
  }

    return(0.0);
}


gr_fuzzification(float *val, gr_fctAppartenance_t *fct, gr_descripteur_t *desc)
{                              
    int i, j;
    /* pour toutes les entrees du systeme */
    for(i=0; i<fct->nb_entree-1; i++)
	/*pour toutes les fonctions d'appartenance dcrivant l'entre i*/
	for(j=0; j<fct->partition[i].nb_fct; j++)
	    /* calcul du degre d'appartenance*/
	  desc->descript[i][j] = gr_appartenance(val[i],fct->partition[i].part[j]);
  
}



/* ************ defuzzification *******/
int gr_defuzzification(float *result, gr_fctAppartenance_t *fct, gr_descripteur_t * desc, int t) 
{
    int i; 
    float max=0.0, num=0.0, denum=0.0;
    int mot=0;

    switch(t){
    case 0:
	/* la classe max est retourne et tous les  degres vont dans result*/
	for(i=0; i<fct->partition[fct->nb_entree-1].nb_fct; i++) {
	    result[i]=desc->descript[fct->nb_entree-1][i];
	    if( desc->descript[fct->nb_entree-1][i] > max ){
		max = desc->descript[fct->nb_entree-1][i];
		mot = i;
	    }
	}
	return(mot);
	break;
    case 1:
	/* */
	for(i=0; i<fct->partition[fct->nb_entree-1].nb_fct; i++){
	    num = num + desc->descript[fct->nb_entree-1][i] * fct->partition[fct->nb_entree-1].part[i].mode;
	    denum = denum + desc->descript[fct->nb_entree-1][i];
	}
	result[0] = (float)(num/denum);
	break;
    default:
	printf("type defuzzification incorrect\n");
	exit(1);
    }
 }


/***************** inference *********/
int
gr_inference(
	gr_fctAppartenance_t*  	fct,
	gr_lesRegles_t*		rules,
	gr_descripteur_t*		desc,
	int (*oper2) (),
	int (*oper1) (),
	int (*oper3) ()
){
  int s, r, k;
    float premisse, combinaison, projection;

    /*Pour tous les mots definisants la variable de  sorties*/
    for (s=0; s<fct->partition[fct->nb_entree-1].nb_fct; s++)
	{
	  k=0;
            /*test de toutes les regles*/
	    for(r=0; r<rules->nb_regle; r++)
	      {	    if( rules->regle[r].nom_out == s)
			{
			  k++;
			    /*si regle OK alors calcul de la premisse correspondante*/
			    premisse = gr_calcul_premisse(fct, desc, rules->regle[r].liste_in, (*oper3));
			    /*combinaison*/
			    (*oper2)( &premisse, &rules->regle[r].ponderation, &combinaison);
			    if (k==1)
			      projection=combinaison;
			    else
			    /*projection*/
			    (*oper1)( &projection, &combinaison, &projection );
			}
		}
	    desc->descript[desc->nb_descripteur-1][s] = projection;
	    //printf("%.2f/%s ", projection, fct->partition[fct->nb_entree-1].part[s].nom );
	}
 }

/**
\\
Cet oprateur ralise l'aggrgation de plusieurs grandeurs d'entres. Chaque entre est dfinie par plusieurs mots. Il n'y a pas de restriction sur le nombre d'entres, ni sur le nombre de mots dcrivant chaque entre. Cet oprateur va lire tous les paramtres d'initialisation du systme dans un fichier. La constitution de ce fichier est explique dans la doc qui suit. Le nom de ce fichier est demand  l'utilisateur par la fonction grClassif_lect. Cette fusion passe par une fuzzification des entres  l'aide des fonctions d'appartenance. Pour le moment, cet oprateur gre 6 types de fonctions d'appartenance:

              0: a constant membership function, it will return 0 all along the universe of discourse
              1: similar to 0, but will return 1
                                                   ___
              2: linear function with the form ___/   , it streches from 0 to 1 and requires 2 values which define it

                                               ___
              3: linear function with the form    \___, it streches from 0 to 1 and requires 2 values which define it

                                                  ____
	      4: linear function with the form __|    |__, it streches from 0 to 1 and requires 2 values which define it

                                               __      __
	      5: linear function with the form   |____|   , it streches from 0 to 1 and requires 2 values which define it

       New membership function types may be added at any time, provided that the "gr_appartenance" function is modified in order to implement them in the case-structure it uses


Aprs la fuzzification, l'tape d'infrence permet de fusionner les reprsentation floues symboliques des entres,  l'aide d'un jeux de rgles et d'oprateurs d'aggrgation (t-normes, t-conormes et l'implication de Rescher-Gaines). La rgle d'infrence utilise ici est la rgle du  modus ponens gnralise. Le jeux de rgles doit tre dcrit dans le fichier de configuration du systme de fusion, de la manire suivante:
Une rgle est compose d'une mot pour chaque entre, suivait d'un mot caractrisant l'tat de la sortie puis d'un coefficient de pondration compris entre 0 et 1. Il n'est pas obligatoire de rentrer la totalit des rgles. Une rgle non prsente est considre comme ayant une pondration nulle. C'est pourquoi l'utilisateur doit prciser le nombre de rgles prsentes dans la base afin d'allouer l'espace ncessaire. Exemple de rgle: 

  grand petit moyen 0.8  ---> si entre1 est grand et entre2 est petit alors sortie est moyen avec un coef de pondration gal  0.8. Il faut absolument respecter l'ordre d'apparition des entres dans la rgle.

 Les T-normes et T-conormes qui sont implementes sont: Zadeh, probabiliste, Lukasiewicz et Weber

Voici les diffrentes mthodes de dfuzzification utilisables pour le moment:

 Recherche du mot de sortie ayant le maximum de degr d'appartenance. C'est une classification. Le numro de la classe est retourn par la fonction ff_symb_calc(). Dans la variable sortie se trouve les degrs d'appartenances pour chacunes des classes.
 Calculate the output value by means of the weighted mean of the maximums of the output fuzzy sets. The weights are the corresponding membership degrees. The result is given in the parameter sortie[0] of the function ff_symb_calc().


Voici la composition exact du fichier d'initialisation de l'oprateur:

\*Nombre_de_variable_floue(Nb entre + 1 sortie)

Nom_entre_1 Nb_mot\\
mot1 type [valeurs]\\
mot2 type [valeurs]\\
... \\
\\
Nb_rgle\\
mot_entree1 mot_entree2 ... mot_sortie pondration\\
...
}\\ 
\\
Descripteur :

     int oper1:         choix de l'operateur projction 
     int oper2:         choix de l'operateur combinaison 
     int oper3:         choix de l'operateur prmisse 
     int defuzz:         choix de la methode de dfuzzification 
     gr_fctAppartenance_t    fonction:  ensemble des fonctions d'appartenace 
     gr_lesRegles_t rgl:     base de regles 
     gr_descripteur_t des:    les descripteurs floues 
     float *data:        les valeurs des variables d'entres 
     char *nom:          gr_chaine_t de caractere represantant le fichier des paramtres 


@name lfusion_floue_symb
@author Lionel VALET
@version 1.0 (02/12/99); Include : ff_symb.h
@see mfus_flou_symb
*/
/*@{ */


/** lecture des parametres de l'operateur ff_symb. \\

     nom: chaine de caractere represantant le fichier des paramtres\\
     oper3: choix de l'operateur pour la prmisse\\
     oper2: choix de l'operateur pour la combinaison\\
     oper1: choix de l'operateur pour la projction\\
     defuzz: choix de la methode de dfuzzification\\
   
    @name ff_symb_lect
    @param des  pointeur descripteur     
    @param ptp  pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return  renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author L. VALET
    @version 1.0 (02/12/99); Include : ff_symb.h
*/
param *grClassif_lect( gr_fusionFloue_t *d, param *ptp, char *debq)
{
    char question[500];

    sprintf(question, "%s fichier d'initialisation", debq); 
    lec_param(question, ptp);
    d->nom = ptp->rep;
    ptp = ptp->next;

    sprintf(question, "%s type de l'operateur pour premisse (0-8)", debq);
    lec_param(question, ptp);
    d->oper[2] = atoi(ptp->rep);
    ptp = ptp->next;

    sprintf(question, "%s type de l'operateur pour combinaison (0-8)", debq);
    lec_param(question, ptp);
    d->oper[1] = atoi(ptp->rep);
    ptp = ptp->next;

    sprintf(question, "%s type de l'operateur pour projection (0-8)", debq);
    lec_param(question, ptp);
    d->oper[0] = atoi(ptp->rep);
    ptp = ptp->next;

    sprintf(question, "%s type de dfuzzification (0-1)", debq);
    lec_param(question, ptp);
    d->defuzz = atoi(ptp->rep);
    ptp = ptp->next;



    return(ptp);
 }



/** initialisation de l'operateur ff_symb.\\
    lecture du fichier de configuration.
    @name ff_symb_init
    @param des  pointeur descripteur
    @author L. VALET
    @version 1.0 (19/10/99); Include : gr_ff_sym.h
*/
int grClassif_init(gr_fusionFloue_t *d){
    gr_init(d->nom, &d->fonction, &d->rgl, &d->des);
    grClassif_init_set_pfi( d);
 }



/** Fusion floue symbolique .\\
    @name grClassif_calc
    @param des  pointeur descripteur
    @param data  pointeur sur un tableau de float contenant les donnes d'entre
    @param result pointeur sur le resultat, celui-ci depend de la methode de deffuzzification utilise. Il est allour par l'operateur.
    @author L. VALET
    @version 1.0 (19/10/99); Include : distance_ima.h
*/
int
grClassif_calc(
		gr_fusionFloue_t*	d,
		gr_degreApp_t*	data,
		gr_degreApp_t*	result
){

    gr_fuzzification(data, &d->fonction, &d->des);
    gr_inference(	&d->fonction,
		&d->rgl,
		&d->des,
		d->op[1],
		d->op[0],
                d->op[2] 
		);

 int res=gr_defuzzification(result, &d->fonction, &d->des, d->defuzz);

 return res;

 }

/*@}*/
