/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
 

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include "image.h"
#include "proto2D.h"
#include "gradual_rules_learning.h"
//#include "ff_symb_copy.h"
#include "classifier.h"

double minim (double a, double b)
{
  if ((a < b) || (a <= b))
    return a;
  return b;
}

double maxim (double a, double b)
{
  if ((a > b) || (a >= b))
    return a;
  return b;
}

int sum (int i, lgr_d id)
{
  int j, a = 0;
  for (j = 0; j < i; j++)
    a = a + id.nb_pts[j];
  return (a);
}

int intersch (double *a, double *b)
{double x;
  x = *a;
  *a = *b;
  *b = x;
  return(0);
}

double semn(double a)
{
  if (a == 0)
    return 0.0;
  if (a < 0)
    return -1.0;
  return 1.0;
}


/**the functionalities of the learning_gradual_rules_lect:
 The parameters that it requires are:

 - Data file is the file containing the learning points. For each class it contains a certain number of points, given by a number of attributes (both are read from the keyboard). At the end of each line there is an integer value giving the number of associated class, which allows the construction of the system, since it is a supervized one...
 - File for the coordinates of the corners will contain the values corresponding to each of the corners of the convex hull wich enclosures the points of every class that is defined in the Data file for each pair of two attributes that is chosen. This file is an intermediary one, it can be used for verifying the results, but it will be probably eliminated
 -  File for the obtained rules will contain all the needed information in order to construct the rules file (in the format that is asked by the cf_ff_symb application)
    One rule will be defined using 2 membership functions, one for each of the 2 attributes, of the type:
       Func_name Func_type [Func_values], where
 
       *Func_name is a string that will be used by the classification application in order to identify the membership function
       *Func_type is, at this moment, a number between 0 and 5 which defines the form of the membership function and indicates if the values are necessary or not:

              0: a constant membership function, it will return 0 all along the universe of discourse
              1: similar to 0, but will return 1
                                                   ___
              2: linear function with the form ___/   , it streches from 0 to 1 and requires 2 values which define it (the values on x-axis)

                                               ___
              3: linear function with the form    \___, it streches from 0 to 1 and requires 2 values which define it

                                                  ____
	      4: linear function with the form __|    |__, it streches from 0 to 1 and requires 2 values which define it

                                               __      __
	      5: linear function with the form   |____|   , it streches from 0 to 1 and requires 2 values which define it

 

    These membership functions have a name-grouping that should be obvious just from looking at the names of the functions, so that the final rules will be obtained by associating the corresponding rules for every of the two attributes (pay attention to the order!) and placing at the end of the line the class that they define and the membership degree that they induce to this class (unless graduality is involved, this should be 1). For example, a natural deducted rule can be:

 A0 B0 C1 1 --> meaning that the mebership degree of the first attribute given by the mebership function A0 combined (the combination which is given by the premise operator) with the membership degree of the second attribute given by the mebership function B0 gives a membership of 1 to the class C1
*/

param *learning_gradual_rules_lect(lgr_d *a, param *ptp, char *debq)
{
  char question[500];
  int i, j;

    sprintf(question, "%s Data file containing the learning points", debq);
    lec_param(question, ptp);
    a->fisin = ptp->rep;
    ptp = ptp->next;

    sprintf(question, "%s Does it come from the preprocessing stage? (0/1)", debq);
    lec_param(question, ptp);
    a->isProcessed = atoi(ptp->rep);
    ptp = ptp->next;

    sprintf(question, "%s File for the coordinates of the corners", debq);
    lec_param(question, ptp);
    a->fisout1 = ptp->rep;
    ptp = ptp->next;

    sprintf(question, "%s File for the obtained rules", debq);
    lec_param(question, ptp);
    a->fisout2 = ptp->rep;
    ptp = ptp->next;

    sprintf(question, "%s Number of attributes in the curent file", debq); 
    lec_param(question, ptp);
    a->nb_attr = atoi(ptp->rep);
    ptp = ptp->next;

    if (a->isProcessed == 0)
      {
	sprintf(question, "%s Number of classes in the current file", debq);
	lec_param(question, ptp);
	a->nb_class = atoi(ptp->rep);
	ptp = ptp->next;

	a->nb_pts = (int*)malloc(a->nb_class*sizeof(int));

	for (i = 0; i < a->nb_class; i++){
	  sprintf(question, "%s Number of learning points in class %d", debq, i); 
	  lec_param(question, ptp);
	  a->nb_pts[i] = atoi(ptp->rep);
	  ptp = ptp->next;
	}

      }
    else 
      if (a->isProcessed != 1)
	{
	  printf("Please confirm that the data file comes from the preprocessing stage or not\n");
	  exit(1);
	}
      else a->nb_class = 0;

//    printf("Am terminat lect\n");

    return(ptp);
}


/**the function learning_gradual_rules_init makes the dynamic allocation of the variables that will contain the results of the procedures*/
int learning_gradual_rules_init (lgr_d *id)
{
  int i;

  if (id->isProcessed)
    id->nb_pts = (int*)malloc(id->nb_class*sizeof(int));

 return(0);
}



double isleft(elem_contour *p, elem_contour *q)
{

  return ( - (q->x - p->x)*(q->next->y - p->y) + (q->next->x - p->x)*(q->y - p->y));

}

/**
The conv_hull function calculates the convex hull of a learning set. It takes as argument the double-chained list containing the points which form the contour of the points cloud. The double-chained list that it returns contains the points in anti-trigonometrical order
*/
elem_contour *conv_hull(elem_contour *first)
{

  elem_contour *current, *interm, *firstf;
  int k = 0, m, f;
  

  interm = first;
  current = first->next;

  while (current != first)
    {
      if (current->y < interm->y)
	interm = current;
      else
	if ((current->y == interm->y) && (current->x > interm->x))
	  interm = current;
      current = current->next;
    }

  first = interm;

  while (!k)
    {
      k = 1;
      current = first->next;
      while (current->next != first)
	{
	  if (isleft(first, current) < 0)
	    {
	      intersch(&current->x, &current->next->x);
	      intersch(&current->y, &current->next->y);
	      k = 0;
	    }
	  if (!isleft(first, current))
	    if ((current->y >= current->next->y) || ((current->y == current->next->y) && (current->x <= current->next->x)))
	      if (current != current->next)
	      {
		interm = current->next;
		current->next = interm->next;
		current->next->ant = current;
		if (interm == first)
		  first = current;
		free(interm);
	      }
	    else 
	      if ((current->y <= current->next->y) || ((current->y == current->next->y) && (current->x >= current->next->x)))
		if (current != current->next)
		{
		  interm = current;
		  current = current->next;
		  current->ant = interm->ant;
		  interm->ant->next = current;
		  if (interm == first)
		    first = current;
		  free(interm);
		}
	  current = current->next;
	}
    }


  current = first->next->next;
  k = 1;
  while (k)
    {
      if (current == first)
	k = 0;
      if (isleft(current->ant->ant, current->ant) <= 0)
	{
	  interm = current->ant;
	  current->ant = interm->ant;
//	  printf("The erased point: %f %f\n", interm->x, interm->y);
	  interm->ant->next = current;
	  free(interm);
	}
      else
	current = current->next;
    }

  firstf = first;
  f = 1;
  while (f)
    {
      current = first;
      k = 1;
      while (k)
	{
	  m = 1;
	  if (current->x == first->x && current->y == first->y)
	    if (current != first)
	      {
		interm = current;
		current = current->next;
		current->ant = interm->ant;
		interm->ant->next = current;
		free(interm);
		m = 0;
	      }
	  current = current->next;
	  if (current == first)
	    k = 0;
	}
      first = first->next;
      if (first == firstf)
	f = 0;
    }

  return first;

}

/**
Given a learning set this function obtains its contour, meaning the extremum points. The result is a double-chained list containing the coordinates (attributes) of these points.
*/

elem_contour *contour_extraction(learning_set iris, int start, int stop, int att1, int att2)
{
  elem_contour *first, *current, *new;
  elem_contour *firstm;
  int i, k, j, m, cont = 1;



  current = (elem_contour*)malloc(sizeof(elem_contour));
  current->x = iris.input_ref.attributes[att1][start];
  current->y = iris.input_ref.attributes[att2][start];
  current->next = current;
  current->ant = current;
  first = current;

  cont = 1;


  for (j = start; j < stop; j++)
    {
      k = 1; m = 0;
      current = first->next;
      while (k)
	{
	  if (current == first)
	  k = 0;
	  if (current->y == iris.input_ref.attributes[att2][j])
	    {
	      m = 1;
	      if (current->x < iris.input_ref.attributes[att1][j])
		current->x = iris.input_ref.attributes[att1][j];
	    }
	  current = current->next;
	}
      if(!m)
	{
	  cont++;
	  new = (elem_contour*)malloc(sizeof(elem_contour));
	  new->x = iris.input_ref.attributes[att1][j];
	  new->y = iris.input_ref.attributes[att2][j];
	  new->next = current;
	  new->ant = current->ant;
	  current->ant->next = new;
	  current->ant = new;
	}
    }


  current = (elem_contour*)malloc(sizeof(elem_contour));
  current->x = iris.input_ref.attributes[att1][start];
  current->y = iris.input_ref.attributes[att2][start];
  current->next = current;
  current->ant = current;
  firstm = current;

  for (j = start; j < stop; j++)
    {
      k = 1; m = 0;
      current = firstm->next;
      while (k)
	{
	  if (current == firstm)
	    k = 0;
	  if (current->y == iris.input_ref.attributes[att2][j])
	    {
	      m = 1;
	      if (current->x > iris.input_ref.attributes[att1][j])
		current->x = iris.input_ref.attributes[att1][j];
	    }
	  current = current->next;
	}
      if(!m)
	{
	  new = (elem_contour*)malloc(sizeof(elem_contour));
	  new->x = iris.input_ref.attributes[att1][j];
	  new->y = iris.input_ref.attributes[att2][j];
	  new->next = current;
	  new->ant = current->ant;
	  current->ant->next = new;
	  current->ant = new;
	}
    }


  current = first->ant;
  new = firstm->ant;
  first->ant = new;
  firstm->ant = current;
  current->next = firstm;
  new->next = first;

  firstm = conv_hull(firstm);


  return(firstm);
}

/**
rules_calc calculates the membership functions that will define the rules to be used in order to obtain the current class. It will result in 2 rules (4 membership functions) that reduce the universe of discourse to the smallest quadrangle that contains the convex hull and [N-4, N] rules (*2 membership functions of type 2-3), where N is the number of sides of the convex hull
*/

int rules_calc(elem_contour *first, lgr_d *id)
{

  elem_contour *current, *interm;
  int k, c;
  FILE *myfile, *myfilec;
  float mx, Mx, my, My;
  
 if ((myfilec=fopen(id->fisout1, "a")) == NULL) {
   printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisout1);
	exit(1);
 }

 if ((myfile=fopen(id->fisout2, "a")) == NULL) {
   printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisout2);
	exit(1);
 }

 mx = Mx = first->x;
 my = My = first->y;
 k = 1; c = 0;
  current = first;


  if (isleft(current, current->next) < 0)
    while (k)
      {
	fprintf(myfilec, "%f %f\n", current->x, current->y);
	if (mx > current->x)
	  mx = current->x;
	if (Mx < current->x)
	  Mx = current->x;
	if (my > current->y)
	  my = current->y;
	if (My < current->y)
	  My = current->y;
	if (current->x != current->next->x && current->y != current->next->y)
	  {
	    if (current->x > current->next->x)
	      if (current->y > current->next->y)
		fprintf(myfile, "A%d 3 %f %f\nB%d 3 %f %f\n", c, current->next->x, current->x, c, current->next->y, current->y);
	      else
		fprintf(myfile, "A%d 2 %f %f\nB%d 3 %f %f\n", c, current->next->x, current->x, c, current->y, current->next->y);
	    else
	      if (current->y > current->next->y)
		fprintf(myfile, "A%d 3 %f %f\nB%d 2 %f %f\n", c, current->x, current->next->x, c, current->next->y, current->y);
	      else
		fprintf(myfile, "A%d 2 %f %f\nB%d 2 %f %f\n", c, current->x, current->next->x, c, current->y, current->next->y);
	    c++;
	  }
	current = current->next;
	if (current == first)
	  k = 0;
      }
  else
    while (k)
      {
	fprintf(myfilec, "%f %f\n", current->x, current->y);
	if (mx > current->x)
	  mx = current->x;
	if (Mx < current->x)
	  Mx = current->x;
	if (my > current->y)
	  my = current->y;
	if (My < current->y)
	  My = current->y;
	if (current->x != current->next->x && current->y != current->next->y)
	  {
	    if (current->x > current->next->x)
	      if (current->y > current->next->y)
		fprintf(myfile, "A%d 2 %f %f\nB%d 2 %f %f\n", c, current->next->x, current->x, c, current->next->y, current->y);
	      else
		fprintf(myfile, "A%d 3 %f %f\nB%d 2 %f %f\n", c, current->next->x, current->x, c, current->y, current->next->y);
	    else
	      if (current->y > current->next->y)
		fprintf(myfile, "A%d 2 %f %f\nB%d 3 %f %f\n", c, current->x, current->next->x, c, current->next->y, current->y);
	      else
		fprintf(myfile, "A%d 3 %f %f\nB%d 3 %f %f\n", c, current->x, current->next->x, c, current->y, current->next->y);
	    c++;
	  }
	current = current->next;
	if (current == first)
	  k = 0;
      }

  fprintf(myfile, "A%d 5 %f %f\nB%d 0\n", c, mx, Mx, c);
  c++;
  fprintf(myfile, "A%d 1\nB%d 4 %f %f\n", c, c, my, My);

  fprintf(myfile, "*************************\n");
  fprintf(myfilec, "*************************\n");

  fclose(myfile);
  fclose(myfilec);



  return 0;
}

int learning_set_process(learning_set ls, lgr_d *id, int class)
{
  int atr1, atr2, i, k, m;
  int nb_points, a;
  elem_contour *conv, *current, *interm;
  float p, n;
  FILE *myfile;





//this file will contain the obtained rules: for each pair of attributes the rules for 1st attribute 1st class, 2nd attribute 1st class, 1st attribute 2nd class and so on
 if ((myfile=fopen(id->fisout2, "a")) == NULL) {
   printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisout2);
	exit(1);
 }

 fprintf(myfile, "\n****%d****\n", class);

 fclose(myfile);

//in this file will be the points that define the surface corresponding to one class - for all the existing classes
 if ((myfile=fopen(id->fisout1, "a")) == NULL) {
   printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisout1);
	exit(1);
 }

 fprintf(myfile, "\n****%d****\n", class);

 fclose(myfile);

  for (atr1 = 0; atr1 < (ls.input_ref.nb_attr)-1; atr1++)
    for (atr2 = atr1+1; atr2 < ls.input_ref.nb_attr; atr2++)
      {
	nb_points = 0;
	for (i = 0; i < ls.output_ref.nb_class; i++)
	  {
	   /*  printf("Setul pe care lucrez:\n"); */
/* 	    for (k = nb_points; k < nb_points + id->nb_pts[i]; k++) */
/* 	      printf("%f %f\n", ls.input_ref.attributes[atr1][k], ls.input_ref.attributes[atr2][k]); */
//	    printf("Deci convex hull:\n");
	    conv = contour_extraction(ls, nb_points, nb_points+id->nb_pts[i], atr1, atr2);
	    if (rules_calc(conv, id))
	      {
		printf("Erreur dans la fonction d'extraction des regles\n");
		exit(1);
	      }
	    current = conv;
	    k = 1;
	    m = 0;
	    while (k)
	      {
		ls.input_ref.attributes[atr1][nb_points+m] = current->x;
		ls.input_ref.attributes[atr2][nb_points+m] = current->y;
		current = current->next;
		//	printf("%f %f\n", current->x, current->y);
		m++;
		if (current == conv)
		    k = 0;
	      }

	    nb_points = nb_points + id->nb_pts[i];
	  }
      }


  return(0);
}


int learning_gradual_rules_calc(lgr_d *id)
{
  FILE *myfile, *myfile1;
  int a, i, j, atr1, atr2, nb_points, k;
  learning_set iris;
  connex_comp comp;
  elem_contour *cont, *conv_h, *current;
  double P1, P2, P_conv, convexity;


//this file will contain the obtained rules: for each pair of attributes the rules for 1st attribute 1st class, 2nd attribute 1st class, 1st attribute 2nd class and so on
 if ((myfile=fopen(id->fisout2, "w")) == NULL) {
   printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisout2);
	exit(1);
 }

 fclose(myfile);

//in this file will be the 4 points that define the surface corresponding to one class - for all the existing classes
 if ((myfile=fopen(id->fisout1, "w")) == NULL) {
   printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisout1);
	exit(1);
 }

 fclose(myfile);

 if (id->nb_class > 0)
   {
     nb_points = 0;
     for (i = 0; i < id->nb_class; i++)
       nb_points = nb_points + id->nb_pts[i];

     iris.input_ref.nb_attr = id->nb_attr;
     iris.input_ref.nb_pts = nb_points;
     iris.output_ref.nb_class = id->nb_class;
     iris.output_ref.nb_pts = nb_points;
     iris.output_ref.type = CRISP_CHOICE;


     if ((myfile=fopen(id->fisin, "r")) == NULL) {
       printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisin);
       exit(1);
     }

     if ((a = alloc_classifier_data_input_completely(&iris.input_ref)) != 0)
       printf("Probleme d'allocation memoire pour les donnees d'entree\n");

     a = alloc_classifier_data_output(&iris.output_ref);

     for (i = 0; i < nb_points; i++)
       {
	 for (j = 0; j < id->nb_attr; j++)
	   {
	     fscanf(myfile, "%f", &iris.input_ref.attributes[j][i]);
	     //    printf("%f ", iris.input_ref.attributes[j][i]); 
	   }
	 fscanf(myfile, "%d", &a);
	 //	 printf("\n");
       }

     fclose(myfile);
     myfile = fopen(id->fisout1, "a");
     fprintf(myfile, "0 1\n");
     fclose(myfile);
     myfile = fopen(id->fisout2, "a");
     fprintf(myfile, "0 1\n");
     fclose(myfile);

     learning_set_process(iris, id, -1);

   }
 else
   {
     if((myfile = fopen(id->fisin, "r")) == NULL)
       {
	 printf("ERREUR: impossible d'ouvrir le fichier %s\n", id->fisin);
	 exit(1);
       }
     fscanf(myfile, "%d", &id->nb_class);
     for (atr1 = 0; atr1 < id->nb_attr - 1; atr1++)
       for (atr2 = atr1+1; atr2 < id->nb_attr; atr2++)
	 {
	   fscanf(myfile, "%d %d", &i, &i);
	   for (k = 0; k < id->nb_class; k++)
	     {
	       fscanf(myfile, "%d", &iris.output_ref.nb_class);
	       iris.input_ref.nb_pts = iris.output_ref.nb_pts = 0;
	       id->nb_pts = (int*)malloc(iris.output_ref.nb_class*sizeof(int));
	       for (i = 0; i < iris.output_ref.nb_class; i++)
		 {
		   fscanf(myfile, "%d", &id->nb_pts[i]);
		   iris.input_ref.nb_pts = iris.output_ref.nb_pts = iris.output_ref.nb_pts + id->nb_pts[i];
		 }
	       iris.input_ref.nb_attr = 2;
	       iris.output_ref.type = CRISP_CHOICE;
	       if ((a = alloc_classifier_data_input_completely(&iris.input_ref)) != 0)
		 printf("Probleme d'allocation memoire pour les donnees d'entree\n");

	       if ((a = alloc_classifier_data_output(&iris.output_ref)) != 0)
		 printf("Probleme d'allocation memoire pour les donnees d'entree\n");

//	       printf("%d\n", iris.input_ref.nb_pts);

	       for (i = 0; i < iris.input_ref.nb_pts; i++)
		 fscanf(myfile, "%f %f", &iris.input_ref.attributes[0][i], &iris.input_ref.attributes[1][i]);

	       myfile1 = fopen(id->fisout1, "a");
	       fprintf(myfile1, "\n%d %d", atr1, atr2);
	       fclose(myfile1);

	       myfile1 = fopen(id->fisout2, "a");
	       fprintf(myfile1, "\n%d %d", atr1, atr2);
	       fclose(myfile1);

	       learning_set_process(iris, id, k);
	       free(id->nb_pts);
	       free_data_input(&iris.input_ref);
	       free_data_output(&iris.output_ref);
	     }
	 }
     fclose(myfile);
   }



  return(0);
}

int main(int argc, char *argv[]){
  lgr_d id;
  param par0, *ptp;



  param_debut(argc, argv, &par0);
  ptp = &par0;
  ptp = learning_gradual_rules_lect(&id, ptp, "");
  param_fin(argc, argv, &par0);

  learning_gradual_rules_init(&id);


  if (learning_gradual_rules_calc(&id) != 0)
    {
      printf("Erreur dans la fonction de calcul\n");
      exit(1);
    }

 return(0);


}
