/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

 /* ********* traitement de donnes d'apprentissages  **************************/
/*                                                                            */
/*                                                                            */
/*                                                                            */
/*  Derniere modif : Lavinia DARLEA, 27/07/06                                 */
/* ****************************************************************************/



/**
\ingroup trait_donnees
\defgroup analyse et prtraitement d'un ensemble de points d'apprentissages
*/



/**
    \ingroup trait_donnees
    @name Oprateur
 * \n
 * doc a mettre
 */
//@{


#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include "image.h"
#include "proto2D.h"
#include "ff_symb.h"
#include "classifier.h"
#include "traitement_point_apprentissage.h"

int intersch (double *a, double *b)
{double x;
  x = *a;
  *a = *b;
  *b = x;
  return(0);
}

int sum (int i, lgr_d id)
{
  int j, a = 0;
  for (j = 0; j < i; j++)
    a = a + id.nb_pts[j];
  return (a);
}

double min(double a, double b)
{
  if (a < b)
    return a;
  return b;
}

double max(double a, double b)
{
  if (a > b)
    return a;
  return b;
}

/* *************************  LECTURE  *******************************/
/*! \brief Fonction to do
 *
 *   The function will ask the user the name of the input and output file
 *  and some information about the learning set that the input file contains
 *
 *
 *  \param a pointeur descripteur
 *  \param ptp pointeur sur parametre courant dj allou (a lire),
 *  \param debq pointeur de chaine de caractres place en debut de question
 *  \return renvoie nouveau pointeur de paramtres dj allou (pour la suite)
 *  \author Lavinia DARLEA
 *  \version 27/07/06
*/
param *traitement_donnees_lect(lgr_d *a, param *ptp, char *debq){

  char question[500];
  int i;

//file that is to be processed
  sprintf(question, "%s Data file containing the learning points", debq);
  lec_param(question, ptp);
  a->fisin = ptp->rep;
  ptp = ptp->next;

  sprintf(question, "%s Number of classes in the current file", debq);
  lec_param(question, ptp);
  a->nb_class = atoi(ptp->rep);
  ptp = ptp->next;

  a->nb_pts = (int*)malloc(a->nb_class*sizeof(int));

  for (i = 0; i < a->nb_class; i++){
    sprintf(question, "%s Number of learning points in class %d", debq, i); 
    lec_param(question, ptp);
    a->nb_pts[i] = atoi(ptp->rep);
    ptp = ptp->next;
  }

  sprintf(question, "%s Number of attributes in the curent file", debq); 
  lec_param(question, ptp);
  a->nb_attr = atoi(ptp->rep);
  ptp = ptp->next;

//output file, it will contain the processed data
  sprintf(question, "%s Data file containing the processed learning set", debq);
  lec_param(question, ptp);
  a->fisout1 = ptp->rep;
  ptp = ptp->next;

  return(ptp);
}

/* *************************  INITIALISATION  ***************************/
/*! \brief Fonction todo
 *
 *    function that initializes the learning_set variable based on the
 *    input file and on the information obtained from the user with "lect"
 *
 *    \param a pointeur descripteur
 *    \param ls to do
 *    \return renvoie un entier qui indique todo
 *    \author Lavinia DARLEA
 *    \version 27/07/06
*/
int traitement_donnees_init(lgr_d a, learning_set *ls)
{
  int nb_points, i, j, m;
  FILE * myfile;

//initialization of the learning_set variable
 if ((myfile=fopen(a.fisin, "r")) == NULL) {
   printf("ERREUR: impossible d'ouvrir le fichier %s\n",a.fisin);
	exit(1);
 }

  nb_points = 0;
  for (i = 0; i < a.nb_class; i++)
    nb_points = nb_points + a.nb_pts[i];



  ls->input_ref.nb_attr = a.nb_attr;
  ls->input_ref.nb_pts = nb_points;
  ls->output_ref.nb_class = a.nb_class;
  ls->output_ref.nb_pts = nb_points;
  ls->output_ref.type = CRISP_CHOICE;

  if ((m = alloc_classifier_data_input_completely(&ls->input_ref)) != 0)
    printf("Probleme d'allocation memoire pour les donnees d'entree\n");

  m = alloc_classifier_data_output(&ls->output_ref);


 for (i=0; i<nb_points; i++)
   { for (j = 0; j < a.nb_attr; j++)
       fscanf(myfile,"%f ", &(ls->input_ref.attributes[j][i]));
/*      printf("%f %f\n", ls->input_ref.attributes[0][i], ls->input_ref.attributes[1][i]); */
       fscanf(myfile, "%d", &a);
   }

 fclose(myfile);

 return 1;
}

 float minim(float a, float b)
 {
   if (a > b)
       return b;

   return a;

 }

 float maxim(float a, float b)
 {
   if (a < b)
       return b;

   return a;

 }

float dmin(learning_set ls, int pos_start, int pos_stop, int att)
{
  int i, j;
  float dm;


//intermediary function, computes the minimum distance between 2 points in a cloud that corresponds to a class
  for (i = pos_start; i < pos_stop - 1; i++)
    for (j = i + 1; j < pos_stop; j++)
      if (fabs(ls.input_ref.attributes[att][i] - ls.input_ref.attributes[att][j]) != 0.0)
	{
	  dm = fabs(ls.input_ref.attributes[att][i] - ls.input_ref.attributes[att][j]);
	  i = pos_stop;
	  j = pos_stop;
	}

  for (i = pos_start; i < pos_stop - 1; i++)
    for (j = i + 1; j < pos_stop; j++)
      if (fabs(ls.input_ref.attributes[att][i] - ls.input_ref.attributes[att][j]) != 0.0)
	{
	  dm = minim(dm, fabs(ls.input_ref.attributes[att][i] - ls.input_ref.attributes[att][j]));
	  //printf("%f %f \n", dm, fabs(ls.input_ref.attributes[att][i] - ls.input_ref.attributes[att][j]));
	}

  return (dm);
}


//reorganization of the learning set: adding information on the frequency, reduction of the dimensions by eliminating the multiple points
int *clean_learning_set(learning_set *iris, lgr_d id)
{

  int i, j, k = -1, a, att, p, n, m;
  int *sim, *verif;
  FILE *myfile;

// The function reduces the dimension of the learning set by eliminating the points that repeat themselves. A recall of the number of appearances for every point that is kept is also available in the variable sim. The number of the vector corresponds to the point in the learning set, for the order has been kept


  myfile = fopen("learn_clean.data", "a");

  sim = (int*)malloc(iris->input_ref.nb_pts*sizeof(int));
  verif = (int*)malloc(iris->input_ref.nb_pts*sizeof(int));

    for (j = 0; j < iris->input_ref.nb_pts; j++)
      {
	sim[j] = 0;
	verif[j] = 1;
      }

    n = 0;


  for (i = 0; i < iris->output_ref.nb_class; i++)
    {
      n = n + id.nb_pts[i];
      for (j = 0; j < id.nb_pts[i]; j++)
	{
	  k++;
	  if (verif[k])
	    {
	      sim[k] = 1;
	      for (m = k+1; m < n; m++)
		{
		  a = 0;
		  for (att = 0; att < iris->input_ref.nb_attr; att++)
		    if (iris->input_ref.attributes[att][m] != iris->input_ref.attributes[att][k])
		      a = 1;
		  if (!a)
		    {
		      sim[k]++;
		      verif[m] = 0;
		    }
		}
	    }
	}
    }


  n = 0;
  m = 0; k = -1;
  for (i = 0; i < iris->output_ref.nb_class; i++)
    {
      p = 0;
      for (j = 0; j < id.nb_pts[i]; j++)
	{
	  k++;
	  if (sim[k])
	    {
	      for (att = 0; att < iris->input_ref.nb_attr; att++)
		{
		  iris->input_ref.attributes[att][m] = iris->input_ref.attributes[att][k];
		  fprintf(myfile, "%f ", iris->input_ref.attributes[att][m]);
		}
	      fprintf(myfile, "%d\n", sim[k]);
	      verif[m] = sim[k];
	      m++;
	      p++;
	    }
	}
      fprintf(myfile, "\n");
      id.nb_pts[i] = p;
    }

  iris->input_ref.nb_pts = 0;
  for (i = 0; i < iris->output_ref.nb_class; i++)
    iris->input_ref.nb_pts = iris->input_ref.nb_pts + id.nb_pts[i];

  free(sim);

  sim = (int*)malloc(iris->input_ref.nb_pts * sizeof(int));

  for (i = 0; i < iris->input_ref.nb_pts; i++)
    sim[i] = verif[i];

  free(verif);
  fclose(myfile);

  return(sim);
}

float val_min(learning_set ls, int pos_start, int pos_stop, int att)
{
  float m;
  int i;

  m = ls.input_ref.attributes[att][pos_start];

  for (i = pos_start+1; i < pos_stop; i++)
    m = minim(m, ls.input_ref.attributes[att][i]);

  return m;

}

float val_max(learning_set ls, int pos_start, int pos_stop, int att)
{
  float m;
  int i;

  m = ls.input_ref.attributes[att][pos_start];
  for (i = pos_start+1; i < pos_stop; i++)
    m = maxim(m, ls.input_ref.attributes[att][i]);

  return m;

}


//Connex components
int competitive_agglomeration(learning_set *iris, int start, int stop, int att1, int att2, int classe)
{
  const nb_pts = stop - start;
  const C = min(10, max(2,nb_pts/10));
  double *exemple[2], card[C], *prot[2], d, dmin, *d_intr[2];
  int i, j, l, k = -1, n, m = 0, *class, cl = C, flag = 1, t, s;
  FILE *myfile;


   if ((myfile = fopen("compet_agg.txt", "a")) == NULL) 
     printf("Error in opening the text file for the unsupervised clustering\n"); 


  exemple[0] = (double*)malloc(nb_pts * sizeof(double));
  exemple[1] = (double*)malloc(nb_pts * sizeof(double));
  class = (int*)malloc(nb_pts * sizeof(int));
  prot[0] = (double*)malloc(C * sizeof(double));
  prot[1] = (double*)malloc(C * sizeof(double));


  for (i = 0; i < nb_pts; i++)
    {
      exemple[0][i] = iris->input_ref.attributes[att1][i+start];
      exemple[1][i] = iris->input_ref.attributes[att2][i+start];
    }

//the initial partition and the initial prototypes
  n = nb_pts / C;
  k = -1;
  for (l = 0; l < C-1; l++)
    {
      prot[0][l] = prot[1][l] = card[l] = 0.0;
      for (j = 0; j < n; j++)
	{
	  k++;
	  class[k] = l;
	  prot[0][l] = prot[0][l] + exemple[0][k];
	  prot[1][l] = prot[1][l] + exemple[1][k];
	  card[l]++;
	}
      prot[0][l] = prot[0][l] / card[l];
      prot[1][l] = prot[1][l] / card[l];
    }
  prot[0][l] = prot[1][l] = card[l] = 0.0;
  for (j = k+1; j < nb_pts; j++)
    {
      class[j] = C - 1;
      card[l]++;
      prot[0][l] = prot[0][l] + exemple[0][j];
      prot[1][l] = prot[1][l] + exemple[1][j];
    }
  prot[0][l] = prot[0][l] / card[l];
  prot[1][l] = prot[1][l] / card[l];


  //basic isodata
  k = 1; j = 0;
  while(k && j < 1000)
    {
      k = 0; j++;
      for (i = 0; i < nb_pts; i++)
	{
	  dmin = sqrt(pow(exemple[0][i] - prot[0][class[i]], 2) + pow(exemple[1][i] - prot[1][class[i]], 2));
	  for (l = 0; l < C; l++)
	    {
	      d = sqrt(pow(exemple[0][i] - prot[0][l], 2) + pow(exemple[1][i] - prot[1][l], 2));
	      if (d < dmin)
		{
		  dmin = d;
		  class[i] = l;
		  k = 1;
		}
	    }
	}
      for (l = 0; l < C; l++)
	prot[0][l] = prot[1][l] = card[l] = 0;
      for (i = 0; i < nb_pts; i++)
	{
	  card[class[i]]++;
	  prot[0][class[i]] = prot[0][class[i]] + exemple[0][i];
	  prot[1][class[i]] = prot[1][class[i]] + exemple[1][i];
	}
      for (l = 0; l < C; l++)
	{
	  prot[0][l] = prot[0][l] / card[l];
	  prot[1][l] = prot[1][l] / card[l];
	}
    }


//"cluster fusion" - the points that belong to neighbour clusters are allocated to the same cluster if they are really close. They will be allocated to the cluster whose cardinality is greatest
 k = 0;
 m = nb_pts;
     d_intr[0] = (double*)malloc(cl*sizeof(double));
     d_intr[1] = (double*)malloc(cl*sizeof(double));
     flag = 0;
     for (j = 0; j < nb_pts; j++)
       {
	 for (n = j+1; n < nb_pts; n++)
	   if (class[j] == class[n])
	     {
	       d = fabs(exemple[0][j] - exemple[0][n]);
	       dmin = fabs(exemple[1][j] - exemple[1][n]);
	       n = m;
	     }
	 for (n = j+1; n < nb_pts; n++)
	   if (class[j] == class[n])
	     {
	       d = minim(fabs(exemple[0][j] - exemple[0][n]), d);
	       dmin = minim(fabs(exemple[1][j] - exemple[1][n]), dmin);
	       n = m;
	     }
	 d_intr[0][class[j]] = d_intr[0][class[j]] + d;
	 d_intr[1][class[j]] = d_intr[1][class[j]] + dmin;
       }
     for (l = 0; l < cl; l++)
       {
	 d_intr[0][l] = d_intr[0][l] / card[l];
	 d_intr[1][l] = d_intr[1][l] / card[l];
       }
       while(!flag)
	 {
	   flag = 1;
	   for (j = k; j < m - 1; j++)
	     for (n = k; n < m; n++)
		 if (class[j] != class[n])
		   {
		     d = fabs(exemple[0][j] - exemple[0][n]);
		     dmin =  fabs(exemple[1][j] - exemple[1][n]);
		     if ((d < 1.5 * minim(d_intr[0][class[j]], d_intr[0][class[n]])) && (dmin < 1.5 * minim(d_intr[1][class[j]], d_intr[0][class[n]])))
		       {
			 flag = 0;
			 if (card [class[j]] > card[class[n]])
			   {
			     card[class[n]]--;
			     card[class[j]]++;
			     class[n] = class[j];
			   }
			 else
			   {
			     card[class[j]]--;
			     card[class[n]]++;
			     class[j] = class[n];
			   }
		       }
		   }
	 }
/*      free(d_intr[0]); */
/*      free(d_intr[1]); */
       k =0;
     for (l = 0; l < cl; l++)
       if (card[l])
	 {
	   for (j = 0; j < nb_pts; j++)
	     if (class[j] == l)
	       class[j] = k;
	   k++;
	 }
     cl = k;



//Modification of the learning set as to separate the learning set into the connex parts that were identified
  t = 0;
  if (cl > 1)
    {
      for (l = 0; l < cl-1; l++)
	{
	  card[l] = 0;
	  for (j = 0; j < nb_pts; j++)
	    if (class[j] == l)
	      {
		intersch(&exemple[0][j], &exemple[0][t]);
		intersch(&exemple[1][j], &exemple[1][t]);
		n = class[j];
		class[j] = class[t];
		class[t] = n;
		t++;
		card[l]++;
	      }
	}
    }

 for (i = 0; i < nb_pts; i++)
   {
     iris->input_ref.attributes[att1][i+start] = exemple[0][i];
     iris->input_ref.attributes[att2][i+start] = exemple[1][i];
  }

   for (j = 0; j < nb_pts; j++)
     fprintf(myfile, "%f %f %d \n", exemple[0][j], exemple[1][j], class[j]);
  fprintf(myfile, "\n");


  fclose(myfile);

  myfile = fopen("learning_set_processing.txt", "a");
  fprintf(myfile, "%d\n", cl);
  fclose(myfile);

  m = start;
  if (cl > 1)
    {
      for (l = 0; l < cl-1; l++)
	{
	  t = m;
	  m = m + card[l];
	  convex_verif(*iris, t, m, att1, att2);
	}
      convex_verif(*iris, m, stop, att1, att2);
    }

/*   free(exemple[0]); */
/*   free(exemple[1]);  */
/*   free(class);  */
/*   free(u); */
/*   free(prot[0]); */
/*   free(prot[1]); */


  return(cl);

}





elem_contour *contour_extraction(learning_set iris, int start, int stop, int att1, int att2)
{
  elem_contour *first, *current, *new;
  elem_contour *firstm;
  int i, k, j, m, cont = 1;
//  FILE *myfile, *myfile1;

/*   myfile = fopen("contour.txt", "w"); */
/*   myfile1 = fopen("convex_hull.txt", "a"); */


//The contour on taking the vertical extreme points


  current = (elem_contour*)malloc(sizeof(elem_contour));
  current->x = iris.input_ref.attributes[att1][start];
  current->y = iris.input_ref.attributes[att2][start];
  current->next = current;
  current->ant = current;
  first = current;

  cont = 1;


  for (j = start; j < stop; j++)
    {
      k = 1; m = 0;
      current = first->next;
      while (k)
	{
	  if (current == first)
	  k = 0;
	  if (current->y == iris.input_ref.attributes[att2][j])
	    {
	      m = 1;
	      if (current->x < iris.input_ref.attributes[att1][j])
		current->x = iris.input_ref.attributes[att1][j];
	    }
	  current = current->next;
	}
      if(!m)
	{
	  cont++;
	  new = (elem_contour*)malloc(sizeof(elem_contour));
	  new->x = iris.input_ref.attributes[att1][j];
	  new->y = iris.input_ref.attributes[att2][j];
	  new->next = current;
	  new->ant = current->ant;
	  current->ant->next = new;
	  current->ant = new;
	}
    }


  current = (elem_contour*)malloc(sizeof(elem_contour));
  current->x = iris.input_ref.attributes[att1][start];
  current->y = iris.input_ref.attributes[att2][start];
  current->next = current;
  current->ant = current;
  firstm = current;

  for (j = start; j < stop; j++)
    {
      k = 1; m = 0;
      current = firstm->next;
      while (k)
	{
	  if (current == firstm)
	    k = 0;
	  if (current->y == iris.input_ref.attributes[att2][j])
	    {
	      m = 1;
	      if (current->x > iris.input_ref.attributes[att1][j])
		current->x = iris.input_ref.attributes[att1][j];
	    }
	  current = current->next;
	}
      if(!m)
	{
	  new = (elem_contour*)malloc(sizeof(elem_contour));
	  new->x = iris.input_ref.attributes[att1][j];
	  new->y = iris.input_ref.attributes[att2][j];
	  new->next = current;
	  new->ant = current->ant;
	  current->ant->next = new;
	  current->ant = new;
	}
    }


  current = first->ant;
  new = firstm->ant;
  first->ant = new;
  firstm->ant = current;
  current->next = firstm;
  new->next = first;

//  firstm = conv_hull(firstm);


/*  k = 1; */
/*   while (k) */
/*     { */
/*       fprintf(myfile1, "%f %f\n", current->x, current->y); */
/*       current = current->next; */
/*       if (current == firstm) */
/* 	k = 0; */
/*     } */

/*   fprintf(myfile1, "\n"); */

/*   fclose(myfile); */
/*   fclose(myfile1); */

  return(firstm);
}


//is the point cloud convex?
int convex_verif(learning_set ls, int start, int stop, int att1, int att2)
{
  elem_contour *first, *current;
  double d_minx, d_miny, stp_x, stp_y, Nx, Ny, m, n;
  double xmin, xmax, ymin, ymax;
  int k, l, i, j, t, indic, **density, *cadran, card[4] = {0, 0, 0, 0};
  int x1, y1, x2, y2, x, y;
  int S, ind1, ind2, s_ls, ns;
  FILE *myfile;

//  matrice = fopen("matrice.txt", "w");


  if ((stop - start) > 3)
    {
      first = contour_extraction(ls, start, stop, att1, att2);



      d_minx = dmin(ls, start, stop, att1);
      d_miny = dmin(ls, start, stop, att2);
      stp_x = 3 * d_minx;
      stp_y = 3 * d_miny;
      xmin = val_min(ls, start, stop, att1 );
      ymin = val_min(ls, start, stop, att2);
      xmax = val_max(ls, start, stop, att1) + stp_x;
      ymax = val_max(ls, start, stop, att2) + stp_y;
      Nx = ceil((xmax - xmin)/stp_x);
      Ny = ceil((ymax - ymin)/stp_y);
      //In case we want an equal fixed nomber of intervals on both axes
      /*  	      xmax = val_max(ls, n, m, at1); */
      /* 	      ymax = val_max(ls, n, m, at2); */
      /* 	      stp_x = (xmax - xmin) / 30.0; */
      /* 	      stp_y = (ymax - ymin) / 30.0; */
      /* 	      Nx = 31; */
      /* 	      Ny = 31; */
      density = (int**)malloc((Nx + 2) * sizeof(int));
      for (j = 0; j < Nx + 2; j++)
	density[j] = (int*)malloc((Ny + 2) * sizeof(int));
      for (k = 0; k < Nx + 2; k++)
	for (l = 0; l < Ny + 2; l++)
	  density[k][l] = 0;
      current = first;
      k = 1;
      indic = 1;
      while (k)
	{
	  i = floor((current->x - xmin)/stp_x) +1;
	  j = floor((current->y - ymin)/stp_y) +1;
//	  printf("%d %d, date de %f %f\n", i, j, current->x, current->y);
	  if (!density[i][j])
	    {
	      density[i][j] = indic;
	      indic++;
	    }
	  current = current->next;
	  if (current == first)
	    k = 0;
	}

      for (i = 1; i < indic; i++)
	{
	  m =0;
	  for (k = 0; k < Nx + 2; k++)
	    for (l = 0; l < Ny + 2; l++)
	      {
		if (density[k][l] == i)
		  {
		    x1 = k;
		    y1 = l;
		    m++;
		  }
		if (density[k][l] == i % (indic-1) + 1)
		  {
		    x2 = k;
		    y2 = l;
		    m++;
		  }
		if (m == 2)
		  {
		    k = Nx+3;
		    l = Ny+3;
		  }
	      }
	  if (fabs(x1 - x2) > 1 || fabs(y1 - y2) > 1)
	    {
	      if (x2 - x1)
		m = (y2 - y1)/(x2 - x1+0.0);
	      else
		m = (y2 - y1)/0.001;
	      n = y1 - (float)m * x1;
	      if (fabs(x1 - x2) > fabs(y1 - y2))
		for (k = min(x1, x2) + 1; k < max(x1, x2); k++)
		  {
		    y = floor(m * k + n);
		    if (!density[k][y])
		      density[k][y] = -1;
		  }
	      else
		for (l = min(y1, y2) + 1; l < max(y1, y2); l++)
		  {
		    x = floor((l - n)/m);
		    if (!density[x][l])
		      density[x][l] = -1;
		  }
	    }
	}


      S = 0;
      for (j = 0; j < Ny+2; j++)
	{
	  m = 0;
	  for (k = 0; k < Nx+2; k++)
	    if(density[k][j])
	      {
		ind1 = k;
		k = Nx+2;
		m = 1;
	      }
	  for (k = Nx+1; k > -1; k--)
	    if(density[k][j])
	      {
		ind2 = k;
		k = -1;
	      }
	  if (m)
	    S = S + (ind2 - ind1) + 1;
	}

      s_ls = 0;

      for (k = 0; k < Nx + 2; k++)
	for (l = 0; l < Ny + 2; l++)
	  density[k][l] = 0;
      for (i = start; i < stop; i++)
	{
	  k = floor((ls.input_ref.attributes[att1][i] - xmin)/stp_x) + 1;
	  l = floor((ls.input_ref.attributes[att2][i] - ymin)/stp_y) + 1;
	  if (!density[k][l])
	    {
	      density[k][l] = 1;
	      s_ls++;
	    }
	}
      //  printf("suprafata acoperita: %d\n", s_ls);

      /*  for (k = 0; k < Nx + 2; k++) */
      /*    { */
      /*     for (l = 0; l < Ny + 2; l++) */
      /*       fprintf(matrice, "%d ", density[k][l]); */
      /*     fprintf(matrice, "\n"); */
      /*    } */

      card[0] = stop - start;

      if ((s_ls / (float)S) < 0.7)
	{
	  cadran = (int*)malloc((stop - start + 1)*sizeof(int));
	  for (i = start; i < stop; i++)
	    if (ls.input_ref.attributes[att1][i] > (xmin + xmax)/2)
	      if (ls.input_ref.attributes[att2][i] > (ymin + ymax)/2)
		cadran[i-start] = 0;
	      else
		cadran[i-start] = 3;
	    else
	      if (ls.input_ref.attributes[att2][i] > (ymin + ymax)/2)
		cadran[i-start] = 1;
	      else
		cadran[i-start] = 2;
	  t = 0;    
	  for (l = 0; l < 3; l++)
	    {
	      card[l] = 0;
	      for (j = 0; j < stop - start; j++)
		if (cadran[j] == l)
		  {
		    m = ls.input_ref.attributes[0][j+start];
		    ls.input_ref.attributes[att1][j+start] = ls.input_ref.attributes[att1][t+start];
		    ls.input_ref.attributes[att1][t+start] = m;
		    m = ls.input_ref.attributes[1][j+start];
		    ls.input_ref.attributes[att2][j+start] = ls.input_ref.attributes[att2][t+start];
		    ls.input_ref.attributes[att2][t+start] = m;
		    n = cadran[j];
		    cadran[j] = cadran[t];
		    cadran[t] = n;
		    t++;
		    card[l]++;
		  }
	    }
	  card[3] = stop - start - card[0] - card[1] - card[2];
	}
    }
  else
    card[0] = stop - start;

  ns = 0;
  for (l = 0; l < 4; l++)
    if (card[l])
      ns ++;



//Writting in the file that will contain the final processed learning set, in the fix format required by the rules learning operator
 if ((myfile = fopen("learning_set_processing.txt", "a")) == NULL)
   printf("Impossible to open the learning_set_processing.txt file\n");

 fprintf(myfile, "%d\n", ns);
 for (i = 0; i < ns; i++)
   fprintf(myfile, "%d ", card[i]);
 fprintf(myfile, "\n");

 t = start - 1;
 for (i = 0; i < ns; i++)
   for (j = 0; j < card[i]; j++)
     {
       t++;
       fprintf(myfile, "%f %f \n", ls.input_ref.attributes[att1][t], ls.input_ref.attributes[att2][t]);
     }



 //free(cadran);
// fclose(matrice);
 fclose(myfile);


  return(0);
}

//filtering and chaining all the 3 processings
int process_chain(learning_set ls, int att1, int att2, lgr_d a)
{
  int i, j, k, l, m, n, at1, at2, ns, nb_pts = 0;
  int Nx, Ny;
  int sum, card, av;
  float d_minx, d_miny, xmin, ymin, xmax, ymax;
  float stp_x, stp_y;
  int *freq, **density;
  short flag;
  FILE * myfile1, *myfile;


  freq = clean_learning_set(&ls, a);


//the filtering of the learning set resulted after the cleaning: establish the grid, determines the points that are to be eliminated and eliminates them...


  m = 0;
  for (i = 0; i < ls.output_ref.nb_class; i++)
    {
      n = m;
      m = m + a.nb_pts[i];
      if (i == 0)
	{
	  myfile1 = fopen("learning_set_processing.txt", "a");
	  fprintf(myfile1, "%d %d\n", att1, att2);
	  fclose(myfile1);
	}
      d_minx = dmin(ls, n, m, 0);
      d_miny = dmin(ls, n, m, 1);
      if (!d_minx)
	d_minx = 0.001;
      if (!d_miny)
	d_miny = 0.001;
      stp_x = 3 * d_minx;
      stp_y = 3 * d_miny;
      xmin = val_min(ls, n, m, 0);
      ymin = val_min(ls, n, m, 1);
      xmax = val_max(ls, n, m, 0) + stp_x;
      ymax = val_max(ls, n, m, 1) + stp_y;
      Nx = ceil((xmax - xmin)/stp_x);
      Ny = ceil((ymax - ymin)/stp_y);
      //In case we want an equal fixed nomber of intervals on both axes
      /*  	      xmax = val_max(ls, n, m, at1); */
      /* 	      ymax = val_max(ls, n, m, at2); */
      /* 	      stp_x = (xmax - xmin) / 30.0; */
      /* 	      stp_y = (ymax - ymin) / 30.0; */
      /* 	      Nx = 31; */
      /* 	      Ny = 31; */
      density = (int**)malloc((Nx + 2) * sizeof(int));
      for (j = 0; j < Nx + 2; j++)
	density[j] = (int*)malloc((Ny + 2) * sizeof(int));
      for (k = 0; k < Nx + 2; k++)
	for (l = 0; l < Ny + 2; l++)
	  density[k][l] = 0;
      for (j = n; j < m; j++)
	{
	  k = floor((ls.input_ref.attributes[0][j] - xmin) / stp_x) + 1;
	  l = floor((ls.input_ref.attributes[1][j] - ymin) / stp_y) + 1;
	  density[k][l] = density[k][l] + freq[j];
	}
      sum = card =0;
      for (k = 0; k < Nx + 2; k++)
	for (l = 0; l < Ny + 2; l++)
	  if (density[k][l])
	    {
	      sum = sum + density[k][l];
	      card++;
	    }
      av = sum/card;
      flag = 0;
      while (!flag)
	{
	  flag = 1;
	  for (k = 1; k < Nx + 1; k++)
	    for (l = 1; l < Ny + 1; l++)
	      {
		if (density[k][l])
		  if (density[k][l] < (av/2))
		    {
		      density[k][l] = 0;
		      flag = 0;
		    }
	      }
	}
      flag = n;
      for (j = n; j < m; j++)
	{
	  k = floor((ls.input_ref.attributes[0][j] - xmin) / stp_x) + 1;
	  l = floor((ls.input_ref.attributes[1][j] - ymin) / stp_y) + 1;
	  if (density[k][l])
	    {
	      ls.input_ref.attributes[0][flag] = ls.input_ref.attributes[0][j];
	      ls.input_ref.attributes[1][flag] = ls.input_ref.attributes[1][j];
	      flag++;
	    }
	}

      /* 	    fprintf(myfile, "%d %d %d %d\n", i, at1, at2, flag - n); */
      /* 	    for (j = n; j < flag; j++) */
      /* 	      fprintf(myfile, "%f %f\n", ls.input_ref.attributes[at1][j], ls.input_ref.attributes[at2][j]); */
      for (k = 0; k < Nx; k++)
	free(density[k]);
      if(competitive_agglomeration(&ls, n ,flag, 0, 1, i) == 1)
	convex_verif(ls, n, flag, 0, 1);
    }

      

}


/* *************************  CALCUL  ***************************/
/*!
    \brief Fonction todo

    obtains the results of the processing chain and puts it on the output file

    \param ls1 todo
    \param desc pointeur descripteur
    \return renvoie un entier qui indique todo
    \author Lavinia DARLEA
    \version 27/07/2006
*/
int traitement_donnees_calc(learning_set ls1, lgr_d a)
{

  int i, j, k, l, sum, at1, at2, *freq, **density;
  int ns, nb_pts, m, n, *nb_pct;
  FILE *myfile, *myfile1;
  learning_set ls;

  nb_pct = (int*)malloc(a.nb_class * sizeof(int));

  myfile = fopen("learn_clean.data", "w");

  fclose(myfile);

  myfile = fopen("compet_agg.txt", "w");

  fclose(myfile);

/*   myfile = fopen("convex_hull.txt", "w"); */

/*   fclose(myfile); */

  myfile1 = fopen("learning_set_processing.txt", "w");

  fprintf(myfile1, "%d\n", a.nb_class);

  fclose(myfile1);

  myfile = fopen(a.fisout1, "w");

  for (i = 0; i < a.nb_class; i++)
    nb_pct[i] = a.nb_pts[i];

  if (ls1.input_ref.nb_attr == 2)
    {
      process_chain(ls1, 0, 1, a);
      ls = ls1;
    }
  else 
    if (ls1.input_ref.nb_attr < 2)
      {
	printf("The number of attributes is not enough to construct the classification rules\n");
	exit(1);
      }
    else 
      for (at1 = 0; at1 < a.nb_attr - 1; at1++)
	for (at2 = at1 + 1; at2 < a.nb_attr; at2++)
	  {
	    ls.input_ref.nb_attr = 2;
	    ls.input_ref.nb_pts = ls1.input_ref.nb_pts;
	    ls.output_ref.nb_class = ls1.output_ref.nb_class;
	    ls.output_ref.nb_pts = ls1.output_ref.nb_pts;
	    ls.output_ref.type = CRISP_CHOICE;

	    if ((m = alloc_classifier_data_input_completely(&ls.input_ref)) != 0)
	      printf("Probleme d'allocation memoire pour les donnees d'entree\n");

	    m = alloc_classifier_data_output(&ls.output_ref);

	    for (i = 0; i < ls.input_ref.nb_pts; i++)
	      {
		ls.input_ref.attributes[0][i] = ls1.input_ref.attributes[at1][i];
		ls.input_ref.attributes[1][i] = ls1.input_ref.attributes[at2][i];
	      }

	    process_chain(ls, 0, 1, a);

	    free_data_input(&ls.input_ref);
	    free_data_output(&ls.output_ref);

	    for (i = 0; i < a.nb_class; i++)
	      a.nb_pts[i] = nb_pct[i];
	  }


  if ((myfile1 = fopen("learning_set_processing.txt", "r")) == NULL)
    printf("Impossible to open the learning_set.processing.txt file for reading ");

  fscanf(myfile1, "%d", &k);
  fprintf(myfile, "%d\n", k);


  for (at1 = 0; at1 < a.nb_attr - 1; at1++)
    for (at2 = at1 + 1; at2 < a.nb_attr; at2++)
      {
	fprintf(myfile, "%d %d\n", at1, at2);
	fscanf(myfile1, "%d %d", &i, &i);
	for (i = 0; i < a.nb_class; i++)
	  {
	    sum = 0;
	    fscanf(myfile1, "%d", &ns);
	    density = (int**)malloc(ns*sizeof(int));
	    freq = (int*)malloc(ns*sizeof(int));
	    nb_pts = 0;
	    for (j = 0; j < ns; j++)
	      {
		fscanf(myfile1, "%d", &freq[j]);
		density[j] = (int*)malloc(freq[j]*sizeof(int));
		sum = sum + freq[j];
		n = nb_pts;
		for (l = 0; l < freq[j]; l++)
		  {
		    fscanf(myfile1, "%d", &density[j][l]);
		    nb_pts = nb_pts + density[j][l];
		  }
		for (m = n; m < nb_pts; m++)
		  fscanf(myfile1, "%f %f", &ls1.input_ref.attributes[0][m], &ls1.input_ref.attributes[1][m]);
	      }
	    fprintf(myfile, "%d\n", sum);
	    for (k = 0; k < ns; k++)
	      for (j = 0; j < freq[k]; j++)
		fprintf(myfile, "%d ", density[k][j]);
	    fprintf(myfile, "\n");
	    for (m = 0; m < nb_pts; m++)
	      fprintf(myfile, "%f %f\n", ls1.input_ref.attributes[0][m], ls1.input_ref.attributes[1][m]);
	  }
      }

  free(nb_pct);
  fclose(myfile1);
  fclose(myfile);

}




//@}
