/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "choquet.h"
#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */





int main(int argc, char *argv[])
	{
	float *donnee;
	float resultat;
	int i,c;
	char fileCriteres[TAILLE_NOM_FICHIER];
	//char *fileCriteres;
	FILE *fp;


	/* declaration de variables locales */
	choquet_interaction_t descripteur;


	/* main : variables et parametres propres au main*/
	param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */


	/* LECTURE PARAMETRES */

	/* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
	param_debut(argc, argv, &par0);
	ptp = &par0;      /* regle : ptp pointe sur la structure du parametre suivant */

	/* operateurs: ptp est passe en argument, return fournit la nouvelle position */
	printf("\n\nLECTURE EN COURS...\n");
	ptp = choquet_interaction_lect(&descripteur, ptp, "");
	printf("\n...LECTURE OK.\n");


	/* fin: sauvegarde des parametres utilises en mode MANUEL ou FICHIER */
	param_fin(argc, argv, &par0);

	

	
	

	printf("\n-> Le fichier des coefficients est : \t\t%s",(descripteur.fichier));



	/* appel de la fonction d'initialisation */
	printf("\n\nINITIALISATION EN COURS...\n\n");
	choquet_interaction_init(&descripteur);
	printf("\n...FIN INITIALISATION.\n");



	switch(descripteur.erreur)
		{
				case AUCUNE_ERREUR : 	printf("\n\nPAS D'ERREUR => PREPARATION AU CALCUL :\n");
							/* rentrer les criteres */
							printf("\nDonner le nom du fichier des criteres : ");
							scanf("%s",fileCriteres);
							printf("-> Le fichier des criteres est : \t\t%s",fileCriteres);

							printf("\n-> Allocation memoire (%d coeff)...",(descripteur.nbcoeff));
							donnee = (float *)malloc((descripteur.nbcoeff)*sizeof(float));
							printf("\t\tOK.\n");

							printf("-> Ouverture du fichier...");
							fp = fopen(fileCriteres,"rt");
							printf("\t\t\tOK\n");

							for(i=0;i<(descripteur.nbcoeff);i++)
								fscanf(fp,"%f",&donnee[i]);

							fclose(fp);

							printf("-> Criteres rentres et fichier ferme...\t\tOK\n");

							printf("\nAffichage des coefficients d'interactions :\n");
							for(i=0;i<(descripteur.nbcoeff);i++)
								{
								for(c=0;c<(descripteur.nbcoeff);c++)
									printf(" %f\t ",descripteur.ptparam[i][c]);
								printf("\n");
								}

							printf("\nAffichage des criteres :\n");
							for(i=0;i<(descripteur.nbcoeff);i++)
								{
								printf(" %f\t ",donnee[i]);
								printf("\n");
								}

							/* appel de la fonction de calcul */
							printf("\n\nCALCUL EN COURS...\n\n");
							choquet_interaction_calc(&descripteur,donnee,&resultat);
							printf("\n...CALCUL OK.");

							/* on affiche le resultat */
							printf("\n\n\n***************************\n");
							printf("RESULTAT = %2.4f\n",resultat);
							printf("***************************\n\n");

							printf("\n-> Libration mmoire associe aux critres...");
							for (i = 0 ; i < (descripteur.nbcoeff) ; i++ )
								free(donnee);
							printf("\tOK\n");
							printf("\n-> Libration mmoire associe aux coefficients...");
							choquet_interaction_ferm(&descripteur);
							printf("OK\n\n");
							printf("\nFin.\n\n");
							break;

		case ERREUR_CHOQUET_FILE :		printf("\nERREUR DETECTEE A L'OUVERTURE DU FICHIER : \nEchec ouverture.\n");
							printf("\nFin.\n");
							break;

		case ERREUR_CHOQUET_LIGNE_MATRICE :	printf("\nERREUR DETECTEE A L'INTERIEUR DU FICHIER : \nIncoherence entre le nombre de coefficients effectifs et nbcoeff => matrice non carree et probleme sur le nombre de lignes.\n");
							printf("\nFin.\n");
							break;

		case ERREUR_CHOQUET_COLONNE_MATRICE :	printf("\nERREUR DETECTEE A L'INTERIEUR DU FICHIER : \nIncoherence entre le nombre de coefficients effectifs et nbcoeff => matrice non carree et probleme sur le nombre de colonnes.\n");
							printf("\nFin.\n");
							break;

		case ERREUR_CHOQUET_NU :		printf("\nERREUR DETECTEE SUR LES COEFFICIENTS DE PONDERATION : \nleur somme doit faire 1.\n");
							printf("\nFin.\n");
							break;

		case ERREUR_CHOQUET_INTERACTION :       printf("\nERREUR DETECTEE SUR LES COEFFICIENTS D'INTERACTION : \nils ne verifient pas la condition permettant le passage a la mesure floue.\n");
							printf("\nFin.\n");
							break;

		}


	exit(1);
}



