/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* *****************************radar.h**********************************/
/*                                                                      */
/*  descripteurs et prototypes de fonctions associees aux operateurs    */
/*                  de traitement des images radar                      */
/*                                                                      */
/*  Derniere modif : E. TROUVE, 25/11/99                                */
/* **********************************************************************/

/* pas de code avant cette ligne*/
#ifndef __radar
#define __radar


                    /* PROTOTYPES FONCTIONS COMMUNES AU RADAR */
/* dans lkuan.c */
double filt_kuan(double amp0, double moyamp, double moycarre, double cv2, int* nbneg); 
/* dans llee_add.c */
double filt_lee_add(double amp0, double moyamp, double moycarre, double cv2, int* nbneg); 



                        /* OPERATEURS COMMUNS AU RADAR */

/* **************************** lconv16to8.c *******************************/
/* conversion 16 bit en 8 bit:                                         */
/*    entree : imau2 : donnes en amplitude                            */
/*    sortie : imau1 : amplitude convertie en 8 bits                   */
/* *********************************************************************/
/** \ingroup conv16to8
 decripteur*/
typedef struct {
   int methode;     
  /**< methode de conversion : 1=[min,moy+K*sigma], 2=[n(K%), n(100-K%)], 3=[0, K]*/
   float K;        /**< parametrage selon la methode       */
   int his[65536];  /**< histogramme */ 
   pixu1 lut[65536]; /**< table de conversion */
   char legende[500]; /**< interval de conversion */
} conv16to8_t;

param *conv16to8_lect(conv16to8_t *des, param *ptp, char *debq);
int conv16to8_init(conv16to8_t *des, imau2 im0, imau1 *imres);     
int conv16to8_calc(conv16to8_t *des, imau2 im0, imau1 *imres);   


/* **************************** lkuan.c ****************************************/
/* filtre de kuan :                                                            */
/*    entree : imaXX : donnes en intensite   XX = u2 | fl | db                */
/*    sortie : imaXX : intensite filtree      XX identique a l'entree          */
/* *****************************************************************************/
/** \ingroup kuan
 descripteur*/
typedef struct {
   int dimX, dimY; /**< dimension de la fenetre de moyennage       */
   double nbvue;    /**< nombre de vues (ou equivalent)            */
} kuan_t;

param *kuan_lect(kuan_t *des, param *ptp, char *debq);
/* init */
int kuanu2_init(kuan_t *des, imau2 im0, imau2 *imres);     
int kuanfl_init(kuan_t *des, imafl im0, imafl *imres);     
int kuandb_init(kuan_t *des, imadb im0, imadb *imres);     
/* calc */
int kuanu2_calc(kuan_t *des, imau2 im0, imau2 moyint, imafl moycarre, imau2 *imres);   
int kuanfl_calc(kuan_t *des, imafl im0, imafl moyint, imafl moycarre, imafl *imres);   
int kuandb_calc(kuan_t *des, imadb im0, imadb moyint, imadb moycarre, imadb *imres);   


/* **************************** llee_add.c ****************************************/
/* filtre de lee_add :                                                            */
/*    entree : imaXX : donnes en amplitude   XX = u1 | u2 | fl | db           */
/*    sortie : imaXX : amplitude filtree      XX identique a l'entree          */
/* *****************************************************************************/
/** \ingroup lee_add
    descripteur*/
typedef struct {
   int dimX, dimY; /**< dimension de la fenetre de moyennage       */
   double vbr;     /**< variance du bruit dans l'image initiale   */
} lee_add_t;

param *lee_add_lect(lee_add_t *des, param *ptp, char *debq);
/* init */
int lee_addu1_init(lee_add_t *des, imau1 im0, imau1 *imres);     
int lee_addu2_init(lee_add_t *des, imau2 im0, imau2 *imres);     
int lee_addfl_init(lee_add_t *des, imafl im0, imafl *imres);     
int lee_adddb_init(lee_add_t *des, imadb im0, imadb *imres);     
/* calc */
int lee_addu1_calc(lee_add_t *des, imau1 im0, imau1 moyamp, imafl moycarre, imau1 *imres);   
int lee_addu2_calc(lee_add_t *des, imau2 im0, imau2 moyamp, imafl moycarre, imau2 *imres);   
int lee_addfl_calc(lee_add_t *des, imafl im0, imafl moyamp, imafl moycarre, imafl *imres);   
int lee_adddb_calc(lee_add_t *des, imadb im0, imadb moyamp, imadb moycarre, imadb *imres);   


/* OPERATEUR LOGDCT */
/** \ingroup logdct
    descripteur*/
typedef struct {
   int N;              /**< nombre de points */
   float tablog[65536]; /**< valeurs du log tabulees      */
   float *v0;           /**< vecteur a transformer */
   float **matdct;     /**< matrice de calcul de DCT N points, 1 ligne par coef */
} logdct_t;
param * logdct_lect(logdct_t *des, param *ptp, char *debq);
int logdct_u2_init(logdct_t *des, imau2 *pt_im0, imafl *pt_imres);     
int logdct_u2_calc(logdct_t *des, imau2 *pt_im0, imafl *pt_imres);     


/* OPERATEUR IDCTEXP */
/** \ingroup idctexp
    descripteur*/
typedef struct {
   int N;              /**< nombre de points */
   float B;            /**< biais attendu pour correction apres l'exponentielle  */
   float *v0;           /**< vecteur a transformer */
   float **matidct;     /**< matrice de calcul de DCT N points, 1 ligne par coef */
} idctexp_t;
param * idctexp_lect(idctexp_t *des, param *ptp, char *debq);
int idctexp_u2_init(idctexp_t *des, imafl *pt_imdct, imau2 *pt_imres);     
int idctexp_u2_calc(idctexp_t *des, imafl *pt_imdct, imau2 *pt_imres);     





/*pas de code aprs cette ligne */
#endif


/* **************************** lestimR.c ****************************************/
/* estimiation de la Reflectivite par moyennage spatiale des intensites (| |^2)  */
/*        SUPPRIME LE 28/02/2000                                                 */
/*        REMPLACE PAR moycarre2DXX avec XX = u1 | u2 | fl | db (dans proto2D.h) */
/* *******************************************************************************/
