/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "image.h"


/** \ingroup utility 
conversion d'une image Teinte-Luminance-Saturation au format Rouge-Vert-Bleu.
    
Cette fonction convertie une image TLS au format RVB. ATTENTION l'image d'entre est au format imarvb o cette fois le champ "r" reprsente l'information de teinte, "v" la luminance et "b" la saturation. L'image de sortie est aussi une image de type imarvb contenant bien les information de rouge, vert et de bleu. Ce programme a t crit le 16 novembre 1992.	

   @param ima_tls pointeur sur une image de type imarvb contenant les informations de teinte, luminance, saturation (r->t, v->l, b->s)
   @param ima_rvb pointeur sur une image de type imarvb reprsentant l'image de sortie au format rvb
   @author T. CARRON, P. LAMBERT, L. VALET 
   @version 1.0 (02/12/99); Include : proto2D.h
   @see conv_RVB_TLS
*/

void conv_TLS_RVB(imarvb *ima_tls, imarvb *ima_rvb)
{
	unsigned char *ima_L,*ima_T,*ima_S;
	int Y,X;
	unsigned char *ima_R,*ima_V,*ima_B;


	unsigned char *R,*V,*B;
	unsigned char *L,*T,*S;
 	double		L_f,T_f,S_f,R_f, V_f, B_f;	
	double		DEUX_PI;
	double		C1,C2,angle;
	double		Fact,F_1_3,F_2_3;
	register	i,j;
	
	ima_T = ima_tls->r[0];
	ima_L = ima_tls->v[0];
	ima_S = ima_tls->b[0];
	Y = ima_tls->nr;
	X = ima_tls->nc;
	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];

	R=ima_R;
	V=ima_V;
	B=ima_B;
	L=ima_L;
	T=ima_T;
	S=ima_S;

	DEUX_PI = 2*M_PI;
	F_1_3  = 1.0/3.0;
	F_2_3  = 2.0/3.0;
	Fact   = sqrt(3.0)/3.0;

	for(i=Y;i--;)
		for(j=X;j--;)
			{
						/* Modif teinte */
				if( *T > 192)
	  			    T_f = (double)((int)*T-255);
				else
				  	T_f = (double)*T;
			
				L_f = (double)*L;
				S_f = (double)*S;

						/* calcul de C1 et C2 */
				angle = T_f*DEUX_PI/255.0;
				C1 = S_f*cos(angle);
				C2 = S_f*sin(angle);

						/* calcul de R,V,B	flotant*/
				R_f = (L_f + (F_2_3)*C1);
				V_f = (L_f - (F_1_3)*C1 + Fact*C2);
				B_f = (L_f - (F_1_3)*C1 - Fact*C2);
						/* test si valeur non negative ou > 255 */
				if (R_f <0 )
					R_f = 0;
				if (R_f > 255)
					R_f = 255;
				if (V_f <0 )
					V_f = 0;
				if (V_f > 255)
					V_f = 255;
				if (B_f <0 )
					B_f = 0;
				if (B_f > 255)
					B_f = 255;
				/*
				*R = (unsigned char)irint(R_f);
				*V = (unsigned char)irint(V_f);
				*B = (unsigned char)irint(B_f);
				*/
				*R = (unsigned char)rint(R_f);
				*V = (unsigned char)rint(V_f);
				*B = (unsigned char)rint(B_f);

						/* Passage pixel suivant */
				R++,V++,B++;
				L++,T++,S++;
			}
}



/** \ingroup utility 
conversion d'une image Rouge-Vert-Bleu au format Teinte-Luminance-Saturation.\\
    
Cette fonction convertie une image RVB au format TLS. L'image d'entre est une image de type imarvb contenant bien les info de rouge, vert et de bleu. ATTENTION l'image de sortie est aussi au format imarvb ou cette fois le champ "r" reprsente l'information de teinte, "v" la luminance et "b" la saturation. L'image d'entre est aussi une image de type imarvb contenant bien les info de rouge, vert et de bleu. Ce programme a t crit le 23 juillet 1993.	

   @param ima_rvb pointeur sur une image de type imarvb reprsentant l'image de sortie au format rvb
   @param ima_tls pointeur sur une image de type imarvb contenant les info de teinte, luminance, saturation (r->t, v->l, b->s)
   @author T. CARRON, P. LAMBERT, L. VALET 
   @version 1.0 (02/12/99); Include : proto2D.h
   @see conv_TLS_RVB
*/
void conv_RVB_TLS(imarvb *ima_rvb, imarvb *ima_tls)
{
	unsigned char *ima_R,*ima_V,*ima_B;
	int Y,X;
	unsigned char *ima_L,*ima_T,*ima_S;

	unsigned char *R,*V,*B;
	unsigned char *L,*T,*S;
	double		DEUX_PI;
	double		C1,C2,calc_inter,double_T;
	double		Fact_T,Fact_C2;
	register	i,j;

	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];
	Y = ima_rvb->nr;
	X = ima_rvb->nc;
	ima_T = ima_tls->r[0];
	ima_L = ima_tls->v[0];
	ima_S = ima_tls->b[0];

	R=ima_R;
	V=ima_V;
	B=ima_B;
	L=ima_L;
	T=ima_T;
	S=ima_S;

	DEUX_PI = 2*M_PI;
	Fact_C2 = sqrt(3.0)/2.0;
	Fact_T = (double)(255.0 / DEUX_PI);	/* Norm Couleur entre 0..255 */

	for(i=Y;i--;)
		for(j=X;j--;)
			{
						/* calcul de la luminance */
				;
				/* *L=(unsigned char)irint(((double)*R + (double)*V  + (double)*B)/3.0 ); */
				*L=(unsigned char)rint(((double)*R + (double)*V  + (double)*B)/3.0 );
						/* calcul facteur C1 et C2 */
				C1 = (double)(*R) - ((double)(*V) + (double)(*B)) / 2.0;
				C2 = ((double)(*V) - (double)(*B)) * Fact_C2;

						/* Calcul de la teinte */
				if (C1 == 0.0)
				{
				   if (C2 > 0)
				      *T = 64;
				   else
				      *T = 192;
				}
				else
				{
				   calc_inter = (double)C2 / (double)C1;
				   if (C1 > 0.0) 
				      double_T = atan(calc_inter) * Fact_T;
				   else
		      		  double_T = (M_PI + atan(calc_inter)) * Fact_T;

					if (double_T >= 0)
					{
						if (double_T - (int)double_T < 0.4999)
						{
							*T = (unsigned char)((int)double_T );
						}
						else
						{
							*T = (unsigned char)((int)double_T + 1);
						}
					}
					else
					{
						if (double_T - (int)double_T  > 0.5)
						{
							*T = (unsigned char)(255 + (int)double_T + 1);
						}
						else
						{
							*T = (unsigned char)(255 + (int)double_T);
						}
			
					}
 
				}

						/* Calcul de la saturation */
				calc_inter = (double)(C1) * (double)C1  + (double)(C2) * (double)C2;
				/* *S = (unsigned char)irint(sqrt(calc_inter)); */
				*S = (unsigned char)rint(sqrt(calc_inter));
						/* Passage pixel suivant */
				R++,V++,B++;
				L++,T++,S++;
			}
}


