/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* **************************** lbidon.c ******************************/
/*                                                                    */
/* ********************************************************************/

/** \defgroup bidon 
operateur bidon : exemple d'operateur type.\\
Illustration du decoupage d'un operateur en 3 fonctions (lect, init, calc).\\
Ici simple recopie de l'image entrante dans l'image sortant\\
\\
Descripteur : \\
    int dimX, dimY    :  exemple de parametre entier \\  
    float gain        :  exemple de parametre reel \\
    char forme[200]   : exemple de parametre chaine de charactere \\
    imafl msq         : exemple de variable : coeffs d'un masque (fl) \\
\\
Type entree : imau1 | ... (selon les types)\\
Type sortie : imau1 | ... (respectivement)
@name lbidon
@author E. Trouve
@version 1.0 (25/11/99); Include : image.h, proto2D.h, bidon.h
@see mbidon_XX,  XX = u1 | ... .
*/

#include "image.h"
#include "proto2D.h"
#include "bidon.h"
#include <stdlib.h>
#include <string.h>

/* *************************  LECTURE  *******************************/
/** \ingroup bidon
    lecture des parametres de l'operateur bidon. \\
    dimX, dimY : exemple d'entiers : nombre de colonnes, de lignes d'un masque\\
    gain : exemple de reel\\
    forme : exemple de chaine de charactere : rectangle/triangle
    @name bidon_lect
    @param des pointeur descripteur     
    @param ptp pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author E. Trouve
    @version 1.0 (25/11/99); Include : image.h, proto2D.h, bidon.h
*/
param *bidon_lect( bidon_t *des, param *ptp, char *debq){
  char question[500];

  sprintf(question, "%s dimX (entier)", debq); 
  lec_param(question, ptp);
  des->dimX = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s dimY (entier)", debq); 
  lec_param(question, ptp);
  des->dimY = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s gain (reel)", debq); 
  lec_param(question, ptp);
  des->gain = (float)atof(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s forme (nom)", debq); 
  lec_param(question, ptp);
  strcpy(des->forme, ptp->rep);
  ptp = ptp->next;

  return(ptp);
}

/* *************************  INITIALISATION  ***************************/
/** \ingroup bidon
    XX = u1 | etc : initialisation de l'operateur bidon.\\
    allocation image resultat et images de travail, 
    calcul des coeff du masque
    @name bidonXX_init 
    @param des  pointeur descripteur
    @param im0  image initiale (taille connue)
    @param imres  pointeur image resultat
    @author E. Trouve
    @version 1.0 (19/10/99); Include : image.h, proto2D.h, bidon.h
*/
int bidonu1_init(bidon_t *des, imau1 im0, imau1 *imres){
  int i, j;
 
  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s_bidon_%dx%d", im0.nom, des->dimX, des->dimY);
  alloc_imau1(imres);

  /* variables de l'operateur */
  des->msq.nc = des->dimX;
  des->msq.nr = des->dimY;
  alloc_imafl(&(des->msq));

  /* calcul du masque selon la forme */
  if( strcmp(des->forme, "rectangle") == 0 ){
      printf("\n>>bidonu1_init : masque %s %dx%d\n", 
	     des->forme, des->dimX, des->dimY); 
      for(j=0; j<des->dimY; j++)
	  for(i=0; i<des->dimX; i++){
	      des->msq.p[j][i] = des->gain;
	  }
  }
  else if( strcmp(des->forme, "triangle") == 0 ){
      printf("\n>>bidonu1_init : masque %s %dx%d\n", 
	     des->forme, des->dimX, des->dimY); 
      /* ... */
  }
  else{
      printf("\n>>ERREUR bidonu1_init : masque %s inconnu\n", 
	     des->forme, des->dimX, des->dimY); 
      exit(1);
  }    
}


/* *************************  CALCUL  ***************************/
/** \ingroup bidon
    XX = u1 | etc : calcul de l'operateur bidon.\\
    Calcul du resultat de l'operateur bidon (ici simple recopie!)
    @name bidonXX_calc  
    @param des  pointeur descripteur
    @param im0  image initiale 
    @param imres  pointeur image resultat
    @author E. Trouve
    @version 1.0 (19/10/99); Include : image.h, proto2D.h, bidon.h
*/
int bidonu1_calc(bidon_t *des, imau1 im0, imau1 *imres){
  int i, j;
      
  for(j=0; j<im0.nr; j++)
      for(i=0; i<im0.nc; i++)
	  imres->p[j][i] = im0.p[j][i];

}


/*@} */  /* fin doc en sous partie */   
