/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

/** \ingroup divers
    \defgroup chanfrein_imafl Distance sur des images flottantes
    Operateur chanfrein_imafl : construit une image de DISTANCE (au format imafl donc float)   partir d'une image binaire (au format imau1)
 et ceci avec un MAILLAGE CARRE (h*l=1). Les distances sont calcules avec un masque de chanfrein de taille 5*5. 
Les coefficients du masque sont les suivants (X reprsente le pixel central du masque):
\begin{center}
\begin{tabular}{ccc}
  & C & \\
A & B & C\\
X & A & \\
\end{tabular}
\end{center}
Les valeurs de ces coefficients sont les distances Euclidiennes. A=1, B=sqrt(2), C=sqrt(5). Ils permettent d'obtenir une distance discrete (obtenue par propagation d'information locale de distance).On a conserv les coefficients du masque de chanfrein sous leur forme relle  permettant d'avoir la distance euclidienne.
\\
Dans l'image binaire d'entre, les pixels reprsentant le fond ont un niveau de gris gal  255 (blanc) tandis que les objets sont au niveau 0 (noir). Pour chaque pixel de l'image, la distance au plus proche objet est dtermine  l'aide du masque de chanfrein. Les valeurs de distances sont relles.\\
L'image de sortie est alloue dans la fonction chanfrein_imafl_init, donc pas besion de le faire avant.\\
\\

@name lchanfrein_imafl
@author Lionel VALET & Didier COQUIN
@version 1.0 (11/09/02); Include : chanfrein_imafl.h
@see mchanfrein_imafl
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "image.h"
#include "proto2D.h"
#include "chanfrein_imafl.h"


#define   max(x,y)  (((x)>(y))?(x):(y))
#define   min(x,y)  (((x)<(y))?(x):(y))
/*#define   abs(x) ((x>0)?x:-x)*/

/*typedef  unsigned char Byte;*/



/* ****************************************************************************/
/*                             FONCTION MINIMUN                               */
/*   Cette fonction retourne le minimum d'un tableau de 25 elements .         */
/*   ainsi que sa position x0 et y0					      */
/* ****************************************************************************/


float  minimum( float masque[5][5], unsigned int *x0, unsigned int *y0)
{
     register int i,j;
     float mini, mini1;

     mini=masque[0][0];
     *x0 = 0;
     *y0 = 0;
     for ( i=0 ; i<5 ; i++ )
         for ( j=0 ; j<5 ; j++ )
         {            
             mini1=min(masque[i][j],mini);
             if (mini1 < mini)
             {
             	mini = mini1;
             	*x0 = i;
             	*y0 = j;
             }  
         }

         return   mini;               
     }


/* ***************************************************************************/
/*                       FIN  FONCTION MINIMUN                               */
/* ***************************************************************************/


/* ***************************************************************************/
/*                             FONCTION DISTANCE                             */
/*   Cette fonction calcule la distance de chaque pixel d'une image au       */
/*                    fond ( Background ).                                   */
/* ***************************************************************************/

void  distance(imau1 *im1, imafl *im2)
{

       register int   x,y;
       float          maximum,niveau,recop,recop1;
       int            w,compt,fin;
       unsigned int   x1,y1;
       double         val;
       float          valeur,inter;
       float          tt,vv,s;
       float          n1,n2,n3,num,denom;
       float          delta1,delta2,delta3;
       float          A,B,C,D,E;
       int            ydebut;
       float          *ima_dist;
       float          *ima_dist1;
       float          *ima_sir;
       float          fen[5][5];
       int            largeur, hauteur;
       unsigned char  *ima1;
       float          *ima2;

       	       /*-----------------------------------------------------*/
               /*      CALCUL DES COEFFICIENTS OPTIMAUX REELS  A,B,C,D,E.    */
               /*-----------------------------------------------------*/

       largeur = im1->nc;
       hauteur = im1->nr;
       ima1 = im1->p[0];
       ima2 = im2->p[0];

       ima_dist=(float *)calloc(largeur*hauteur,sizeof(float));
       ima_dist1=(float *)calloc(largeur*hauteur,sizeof(float));
       ima_sir=(float *)calloc(largeur*hauteur,sizeof(float)); 
         
       

       A=1.0;
       B=sqrt(2.0);
       C=sqrt(5.0);
       /* Comme le Maillage est CARRE */
       D=A;
       E=C;


       printf("Coefficients reels du masque 5x5 de Chanfrein (Maillage Carre)\n");
       printf("A=%d  B=%d  C=%d\n",A,B,C); 


            /*----------------------------------------------*/
            /*          MISE EN FORME DE L'IMAGE SOURCE     */
            /*----------------------------------------------*/
/* On traite une image binaire: Objet = 0,  Fond = 255   *****/
/* les points du fond initialement a 255 sont mis a l'infini ***/
/* les autres, donc l'objet reste a 0		        ***/
   
       fin=(largeur)*(hauteur);

	   printf("largeur= %d  hauteur= %d\n",largeur,hauteur);       
      
       for ( x=0 ; x<fin ; x++ )
           {
           if ( ima1[x]!=0)
           ima_sir[x]=20000;
           else  ima_sir[x]=0.0; 
           }

  
/* ima_sir = image de reference utiliser pour calculer l'image de distance **/
/* ima_dist1 = image de distance apres le passage du masque_avant     *******/
/* ima_dist  = image de distance finale   *****/      

           /*----------------------------------------------*/
           /*               BOUCLE ALLER                   */
           /*  Balayage de gche a drte et de haut en bas   */
           /*----------------------------------------------*/
	
       for ( y=2 ; y<hauteur-2 ; y++ )
           for ( x=2 ; x<largeur-2 ; x++ )
               {
               w=x+y*largeur;
               if (ima_sir[w]!=0.0)
               {
  				/* comparaison avec le masque avant  ****/		                                 
                  fen[0][0]=(2*B)+(ima_sir[w-2*largeur-2]);
                  fen[0][1]=(E)+(ima_sir[w-2*largeur-1]);  
                  fen[0][2]=(2*D)+(ima_sir[w-2*largeur]);  
                  fen[0][3]=(E)+(ima_sir[w-2*largeur+1]);  
                  fen[0][4]=(2*B)+(ima_sir[w-2*largeur+2]);  
                   
                  fen[1][0]=(C)+(ima_sir[w-largeur-2]);      
                  fen[1][1]=(B)+(ima_sir[w-largeur-1]);    
                  fen[1][2]=(D)+(ima_sir[w-largeur]);    
                  fen[1][3]=(B)+(ima_sir[w-largeur+1]);   
                  fen[1][4]=(C)+(ima_sir[w-largeur+2]);

                  fen[2][0]=(2*A)+(ima_sir[w-2]);
                  fen[2][1]=(A)+(ima_sir[w-1]);    
                  fen[2][2]=20000;   
                  fen[2][3]=20000;                 
                  fen[2][4]=20000; 
                     
                  fen[3][0]=20000;                
                  fen[3][1]=20000;                 
                  fen[3][3]=20000;                  
                  fen[3][4]=20000;
                  fen[3][2]=20000;
                  fen[4][0]=fen[4][1]=fen[4][2]=fen[4][3]=fen[4][4]=20000;
 
                  
                  ima_dist1[w]=minimum(fen,&x1,&y1);
                  ima_sir[w]=ima_dist1[w];
                }
                else ima_dist[w] = 0.0;
               }                        /*    fin du for */


      /*-------------------------------------------------------*/             
      /*                     BOUCLE RETOUR                     */
      /*  Balayage de l'image de drte a gche et de bas en haut */
      /*-------------------------------------------------------*/
	
       for ( y=hauteur-3 ; y>1 ; y-- )
           for ( x=largeur-3 ; x>1 ; x-- )
               {
               w=x+(y)*largeur;
               if (ima_sir[w]!=0.0)
               {
   				/* comparaison avec le masque arriere  ****/

                  fen[4][0]=(2*B)+(ima_sir[w+2*largeur-2]);
                  fen[4][1]=(E)+(ima_sir[w+2*largeur-1]);  
                  fen[4][2]=(2*D)+(ima_sir[w+2*largeur]);  
                  fen[4][3]=(E)+(ima_sir[w+2*largeur+1]);  
                  fen[4][4]=(2*B)+(ima_sir[w+2*largeur+2]);  
                   
                  fen[3][0]=(C)+(ima_sir[w+largeur-2]);      
                  fen[3][1]=(B)+(ima_sir[w+largeur-1]);    
                  fen[3][2]=(D)+(ima_sir[w+largeur]);    
                  fen[3][3]=(B)+(ima_sir[w+largeur+1]);   
                  fen[3][4]=(C)+(ima_sir[w+largeur+2]);

                  fen[2][4]=(2*A)+(ima_sir[w+2]);
                  fen[2][3]=(A)+(ima_sir[w+1]);    
                  fen[2][2]=20000;   
                  fen[2][0]=20000;                 
                  fen[2][1]=20000;                      
                  fen[0][0]=fen[0][1]=fen[0][2]=fen[0][3]=fen[0][4]=20000;

                  fen[1][0]=20000;                  
                  fen[1][1]=20000;
                  fen[1][3]=20000;                 
                  fen[1][4]=20000;
                  fen[1][2]=20000;
             
                  recop1=minimum(fen,&x1,&y1);
                  recop=ima_dist1[w];                 
                  ima_dist[w]=min(recop,recop1);
                  ima_sir[w]=ima_dist[w];
                }
                else ima_dist[w]=0.0;        
                             
              }              /*    fin du for */



	for ( y=(fin-1) ; y>=0 ; y-- )
	    ima2[y] = ima_dist[y];
      



        printf("\nFIN IMAGE DE DISTANCE: \n");
        printf("\n");
		
        cfree(ima_sir);
        cfree(ima_dist);
        cfree(ima_dist1);  

}                      /*    fin de distance    */


/** \ingroup chanfrein_imafl
    initialisation de l'operateur chanfrein_imafl.\\
    allocation de l'image rsultat.
    @name distance_imau1_init
    @param des  pointeur descripteur
    @param imaO  pointeur image initiale
    @param imaRes pointeur image rsultat
    @author D. COQUIN
    @version 1.0 (11/09/02); Include : chanfrein_imfl.h
*/
int chanfrein_imafl_init( imau1 *imO, imafl *imRes)
{
	imRes->nr = imO->nr;
	imRes->nc = imO->nc;
	alloc_imafl(imRes);
    
}

/** \ingroup chanfrein_imafl
    Calcul de l'image de distance .\\
    @name chanfrein_imafl_calc
    @param des  pointeur descripteur
    @param imaO  pointeur image initiale
    @param imaRes pointeur image rsultat
    @author D. COQUIN
    @version 1.0 (11/09/02); Include : chanfrein_imfl.h
*/
int chanfrein_imafl_calc( imau1 *imO, imafl *imRes)
{
    distance( imO, imRes);
}

