/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************* lconv_img.c ******************************/
/*                                                                    */
/* ********************************************************************/

/**
    conversion d'une image Teinte-Luminance-Saturation au format Rouge-Vert-Bleu.\\
  
Cette fonction convertie une image TLS au format RVB. ATTENTION l'image d'entre est au format imarvb o cette fois le champ "r" reprsente l'information de teinte, "v" la luminance et "b" la saturation. L'image de sortie est aussi une image de type imarvb contenant bien les information de rouge, vert et de bleu..	


conversion d'une image Rouge-Vert-Bleu au format Teinte-Luminance-Saturation.
    
Cette fonction convertie une image RVB au format TLS. L'image d'entre est une image de type imarvb contenant bien les info de rouge, vert et de bleu. ATTENTION l'image de sortie est aussi au format imarvb ou cette fois le champ "r" reprsente l'information de teinte, "v" la luminance et "b" la saturation. L'image d'entre est aussi une image de type imarvb contenant bien les info de rouge, vert et de bleu.

   
conversion d'une image Rouge-Vert-Bleu au format Teinte-Saturation-Value. (HSV) 
    
Cette fonction convertie une image RVB au format TSV (HSV).
ATTENTION 
L'image d'entre est au format imarvb o le champ "r" reprsente l'information de rouge, "v" le vert et "b" le bleu (la variation 0 - 255 pour chaque composante).
L'image de sortie est sauvee dans 3 pointer double de type pixdb contenant les information de Teinte (la variation 0.0 - 360.0), Saturation (variation 0.0 - 1.0), Value (variation 0.0 - 1.0).



conversion d'une image Teinte-Saturation-Value (HSV) au format Rouge-Vert-Bleu 
    
Cette fonction convertie une image TSV (HSV) au format RVB.
ATTENTION 
L'image d'entre est lu de 3 pointer double de type pixdb contenant les information de Teinte (la variation 0.0 - 360.0), Saturation (variation 0.0 - 1.0), Value (variation 0.0 - 1.0).
L'image de sortie est au format imarvb o le champ "r" reprsente l'information de rouge, "v" le vert et "b" le bleu (la variation 0 - 255 pour chaque composante) 
   

conversion d'une image Rouge-Vert-Bleu au format Lab
    
Cette fonction convertie une image RVB au format Lab.
ATTENTION 
L'image d'entre est au format imarvb o le champ "r" reprsente l'information de rouge, "v" le vert et "b" le bleu. 
L'image de sortie est lu de 3 pointer double de type pixdb contenant les information de Luminance - L , premiere composante chromatique - a, deuxieme composante chromatique - b.


conversion d'une image Rouge-Vert-Bleu au format Luv
    
Cette fonction convertie une image RVB au format Luv.
ATTENTION 
L'image d'entre est au format imarvb o le champ "r" reprsente l'information de rouge, "v" le vert et "b" le bleu. 
L'image de sortie est lu de 3 pointer double de type pixdb contenant les information de Luminance - L , premiere composante chromatique - u, deuxieme composante chromatique - v

@name lconv_img
@author B. Ionescu
@version 1.0 (10/01/01); Include : proto2D.h, conv_img.h
*/

/*@{*/  /* doc suivantes en sous-partie */  

#include "image.h"
#include "proto2D.h"
#include "conv_img.h"
#include <stdlib.h>


void conv_TLS_RVB2(space_struct *des, imarvb *ima_tls, imarvb *ima_rvb)
{
	unsigned char *ima_L,*ima_T,*ima_S;
	int Y,X;
	unsigned char *ima_R,*ima_V,*ima_B;

	unsigned char *R,*V,*B;
	unsigned char *L,*T,*S;
 	double		L_f,T_f,S_f,R_f, V_f, B_f;	
	double		DEUX_PI;
	double		C1,C2,angle;
	double		Fact,F_1_3,F_2_3;
	register	i,j;
	
	ima_T = ima_tls->r[0];
	ima_L = ima_tls->v[0];
	ima_S = ima_tls->b[0];
	Y = ima_tls->nr;
	X = ima_tls->nc;
	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];

	R=ima_R;
	V=ima_V;
	B=ima_B;
	L=ima_L;
	T=ima_T;
	S=ima_S;

	DEUX_PI = 2*M_PI;
	F_1_3  = 1.0/3.0;
	F_2_3  = 2.0/3.0;
	Fact   = sqrt(3.0)/3.0;

	for(i=Y;i--;)
		for(j=X;j--;)
			{

			  /* ************************************* */
			  *T += (unsigned char) des->comp_a;
			  *L += (unsigned char) des->comp_b;
			  *S += (unsigned char) des->comp_c;
			  /* ************************************* */

						/* Modif teinte */
				if( *T > 192)
	  			    T_f = (double)((int)*T-255);
				else
				  	T_f = (double)*T;
			
				L_f = (double)*L;
				S_f = (double)*S;

						/* calcul de C1 et C2 */
				angle = T_f*DEUX_PI/255.0;
				C1 = S_f*cos(angle);
				C2 = S_f*sin(angle);

						/* calcul de R,V,B	flotant*/
				R_f = (L_f + (F_2_3)*C1);
				V_f = (L_f - (F_1_3)*C1 + Fact*C2);
				B_f = (L_f - (F_1_3)*C1 - Fact*C2);
						/* test si valeur non negative ou > 255 */
				if (R_f <0 )
					R_f = 0;
				if (R_f > 255)
					R_f = 255;
				if (V_f <0 )
					V_f = 0;
				if (V_f > 255)
					V_f = 255;
				if (B_f <0 )
					B_f = 0;
				if (B_f > 255)
					B_f = 255;
				/*
				*R = (unsigned char)irint(R_f);
				*V = (unsigned char)irint(V_f);
				*B = (unsigned char)irint(B_f);
				*/
				*R = (unsigned char)rint(R_f);
				*V = (unsigned char)rint(V_f);
				*B = (unsigned char)rint(B_f);

						/* Passage pixel suivant */
				R++,V++,B++;
				L++,T++,S++;
			}
}



void  conv_RVB_TLS2(space_struct *des, imarvb *ima_rvb, imarvb *ima_tls)
{
	unsigned char *ima_R,*ima_V,*ima_B;
	int Y,X;
	unsigned char *ima_L,*ima_T,*ima_S;

	unsigned char *R,*V,*B;
	unsigned char *L,*T,*S;
	double		DEUX_PI;
	double		C1,C2,calc_inter,double_T;
	double		Fact_T,Fact_C2;
	register	i,j;


	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];
	Y = ima_rvb->nr;
	X = ima_rvb->nc;
	ima_T = ima_tls->r[0];
	ima_L = ima_tls->v[0];
	ima_S = ima_tls->b[0];

	R=ima_R;
	V=ima_V;
	B=ima_B;
	L=ima_L;
	T=ima_T;
	S=ima_S;

	DEUX_PI = 2*M_PI;
	Fact_C2 = sqrt(3.0)/2.0;
	Fact_T = (double)(255.0 / DEUX_PI);	/* Norm Couleur entre 0..255 */

	for(i=Y;i--;)
		for(j=X;j--;)
			{
						/* calcul de la luminance */
				;
				/* *L=(unsigned char)irint(((double)*R + (double)*V  + (double)*B)/3.0 ); */
				*L=(unsigned char)rint(((double)*R + (double)*V  + (double)*B)/3.0 );
						/* calcul facteur C1 et C2 */
				C1 = (double)(*R) - ((double)(*V) + (double)(*B)) / 2.0;
				C2 = ((double)(*V) - (double)(*B)) * Fact_C2;

						/* Calcul de la teinte */
				if (C1 == 0.0)
				{
				   if (C2 > 0)
				      *T = 64;
				   else
				      *T = 192;
				}
				else
				{
				   calc_inter = (double)C2 / (double)C1;
				   if (C1 > 0.0) 
				      double_T = atan(calc_inter) * Fact_T;
				   else
		      		  double_T = (M_PI + atan(calc_inter)) * Fact_T;

					if (double_T >= 0)
					{
						if (double_T - (int)double_T < 0.4999)
						{
							*T = (unsigned char)((int)double_T );
						}
						else
						{
							*T = (unsigned char)((int)double_T + 1);
						}
					}
					else
					{
						if (double_T - (int)double_T  > 0.5)
						{
							*T = (unsigned char)(255 + (int)double_T + 1);
						}
						else
						{
							*T = (unsigned char)(255 + (int)double_T);
						}
			
					}
 
				}

						/* Calcul de la saturation */
				calc_inter = (double)(C1) * (double)C1  + (double)(C2) * (double)C2;
				/* *S = (unsigned char)irint(sqrt(calc_inter)); */
				*S = (unsigned char)rint(sqrt(calc_inter));

				 /* ************************************* */
				*T += (unsigned char) des->comp_a;
				*L += (unsigned char) des->comp_b;
				*S += (unsigned char) des->comp_c;
				/* ************************************* */

				des->comp0[i][j] = (double) *T;
				des->comp1[i][j] = (double) *L;
			        des->comp2[i][j] = (double) *S;

	       			/* Passage pixel suivant */
				R++,V++,B++;
				L++,T++,S++;
       			     
			}
}

////////////////////////////////////////////////////////////
/*L'espace HSV*/


double maxim(double i, double j)
{
  if(i > j)
    return(i);
  else
    return(j);
}

double minim(double i, double j)
{
  if(i < j)
    return(i);
  else
    return(j);
}



void  conv_RVB_HSV(space_struct *des, imarvb *ima_rvb)
{

unsigned char *ima_R, *ima_V, *ima_B;
int Y, X;
pixdb *ima_H, *ima_S, *ima_Val;

unsigned char *R, *V, *B;
pixdb *H, *S, *Val;

double *RD, *VD, *BD, *HD, *SD, *ValD;

register i, j;

double imax, imin, rc, vc, bc;


       	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];
	Y = ima_rvb->nr;
	X = ima_rvb->nc;
	ima_H = des->comp0[0];
	ima_S = des->comp1[0];
	ima_Val = des->comp2[0];

	R=ima_R;
	V=ima_V;
	B=ima_B;
	H=ima_H;
	S=ima_S;
	Val=ima_Val;

        RD = (double *)ima_R; VD = (double *) ima_V; BD = (double *) ima_B; 
        HD = (double *)ima_H; SD = (double *) ima_S; ValD = (double *) ima_Val; 

	for(i=Y;i--;)             
            for(j=X;j--;)
	       	{
          
      *RD = (double)(*R);  *VD = (double)(*V);  *BD = (double)(*B);
      *RD = (*RD)/ 255.0;  *VD = (*VD)/ 255.0;  *BD = (*BD)/ 255.0;

           imax = maxim(*RD, maxim(*VD, *BD));
	   imin = minim(*RD, minim(*VD, *BD));
	   
 if (*RD<0.0 || *RD>1.0 || *VD<0.0 || *VD>1.0 || *BD<0.0 || *BD>1.0 ){
   printf("\nERROR: Les valeurs de *R, *V et *B sont dehors l'interval [0,1].");
   printf("\nOperateur:  conv_RVB_HSV() \n"); 
   exit(1);}
   

    *ValD = imax;
    if (imax != 0) {
	*SD = (imax - imin) / imax;
    } else {
	*SD = 0;
    }

    if (*SD == 0) {
	*HD = -1;
    } else {
	rc = (imax - *RD) / (imax - imin);
	vc = (imax - *VD) / (imax - imin);
	bc = (imax - *BD) / (imax - imin);
	if (*RD == imax) {
	    *HD = bc - vc;
	} else if (*VD == imax) {
	    *HD = 2.0 + rc - bc;
	} else {
	    *HD = 4.0 + vc - rc;
	}
	*HD *= 60.0;
	if (*HD < 0.0) {
	    *HD += 360.0;
	}
    }    

      *HD += des->comp_a;   *SD += des->comp_b;   *ValD += des->comp_c;

      des->comp0[i][j] = *HD; des->comp1[i][j] = *SD; des->comp2[i][j] = *ValD;

      R++, V++, B++;
      H++, S++, Val++;
     }
}




void conv_HSV_RVB(space_struct *des, imarvb *ima_rvb)
{
  
pixdb *ima_H, *ima_S, *ima_Val;
unsigned char *ima_R, *ima_V, *ima_B;

int Y, X;

unsigned char *R, *V, *B;
pixdb *H, *S, *Val;

register i, j;
int z;
double f, p, q, t;

double *RD, *VD, *BD, *HD, *SD, *ValD;

        ima_H = des->comp0[0];
	ima_S = des->comp1[0];
	ima_Val = des->comp2[0];
       	Y = des->nr;
	X = des->nc;
       	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];
	
	R=ima_R;
	V=ima_V;
	B=ima_B;
	H=ima_H;
	S=ima_S;
	Val=ima_Val;

	HD = (double *)ima_H; SD = (double *) ima_S; ValD = (double *) ima_Val; 
    	RD = (double *)ima_R; VD = (double *) ima_V; BD = (double *) ima_B; 

   for(i=Y;i--;)
      for(j=X;j--;)
	 {
      *HD = des->comp0[i][j];  *SD = des->comp1[i][j];  *ValD =  des->comp2[i][j];
      *HD += des->comp_a;      *SD += des->comp_b;      *ValD += des->comp_c;

    if (*SD == 0) {
	*RD = *VD = *BD = *ValD;
    } else {
	if (*HD == 360.0) {*HD = 0;}

	*HD /= 60.0;

	z = (int)floor(*HD);
	f = *HD - z;
	p = *ValD * (1.0 - *SD);
	q = *ValD * (1.0 - (*SD * f));
	t = *ValD * (1.0 - (*SD * (1.0 - f)));

	switch (z) {
	    case 0:
		*RD = *ValD;
		*VD = t;
		*BD = p;
		break;

	    case 1:
		*RD = q;
		*VD = *ValD;
		*BD = p;
		break;

	    case 2:
		*RD = p;
		*VD = *ValD;
		*BD = t;
		break;

	    case 3:
		*RD = p;
		*VD = q;
		*BD = *ValD;
		break;

	    case 4:
		*RD = t;
		*VD = p;
		*BD = *ValD;
		break;

	    case 5:
		*RD = *ValD;
		*VD = p;
		*BD = q;
		break;
	 }
    }

     *RD = (*RD) * 255.0;     *VD = (*VD) * 255.0;     *BD = (*BD) * 255.0;
     *R = (unsigned char)(*RD); *V = (unsigned char)(*VD); *B = (unsigned char)(*BD);
 
      R++, V++, B++;
      H++, S++, Val++;

      }
}

/* ********************************************************* */

/* Transforme la carte d'intensite RGB to XYZ */

void toetRGB_to_XYZ(RD, VD, BD, X, Y, Z)
double RD, VD, BD, *X, *Y, *Z;
{
   if (RD<0.0 || RD>1.0 || VD<0.0 || VD>1.0 || BD<0.0 || BD>1.0 ){
	printf("\nEREURE: Les valeurs de *R, *V et *B sont dehors l'interval [0,1].");
	printf("\nOperateur:   toetRGB_to_XYZ() \n"); 
	exit(1);}

   *X= 0.040*RD + 0.084*VD - 0.041*BD;
   *Y= 0.021*RD + 0.107*VD - 0.038*BD;
   *Z=-0.005*RD + 0.011*VD + 0.024*BD;

}

/* Transforme la carte d'intensite XYZ to RGB */

void ceiRGB_to_XYZ(RD, VD, BD, X, Y, Z)
double RD, VD, BD, *X, *Y, *Z; 
{
   if (RD<0.0 || RD>1.0 || VD<0.0 || VD>1.0 || BD<0.0 || BD>1.0 ){
	printf("\nEREURE: Les valeurs de *R, *V et *B sont dehors l'interval [0,1].");
	printf("\nOperateur:  ceiRGB_to_XYZ() \n"); 
	exit(1);}
   
   *X = 0.490*RD + 0.310*VD + 0.200*BD;
   *Y = 0.177*RD + 0.813*VD + 0.011*BD;
   *Z = 0.000*RD + 0.010*VD + 0.990*BD;

}

void XYZ_to_ceiuvY(X,Y,Z,u,v,Y1)
double X,Y,Z;
double *u, *v, *Y1;
{
  double w;

  /*  
if (X<0.0 || X>1.0 || Y<0.0 || Y>1.0 || Z<0.0 || Z>1.0 ){
   printf("\nEREURE: Les valeurs de X, Y et Z sont dehors l'interval [0,1].");
   printf("\nOperateur: XYZ_to_ceiuvY() \n"); 
   exit(1);}
   */

  w=X+15.0*Y+3.0*Z;
  if(w!=0.0)
    {
      *Y1=Y;
      *u=(4.0*X)/w;
      *v=(6.0*Y)/w;
    }
  else{
    *Y1=0.0;
    *u=0.0;
    *v=0.0;
  }
}


double f(double t)
{
  if( t> 0.008856)
  return(pow(t,(1.0/3.0)));
else 
  return(0.787*t+16.0/116.0);
}



void conv_RVB_Lab(space_struct *des, imarvb *ima_rvb)
{
  
unsigned char *ima_R, *ima_V, *ima_B;
int N, M;
pixdb *ima_L, *ima_a, *ima_b;

unsigned char *R, *V, *B;
pixdb *L, *a, *b;

double *RD, *VD, *BD, *LD, *aD, *bD;

register i, j;

double X, Y, Z, u, v, Y_Yo, Xo, Yo, Zo;

       	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];
	N = ima_rvb->nr;
	M = ima_rvb->nc;
	ima_L = des->comp0[0];
	ima_a = des->comp1[0];
	ima_b = des->comp2[0];

	R=ima_R;
	V=ima_V;
	B=ima_B;
	L=ima_L;
	a=ima_a;
	b=ima_b;

	 RD = (double *)ima_R; VD = (double *) ima_V; BD = (double *) ima_B; 
	 LD = (double *)ima_L; aD = (double *) ima_a; bD = (double *) ima_b;

	for(i=N;i--;)             
            for(j=M;j--;)
	       	{
		  
      *RD = (double)(*R);   *VD = (double)(*V);    *BD = (double)(*B);      
      *RD = (*RD)/ 255.0;   *VD = (*VD)/ 255.0;    *BD = (*BD)/ 255.0;      
      
      if (*RD<0.0 || *RD>1.0 || *VD<0.0 || *VD>1.0 || *BD<0.0 || *BD>1.0 ){
	printf("\nERROR: Les valeurs de *R, *V et *B sont dehors l'interval [0,1].");
	printf("\nOperateur:  conv_RVB_HSV() \n"); 
	exit(1);}
      
      /*Les coordonees de reference pour blanc  toetRGB */
      if(des->transf_interm == 0)
      	toetRGB_to_XYZ(1.0,1.0,1.0, &Xo, &Yo, &Zo);
      else if(des->transf_interm == 1)
        ceiRGB_to_XYZ(1.0,1.0,1.0, &Xo, &Yo, &Zo);

      /* Conversion de toetRGB a XYZ de CEI*/
      if(des->transf_interm == 0)
      	toetRGB_to_XYZ(*RD, *VD, *BD, &X, &Y, &Z);
      else if(des->transf_interm == 1)
      	ceiRGB_to_XYZ(*RD, *VD, *BD, &X, &Y, &Z);
             /*	printf("\nOJO: Conversion RGB-XYZ Standar\n"); */

      /* Les coordonees Lab  CEI */
      Y_Yo=Y/Yo;
      if( Y_Yo> 0.008856)
	 *LD=116.0*pow(Y_Yo,(1.0/3.0))-16.0;
      else if( Y_Yo <= 0.008856)
	 *LD=903.3*Y_Yo;

      *aD=500.0*(f(X/Xo)-f(Y/Yo));
      *bD=200.0*(f(Y/Yo)-f(Z/Zo));
     
      *LD += des->comp_a;      *aD += des->comp_b;      *bD += des->comp_c;

      des->comp0[i][j] = *LD; des->comp1[i][j] = *aD; des->comp2[i][j] = *bD;
     
      R++, V++, B++;
      L++, a++, b++;      
       	}  
}



void conv_RVB_Luv(space_struct *des, imarvb *ima_rvb)
{
     
unsigned char *ima_R, *ima_V, *ima_B;
int N, M;
pixdb *ima_L, *ima_u, *ima_v;

unsigned char *R, *V, *B;
pixdb *L, *U, *Va;

double *RD, *VD, *BD, *LD, *uD, *vD;

register i, j;

double X, Y, Z, u, v, Y_Yo, uo, vo, Yo;

       	ima_R = ima_rvb->r[0];
	ima_V = ima_rvb->v[0];
	ima_B = ima_rvb->b[0];
	N = ima_rvb->nr;
	M = ima_rvb->nc;
	ima_L = des->comp0[0];
	ima_u = des->comp1[0];
	ima_v = des->comp2[0];

	R=ima_R;
	V=ima_V;
	B=ima_B;
	L=ima_L;
	U=ima_u;
	Va=ima_v;

	 RD = (double *)ima_R; VD = (double *) ima_V; BD = (double *) ima_B; 
	 LD = (double *)ima_L; uD = (double *) ima_u; vD = (double *) ima_v; 

	for(i=N;i--;)             
            for(j=M;j--;)
	       	{		  

      *RD = (double)(*R);   *VD = (double)(*V);    *BD = (double)(*B);      
      *RD = (*RD)/ 255.0;   *VD = (*VD)/ 255.0;    *BD = (*BD)/ 255.0;      
      
      if (*RD<0.0 || *RD>1.0 || *VD<0.0 || *VD>1.0 || *BD<0.0 || *BD>1.0 ){
	printf("\nERROR: Les valeurs de *R, *V et *B sont dehors l'interval [0,1].");
	printf("\nOperateur:  conv_RVB_HSV() \n"); 
	exit(1);}
      
        /*Les coordonees de reference pour blanc  toetRGB */
      if(des->transf_interm == 0)
	 toetRGB_to_XYZ(1.0,1.0,1.0, &X, &Y, &Z);
      else if(des->transf_interm == 1)
      	ceiRGB_to_XYZ(1.0,1.0,1.0, &X, &Y, &Z);
      
      XYZ_to_ceiuvY(X,Y,Z,&uo,&vo,&Yo);

      /* Conversion de toetRGB a XYZ de CEI*/
      if(des->transf_interm == 0)
      	toetRGB_to_XYZ(*RD, *VD, *BD, &X, &Y, &Z);
      else  if(des->transf_interm == 1) 
        ceiRGB_to_XYZ(*RD, *VD, *BD, &X, &Y, &Z);
        
      XYZ_to_ceiuvY(X,Y,Z,&u,&v,&Y_Yo);

      /* Les coordonees Luv  CEI */

      Y_Yo/=Yo;
      if( Y_Yo > 0.008856)
	*LD = 116.0 * pow(Y_Yo,(1.0/3.0)) - 16.0;
      else if( Y_Yo <= 0.008856)
	*LD = 903.3 * Y_Yo;

      *uD=13.0*(*LD)*(u-uo);
      *vD=13.0*(*LD)*(v-vo);    

      *LD += des->comp_a;      *uD += des->comp_b;      *vD += des->comp_c;

       des->comp0[i][j] = *LD; des->comp1[i][j] = *uD; des->comp2[i][j] = *vD;
    
      R++, V++, B++;
      L++, U++, Va++;
     }      
}




/* *************************  LECTURE  ****************************** */
/** lecture des parametres de l'operateur de conversion image. \\

    @name conv_img_lect
    @param des pointeur descripteur     
    @param ptp pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author Benone Ionescu
    @version 1.0 (10/12/00); Include : image.h, proto2D.h, conv_img.h
*/
param *conv_img_lect(space_struct *des, param *ptp, char *debq)
{
  char question[500];

  sprintf(question, "%s Tipe de conversion (entier)", debq); 
  lec_param(question, ptp);
  des->type_conv = atoi(ptp->rep);
  ptp = ptp->next;
   
  switch(des->type_conv)
    {
    case 1:
    case 2: { sprintf(question, "%s Teinte (TLS) (entier)", debq); 
              lec_param(question, ptp);
              des->comp_a = atof(ptp->rep);
              ptp = ptp->next; 
   
              sprintf(question, "%s Luminance (TLS) (entier)", debq); 
              lec_param(question, ptp);
              des->comp_b = atof(ptp->rep);
              ptp = ptp->next;  

	      sprintf(question, "%s Saturation (TLS) (entier)", debq); 
              lec_param(question, ptp);
              des->comp_c = atof(ptp->rep);
              ptp = ptp->next;

              break;}

    case 3: 
    case 4: { sprintf(question, "%s Hue - Teinte (HSV) (real) 0 - 360 ", debq); 
              lec_param(question, ptp);
              des->comp_a = atof(ptp->rep);
              ptp = ptp->next; 
   
              sprintf(question, "%s Saturation (HSV) (real) 0 - 1 ", debq); 
              lec_param(question, ptp);
              des->comp_b = atof(ptp->rep);
              ptp = ptp->next;  

	      sprintf(question, "%s Value (HSV) (real) 0 - 1 ", debq); 
              lec_param(question, ptp);
              des->comp_c = atof(ptp->rep);
              ptp = ptp->next;

              break;}

    case 5: { sprintf(question, "%s L (Lab) (real) ", debq); 
              lec_param(question, ptp);
              des->comp_a = atof(ptp->rep);
              ptp = ptp->next; 
   
              sprintf(question, "%s a (Lab) (real) ", debq); 
              lec_param(question, ptp);
              des->comp_b = atof(ptp->rep);
              ptp = ptp->next;  

	      sprintf(question, "%s b (Lab) (real) ", debq); 
              lec_param(question, ptp);
              des->comp_c = atof(ptp->rep);
              ptp = ptp->next;
              
              sprintf(question, "%s (Transformation intermediaire) (0 - Transforme les intensite toetRGB en XYZ apres en Lab) (1 - Transforme les intensite ceiRGB en XYZ apres en Lab)", debq); 
              lec_param(question, ptp);
              des->transf_interm  = atoi(ptp->rep);
              ptp = ptp->next;

              break;}

    case 6:  { sprintf(question, "%s L (Luv) (real) ", debq); 
              lec_param(question, ptp);
              des->comp_a = atof(ptp->rep);
              ptp = ptp->next; 
   
              sprintf(question, "%s u (Luv) (real) ", debq); 
              lec_param(question, ptp);
              des->comp_b = atof(ptp->rep);
              ptp = ptp->next;  

	      sprintf(question, "%s v (Luv) (real) ", debq); 
              lec_param(question, ptp);
              des->comp_c = atof(ptp->rep);
              ptp = ptp->next;
		
	      sprintf(question, "%s (Transformation intermediaire) (0 - Transforme les intensite toetRGB en XYZ apres en Lab) (1 - Transforme les intensite ceiRGB en XYZ apres en Lab)", debq); 
              lec_param(question, ptp);
              des->transf_interm = atoi(ptp->rep);
              ptp = ptp->next;
	
              break;}

    default: printf("Nombre incorect");
    }

  return(ptp);
}


/* *************************  INITIALISATION  ************************** */
/** initialisation de l'operateur conversion image.\\

Attention : on alluer l'espace pour l'image resultant imres (TLS) et pour le descripteur contenant les 3 composantes de l'image resultant (HSV, Lab et Luv)
    @name conv_img_init 
    @param des  pointeur descripteur
    @param im0  image initiale
    @param imres image resultante
    @author B. Ionescu
    @version 1.0 (10/12/00); Include : image.h, proto2D.h, conv_img.h
*/
int conv_img_init(space_struct *des, imarvb im0, imarvb *imres0)
{   
  imres0->nr = im0.nr;
  imres0->nc = im0.nc;
  alloc_imarvb(imres0);

  des->nr = im0.nr;
  des->nc = im0.nc;
  alloc_space_struct(des, &im0);  
}


/* *************************  CALCUL  ************************** */
/** calcul de l'operateur conversion image.\\

    @name conv_img_calc   
    @param des  pointeur descripteur
    @param im0  image initiale
    @param imres image resultante
    @author B. Ionescu
    @version 1.0 (10/12/00); Include : image.h, proto2D.h, conv_img.h
*/
int  conv_img_calc(space_struct *des, imarvb im0, imarvb *imres0)
{
    
switch(des->type_conv)
    {
    case 1: {conv_RVB_TLS2(des, &im0, imres0); break;}
    case 2: {conv_TLS_RVB2(des, &im0, imres0); break;}
    case 3: {conv_RVB_HSV(des, &im0); break;}
    case 4: {conv_HSV_RVB(des, imres0); break;}
    case 5: {conv_RVB_Lab(des, &im0); break;}
    case 6: {conv_RVB_Luv(des, &im0); break;}
    default: printf("Nombre incorect");
    }

}




/* ************************* alloc_space_struct ************************* */
/* TYPES space_struct
    allocation de memoire pour la structure de donnes space_struct.\\
    allocation en une fois permettant l'adressage 1D : im->p[0] + j*im->nc + i
    ou l'adressage 2D : im->p[j][i]
    @param space  pointeur space_structure a allouer pour les 3 vecteurs contenant les composantes de l'image convertie
    @param im  pointeur image a convertir, nombre de colonnes (im->nc) et 
    nombre de lignes (im->nr) connus
    @author B. Ionescu
    @version 1.0 (10/12/00); Include : proto2D.h   lconv.h
*/

int alloc_space_struct(space_struct *space, imarvb *im)
{
   int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (space->comp0 = (pixdb **)malloc(im->nr*sizeof(pixdb *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
  if( (space->comp1 = (pixdb **)malloc(im->nr*sizeof(pixdb *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
  if( (space->comp2 = (pixdb **)malloc(im->nr*sizeof(pixdb *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (space->comp0[0] = (pixdb *)malloc(im->nc*im->nr*sizeof(pixdb))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
  if( (space->comp1[0] = (pixdb *)malloc(im->nc*im->nr*sizeof(pixdb))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
  if( (space->comp2[0] = (pixdb *)malloc(im->nc*im->nr*sizeof(pixdb))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++){
        space->comp0[j] =  space->comp0[0] + j*(im->nc);
        space->comp1[j] =  space->comp1[0] + j*(im->nc);
        space->comp2[j] =  space->comp2[0] + j*(im->nc);
  }
}

/* ********************* liberation memeoire image  ********************** */
/* TYPES space_struct
    desallocation d'une structure.
    \\
    @param space  pointeur space_struct  a desallouer, 
    @author Benone Ionescu
    @version 1.0 (10/12/00); Include : proto2D.h  conv_img.h
*/

int free_space_struct(space_struct *space)
{
  free(space->comp0[0]);  free(space->comp1[0]); free(space->comp2[0]);
  free(space->comp0);  free(space->comp1);  free(space->comp2);
}



/*@}*/  /* fin doc en sous partie */   
