/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* **************************** llabelcc.c ******************************/
/*                                                                    */
/* ********************************************************************/

/** \ingroup divers
    \defgroup labelcc operateur d'etiquetage en composante connexe d'image 8 bit seuillee.
Choix d'un seuil, du contraste  (>=0 : objet=[seuil, 255]; <0 : objet=[0, seuil]) et 
de la connexite (4 ou 8). \\
Resultat : image 16 bit des etiquettes, 0 pour le fond.\\
N.B. Cette operateur ne fait qu'appeler un fonction ecrite par Michel Roux a l'ENST\\
\\
Descripteur : \\
    int connex   :  connexite 4 ou 8  \\  
    int seuil   :  seuil de demarage des niveaux de gris des objets  \\  
    int contraste   :  <0 pour objets sombres, >=0 pour objets claires  \\  
\\
Type entree : imau1 \\
Type sortie : imau2 : les etiquettes\\
@name llabelcc
@author M. Roux, E. Trouve
@version 1.0 (15/06/00); Include : image.h, proto2D.h, labelcc.h
@see mlabelcc_u1
*/

#include "image.h" 
#include "proto2D.h"
#include "labelcc.h" 
#include <stdlib.h>
#include <fcntl.h>

/* ENST : =============================================================*/
#define TAILLE_TABLE 0x10000
#define new_object ((nbfs >= 2) ? ++objets : 1 + (objets++ % 255))

	/* Variables statiques */
static pixu2 *t_lut;
static pixu2 etiq;
static int connex_8;
static int masque;
static int seuil;
static int negatif;
static int np , nl ;

pixu1 **ENTREE ;
pixu2 **SORTIE ;

/* FIN ENST ===================================================================*/


/* *************************  LECTURE  *******************************/
/** \ingroup labelcc
    lecture des parametres de l'operateur labelcc. \\
    connex : connexite (4 ou 8) \\
    seuil : seuil de demarage des niveaux de gris des objets\\
    contraste : <0 : objets sombres, >=0 objets claires 
    @name labelcc_lect
    @param des pointeur descripteur     
    @param ptp pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author E. Trouve
    @version 1.0 (09/06/00); Include : image.h, proto2D.h, labelcc.h
*/
param *labelcc_lect( labelcc_t *des, param *ptp, char *debq){
  char question[500];

  sprintf(question, "%s connexite : 4 ou 8", debq); 
  lec_param(question, ptp);
  des->connex = atoi(ptp->rep);
  ptp = ptp->next;
  if(des->connex!=4 && des->connex!=8 ){
      printf("\n>>ERREUR labelcc_lect : connexite %d inconnu (4 ou 8)\n", 
	     des->connex); 
      exit(1);
  }

  sprintf(question, "%s seuil : premier niveau de gris des objets", debq); 
  lec_param(question, ptp);
  des->seuil = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s contraste : <0 = objets sombres, >=0 = objets claires",
	  debq); 
  lec_param(question, ptp);
  des->contraste = atoi(ptp->rep);
  ptp = ptp->next;

  return(ptp);
}

/* *************************  INITIALISATION  ***************************/
/** \ingroup labelcc
    XX = u1  : initialisation de l'operateur labelcc.\\
    allocation image resultat 
    @name labelccXX_init 
    @param des  pointeur descripteur
    @param im0  image initiale (taille connue)
    @param imres  pointeur image resultat type u2 (16 bit)
    @author E. Trouve
    @version 1.0 (09/06/00); Include : image.h, proto2D.h, labelcc.h
*/
int labelccu1_init(labelcc_t *des, imau1 im0, imau2 *imres){     
  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s_label_s%d", im0.nom, des->seuil);
  alloc_imau2(imres);
}


/* *************************  CALCUL  ***************************/
/** \ingroup labelcc
    XX = u1 : calcul de l'operateur labelcc.\\
    Calcul de l'image des etiquettes.
    @name labelccXX_calc  
    @param des  pointeur descripteur
    @param im0  image initiale 
    @param imres  pointeur image resultat (type u2 : 16 bit)
    @return nombre d'objets 
    @author E. Trouve
    @version 1.0 (09/06/00); Include : image.h, proto2D.h, labelcc.h
*/
int labelccu1_calc(labelcc_t *des, imau1 im0, imau2 *imres){
int nbo;
int nbfs=2;

nbo = ETIQ(&im0, imres, &(des->connex), &(des->seuil), &(des->contraste), &nbfs);

return(nbo);
}    


/* ENST : =============================================================*/
/* **********************************************************************
 * NOM 
 * 
 *      etiq.c : routine pour l'etiquetage des parties connexes d'une image 
 *               codee sur un octet. Retourne le nombre d'objets dans l'image.
 *
 * ****************************************************************************
 * SYNOPSIS 
 *
 * int ETIQ ( IMA_IN, IMA_EX, NP, NL, CONNEX, SEUIL, CONTRASTE, NBFS)
 *      imau1	*IMA_IN ;
 *      imau2 *IMA_EX ;
 *      int *CONNEX;
 *      int *SEUIL;
 *      int *CONTRASTE;
 *      int *NBFS;
 * 
 * void save_label(imau2 label, char *message)
 *
 * ****************************************************************************
 * DESCRIPTION
 *
 *      Cette routine effectue l'etiquetage des parties connexes d'une image
 *      codee sur 1 octet. Dans l'image de sortie, les pixels d'une meme zone 
 *      connexe ont la meme valeur (etiquette) codee sur 2 octets.
 *      
 *
 * ****************************************************************************
 * PARAMETRES
 *
 * IMA_IN (*imau1)
 *      Image originale codee sur 1 octet.
 *
 * IMA_EX (*imau2)
 *      Image des etiquettes codee sur 2 octets. L'allocation memoire 
 *      de ce tableau doit avoir ete effectuee precedemment.
 *
 * CONNEX (*int)
 *      Pointeur sur la connexite (4 ou 8) qui doit etre utilisee pour 
 *      determinee les objets presents dans l'image. N.B. Si *CONNEX n'est
 *      pas egal a 4, alors on se place en 8-connexite.
 *
 * SEUIL (*int)
 *      Pointeur sur le seuil (en niveau de gris) entre les objets et le fond.
 *      
 * CONTRASTE (*int)
 *      Pointeur donnant le contraste des objets recherches. Si *CONTRASTE
 *      est strictement negatif, on cherche des objets sombres (val <= *SEUIL),
 *      sinon on cherche des objets clairs (val >= *SEUIL).
 *
 * NBFS (*int)
 *      Pointeur donnant le nombre d'octets utilises pour coder les pixels
 *      dans l'image des etiquettes.
 *      N.B. : NBFS doit etre egal a 2.
 *
 * ****************************************************************************
 * HISTORIQUE
 *
 *
 * 25-oct-95  Emmanuel trouve : adaptation aux structure personnelles
 *  		ajout de save_label
 * 22-mars-95 Michel Roux (mroux@ima), TELECOM Paris / IMA.
 *     Utilisation des structures Image et Ima16.
 *     Suppression des parametres NP et NL
 *
 * 17-fev-95 Michel Roux (mroux@ima), TELECOM Paris / IMA.
 *     Addition de cet entete.
 *
 * 18-nov-92 Alain Clainchard (clainchard@ima), TELECOM Paris / IMA.
 *     Creation
 *
 */

int ETIQ ( IMA_IN, IMA_EX, CONNEX, SEUIL, CONTRASTE, NBFS)
     imau1 *IMA_IN ;
     imau2 *IMA_EX ;
     int *CONNEX;
     int *SEUIL;
     int *CONTRASTE;
     int *NBFS;
{

register unsigned int i, j;
int objets;
int nlf = 2;
int bord = 1;
int nbm = sizeof(pixu1);
int nbfs;
int nargs;

		/* Controle des arguments */

	connex_8 = (*CONNEX != 4);
	negatif = (*CONTRASTE < (int) 0);
	seuil = negatif ? 0 : 1;
	if (*SEUIL > seuil) seuil = *SEUIL;
	nbfs = (*NBFS > 0) ? *NBFS : 0;
	masque = connex_8 ? 0xF : 0x7;
	ENTREE = IMA_IN->p ; SORTIE = IMA_EX->p ;
	np = IMA_IN->nc ; nl = IMA_IN->nr ;

		/* Initialisation de la table des etiquettes */

	if ((t_lut = (pixu2 *) calloc (TAILLE_TABLE, sizeof(pixu2))) == NULL) {
		fprintf (stderr,"\n*** ETIQ *** Memoire insuffisante.\n");
		exit(2);
	}
	t_lut[0] = 0;


		/* Premiere passe: identification des objets */

	ETIQ_S1 () ;

		/* Generation des etiquettes definitives */

	for (objets = 1; objets <= (int) etiq; objets++) {
		i = objets; do { j = i; i = t_lut[i]; } while (i != j);
		t_lut[objets] = i;
	}
	objets = 0;
	for (i = 1; i <= (int) etiq; i++) {
		j = t_lut[i];
		t_lut[i] = (pixu2) ((j == i) ? new_object : t_lut[j]);
	}
	fprintf (stdout, "\n*** ETIQ *** %u objets dnombrs.\n", objets);

		/* Deuxieme passe: etiquetage des objets */

	ETIQ_S2 () ;


	free (t_lut);

	return ((int) objets);

}
#undef new_object

/*========================================================================
*/

/* PREMIERE PASSE: identification des objets */

#define new_label(X)							\
	if (etiq == (pixu2) (TAILLE_TABLE - 1)) {                	\
		fprintf (stderr, "\n*** ETIQ *** table sature\n");	\
		exit (2);						\
	}                                                        	\
	X = ++etiq;

#define merge(X, Y)	if (X < Y) Y = t_lut[Y] = X; else X = t_lut[X] = Y;

ETIQ_S1 ()
{

register int i,j;
register unsigned int ouest, aux;
unsigned int nord, nord_ouest;
int status;
pixu2 *ptr1, *ptr2 , *bid ;

		
	ptr1 = (pixu2 *) malloc ( np * sizeof ( pixu2 ) ) ;
	ptr2 = (pixu2 *) malloc ( np * sizeof ( pixu2 ) ) ;

	for ( i = 0 ; i < np ; ptr2[i++] = 0 ) ;

	etiq = 0 ;

/* Cas general */

	for ( j = 0 ; j < nl ; j ++ ) {
		bid = ptr1 ;
		ptr1 = ptr2 ;
		ptr2 = bid ;
		
		if (negatif) for (i = 0 ; i < np ; i++)
			ptr2[i] = (pixu2) (( (int) ENTREE[j][i] <= seuil) ? -1 : 0);
		else 
			if (seuil > 1) for (i = 0 ; i < np ; i++)
			   ptr2[i] = (pixu2)(( (int) ENTREE[j][i] >= seuil) ? -1 : 0);
			else for ( i = 0 ; i < np ; i++ ) 
			   ptr2[i] = (pixu2) ENTREE[j][i] ;


		nord_ouest = ouest = status = 0;
		for (i = 0 ; i < np ; i++) {
			status = (status << 2) & masque;
			if ((nord = ptr1[i]) != 0) {
				while ((aux = t_lut[nord]) != nord) nord = aux;
				status += 2;
				}
			if (ptr2[i] != 0) status++;
			switch (status) {
				case 3:
				case 11:
					ouest = nord;
				case 5:
				case 13:
				case 15:
					break;

				case 1:
					new_label (ouest);
					t_lut[ouest] = ouest;
					break;

				case 7:
					merge (nord, ouest);
					break;

				case 9:
					ouest = nord_ouest;
					break;

				case 6:
					if (connex_8) merge (nord, ouest);
				default:
					ouest = 0;
				}
			ptr2[i] = (pixu2) ouest; nord_ouest = nord;
			}
		}
}
#undef merge
#undef new_label

/*========================================================================
*/

/* DEUXIEME PASSE: ETIQUETAGE DES OBJETS */

#define merge(X, Y)	if (X < Y) Y = X; else X = Y;

ETIQ_S2 ()
{

register int i,j;
register unsigned int ouest, aux;
unsigned int nord, nord_ouest;
int status;
pixu2 *ptr1, *ptr2 , *bid ;

	ptr1 = (pixu2 *) malloc ( np * sizeof ( pixu2 ) ) ;
	ptr2 = (pixu2 *) malloc ( np * sizeof ( pixu2 ) ) ;

	etiq = 0 ;

	for ( i = 0 ; i < np ; ptr2[i++] = 0 ) ;


/* Cas general */

	for ( j = 0 ; j < nl ; j ++ ) {
		bid = ptr1 ;
		ptr1 = ptr2 ;
		ptr2 = bid ;
		
		if (negatif) for (i = 0 ; i < np ; i++)
			ptr2[i] = (pixu2) (( (int) ENTREE[j][i] <= seuil) ? -1 : 0);
		else 
			if (seuil > 1) for (i = 0 ; i < np ; i++)
			   ptr2[i] = (pixu2)(( (int) ENTREE[j][i] >= seuil) ? -1 : 0);
			else for ( i = 0 ; i < np ; i++ ) 
			   ptr2[i] = (pixu2) ENTREE[j][i] ;


		nord_ouest = ouest = status = 0;
		for (i = 0; i < np ; i++) {
			status = (status << 2) & masque;
			if ((nord = ptr1[i]) != 0) status += 2;
			if (ptr2[i] != 0) status++;
			switch (status) {
				case 3:
				case 11:
					ouest = nord;
				case 5:
				case 13:
				case 15:
					break;

				case 1:
					ouest = ++etiq;
					break;

				case 7:
					merge (nord, ouest);
					break;

				case 9:
					ouest = nord_ouest;
					break;

				case 6:
					if (connex_8) merge (nord, ouest);
				default:
					ouest = 0;
			}
			ptr2[i] = (pixu2) ouest; 
			nord_ouest = nord;
			SORTIE[j][i] = (pixu2) t_lut[ouest];
		}
	}
}

