/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \ingroup filtrage
    \defgroup medhist operateur median 2D
TRAITEMENT BIDIMENSIONNEL (D'UNE IMAGE) PAR LA METHODE DE L'HISTOGRAMME
AVEC UNE FENETRE RECTANGULAIRE (TRAITEMENT NON SEPARABLE)


Descripteur : 
   - int taille   :  dimension de la fenetre 

Type entree : imau1
Type sortie : imau1

@author G. Marinescu, L. Valet, P. Lambert
@version 1.0 (01.04.03); Include : medhist.h 
@see
*/

#include <math.h>
#include <sys/param.h>
#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <fcntl.h>
#include "medhist.h"


/* **********************  LECTURE DE PARAMETRES   *************************/
/** \ingroup medhist
    lecture des parametres de l'operateur medhist. \\
    tf : taille fenetre (impaire) \\

    @param   des  pointeur descripteur	  
    @param   ptp  pointeur sur parametre courant deja alloue 
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return  renvoie nouveau pointeur de parametre deja alloue 
    @author  G. Marinescu, L. Valet, P. Lambert
    @version 1.0 (01.04.03); Include : medhist.h
*/
param *medhist_lect(median_hist_t *des, param *ptp, char *debq){
  char question[500];

  sprintf(question, "%s taillefenetre (impaire)", debq); 
  lec_param(question, ptp);
  des->tf = atoi(ptp->rep);
  ptp = ptp->next; 

  if( (des->tf)%2 == 0){
      printf("\n>> ERREUR taillefenetre : dimensions incorrectes\n");
      exit(1);
  }

  return(ptp);
}


/* **********************  INITIALISATION  *****************************/
/** \ingroup medhist
    XX = u1 : initialisation de l'operateur medhist.\\
    allocation image resultat et colonne de travail\\

    @param des : pointeur descripteur
    @param im0 : image initiale (taille connue)
    @param imres : pointeur image resultat
    @author G. Marinescu, L. Valet, P. Lambert
    @version 1.0 (01.04.03); Include : image.h, proto2D.h
*/
int medhist_init(median_hist_t *des, imau1 im0, imau1 *imres){
  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s(medhist %d)", im0.nom, des->tf);
  alloc_imau1(imres);


  return(0);  
}



/* **************************************************************************/
/*	            premiere fenetre 2D - init. histogramme                    */
/* **************************************************************************/
void prem_fenetre_2d(unsigned char *ima, 
		     int i,
		     int *hist,
		     int th,
		     int *m,
		     int *n,
		     int tf,
		     int largeur)

{
register int   k, l ,tz;

tz=(tf*tf)/2;
for(k=0 ; k<=255 ; ++k) hist[k]=0;       /*initialisation du tableau hist[]*/

for(k=i-th; k<=i+th ; ++k)
	for(l=0 ; l<tf    ; ++l)
			  hist[ima[k*largeur+l]]++;
			  
/*----calculer M ---*/

k=0;
*n=hist[k];
while (*n<=tz) 
	{
	k++;
	(*n)+=hist[k];
	}
*m=k;

/*--calculer N --*/
(*n)-=hist[*m];
}

/* **************************************************************************/
/*	                             traitement ligne                          */
/* **************************************************************************/
void traitligne_2d(unsigned char *ima,
		   int i,
		   int j,
		   int *col_gauche,
		   int *col_droite,
		   int tf,
		   int largeur)

{
register  int  k, d=i-(tf/2);
int	th=tf/2;

for(k=i-th ; k<=i+th ; ++k)
	{
	
	/*mettre colonne gauche de l'ancienne fenetre dans col_gauche[]*/
	
	*(col_gauche + k - d)= ima[k*largeur+j-th];

	/*mettre colonne droite de la nouvelle fenetre dans col_droite[]*/
	
	*(col_droite + k - d)= ima[k*largeur+j+th+1];
	}
 }
 



/* **************************************************************************/
/*	                            Calcul de M et N                       */
/* **************************************************************************/
void calculM_2d(int *n, 
		int *m,
		int th,
		int hist[],
		int *col_gauche,
		int *col_droite,
		int taillefenetre)

{
register int  k,g,tz;

tz=(taillefenetre*taillefenetre)/2;
for(k=0 ; k<taillefenetre ;  ++k )
   { 
     g=*(col_gauche+k);
     hist[g]--;           /* enlever la col. gauche de l'ancienne fenetre */
     if (g<*m)            /* et mettre a jour hist[] et N */
        (*n)--;
        
     g=*(col_droite+k);   
     hist[g]++;           /*ajouter la col. droite de la nouvelle fenetre*/
     if (g<*m)            /*et mettre a jour hist[] et N*/
         (*n)++;
   }
   
   /*calcul de M*/
   
if (*n>tz)
	while(*n>tz) 
	{
	(*m)--;
	(*n)-=hist[*m];
	}
else
	{
    *n +=hist[*m];
    while (*n<=tz)
        { 
        (*m)++;
        (*n)+=hist[*m];
        }
        (*n)-=hist[*m];
      }
}


/* **************************************************************************/
/*	                         MEDIAN STANDARD 2D                        */
/* **************************************************************************/
void mstd2d(unsigned char *ima_src,
	    unsigned char *ima_dest,
	    int largeur,
	    int hauteur,
	    int tf)
{
int	hist[256],*h;	/*histogramme*/
int	M,*m ;		/*valeur  du median dans la fenetre*/
int	N ,*n;		/*nbre de pixels ayant un niveau de gris < M */
int	*col_gauche;	/*colonne gauche de la fenetre */
int	*col_droite;	/*colonne droite de la fenetre */
register int    i, j, th=tf/2;

h=&hist[256];
m=&M; n=&N;
col_gauche=(int *)calloc(tf,sizeof(int));
col_droite=(int *)calloc(tf,sizeof(int));
for(i=th; i<hauteur-th; ++i)	/*pour chaque ligne i faire : */
	{
	prem_fenetre_2d(ima_src,i,hist,th,m,n,tf,largeur);
                           /*construire l'histogramme de la premiere fenetre*/
	ima_dest[i*largeur+th] = M;
	for(j=th  ; j<largeur-th-1;  ++j)	/*j:numero de colonne*/ 
		{
		traitligne_2d(ima_src,i,j,col_gauche,col_droite,tf,largeur);
		calculM_2d(n,m,th,hist,col_gauche,col_droite,tf);
		ima_dest[i*largeur+j+1]=M;      
		}
	}
cfree(col_gauche);
cfree(col_droite);
}



/* *************************  CALCUL  ***************************/
/** \ingroup medhist
    XX = u1 : calcul de l'operateur medhist.

    @param des : pointeur descripteur
    @param im0 : image initiale 
    @param imres : pointeur image resultat
    @author G. Marinescu , L. Valet, P. Lambert
    @version 1.0 (01.04.03); Include : image.h, proto2D.h
*/

int medhist_calc(median_hist_t *des, imau1 im0, imau1 *imres){
  
  mstd2d( im0.p[0], imres->p[0], im0.nc , im0.nr, des->tf);
  return(0);
}


