/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \ingroup filtrage
    \defgroup moycarre2D operateur moyenne des carres 2D glissante

Moyenne glissante des carres en ligne puis en colonne avec moyenne exacte 
des points disponibles sur les bords.\\
\\
Descripteur : \\
   int dimX, dimY   :  dimension de la fenetre de moyennage \\
   double *tab      :  nombres entiers au carre tabules en double \\
   double *tmplig   :  ligne de travail pour les carres des reels \\
   double *tmpcol   : colonne de travail pour traitement colonne \\
\\
Type entree : imau1 | u2 | fl | db\\
Type sortie : imafl (pour u1, u2, fl), db pour db 
@name lmoycarre2D
@author E. Trouve
@version 1.0 (28/02/2000); Include : image.h, proto2D.h 
@see mmoycarre2D
*/


#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <fcntl.h>
#include <limits.h> /* pour  USHRT_MAX */


/* **********************  LECTURE DE PARAMETRES   *************************/
/** \ingroup moycarre2D
    lecture des parametres de l'operateur moycarre2D. \\
    dimX : nombre de colonnes de la fenetre de moyennage (impaire) \\
    dimY : nombre de lignes de la fenetre de moyennage (impaire)   \\
    @name moycarre2D_lect
    @param   des  pointeur descripteur	  
    @param   ptp  pointeur sur parametre courant deja alloue 
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return  renvoie nouveau pointeur de parametre deja alloue 
    @author  E. Trouve
    @version 1.0 (28/02/2000); Include : image.h, proto2D.h
*/
param * moycarre2D_lect(moycarre2D_t *des, param *ptp, char *debq){
  char question[500];

  sprintf(question, "%s nombre de colonnes moyennees (impaire)", debq); 
  lec_param(question, ptp);
  des->dimX = atoi(ptp->rep);
  ptp = ptp->next; 

  sprintf(question, "%s nombre de lignes moyennees (impaire)", debq); 
  lec_param(question, ptp);
  des->dimY = atoi(ptp->rep);
  ptp = ptp->next;

  if( ((des->dimX*des->dimY)%2 == 0) || des->dimX<=0 || des->dimY<=0){
      printf("\n>> ERREUR moycarre2D_lect : dimensions incorrectes\n");
      exit(1);
  }

  return(ptp);
}


/* **********************  INITIALISATION  *****************************/
/** \ingroup moycarre2D
    XX = u1 | u2 | fl | db : initialisation de l'operateur moycarre2D.\\
    allocation image resultat, tableau ou ligne de travail, colonne de travail\\
    @name moycarre2DXX_init
    @param des : pointeur descripteur
    @param im0 : image initiale (taille connue)
    @param imres : pointeur image resultat
    @author E. Trouve
    @version 1.0 (28/02/2000); Include : image.h, proto2D.h
*/
int moycarre2Du1_init(moycarre2D_t *des, imau1 im0, imafl *imres){
  int i;

  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s(moycarre%dx%d)", im0.nom, des->dimX, des->dimY);
  alloc_imafl(imres);

  /* colonne de travail */
  if( (des->tmpcol = (double*)malloc(im0.nr*sizeof(double))) == NULL ){
	  printf("\n>> ERREUR moycarre2Du1_init : allocation impossible\n");
	  exit(1);
  }

  /* modules carres */  
  if( (des->tab = (double*)malloc(256*sizeof(double))) == NULL ){
      printf("\n>> ERREUR moycarre2Du1_init : allocation impossible\n");
      exit(1);
  }
  for(i=0; i<256; i++)
      des->tab[i] = (double)i * (double)i;
  
  return(0);  
}

/* initialisation (16 bit)*/
int moycarre2Du2_init(moycarre2D_t *des, imau2 im0, imafl *imres){
  int i;
 
  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s(moycarre%dx%d)", im0.nom, des->dimX, des->dimY);
  alloc_imafl(imres);

  /* colonne de travail */
  if( (des->tmpcol = (double*)malloc(im0.nr*sizeof(double))) == NULL ){
	  printf("\n>> ERREUR moycarre2Du2_init : allocation impossible\n");
	  exit(1);
  }

  /* modules carres */  
  if( (des->tab = (double*)malloc((USHRT_MAX+1)*sizeof(double))) == NULL ){
      printf("\n>> ERREUR estimRu2_init : allocation impossible\n");
      exit(1);
  }
  for(i=0; i<=USHRT_MAX; i++)
      des->tab[i] = (double)i * (double)i;
  
  return(0);  
}

/* initialisation (float) */
int moycarre2Dfl_init(moycarre2D_t *des, imafl im0, imafl *imres){
  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s(moycarre%dx%d)", im0.nom, des->dimX, des->dimY);
  alloc_imafl(imres);

  /* colonne de travail */
  if( (des->tmpcol = (pixdb *)malloc(im0.nr*sizeof(pixdb))) == NULL ){
	  printf("\n>> ERREUR moycarre2Dfl_init : allocation impossible\n");
	  exit(1);
  }

  /* ligne de travail */
  if( (des->tmplig = (pixdb *)malloc(im0.nc*sizeof(pixdb))) == NULL ){
	  printf("\n>> ERREUR moycarre2Dfl_init : allocation impossible\n");
	  exit(1);
  }  

  return(0);  
}

/* initialisation (double) */
int moycarre2Ddb_init(moycarre2D_t *des, imadb im0, imadb *imres){
  /* image resultat */ 
  imres->nc = im0.nc;
  imres->nr = im0.nr;
  sprintf(imres->nom, "%s(moy%dx%d)", im0.nom, des->dimX, des->dimY);
  alloc_imadb(imres);

  /* colonne de travail */
  if( (des->tmpcol = (pixdb *)malloc(im0.nr*sizeof(pixdb))) == NULL ){
	  printf("\n>> ERREUR moycarre2Ddb_init : allocation impossible\n");
	  exit(1);
  }

  /* ligne de travail */
  if( (des->tmplig = (pixdb *)malloc(im0.nc*sizeof(pixdb))) == NULL ){
	  printf("\n>> ERREUR moycarre2Ddb_init : allocation impossible\n");
	  exit(1);
  }  

  return(0);  
}


/* *************************  CALCUL  ***************************/
/** \ingroup moycarre2D
    XX = u1 | u2 | fl | db : calcul de l'operateur moycarre2D.\\
    Calcul glissant des moyennes des carres en ligne puis en colonne sur toute l'image
    avec moyenne exacte des points disponibles sur les bords. 
    @name moycarre2DXX_calc
    @param des : pointeur descripteur
    @param im0 : image initiale 
    @param imres : pointeur image resultat
    @author E. Trouve
    @version 1.0 (28/02/2000); Include : image.h, proto2D.h
*/
int moycarre2Du1_calc(moycarre2D_t *des, imau1 im0, imafl *imres){
  pixu1 *ptu1;
  int i, j, nc, nr, nbp, dlfX, dlfY;
  float *ptfl;
  double tmpdb, somdb, *ptdb;
  
  
  nc = im0.nc; 
  nr = im0.nr; 
  dlfX = des->dimX/2; /* demi largeur de fenetre en X */
  dlfY = des->dimY/2; /* demi largeur de fenetre en Y */

  /* moyenne ligne */
  for(j=0; j<nr; j++){
      nbp = 0;
      somdb = 0.;
      ptu1 = im0.p[j];
      ptfl = imres->p[j];
      for(i=0; i<dlfX; i++){
	  somdb += des->tab[ptu1[i]];
	  nbp++;
      }
      for(i=0; i<dlfX; i++){
	  somdb += des->tab[ptu1[i+dlfX]];
	  nbp++;
	  ptfl[i] = (float)somdb/nbp; 
      } 
      nbp++;
      for(  ; i<nc-dlfX; i++){
	  somdb += des->tab[ptu1[i+dlfX]];
	  ptfl[i] = (float)somdb/nbp; 	  
	  somdb -= des->tab[ptu1[i-dlfX]];
      }
      nbp--;
      for(  ; i<nc; i++){
	  ptfl[i] = (float)somdb/nbp; 	  
	  somdb -= des->tab[ptu1[i-dlfX]];
	  nbp--;  
      }
  } 

  /* moyenne colonne */   
  ptdb = des->tmpcol;  
  for(i=0; i<nc; i++){
      /* recopie, verif < 0 */
      for(j=0; j<nr; j++)
	  if( (ptdb[j] = (double)imres->p[j][i]) < 0. ){ 
	      printf("\n>> ATT moycarre2Du1_calc : som_lig(%d,%d)=%f mis a 0\n",
		     i, j, ptdb[j]);
	      ptdb[j] = 0.;
	  }
      nbp = 0;
      somdb = 0.;
      for(j=0; j<dlfY; j++){
	  somdb += ptdb[j];
	  nbp++;
      }
      for(j=0; j<dlfY; j++){
	  somdb += ptdb[j+dlfY];
	  nbp++;
	  imres->p[j][i] = (float)somdb/nbp;
      } 
      nbp++;
      for(  ; j<nr-dlfY; j++){
	  somdb += ptdb[j+dlfY];
	  if(somdb<0.){
	      printf("\n>> ATT moycarre2Du1_calc : somme(%d,%d)=%f mis a 0\n",
		     i, j, somdb);
	      somdb = 0.;
	      imres->p[j][i] = 0.;
	  }
	  else
	      imres->p[j][i] = (float)somdb/nbp;
	  somdb -= ptdb[j-dlfY];
      }
      nbp--;
      for(  ; j<nr; j++){
	  if(somdb<0.){
	      printf("\n>> ATT moycarre2Du1_calc : somme(%d,%d)=%f mis a 0\n",
		     i, j, somdb);
	      somdb = 0.;
	      imres->p[j][i] = 0.;
	  }
	  else
	      imres->p[j][i] = (float)somdb/nbp;
	  somdb -= ptdb[j-dlfY];
	  nbp--;  
      }
  }

  return(0);
}

/* calcul (16 bit) */
int moycarre2Du2_calc(moycarre2D_t *des, imau2 im0, imafl *imres){
  pixu2 *ptu2;
  int i, j, nc, nr, nbp, dlfX, dlfY;
  float *ptfl;
  double tmpdb, somdb, *ptdb;
  
  
  nc = im0.nc; 
  nr = im0.nr; 
  dlfX = des->dimX/2; /* demi largeur de fenetre en X */
  dlfY = des->dimY/2; /* demi largeur de fenetre en Y */

  /* moyenne ligne */
  for(j=0; j<nr; j++){
      nbp = 0;
      somdb = 0.;
      ptu2 = im0.p[j];
      ptfl = imres->p[j];
      for(i=0; i<dlfX; i++){
	  somdb += des->tab[ptu2[i]];
	  nbp++;
      }
      for(i=0; i<dlfX; i++){
	  somdb += des->tab[ptu2[i+dlfX]];
	  nbp++;
	  ptfl[i] = (float)somdb/nbp; 
      } 
      nbp++;
      for(  ; i<nc-dlfX; i++){
	  somdb += des->tab[ptu2[i+dlfX]];
	  ptfl[i] = (float)somdb/nbp; 	  
	  somdb -= des->tab[ptu2[i-dlfX]];
      }
      nbp--;
      for(  ; i<nc; i++){
	  ptfl[i] = (float)somdb/nbp; 	  
	  somdb -= des->tab[ptu2[i-dlfX]];
	  nbp--;  
      }
  } 

  /* moyenne colonne */   
  ptdb = des->tmpcol;  
  for(i=0; i<nc; i++){
      /* recopie, verif < 0 */
      for(j=0; j<nr; j++)
	  if( (ptdb[j] = (double)imres->p[j][i]) < 0. ){ 
	      printf("\n>> ATT moycarre2Du2_calc : som_lig(%d,%d)=%f mis a 0\n",
		     i, j, ptdb[j]);
	      ptdb[j] = 0.;
	  }
      nbp = 0;
      somdb = 0.;
      for(j=0; j<dlfY; j++){
	  somdb += ptdb[j];
	  nbp++;
      }
      for(j=0; j<dlfY; j++){
	  somdb += ptdb[j+dlfY];
	  nbp++;
	  imres->p[j][i] = (float)somdb/nbp;
      } 
      nbp++;
      for(  ; j<nr-dlfY; j++){
	  somdb += ptdb[j+dlfY];
	  if(somdb<0.){
	      printf("\n>> ATT moycarre2Du2_calc : somme(%d,%d)=%f mis a 0\n",
		     i, j, somdb);
	      somdb = 0.;
	      imres->p[j][i] = 0.;
	  }
	  else
	      imres->p[j][i] = (float)somdb/nbp;
	  somdb -= ptdb[j-dlfY];
      }
      nbp--;
      for(  ; j<nr; j++){
	  if(somdb<0.){
	      printf("\n>> ATT moycarre2Du2_calc : somme(%d,%d)=%f mis a 0\n",
		     i, j, somdb);
	      somdb = 0.;
	      imres->p[j][i] = 0.;
	  }
	  else
	      imres->p[j][i] = (float)somdb/nbp;
	  somdb -= ptdb[j-dlfY];
	  nbp--;  
      }
  }

  return(0);
}

/* calcul (float) */
int moycarre2Dfl_calc(moycarre2D_t *des, imafl im0, imafl *imres){
  pixfl *ptfl, *ptres;
  int i, j, nc, nr, nbp, dlfX, dlfY;
  pixdb  somdb, tmpdb, *ptdb;
  
  nc = im0.nc; 
  nr = im0.nr; 
  dlfX = des->dimX/2; /* demi largeur de fenetre en X */
  dlfY = des->dimY/2; /* demi largeur de fenetre en Y */
 
  /* moyenne ligne */
  ptdb = des->tmplig;
  for(j=0; j<nr; j++){
      nbp = 0;
      somdb = 0.;
      ptfl = im0.p[j];
      ptres = imres->p[j];
      for(i=0; i<nc; i++){
	  tmpdb = (double)ptfl[i];
	  ptdb[i] =  tmpdb*tmpdb;
      }
      for(i=0; i<dlfX; i++){
	  somdb += ptdb[i];
	  nbp++;
      }
      for(i=0; i<dlfX; i++){
	  somdb += ptdb[i+dlfX];
	  nbp++;
	  ptres[i] = (pixfl)somdb/nbp; 
      } 
      nbp++;
      for(  ; i<nc-dlfX; i++){
	  somdb += ptdb[i+dlfX];
	  ptres[i] = (pixfl)somdb/nbp; 
	  somdb -= ptdb[i-dlfX];
      }
      nbp--;
      for(  ; i<nc; i++){
	  ptres[i] = (pixfl)somdb/nbp; 
	  somdb -= ptdb[i-dlfX];
	  nbp--;  
      }
  }

  /* moyenne colonne */     
  ptdb = des->tmpcol;
  for(i=0; i<nc; i++){
      /* recopie */
      for(j=0; j<nr; j++)
	  ptdb[j] = (pixdb)imres->p[j][i];
      nbp = 0;
      somdb = 0.;
      for(j=0; j<dlfY; j++){
	  somdb += ptdb[j];
	  nbp++;
      }
      for(j=0; j<dlfY; j++){
	  somdb += ptdb[j+dlfY];
	  nbp++;
	  imres->p[j][i] = (pixfl)somdb/nbp;  
      } 
      nbp++;
      for(  ; j<nr-dlfY; j++){
	  somdb += ptdb[j+dlfY];
	  imres->p[j][i] = (pixfl)somdb/nbp;  
	  somdb -= ptdb[j-dlfY];
      }
      nbp--;
      for(  ; j<nr; j++){
	  imres->p[j][i] = (pixfl)somdb/nbp;  
	  somdb -= ptdb[j-dlfY];
	  nbp--;  
      }
  }
  return(0);
}

/* calcul (double) */
int moycarre2Ddb_calc(moycarre2D_t *des, imadb im0, imadb *imres){
  pixdb *ptdb1, *ptdb2, *ptres;
  int i, j, nc, nr, nbp, dlfX, dlfY;
  pixdb  somdb;
  
  nc = im0.nc; 
  nr = im0.nr; 
  dlfX = des->dimX/2; /* demi largeur de fenetre en X */
  dlfY = des->dimY/2; /* demi largeur de fenetre en Y */
 
  /* moyenne ligne */
  ptdb2 = des->tmplig;
  for(j=0; j<nr; j++){
      nbp = 0;
      somdb = 0.;
      ptdb1 = im0.p[j];
      ptres = imres->p[j];
      for(i=0; i<nc; i++)
	  ptdb2[i] = ptdb1[i] * ptdb1[i];
      for(i=0; i<dlfX; i++){
	  somdb += ptdb2[i];
	  nbp++;
      }
      for(i=0; i<dlfX; i++){
	  somdb += ptdb2[i+dlfX];
	  nbp++;
	  ptres[i] = somdb/nbp; 
      } 
      nbp++;
      for(  ; i<nc-dlfX; i++){
	  somdb += ptdb2[i+dlfX];
	  ptres[i] = somdb/nbp; 
	  somdb -= ptdb2[i-dlfX];
      }
      nbp--;
      for(  ; i<nc; i++){
	  ptres[i] = (pixdb)somdb/nbp; 
	  somdb -= ptdb2[i-dlfX];
	  nbp--;  
      }
  }

  /* moyenne colonne */     
  ptdb2 = des->tmpcol;
  for(i=0; i<nc; i++){
      /* recopie */
      for(j=0; j<nr; j++)
	  ptdb2[j] = imres->p[j][i];
      nbp = 0;
      somdb = 0.;
      for(j=0; j<dlfY; j++){
	  somdb += ptdb2[j];
	  nbp++;
      }
      for(j=0; j<dlfY; j++){
	  somdb += ptdb2[j+dlfY];
	  nbp++;
	  imres->p[j][i] = somdb/nbp;  
      } 
      nbp++;
      for(  ; j<nr-dlfY; j++){
	  somdb += ptdb2[j+dlfY];
	  imres->p[j][i] = somdb/nbp;  
	  somdb -= ptdb2[j-dlfY];
      }
      nbp--;
      for(  ; j<nr; j++){
	  imres->p[j][i] = somdb/nbp;  
	  somdb -= ptdb2[j-dlfY];
	  nbp--;  
      }
  }
  return(0);
}


