/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \ingroup detection
    \defgroup reggrow segmentation par etiquettage
Le nom de cet oprateur: reggrow est l'abrviation de Region Growing (croissance de rgions).\\
Il permet une segmentation d'une image 2D par itration succesive de croissance de regions.

Type entree : imau1
Type sortie : imau1 (respectivement)
@name lreggrow
@author G. Marinescu, L. Valet, P. Lambert
@version 1.0 (29/04/03); Include : reggrow.h
@see
*/


#include "image.h"
#include "proto2D.h"
#include <stdlib.h>
#include <math.h>
#include "reggrow.h"

#define ABSOL(x,y)  (((x) > (y))? abs(x-y) : abs(y-x))
#define MAX(x,y) (((x)>(y))? (x):(y))
#define MIN(x,y) (((x)<(y))? (x):(y))
#define ETMAX 65535L 		/* nbre maximum d'etiquettes */


/* **********************  LECTURE DE PARAMETRES   *************************/
/** lecture des parametres de l'operateur reggrow. \\
    int  nb_loop: nombre d'iterations \\
    int  seuil0: seuil de depart aggreg de 2 regions \\
    int  step: increment du seuil\\
    int  taille: taille minimum d'une region \\
    @ingroup reggrow
    @param   des  pointeur descripteur	  
    @param   ptp  pointeur sur parametre courant deja alloue 
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return  renvoie nouveau pointeur de parametre deja alloue 
    @author G. Marinescu, L. Valet
    @version 1.0 (29/04/03); Include : image.h, proto2D.h
*/
param * reggrow_lect(reggrow_t *des, param *ptp, char *debq){
  char question[500];

  sprintf(question, "%s nb d'iteration ", debq); 
  lec_param(question, ptp);
  des->nb_loop = atoi(ptp->rep);
  ptp = ptp->next; 

  sprintf(question, "%s seuil de depart aggreg de 2 regions ", debq); 
  lec_param(question, ptp);
  des->seuil0 = atoi(ptp->rep);
  ptp = ptp->next;
 
  sprintf(question, "%s increment du seuil ", debq); 
  lec_param(question, ptp);
  des->step = atoi(ptp->rep);
  ptp = ptp->next; 

  sprintf(question, "%s taille minimum d'une region ", debq); 
  lec_param(question, ptp);
  des->taille = atoi(ptp->rep);
  ptp = ptp->next;

  return(ptp);
}





etiq(unsigned char *iorg,/* adr de l'image origine (initiale ou segmentee) */
     int *ilab,/* adresse de l'image destination (etiquettes) */
     int Y,/* taille de l'image */
     int X,/* taille de l'image */
     int seuil,/* seuil d'agregation de deux regions */
     int *pnlab,
     int *pneff)
{
	int i,j;
	unsigned char *io;	/* pointeur intermediaire image origine */
	int *il;		/* pointeur intermediaire image etiquettes */

	int reg,k;

	int *ptet;		/* pointeur tableau d'etiquettes */
	
	int count;		/* compteur des etiquettes */
	int cas;				


/* ===========================================================================
*		initialisation du tableau de chainage
* ===========================================================================
*/
		ptet = (int*)calloc(ETMAX,sizeof(*ptet)); /* allocation du tableau */
		if(ptet == NULL){
		  printf("error ALLOC\n");
		  exit(1);
		}

		for(k = 0;k < ETMAX ; k++ )		/* initialisation */
		   ptet[k] = k;  

		count = 0;
		
/* =============================================================================
*		initialisation de l'image des etiquettes
* =============================================================================
*/
		printf("init image\n");
		il = ilab - 1;
		for(i = Y; i--;) 
		  for(j = X; j--;) {
		     ++il;
		     *il = 0;
		}

/* ===========================================================================
*		traitement de la premiere ligne
* ===========================================================================
*/
		il = ilab - 1;
		*++il = 0;		/* premier pixel etiquete a 0 */
		io = iorg - 1;
		++io;

/* etiquetage des autres pixels de la ligne */

		j = X - 1;
		while(j--) {
		   ++io; ++il;
		   cas = 0;
				
		   if(ABSOL(*io,*(io - 1)) < seuil)
		      cas += 2;
		   ordre(cas,il,ptet,&count,X);
		}

/* =========================================================================
*		traitement pour les autres lignes
* =========================================================================
*/
		for(i = Y - 1; i--;)
		   for(j = X; j--;) {
		      ++io; ++il;
				/* traitement de la premiere colonne */

			if(j == X) {		
			   cas = 0;
			   if(ABSOL( *io , *(io - X)) < seuil)
			      cas += 1;
			   ordre(cas,il,ptet,&count,X);
			} else {
			   cas = 0;
			   if(ABSOL(*io , *(io - X)) < seuil)
				cas += 1;
			   if(ABSOL(*io , *(io - 1)) < seuil)
				cas += 2;
			   if(ABSOL(*(io - 1) , *(io - X)) < seuil)
				cas += 4;
			   ordre(cas,il,ptet,&count,X);
			}		
			//printf("count = %d\n", count);
			if( count >= ETMAX) {
			   exit(printf("too many labels nblab=%d\n",count));
			abort();
			}
		   }
		*pnlab = count + 1;
		retiq(ilab,Y,X,ptet,pnlab,&reg,pneff);
		*pnlab = reg;
		cfree(ptet);		/* liberation du tableau */

}



    
etiq1(
      unsigned char *iorg,	/* adresse de l'image origine (initiale ou segmentee) */
      int *ilab,		/* adresse de l'image destination (etiquettes) */
      int Y, int X,		/* taille de l'image */
      int seuil,		/* seuil d'agregation de deux regions */
      int *pnlab,
      int *pneff)
{
		int i,j;
		unsigned char *io;
		int *il;

		int reg,k;

		int *ptet;	/* pointeur d'etiquettes */
		
		int count;	/* compteur des etiquettes */
		int cas;
/*
 * initialisation du tableau de chainage
 */
		ptet = (int*)calloc(ETMAX,sizeof(*ptet)); /* allocation du tableau */

		for(k = 0;k < ETMAX ; k++ )	/* initialisation */
		   ptet[k] = k;  

		count = 0;
		
/* 
*		traitement de la premiere ligne
* 
*/

		il = ilab - 1;
		*++il = 0;		/* premier pixel etiquete a 0 */
		io = iorg - 1;
		++io;

/* etiquetage des autres pixels de la ligne */

		j = X - 1;
		while(j--) {
		   ++io; ++il;
		   cas = 0;
				
		   if(ABSOL(*io,*(io - 1)) < seuil)
		   	cas += 2;
		   ordre(cas,il,ptet,&count,X);
		}

/* 
*		traitement pour les autres lignes
*
*/
	for(i = Y - 1; i--;)
	   for(j = X; j--;) {
		++io; ++il;

/* traitement de la premiere colonne */

		if(j == X) {	
		   cas = 0;
		   if(ABSOL(*io , *(io - X)) < seuil)
		      cas += 1;
		   ordre(cas,il,ptet,&count,X);
		} else {
		   cas = 0;
		   if(ABSOL(*io , *(io - X)) < seuil)
		   	cas += 1;
		   if(ABSOL(*io , *(io - 1)) < seuil)
		   	cas += 2;
		   if(ABSOL(*(io - 1) , *(io - X)) < seuil)
		   	cas += 4;
		   ordre(cas,il,ptet,&count,X);
		}	
		
		if( count >= ETMAX)
		   exit(printf("too many labels nblab=%d\n",count));
	   }
	
	*pnlab = count + 1;
	retiq(ilab,Y,X,ptet,pnlab,&reg,pneff);
	*pnlab = reg;
	cfree(ptet);		/* liberation du tableau */

}



/* **********************  INITIALISATION  *****************************/
/** Initialisation de l'operateur reggrow.\\
    allocation image resultat et colonne de travail\\
    @ingroup reggrow
    @param des : pointeur descripteur
    @param im0 : pointeur image initiale (taille connue)
    @param imres : pointeur image resultat
    @author G. Marinescu, L. Valet
    @version 1.0 (29/04/03); Include : image.h, proto2D.h
*/
int reggrow_init(reggrow_t *des, imau1 *im0, imau1 *imres){
  /* image resultat */ 
  alloc_imau1_as( im0, imres);
  return(0);  
}




moyenne(
     unsigned char *iorg,
  unsigned char *ides,
  int *ilab,
  int Y,int X,
  int nlab,
int taille,			/* taille minimum d'une region */
int *nreg)			/* nombre de  region */
{
	int i,j;
	unsigned char *io,*ds;
	int *il, val;
	int k,n;
	int *lut;
	long *npel,*sums;

	if ((lut = (int*)calloc(nlab,sizeof(*lut))) == NULL)
		exit(printf("no more free space\n"));
	if ((npel = (long *)calloc(nlab,sizeof(*npel))) == NULL)
		exit(printf("no more free space\n"));
	if ((sums = (long *)calloc(nlab,sizeof(*sums))) == NULL)
		exit(printf("no more free space\n"));

	/* initialisation des tableaux */
	for(k = nlab; k--;) {
	  lut[k]=0;
	  npel[k] = 0;
	  sums[k] = 0;
	}

	/* balayage de l'image */

	io = iorg - 1;
	il = ilab - 1;
	for(i = Y; i--;)
	   for(j = X; j--;) {
		val = *++il;
		npel[val]++;
		sums[val] = sums[val] +  (long)*++io;
	   }

	/* calcul de la lut */
	*nreg = 0;
	for (i = 0; i < nlab; i++) {
		if (npel[i] > taille) {
   		   lut[i] = sums[i] / npel[i];
   		   (*nreg)++;
		}
		else 
		   lut[i] = 0x00;
	} 

	/* passage de l'image dans la lut */
	il = ilab - 1;
	ds = ides - 1;
	for (i = Y; i--; )
	   for (j = X; j--; )
	      *++ds = lut[*++il];

	cfree(lut);
	cfree(npel);
	cfree(sums);
}


ordre( int cas, 
       int X,
       int *il,
       int ptet[ETMAX],
       int *pn)
{

	int a,b;
	int count;

	a = *(il - X);
	b = *(il - 1);
	count = *pn;

	switch (cas) {
	case 0 : {
		/* on attribue une nouvelle etiquette
		   ou bien on ne fait rien */
		*il = ++count;
		*pn = count;
		return;
		}
	case 1 : {
		/* propagation de l'etiquette du haut
		*/
		*il = ptet[*(il - X)];
		return;
		}
	case 2 : {
		/* propagation de l'etiquette de gauche
		*/
		*il = *(il - 1);
		return;
		}
	case 3 : {
		/* equivalence des trois etiquettes
		*/
		
		if (ptet[a] != b)
			ptet[MAX(ptet[a],b)] = MIN(ptet[a],b);
		*il = MIN(ptet[a],b);
		return;
		}
	case 4 : {
		/* nouvelle etiquette
		*/
		*il = ++count;
		*pn = count;
		return;
		}
	case 5 : {
		/* equivalence des trois etiquettes
		*/
		
		if (ptet[a] != b)
			ptet[MAX(ptet[a],b)] = MIN(ptet[a],b);
		*il = MIN(ptet[a],b);
		return;
		}
	case 6 : {
		/* equivalence des trois etiquettes
		*/
		
		if (ptet[a] != b)
			ptet[MAX(ptet[a],b)] = MIN(ptet[a],b);
		*il = MIN(ptet[a],b);
		return;
		}
	case 7 : {
		/* equivalence des trois etiquettes
		*/
		
		if (ptet[a] != b)
		ptet[MAX(ptet[a],b)] = MIN(ptet[a],b);
		*il = MIN(ptet[a],b);
		return;
		}
	default :
		exit(printf("ordre: case default\n"));
	}
}





/* ============================================================================
*  re-etiquetage apres mise a jour du chainage
* ============================================================================
*/



retiq(
      int *ilab,
      int Y, int X,
      int ptet[ETMAX],
      int *pnlab,
      int *preg,
      int *pneff)
{
	int *il;
	int i,j;
	int k,kmax;
	int nlab;

	nlab = *pnlab;

	for (k = 0; k < nlab; k++) {
		if (ptet[k] != k) 
		 ptet[k] = ptet[ptet[k]];
	}

	kmax = 0;
	*preg = 0;

	il = ilab - 1;
	for(i = Y; i--;)
		for(j = X; j--;) {
		++il;
		*il = ptet[*il];
		if(*il > *preg) {
			++kmax;
			*preg = *il;
		}
		}
	++kmax; ++(*preg);
	*pneff = kmax;
}



/** calcul de l'operateur reggrow.\\
    Segmentation d'une image par la methode\\
    de croissance de regions.
    @ingroup reggrow
    @param des : pointeur descripteur
    @param imaIn : pointeur image initiale 
    @param imaOut : pointeur image resultat
    @author G. Marinescu, L. Valet
    @version 1.0 (29/04/03); Include : image.h, proto2D.h
*/
int reggrow_calc(reggrow_t *des, imau1 *imaIn, imau1 *imaOut){

  int 		n_ima;	                     	/* compteur d'images */
  unsigned char *iorg=(unsigned char *)imaIn->p[0];
  unsigned char *ides=(unsigned char *)imaOut->p[0];
  int 		*ilab;	                  	/* images etiquettes */
  int 		Y=imaIn->nr,X=imaIn->nc;        /* taille des images */
  int 		nb_loop=des->nb_loop;        	/* nombre d'iteration */
  int		seuil0=des->seuil0;             /* seuil de depart aggreg de 2 regions */
  int		seuil=seuil0;	           	/* seuil aggregation de deux regions */
  int 		step=des->step;	            	/* increment du seuil */
  int 		taille=des->taille;		/* taille minimum d'une region */
  int 		nlab,neff,nreg;            	/* nombre d'etiquettes */
  register int		i,j;
/* ===========================================================================
*				premier etiquettage
* ===========================================================================
*/

  ilab = (int*)calloc( X*Y ,sizeof(int) );
     nlab = 0;
     neff = 0;
     etiq(iorg,ilab,Y,X,seuil,&nlab,&neff);
     printf("nb d'etiquettes utilisees = %d \n",nlab);
     printf("nb d'etiquettes effectives avant agregation = %d\n",neff);

     moyenne(iorg,ides,ilab,Y,X,nlab,0,&nreg);
     printf("nb d'etiquettes effectives apres agregation = %d\n",nreg);
     //if( visu == 1) { aff_sun(ides,Y,X,5); }
    
/* ===========================================================================
*				iteration de l'etiquettage
* ===========================================================================
*/

     for (i = 0; i < nb_loop; i++) {

       printf("iteration %d\n",i);

       seuil += step;
       printf("seuil: %d\n",seuil);
       
       etiq1(ides,ilab,Y,X,seuil,&nlab,&neff);
       printf("nb d'etiquettes utilisees = %d\n",nlab);
       printf("nb d'etiquettes effectives avant agregation = %d\n",neff);
       if (i == nb_loop - 1)
	 {					/* derniere iteration */
	   moyenne(iorg,ides,ilab,Y,X,nlab,taille,&nreg);
	 }
       else
	 {
	   moyenne(iorg,ides,ilab,Y,X,nlab,0,&nreg);
	 }
       printf("nb d'etiquettes effectives apres agregation = %d\n",nreg);

     }
  return(0);
}

