/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/****************************** lwrite_ima.c *****************************/
/*                                                                       */
/*************************************************************************/

/** \ingroup manip2d
    \defgroup write_ima Sauvegarde des images
operateur write_ima : ecriture d'une image en totalite.<BR>
Formats de sortie : .ras | .ima | .imw | .imf | .cxs | .cxf + .dim

Revoir le format rasterFile.



Visualisation par "xima" des formats .ima | .imw | .imf | .cxs | .cxf + .dim et par xv pour le format .ras


modif pour test


Pour utiliser "xv" sur les images 8 bits (toto.ima+.dim), passer par le script 
"xvima" sans l'extension : >xvima toto

Dans le cas des images de type u1,si une palette existe, un fichier d'extension .3lt (3 luts) est cr� dans le cas d'une sauvegarde au format ".ima, .dim". Pour la sauvegarde en rasterfile, la palette est enregistre directement dans l'image. Il est possible d'ajouter une color map particuli�e �une image de type u1 gr�e �la fonction <B> addcolormap()</B>

Descripteur :
   - FILE *pfd : pointeur fichier dimension 
   - int  dfi : numero logique fichier image 
   - char nom[200] : nom du fichier, sans extension 
   - char ext[10] : extension  

Type entree : imau1 | u2 |u4 | fl | cx4 | cx8 | rvb
@author E. Trouve
@version 1.0 (25/11/99); Include : proto2D.h
@see mbidon_XX, lread_ima, addcolormap
*/

#include "image.h"
#include "proto2D.h"
#include "rasterfile.h"
#include "permut.h"
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>


#define PERMS   0644

/* *************************  LECTURE  *******************************/
/** \ingroup write_ima
    lecture des parametres de l'operateur write_ima. \\
    nom : nom du fichier, sans suffix \\
    ext : extension (suffix) ras | ima | imw| imf | cxs | cxf  
    @param des  pointeur descripteur     
    @param ptp  pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return  renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author E. Trouve
    @version 1.0 (25/11/99); Include : image.h, proto2D.h
*/
param *write_ima_lect(write_ima_t *des, param *ptp, char *debq){
  char nom[300], question[500];
  int l;

  sprintf(question, "%s nom complet du fichier image a ecrire", debq); 
  lec_param(question, ptp);
  strcpy(nom, ptp->rep);
  ptp = ptp->next;

  l = strlen(nom);
  do
      l--;
  while(nom[l] != '.'  && l>0);
  if( l>0 ){
      strncpy(des->nom, nom, l);
      des->nom[l] = '\0';
      strcpy(des->ext, nom+l+1);
  }
  else{
      printf("\n>> ERREUR write_ima_lect : nom image incomplet (extension?)\n");
      exit(1);
  }
  return(ptp);
}


/* *************************  INITIALISATION  ***************************/
/** \ingroup write_ima
    initialisation de l'operateur write_ima.\\
    ouverture fichiers donnees + .dim + .3lt si une palette est pr�ente
    @param des  pointeur descripteur
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h
*/
int write_ima_init(write_ima_t *des){ 
  char nom_ima[200], nom_dim[200];

  sprintf(nom_ima, "%s.%s", des->nom, des->ext);
  sprintf(nom_dim, "%s.dim", des->nom);

  if(strcmp(des->ext, "ima")== 0 ||
     strcmp(des->ext, "imw")== 0 ||
     strcmp(des->ext, "iml")== 0 ||
     strcmp(des->ext, "imf")== 0 ||
     strcmp(des->ext, "cxs")== 0 ||
     strcmp(des->ext, "cxf")== 0 ){
      if( (des->pfd=fopen(nom_dim,"w")) == NULL ){
	  printf("\n>> ERREUR write_ima_init: creation fichier %s impossible\n",nom_dim);
	  exit(1);
      }
      if( (des->dfi=creat(nom_ima, PERMS)) == -1 ){
	  printf("\n>> ERREUR write_ima_init: creation fichier %s impossible\n",nom_ima);
	  exit(1);
      }  
  }
  else if(strcmp(des->ext, "ras")== 0){
      if( (des->pfd=fopen(nom_ima,"w")) == NULL ){
	  printf("\n>> ERREUR write_ima_init: creation fichier %s impossible\n",nom_dim);
	  exit(1);
      }
  }
  else{
    printf("\n Erreur : %s extension inconnue",des->ext);
    exit(1);
  }
  return(0);  
}


/* *************************  CALCUL (aucun) ***********************/


/* *************************  FERMETURE  ***************************/
/** \ingroup write_ima
    write_imaXX_ferm <BR>
    XX = u1 | u2 | u4 | fl | cx4 | cx8 : fermeture de l'operateur write_ima .<BR>
    ecriture de l'image en totalite
    @param des  pointeur descripteur
    @param im  image a ecrire
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h
*/
int write_imau1_ferm(write_ima_t *des, imau1 im){   /* .ima , .ras */
  int j;
  char nom_ras[200], nom_lut[200];
  int l,c, tmp;
  struct rasterfile entete;
  unsigned char colormap[768], nb_col_impaire, tampon=0;
  FILE *fid;
    
  /* format .ima+.dim */
  if(strcmp(des->ext, "ima")== 0){
      /*ecriture des dimensions dans .dim*/
      fprintf(des->pfd,"%d %d", im.nc, im.nr);
      fclose(des->pfd);

      /*ecriture du fichier palette si necesaire*/      
      if( im.lutr!=NULL && im.lutv!=NULL && im.lutb!=NULL) {
	  sprintf(nom_lut, "%s.%s", des->nom, "3lt"); 
	  printf("write_ima_ferm: sauvegarde palette dans %s\n", nom_lut);
	  if( (des->pfd=fopen(nom_lut,"w")) == NULL ){ 
	      printf("\n>> ERREUR write_ima_ferm: creation fichier %s impossible\n",nom_lut); 
	      exit(1); 
	  } 
	  fprintf(des->pfd,"%3d ",im.lutr[0]);
	  for(j = 1; j < 256; j++) {
	      if((j % 20) == 0 ) 
		  fprintf(des->pfd,"\n");
	      fprintf(des->pfd,"%3d ", im.lutr[j]);
	  } 
	  fprintf(des->pfd,"\n"); 

	  fprintf(des->pfd,"%3d ",im.lutv[0]);
	  for(j = 1; j < 256; j++) {
	      if((j % 20) == 0 ) 
		  fprintf(des->pfd,"\n");
	      fprintf(des->pfd,"%3d ", im.lutv[j]);
	  } 
	  fprintf(des->pfd,"\n"); 

	  fprintf(des->pfd,"%3d ",im.lutb[0]);
	  for(j = 1; j < 256; j++) {
	      if((j % 20) == 0 ) 
		  fprintf(des->pfd,"\n");
	      fprintf(des->pfd,"%3d ", im.lutb[j]); 
	  }
	  fprintf(des->pfd,"\n"); 
      }
     

      /*ecriture image*/
      lseek(des->dfi, 0L, 0);
      for(j = 0; j < im.nr; j++)				
	  if( write(des->dfi, im.p[j], im.nc) != im.nc ){ 
	      printf("\n>> ERREUR write_imau1_ferm : ecriture du fichier %s.%s\n",
		     des->nom, des->ext); 
	      exit(1); 
	  } 
      
  }
  /* format .ras */
  else  if(strcmp(des->ext, "ras")== 0){ 
    	nb_col_impaire=im.nc%2;
	entete.ras_magic=RAS_MAGIC;
	entete.ras_width=im.nc;
	entete.ras_height=im.nr;
	entete.ras_depth=8;	
	entete.ras_length=im.nr*(im.nc+nb_col_impaire);
	entete.ras_type=1;
	/*cas ecriture avec palette*/
	if(im.lutr!=NULL && im.lutv!=NULL && im.lutb!=NULL){
	  entete.ras_maptype=1;
	  entete.ras_maplength=768;
	}
	/*cas ecriture sans palette*/
	else {
	  entete.ras_maptype=0;
	  entete.ras_maplength=0;
	}
	 
	/* ecriture de l'entete*/
#ifdef INVERSE_DATA
	printf("inversion entete raster comme float\n");
	permuteFloatTab((float *)&entete, 8);
#endif
	fwrite( (char *)&entete,sizeof(struct rasterfile), 1, des->pfd);

	/*ecriture de la palette si elle existe*/	
	if( im.lutr!=NULL && im.lutv!=NULL && im.lutb!=NULL) {
	    fwrite( im.lutr, sizeof(unsigned char), 256, des->pfd); 
	    fwrite( im.lutv, sizeof(unsigned char), 256, des->pfd); 
	    fwrite( im.lutb, sizeof(unsigned char), 256, des->pfd); 
	} 

	/*ecriture des valeurs de l'image*/
	for(l=0; l<im.nr; l++)
	    {
	    for (c=0;c<im.nc;c++)
		fwrite( &im.p[l][c], sizeof(unsigned char), 1, des->pfd);
	    if (nb_col_impaire)
	       fwrite( &tampon, 1, 1, des->pfd);
	    }
	fclose(des->pfd);
  }
  return(0);
}



/* image 16 bit */
int write_imau2_ferm(write_ima_t *des, imau2 im){   /* .imw       */
  int j, nb_oct;
      
  nb_oct = 2*im.nc;
  fprintf(des->pfd,"%d %d", im.nc, im.nr);
  fclose(des->pfd);
  lseek(des->dfi, 0L, 0);

#ifdef INVERSE_DATA
	printf("Swap des octets	des donnees pour ecriture sur fichier\n");
	for( j=0; j<im.nr; j++)
		permuteShortTab( (short *)im.p[j], im.nc);
#endif

  for(j = 0; j < im.nr; j++)				
      if( write(des->dfi, (char *)im.p[j], nb_oct) != nb_oct){ 
	  printf("\n>> ERREUR write_imau2_ferm : ecriture du fichier %s.%s\n",
		 des->nom, des->ext); 
	  exit(1); 
      }
#ifdef INVERSE_DATA
  //printf("Swap des octets	des donnees pour ecriture sur fichier\n");
	for( j=0; j<im.nr; j++)
		permuteShortTab( (short *)im.p[j], im.nc);
#endif
  return(0);
}




/* image 32 bit */
 int write_imau4_ferm(write_ima_t *des, imau4 im)
{   /* .imw       */
   int j, nb_oct;

   nb_oct = 4*im.nc;
   fprintf(des->pfd,"%d %d", im.nc, im.nr);
   fclose(des->pfd);
   lseek(des->dfi, 0L, 0);
 
#ifdef INVERSE_DATA
        printf("Swap des octets des donnees pour ecriture sur fichier\n");
        for( j=0; j<im.nr; j++)
                permuteIntTab( im.p[j], im.nc);
#endif
 
   for(j = 0; j < im.nr; j++)                            
       if( write(des->dfi, (char *)im.p[j], nb_oct) != nb_oct){ 
          printf("\n>> ERREUR write_imau4_ferm : ecriture du fichier %s.%s\n",
                  des->nom, des->ext); 
           exit(1); 
       }
#ifdef INVERSE_DATA
   //printf("Swap des octets     des donnees pour ecriture sur fichier\n");
     for( j=0; j<im.nr; j++)
                permuteIntTab(im.p[j], im.nc);
#endif
   return(0);
 }






/* image float */
int write_imafl_ferm(write_ima_t *des, imafl im){   /* .imf       */
  int j, nb_oct;
      
  nb_oct = 4*im.nc;
  fprintf(des->pfd,"%d %d", im.nc, im.nr);
  fclose(des->pfd);
  lseek(des->dfi, 0L, 0);


#ifdef INVERSE_DATA
	printf("Swap des octets	des donnees pour ecriture sur fichier\n");
	for( j=0; j<im.nr; j++)
		permuteFloatTab( im.p[j], im.nc);
#endif
  
  for(j = 0; j < im.nr; j++)				
      if( write(des->dfi, (char *)im.p[j], nb_oct) != nb_oct){ 
	  printf("\n>> ERREUR write_imafl_ferm : ecriture du fichier %s.%s\n",
		 des->nom, des->ext); 
	  exit(1); 
      }


#ifdef INVERSE_DATA
	//printf("re Swap des octets des donnees float pour traitement ult�ieur\n");
	for( j=0; j<im.nr; j++)
		permuteFloatTab( im.p[j], im.nc);
#endif
  
  return(0);
}

/* image complex short */
int write_imacx4_ferm(write_ima_t *des, imacx4 im){ /* .cxs       */
  int j, nb_oct;
      
  nb_oct = 4*im.nc;
  fprintf(des->pfd,"%d %d", im.nc, im.nr);
  fclose(des->pfd);
  lseek(des->dfi, 0L, 0);

#ifdef INVERSE_DATA
	printf("Swap des octets	des donnees pour ecriture sur fichier\n");
	for( j=0; j<im.nr; j++)
		permuteShortTab( (short *)&im.p[j][0].re, 2*im.nc);
#endif
  for(j = 0; j < im.nr; j++)				
      if( write(des->dfi, (char *)im.p[j], nb_oct) != nb_oct){ 
	  printf("\n>> ERREUR write_imacx4_ferm : ecriture du fichier %s.%s\n",
		 des->nom, des->ext); 
	  exit(1); 
      }
#ifdef INVERSE_DATA
  //printf("Swap des octets	des donnees pour ecriture sur fichier\n");
	for( j=0; j<im.nr; j++)
		permuteShortTab( (short *)&im.p[j][0].re, 2*im.nc);
#endif
  return(0);
}

/* image complex float */
int write_imacx8_ferm(write_ima_t *des, imacx8 im){ /* .cxf       */
  int j, nb_oct;
      
  nb_oct = 8*im.nc;
  fprintf(des->pfd,"%d %d", im.nc, im.nr);
  fclose(des->pfd);
  lseek(des->dfi, 0L, 0);
#ifdef INVERSE_DATA
	printf("Swap des octets	des donnees pour ecriture sur fichier\n");
	for( j=0; j<im.nr; j++)
		permuteFloatTab( (float *)im.p[j], 2*im.nc);
#endif
    for(j = 0; j < im.nr; j++)				
      if( write(des->dfi, (char *)im.p[j], nb_oct) != nb_oct){ 
	  printf("\n>> ERREUR write_imacx8_ferm : ecriture du fichier %s.%s\n",
		 des->nom, des->ext); 
	  exit(1); 
      }
#ifdef INVERSE_DATA
    //printf("Swap des octets	des donnees pour ecriture sur fichier\n");
	for( j=0; j<im.nr; j++)
		permuteFloatTab( (float *)im.p[j], 2*im.nc);
#endif
  return(0);
}

/* image rvb */
int write_imarvb_ferm(write_ima_t *des, imarvb im){   /* .ras       */
    struct rasterfile entete;
    register int l, c;
    unsigned char nb_col_impaire, tampon=0;

    nb_col_impaire = im.nc%2;
    entete.ras_magic = RAS_MAGIC;
    entete.ras_width = im.nc;
    entete.ras_height = im.nr;
    entete.ras_depth = 24;
    /*entete.ras_length = im.nr*(im.nc+nb_col_impaire)*3;*/
    entete.ras_length = im.nr*(im.nc*3+nb_col_impaire);
    /*entete.ras_type = RT_FORMAT_RGB;*/
    entete.ras_type = RT_STANDARD;
    entete.ras_maptype = RMT_NONE;
    entete.ras_maplength = 0;
    /*
    printf("ETAPE ECRITURE\n");
    printf("entete.ras_width=%d\n",entete.ras_width);
    printf("entete.ras_height=%d\n",entete.ras_height);
    printf("entete.ras_depth=%d\n",entete.ras_depth);
    printf("entete.ras_length=%d\n",entete.ras_length);
    printf("entete.ras_type=%d\n",entete.ras_type);
    printf("entete.ras_maptype=%d\n",entete.ras_maptype);
    printf("entete.ras_maplength=%d\n",entete.ras_maplength);
    */

#ifdef INVERSE_DATA
    printf("inversion entete raster image rvb\n");
    permuteFloatTab((float *)&entete, 8);
#endif
		    
    /*Ecriture entete */
    fwrite(&entete,1,sizeof(struct rasterfile),des->pfd);
    /*printf("essai ecriture\n");*/

    /* Ecriture image en BVR*/
    for(l=0; l<im.nr; l++)
        {
            for(c=0; c<im.nc; c++){
                    fwrite( &im.b[l][c], 1, sizeof(pixu1), des->pfd);
                    fwrite( &im.v[l][c], 1, sizeof(pixu1), des->pfd);
                    fwrite( &im.r[l][c], 1, sizeof(pixu1), des->pfd);
            }
            if(nb_col_impaire) {
                fwrite( &tampon, 1, sizeof(pixu1), des->pfd);
                /*fwrite( &tampon, 1, sizeof(pixu1), des->pfd);
                fwrite( &tampon, 1, sizeof(pixu1), des->pfd);*/
            }
        }
    fclose( des->pfd);
}

