/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* **********************************************************************/
/*                   routines.c                                         */
/*                                                                      */
/*      Fonctions communes aux operateurs de traitement d'images        */
/*                                                                      */
/*  Liste (dans l'ordre d'ecriture) :                                   */
/*    I lecture d'entier, float, chaine... avec valeur par defaut       */
/*   II allocations d'images 2D (differents types)                      */
/*                                                                      */
/*  Derniere modif : E. TROUVE, 06/09/00                                */
/* **********************************************************************/

#include "image.h"
#include "proto2D.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>


/* ************************ param_debut *****************************/
/** \ingroup utility
    gestion des parametres en debut de main selon les 3 modes.\\
\\
1. MANUEL : \\
- lancement sans argument,\\
- saisie manuelle des parametres\\
=> param_fin va creer un fichier ctrl avec valeurs rentrees et questions\\
\\
2. FICHIER : \\
- lancement avec 1 seul argument : le nom du fichier de parametre, extension .ctrl\\
- questions avec valeurs par defaut lues dans le fichier\\
=> param_fin va sauvegarder les nouveaux parametres \\
\\
3. AUTOMATIQUE : \\
- lancement avec les parametres passes par argv\\
- pas de question et pas de fichier \\
=> mode utilise par l'interface graphique qui va gerer elle-meme les fichiers .ctrl\\
\\
sortie :\\
- Mode 1 : ptp->qst[0] = ptp->rep[0] = '\0', ptp->next = NULL\\
- Mode 2 : chaine de parametres (questions et reponses) initialisees a partir du fichier\\
- Mode 3 : chaine de parametres (reponses seules) initialisee a partir de argv, qst[0]='\0'
   @param argc nombre de chaines de la ligne de lancement 
   @param argv contenu des chaines de la ligne de lancement 
   @param ptp  pointeur (deja alloue) de structure de parametre : 
   debut de la liste chainee
   @author E. Trouve
   @version 1.0 (19/10/99); Include : proto2D.h  
*/
int param_debut(int argc, char **argv, param *ptp){
  FILE *fp;
  char nom[200], l;
  param *ptpar, tmp_par;
  

  ptp->qst[0] = '\0';
  ptp->rep[0] = '\0';
  ptp->next = NULL;
  printf("\n\n\t\t >>> programme %s <<< \n\n", argv[0]);

  /* lecture manuelle */
  if(argc == 1){
      printf("\n>> param_debut, mode MANUEL\n");
      return(0);
  }

  l = strlen(argv[1]);
  do
      l--;
  while(argv[1][l] != '.'  && l>0);

  /* lecture des valeurs par defaut dans un fichier .ctrl */
  if( (argc == 2) && (strcmp(argv[1]+l, ".ctrl") == 0) ){
      if( (fp=fopen(argv[1], "r")) == NULL ){
          printf("\n>> ERREUR param_debut : fichier %s introuvable\n", argv[1]);
          exit(1);
      }      
      l = 0;
      while( flec_param(fp, ptp) ){
	  ptp = ptp->next;
	  l++;
      }
      printf("\n>> param_debut, mode FICHIER : %d parametres lus dans %s\n", l, argv[1]);
      fclose(fp);
      return(0);
  }  
  
  /* automatique : parametres transmis par argv */
  for(l=1; l<argc; l++){
      ptp->qst[0] = '\0';
      strcpy(ptp->rep, argv[l]);
      alloc_param(ptp);
      ptp = ptp->next;
  } 
  printf("\n>> param_debut, mode AUTO : %d parametres passes en ligne\n", argc-1, argv[1]);
  return(0);
}

/* ************************ param_fin *****************************/
/**  \ingroup utility
    sauvegarde des parametres apres lecture. \\
   - mode MANUEL ou FICHIER : sauvegarde des questions et des reponses dans un 
   fichier .ctrl \\
   - mode  AUTOMATIQUE : rien a faire. 
   @param argc nombre de chaines de la ligne de lancement 
   @param argv contenu des chaines de la ligne de lancement 
   @param ptp  pointeur de structure de parametre (debut de la liste chainee)
   @author E. Trouve
   @version 1.0 (19/10/99); Include : proto2D.h
*/
int param_fin(int argc, char **argv, param *ptp){
  FILE *fp;
  char nom[200];
  int l;

  if( argc == 1 )                         /* lecture manuelle */ 
      sprintf(nom, "%s.ctrl", argv[0]);
  else if ( argc == 2 ){
      l = strlen(argv[1]);
      do
	  l--;
      while(argv[1][l] != '.'  && l>0);
      if(strcmp(argv[1]+l, ".ctrl") == 0) /* lecture issue d'un fichier */
	  strcpy(nom, argv[1]);
      else
	  return(0);  
  }
  else 
      return(0);

  lec_nom(nom, ">> param_fin : fichier de sauvegarde des nouveaux parametres");
  l = strlen(nom);
  do
      l--;
  while(nom[l] != '.'  && l>0);
  if( l == 0 )
      strcat(nom, ".ctrl");
  else if (strcmp(nom+l, ".ctrl") != 0){
      strcat(nom, ".ctrl");
      printf("\n>> ATT param_fin : extension erronee, .ctrl ajoutee => %s\n",
	     nom);
  }
  if( (fp=fopen(nom, "w")) == NULL ){
      printf("\n>> ERREUR param_fin : fichier %s inouvable\n", nom);
      exit(1);
  }
  while(ptp->qst[0] != '\0' && ptp->rep[0] != '\0'){
      fprintf(fp, "%s\t\t#%s\n", ptp->rep, ptp->qst);
      ptp = ptp->next;
  }    
  fclose(fp);
  return(0);
} 


/* ************************* lec_param  **********************************/
/**  \ingroup utility
    lecture de parametre avec valeur par defaut selon les 3 modes.\\
    - Mode 1 -> saisie manuelle \\
    - Mode 2 -> fichier + confirmation manuelle \\
    - Mode 3 -> automatique, pas de qestion \\
    sortie :\\
    ptp->qst, ptp->rep mis a jour selon le mode\\
    ptp->next est alloue si il ne l'etait pas
    @param chaine  pointeur de chaine de caractere : la question    
    @param ptp  pointeur de la structure associee au parametre a lire 
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h  
*/
void lec_param(char *chaine, param *ptp){ 
  char replue[200];
  
  /* Mode 1 : saisie manuelle */
  if( (ptp->qst[0] == '\0') && (ptp->rep[0] == '\0') ){
      strcpy(ptp->qst, chaine);
      do{
	  printf("\n%s? : ", ptp->qst);
	  fgets(ptp->rep, 199, stdin);
	  /*suppression du retour chariot placer par fgets en fin de chaine*/
	  if(ptp->rep[strlen(ptp->rep)-1] == '\n')
	    ptp->rep[strlen(ptp->rep)-1] = '\0';

      }while( ptp->rep == '\0' );
  }
  /* mode 2 : fichier + confirmation manuelle */
  else if ( (ptp->qst[0] != '\0') && (ptp->rep[0] != '\0') ){
      if( strcmp(ptp->qst, chaine) ){
	  printf("\nATT lec_param : questions differentes : \n\t- fichier .ctrl => %s\n\t- source C => %s\n", 
		 ptp->qst, chaine);
	  /* mieux vaut reprendre la question venant du source */
	  strcpy(ptp->qst, chaine); 
      }
      printf("\n%s? [%s] : ", ptp->qst, ptp->rep); 
      fgets(replue, 199, stdin);
      /*suppression du retour chariot placer par fgets en fin de chaine*/
      if(replue[strlen(replue)-1] == '\n')
	replue[strlen(replue)-1] = '\0';

      if( replue[0] != '\0')
	  strcpy(ptp->rep, replue);
  }
  /* mode 3 : automatique, pas de qestion */
  else if ( (ptp->qst[0] == '\0') && (ptp->rep[0] != '\0') ){
      printf("\n%s = %s\n", chaine, ptp->rep);
  }
  /* erreur */
  else{
      printf("\nERREUR lec_param : qst=%s, rep=%s\n", ptp->qst, ptp->rep);
  }
  if( ptp->next == NULL )
      alloc_param(ptp);
}

/* ************************* flec_param  **********************************/
/**  \ingroup utility
    lecture de parametre (question et reponse) dans un fichier ctrl.\\
    sortie :\\
    ptp->qst contient la question (max 199 characteres)\\
    ptp->rep contient la chaine de la valeur par defaut\\
    ptp->next est alloue si il ne l'etait pas
    @return 1 si lecture correcte, 0 sinon
    @param fp  pointeur de fichier positionne en debut de ligne
    @param ptp  pointeur de la structure parametre (deja allouee)  
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h  
*/
int flec_param(FILE *fp, param *ptp){ 
  char tmpc;
  int i=-1;
  
  if( fscanf(fp, "%s", ptp->rep) == 0 ) 
      return(0);

  do
      tmpc = (char)fgetc(fp);
  while( tmpc != '#' && tmpc != EOF );
  if(tmpc == EOF)
      return(0);

  do{
      i++;
      ptp->qst[i] = (pixu1)fgetc(fp);
  }while( ptp->qst[i]!='\n' && ptp->qst[i]!=EOF && i<199 );
  if(ptp->qst[i] == EOF)
      return(0);

  if( i == 199 ){
      printf("\n ATT flec_param : question trop longue (>199 caracteres)\n");
      do
	  tmpc = (pixu1)fgetc(fp);
      while( tmpc != '\n' && tmpc != EOF);      
  }
  ptp->qst[i] = '\0';

  if( ptp->next == NULL )
      alloc_param(ptp);
  return(1);
}

/* ************************* alloc_param  **********************************/
/**  \ingroup utility
    allocation du pointeur next d'une structure parametre.\\
    entree :\\
    ptp : pointeur de structure parametre deja allouee.\\
    sortie :\\
    ptp->next alloue avec ptp->next->qst[0]='\0', ptp->next->rep[0]='\0', 
    ptp->next->next = NULL;
    @param ptp  pointeur sur la structure dont on va allouer le ->next 
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h  
*/
void alloc_param(param *ptp){
  /* allocation parametre suivant */
  if( (ptp->next = (param*)malloc(sizeof(param))) == NULL ){
      printf ("\nERREUR alloc_param : allocation  impossible\n");  
      exit(1);      
  }
  ptp->next->qst[0] = '\0';
  ptp->next->rep[0] = '\0';
  ptp->next->next = NULL;
}

/* ************************* lec_XXXX ************************************/
/*   ancienne lecture d'entier, float, chaine... avec valeur par defaut  */
/* ***********************************************************************/
/**  \ingroup utility
    lecture d'entier avec valeur par defaut.

    @param pt_i  pointeur entier a lire
    @param chaine  pointeur question a poser
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h  
*/
void lec_int(int *pt_i, char *chaine){
  char rep[200];
 
  printf("\n%s? [%d] : ", chaine, *pt_i);
  fgets(rep, 199, stdin);
  /*suppression du retour chariot placer par fgets en fin de chaine*/
  if(rep[strlen(rep)-1] == '\n')
    rep[strlen(rep)-1] = '\0';

  if (rep[0] != '\0')
        *pt_i = atoi(rep);
}

/**  \ingroup utility
    lecture de reel, type float,  avec valeur par defaut.

    @param pt_f  pointeur reel a lire
    @param chaine  pointeur question a poser
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h  
*/
void lec_float(float *pt_f, char *chaine){
  char rep[200];
 
  printf("\n%s? [%.3f] : ", chaine, *pt_f);
  fgets(rep, 199, stdin);
  /*suppression du retour chariot placer par fgets en fin de chaine*/
  if(rep[strlen(rep)-1] == '\n')
    rep[strlen(rep)-1] = '\0';
  if (rep[0] != '\0')
        *pt_f = (float)atof(rep);
}
 
/**  \ingroup utility
    lecture de reel, type double,  avec valeur par defaut.

    @param pt_d  pointeur reel a lire
    @param chaine  pointeur question a poser
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h  
*/
void lec_double(double *pt_d, char *chaine){
  char rep[200];
 
  printf("\n%s? [%.3f] : ", chaine, *pt_d);
  fgets(rep, 199, stdin);
  /*suppression du retour chariot placer par fgets en fin de chaine*/
  if(rep[strlen(rep)-1] == '\n')
    rep[strlen(rep)-1] = '\0';
  if (rep[0] != '\0')
        *pt_d = atof(rep);
}
 
/**  \ingroup utility
    lecture de chaine de characteres avec valeur par defaut.

    @param pt_nom  pointeur chaine de characteres  a lire
    @param chaine  pointeur question a poser
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h  
*/
void lec_nom(char *pt_nom, char *chaine){
  char rep[200];
 
  printf("\n%s? [%s] : ", chaine, pt_nom);
  fgets(rep, 199, stdin);
  /*suppression du retour chariot placer par fgets en fin de chaine*/
  if(rep[strlen(rep)-1] == '\n')
    rep[strlen(rep)-1] = '\0';
  if (rep[0] != '\0')
    strcpy(pt_nom, rep);
}

 
/**  \ingroup utility
    generation du nom suivant (le radical) pour les series d'images.\\
Convention sur la numerotation des images :\\
1) les numeros se suivent!\\
2) ils sont donnes a la fin du nom de fichier, avant le .imw/.imf...\\
3) si les numero se reduisent a 1 chiffre, c'est le chiffre seul (ex: 0, 1, 2...)\\
4) si des numeros a 2 chiffres (>9) sont utilises, tous les numeros sont ecrits avec 2 digits
(ex: 01, 02,...,10, 11...).\\
5) pour le moment, seuls les 2 derniers chiffres sont consideres. 
Ce principe pourra s'etendre a 3 chiffres le jour ou on traitera des series de plus de 100 images...\\
\\
NB : si nom0 ne se termine pas par un nombre a 1 ou 2 chiffres, 
    la fonction retourne 0 et nomres = nom0_01\\

    @return 1 ou 2 si le numero de nom0 est sur 1 ou 2 chiffres, 0 si aucun chiffre
    @param nom0  pointeur chaine de characteres du nom image (N), sans extension
    @param nomres  pointeur chaine de characteres du nom suivant (N+1)
    @author E. Trouve
    @version 1.0 (06/09/00); Include : proto2D.h  
*/
int nom_image_suivante( char *nom0, char *nomres){

int l, n;
char *ptnum;

  strcpy(nomres, nom0);
  l = strlen(nom0);
  if( isdigit( nom0[l-2] ) ) {     /* numero sur 2 chiffres */
      ptnum = nom0 + l - 2;
      n = atoi(ptnum) + 1;
      ptnum = nomres + l - 2;
      sprintf(ptnum, "%02d", n);
      return(2);
  }
  else if( isdigit( nom0[l-1] ) ) { /* numero sur 1 chiffre */
      ptnum = nom0 + l - 1;
      n = atoi(ptnum) + 1;
      ptnum = nomres + l - 1;
      sprintf(ptnum, "%d", n);
      if(n>9){
	  printf("\n>>nom_image_suivante : ATT, la numerotation passe de 1 a 2 chiffres:\n");
	  printf("\t nom initial : %s\n", nom0);
	  printf("\t nom suivant : %s\n", nomres);
      }
      return(1);
  }
  else{                             /* pas de chiffre a la fin */
     strcat(nomres, "_01");
     printf("\n>>nom_image_suivante : ATT, chaine %s incorrecte; nouveau nom : %s\n",
	    nom0, nomres);
     return(0);
  }

}






/* ***********************************************************************/
/*                            alloc_imaXX                                */
/*     allocations d'images 2D (differents types)                        */
/* ***********************************************************************/

/**  \ingroup utility
    TYPES XX = u1 | rvb | u2 | s2 | u4 | s4 | fl | db | cx4 | cx8 | cx16 : 
    allocation d'une image.

  
   \f[
   V(t,x)=\inf_{(\alpha_s)_s}
   I\!\! E_{t,x}
   \left\{\Psi\left(X_\tau\right)
   \right\}
   \f]

    allocation en une fois permettant l'adressage 1D : im->p[0] + j*im->nc + i
    ou l'adressage 2D : im->p[j][i]
    @param im  pointeur image a allouer, nombre de colonnes (im->nc) et 
    nombre de lignes (im->nr) connus
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h
*/
int alloc_imau1(imau1 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixu1 **)malloc(im->nr*sizeof(pixu1 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixu1 *)malloc(im->nc*im->nr*sizeof(pixu1))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* pointeur de LUT */
  im->lutr = NULL;
  im->lutv = NULL;
  im->lutb = NULL;

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imarvb (3 plans bit) */
int alloc_imarvb(imarvb *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->r = (pixu1 **)malloc(im->nr*sizeof(pixu1 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
  if( (im->v = (pixu1 **)malloc(im->nr*sizeof(pixu1 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
  if( (im->b = (pixu1 **)malloc(im->nr*sizeof(pixu1 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->r[0] = (pixu1 *)malloc(im->nc*im->nr*sizeof(pixu1))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
  if( (im->v[0] = (pixu1 *)malloc(im->nc*im->nr*sizeof(pixu1))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
  if( (im->b[0] = (pixu1 *)malloc(im->nc*im->nr*sizeof(pixu1))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++){
        im->r[j] =  im->r[0] + j*(im->nc);
        im->v[j] =  im->v[0] + j*(im->nc);
        im->b[j] =  im->b[0] + j*(im->nc);
  }

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imau2 (16 bit non signe) */
int alloc_imau2(imau2 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixu2 **)malloc(im->nr*sizeof(pixu2 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixu2 *)malloc(im->nc*im->nr*sizeof(pixu2))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imas2 (16 bit signe) */
int alloc_imas2(imas2 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixs2 **)malloc(im->nr*sizeof(pixs2 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixs2 *)malloc(im->nc*im->nr*sizeof(pixs2))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}


/* allocation d'une image type imau4 (entier 32 bit non-signe) */
int alloc_imau4(imau4 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixu4 **)malloc(im->nr*sizeof(pixu4 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixu4 *)malloc(im->nc*im->nr*sizeof(pixu4))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}


/* allocation d'une image type imas4 (entier 32 bit signe) */
int alloc_imas4(imas4 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixs4 **)malloc(im->nr*sizeof(pixs4 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixs4 *)malloc(im->nc*im->nr*sizeof(pixs4))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imafl (reel 4 octets) */
int alloc_imafl(imafl *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixfl **)malloc(im->nr*sizeof(pixfl *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixfl *)malloc(im->nc*im->nr*sizeof(pixfl))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imadb (reel 8 octets) */
int alloc_imadb(imadb *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixdb **)malloc(im->nr*sizeof(pixdb *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixdb *)malloc(im->nc*im->nr*sizeof(pixdb))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imacx4 (complexe short, 4 octet) */
int alloc_imacx4(imacx4 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixcx4 **)malloc(im->nr*sizeof(pixcx4 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixcx4 *)malloc(im->nc*im->nr*sizeof(pixcx4))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imacx8 (complexe float, 8 octet) */
int alloc_imacx8(imacx8 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixcx8 **)malloc(im->nr*sizeof(pixcx8 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixcx8 *)malloc(im->nc*im->nr*sizeof(pixcx8))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}

/* allocation d'une image type imacx16 (complexe double, 16 octet) */
int alloc_imacx16(imacx16 *im){
  int j;
 
  /* tableau d'adresses des debuts de ligne                             */
  if( (im->p = (pixcx16 **)malloc(im->nr*sizeof(pixcx16 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (im->p[0] = (pixcx16 *)malloc(im->nc*im->nr*sizeof(pixcx16))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < im->nr; j++)
        im->p[j] =  im->p[0] + j*(im->nc);

  /* chaine du nom */
  strcpy(im->nom, "");
}

/**  \ingroup utility
    allocation �olu� d'une image �partir d'une image de r��ence.\\
    Le premier param�re est une image d��allou�. Cette fonction alloue l'image de sortie avec des dimensions identiques que l'image de r��ence. De m�e les adresse des palettes couleurs sont copi� dans l'image de sortie. Enfin, l'allocation est r�lis� en une fois permettant l'adressage 1D : im->p[0] + j*im->nc + i ou l'adressage 2D : im->p[j][i], et les valeurs de l'image sont initialis��ZERO.
    @param modele  pointeur image de r��ence
    @param out  pointeur image �allouer
    @author L. VALET
    @version 1.0 (21/12/99); Include : proto2D.h
*/
int alloc_imau1_as(imau1 *modele, imau1 *out){
  int j;
 
  out->nr = modele->nr;
  out->nc = modele->nc;

  out->lutr = modele->lutr;
  out->lutv = modele->lutv;
  out->lutb = modele->lutb;  

  /* tableau d'adresses des debuts de ligne                             */
  if( (out->p = (pixu1 **)calloc(out->nr, sizeof(pixu1 *))) == NULL )
        {printf ("\n allocation image impossible\n");  exit(1);}
 
  /* tableau 1D donnant l'image en 1 seul bloc, ligne apres ligne       */
  if( (out->p[0] = (pixu1 *)calloc(out->nc, out->nr*sizeof(pixu1))) == NULL)
        {printf ("\n allocation image impossible\n");  exit(1); }
 
  /* positionnement des pointeurs de debut de ligne                     */
  for(j = 1; j < out->nr; j++)
        out->p[j] =  out->p[0] + j*(out->nc);

  /* chaine du nom */
  strcpy(out->nom, modele->nom);
}



/* ********************* liberation memeoire image  ***********************/
/**  \ingroup utility
    TYPES XX = u1 | rvb | u2 | s2 | u4 | s4 | fl | db | cx4 | cx8 | cx16 : 
    desallocation d'une image.
    \\
    @param im  pointeur image a desallouer, 
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h
*/
int free_imau1(imau1 *im){
  free(im->p[0]);
  free(im->p);
}
int free_imarvb(imarvb *im){
  free(im->r[0]);  free(im->v[0]); free(im->b[0]);
  free(im->r);  free(im->v);  free(im->b);
}
int free_imau2(imau2 *im){
  free(im->p[0]);
  free(im->p);
}
int free_imas2(imas2 *im){
  free(im->p[0]);
  free(im->p);
}
int free_imau4(imau4 *im){
  free(im->p[0]);
  free(im->p);
}
int free_imas4(imas4 *im){
  free(im->p[0]);
  free(im->p);
}
int free_imafl(imafl *im){
  free(im->p[0]);
  free(im->p);
}
int free_imadb(imadb *im){
  free(im->p[0]);
  free(im->p);
}
int free_imacx4(imacx4 *im){
  free(im->p[0]);
  free(im->p);
}
int free_imacx8(imacx8 *im){
  free(im->p[0]);
  free(im->p);
}
int free_imacx16(imacx16 *im){
  free(im->p[0]);
  free(im->p);
}
