/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <math.h>
#include "proto2D.h"

             /** \ingroup utility
		  Tracing a discrete line using the breshenham algorithm.
                 
		 This function is tracing and inserting a discrete straight line connecting two particular points in an image.
		 We consider the straight line segment is a finite set of grid points of the square raster grid, individuated by a straight line segment of the Euclidean plane by making use of the grid intersection digitization criterion; the grid point closest to each intersection between the grid and the Euclidean segment is an element of the digital straight line segment. Normally this point set has two unique end points, it can be ordered as 8-path from one end point to the other, and the digital straight line segment can be identified with this 8-path.\\
The computation of a digital straight line segment uses consecutive 'small' steps from pixel to pixel, starting at p1=(x1,y1) and ending at p2=(x2,y2). These elementary steps can be restricted to be parallel to he coordinate axes (4-path), or can also include diagonal steps (8-path). The latter case is mostly used for representing a digital curve, and it is also used in the straight line algorithm given below. Besides the given points p1 and p2 the algorithm generates additional

	max{|x1-x2|,|y1-y2|}-1

'new' points in the image for a digital straight line from  p1 to p2.\\

		 @param      image   image in which the line is traced		
                 @param      x1      first coordinate of the startpoint (column)
		 @param      y1      second coordinate of the startpoint (row)
		 @param      x2      first coordinate of the endpoint  (column)
		 @param      y2      second coordinate of the endpoint (row) 
		 @param      intensity intensity (greyvalue) of the line to draw
	
                 @author     C. MAIER 
		 @version 1.0 (03/04/00); Include : proto2D.h

                */


short breshenham(imau1 *image,short x1, short y1, short x2,short y2,short intensity)
    
{
  short switch1;
  short inx1, inxy;
  short  stopX;
  short diffX, diffY, helpdiff;
  short x,y,flag=0;

  /* wenn der Ausganspunkt in der Mitte liegt wird erst in eine Rtg gegangen und dann in die entgegengesetzte, deshalb gibt es neben den eigentlichen x,y,z Zaehlern noch entgegengesetze */
  diffX=x2-x1;
  diffY=y2-y1;
  if (fabs(diffY)>fabs(diffX)){
    flag=1;
    x=y1;
    y=x1;
    stopX=y2;
    helpdiff=diffX;
    diffX=diffY;
    diffY=helpdiff;       
  }
  else{
    flag=0;
    x=x1;
    y=y1;
    stopX=x2;    
  }
 switch1=2*fabs(diffY)-fabs(diffX);
 inx1=2*fabs(diffY);
 inxy=2*(fabs(diffY)-fabs(diffX));    

    
  while  (x!=stopX)  {         
    if (flag==0) {  
      image->p[y][x]=intensity; 
    }
    else{
      image->p[x][y]=intensity;       
    }
     if (diffX<0)
       x--;      
     else
       x++;      
     if (switch1>0){
       if (diffY<0)	 
	 y--;
       else
	 y++;	
       switch1+=inxy;  
     }
     else
       switch1+=inx1;       
  }      
  image->p[y2][x2]=intensity;    
  return 0;
}
