/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** XX = u1 | etc : executable(s) associe(s) a l'operateur "bidon" - programme type.\\
Entree : une image type XX = u1 | etc\\
Sortie : une image type XX = u1 | etc\\
Description : programme type illustrant la saisie de parametres 
et l'utilisation des operateurs read_ima, bidon et write_ima.\\
Effectue une simple recopie de l'image entrante dans l'image sortante.
@name mbidon_XX
@author E. Trouve 
@version 1.0 (19/10/99)
@see lread_ima, lbidon et lwrite_ima
*/
#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
#include "bidon.h"    /* complement pour les operateurs en cours de developpt */ 
#include <stdlib.h>

int main(int argc, char *argv[]){

/* DECLARATIONS */

  /* images */ 
  imau1 im0, imres;

  /* operateurs */
  read_ima_t rea;
  bidon_t bid;
  write_ima_t wri;   

  /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  int iloc;              
  float floc;
  char  nomloc[200];

  
/* LECTURE PARAMETRES */

  /* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
  param_debut(argc, argv, &par0); 
  ptp = &par0;      /* regle : ptp pointe sur la structure du parametre suivant */

  /* operateurs: ptp est passe en argument, return fournit la nouvelle position */
  ptp = read_ima_lect(&rea, ptp, ">> image initiale numero 1 :");
  ptp = bidon_lect(&bid, ptp, ">> parametre bidon :");
  ptp = write_ima_lect(&wri, ptp,  ">> image resultat numero 1 :");

  /* main : en cas de parametres lus dans le main */
  lec_param(">>main, entier iloc", ptp);
  iloc = atoi(ptp->rep);
  ptp = ptp->next;
  lec_param(">>main, float floc", ptp);
  floc = (float)atof(ptp->rep);
  ptp = ptp->next;
  lec_param(">>main, chaine de caracteres nomloc", ptp);
  strcpy(nomloc, ptp->rep);
  ptp = ptp->next;  

  /* fin: sauvegarde des parametres utilises en mode MANUEL ou FICHIER */
  param_fin(argc, argv, &par0);


/* INITIALISATION  */

  /* operateur */
  read_imau1_init(&rea, &im0);
  bidonu1_init(&bid, im0, &imres);
  write_ima_init(&wri);

  /* main : en cas de variable a allouer/initialiser dans le main  */ 

   
/* CALCUL */
   bidonu1_calc(&bid, im0, &imres); 
   printf("\n partie calcul, iloc=%d, floc=%f, nomloc=%s\n", iloc, floc, nomloc);
  /* pas de read_imau1_calc, write_imau1_calc : lecture, ecriture en une fois */


/* FREMETURE */
  write_imau1_ferm(&wri, imres);
  
}

