/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** executable associe a l'operateur "contourRSO", detection de contour dans les images radar.\\
Entree : une image type u2 \\
Sortie : une image type u1/fl selon extension : valeur de ratio \\
Sortie : une image type u1 : direction de contour \\
Description : cf. lcontourRSO.c \\
@name mcontourRSO_u2
@author F. Bujor, E. Trouve
@version 1.0 (15/06/00)
@see lread_ima, lcontourRSO et lwrite_ima
*/
#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
#include "contourRSO.h"  /* complement pour le operateur detecteur de contour */ 
#include <stdlib.h>


int main(int argc, char *argv[]){
  int i, j;
  float tmpfl;

/* DECLARATIONS */
  /* images */ 
  imau2 im0;
  imafl imvalfl;
  imau1 imvalu1;
  imau1 imdir;
  /* operateurs */
  read_ima_t rea;
  contourRSO_t cont;
  write_ima_t wri1;
  write_ima_t wri2;
  add_colormap_t col;
  param par0, *ptp;

/* LECTURE PARAMETRES */
  param_debut(argc, argv, &par0); 
  ptp = &par0;  
  /* operateurs: ptp est passe en argument, return fournit la nouvelle position */
  ptp = read_ima_lect(&rea, ptp, ">> image initiale (u2):");
  ptp = contourRSO_lect(&cont, ptp, ">> contours :");
  ptp = write_ima_lect(&wri1, ptp, ">> image resultat, ratio (u1/fl) :");
  ptp = write_ima_lect(&wri2, ptp, ">> image resultat, direction (u1) :");
  ptp = add_colormap_lect(&col, ptp, ">> LUT direction normale au contour :"); 
  /* fin: sauvegarde des parametres utilises en mode MANUEL ou FICHIER */
  param_fin(argc, argv, &par0);

/* INITIALISATION  */
  /* operateur */
  read_imau2_init(&rea, &im0);
  contourRSOu2_init(&cont, im0, &imvalfl, &imdir);
  write_ima_init(&wri1);
  write_ima_init(&wri2);
  add_colormap_init(&col);

/* CALCUL */
   contourRSOu2_calc(&cont, im0, &imvalfl, &imdir);
   if(strcmp(wri1.ext, "ima") == 0 || strcmp(wri1.ext, "ras") == 0){
       imvalu1.nr = imvalfl.nr;
       imvalu1.nc = imvalfl.nc;
       alloc_imau1(&imvalu1);
       for(i=0; i<imvalfl.nr; i++)
	   for(j=0; j<imvalfl.nc; j++){
	       tmpfl = imvalfl.p[i][j]*255.999;
	       if(tmpfl < 256)
		   imvalu1.p[i][j] = (pixu1)tmpfl;
	       else
		   imvalu1.p[i][j] =255;
	   }
   }
   add_colormap_calc(&col, &imdir);
    
/* FREMETURE */
   if(strcmp(wri1.ext, "ima") == 0 || strcmp(wri1.ext, "ras") == 0){
       write_imau1_ferm(&wri1, imvalu1);
   }
   else{
       write_imafl_ferm(&wri1, imvalfl);
   }
   write_imau1_ferm(&wri2, imdir);
  
}

