/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** conversion d'une image 16 bits (imw) en 8 bits (ima|ras).\\
Entree : une image 16 bits (unsigned short = u2), exetension .imw\\
Sortie : une image 8 bit (unsigned char = u1), extension .ima ou .ras\\
\\
Description : 3 methodes de quantification possibles\\
Methode 1 : a partir des statistiques (moy et ecart-type) : 
            [min,moy+K*sigma] => [0, 255]\\
Methode 2 : en eliminant les K premiers et K derniers % de l'histogramme : 
            [n(K%), n(100-K%)]  => [0, 255]\\
Methode 3 : interval fixe, independant de l'image : 
            [0, K]  => [0, 255]\\
\\
Lecture et ecriture par les operateurs read_ima et write_ima (image entiere).
@name mconv16to8
@author E. Trouve 
@version 1.0 (01/02/2001)
@see lread_ima et lwrite_ima
*/
#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
#include "radar.h" 
#include <stdlib.h>
#include <limits.h> /* pour  USHRT_MAX ... */

int main(int argc, char *argv[]){

/* DECLARATIONS */

  /* images */ 
  imau1 imres;
  imau2 im0;

  /* operateurs */
  read_ima_t rea;
  conv16to8_t conv;
  write_ima_t wri;   

  /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */ 
  

  
/* LECTURE PARAMETRES */

  /* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
  param_debut(argc, argv, &par0); 
  ptp = &par0;      /* regle : ptp pointe sur la structure du parametre suivant */

  /* operateurs: ptp est passe en argument, return fournit la nouvelle position */
  ptp = read_ima_lect(&rea, ptp, ">> image initiale 16 bits (.imw) :");
  ptp = conv16to8_lect(&conv, ptp, "");
  ptp = write_ima_lect(&wri, ptp, ">> image resultat 8 bits (.ima|.ras) :");

  /* param du main */

  /* fin: sauvegarde des parametres utilises en mode MANUEL ou FICHIER */
  param_fin(argc, argv, &par0);


/* INITIALISATION  */

  /* operateur */
  read_imau2_init(&rea, &im0);
  conv16to8_init(&conv, im0, &imres); 
  write_ima_init(&wri);

   
/* CALCUL */
  conv16to8_calc(&conv, im0, &imres); 


/* FREMETURE */
  write_imau1_ferm(&wri, imres);
}

