/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \ingroup programme
\defgroup mcpima
Recopie d'une image dans un autre type parmi u1, u2, u4, fl. avec gain et offset possible <br>
Entree : une image type u1 | u2 | u4 | fl <br>
Sortie : une image type u1 | u2 | u4 | fl <br>
Description : recopie de l'image entrante dans l'image sortante apres multiplication par un gain 
puis ajout d'un offset (NB. G=1, Off=0 => image identique).<br>
Les types sont donnes par les extensions des noms de fichier : 
en entrée : .ima | .ras | .imw/.IMW | .iml/.IML | .imf/.IMF (format IEEE/Intel);
en sortie : .ima | .ras | .imw | .iml | .imf (format IEEE); 
Attention, les conversions en 16 ou 8 bits se font avec simple troncature 
(valeurs <0 mises a 0, > au max mises au max). 
Lecture et ecriture par les operateurs read_ima et write_ima.

@author E. Trouve 
@version 1.0 (28/02/2000)
@see read_ima, write_ima, mcpima.ctrl
*/
#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
#include <stdlib.h>
#include <limits.h> /* pour  USHRT_MAX ... */

int main(int argc, char *argv[]){

/* DECLARATIONS */

  /* images */ 
  imau1 im0u1, imresu1;
  imau2 im0u2, imresu2;
  imau4 im0u4, imresu4;
  imafl im0fl, imresfl;

  /* operateurs */
  read_ima_t rea;
  write_ima_t wri;   

  /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  int i, j, t0, tres, nr, nc;      
  float v0, gain, offset;
  

  
/* LECTURE PARAMETRES */

  /* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
  param_debut(argc, argv, &par0); 
  ptp = &par0;      /* regle : ptp pointe sur la structure du parametre suivant */

  /* operateurs: ptp est passe en argument, return fournit la nouvelle position */
  ptp = read_ima_lect(&rea, ptp, ">> entree format .ima | .ras (8b) | .imw|IMW | .iml|IML | .imf|IMF :");
  ptp = write_ima_lect(&wri, ptp, ">> sortie format .ima | .ras (8b) | .imw | .iml | .imf :");

  /* param du main */
  lec_param(">>mcpima, cefficient multiplicatif = gain (float)", ptp);
  gain = (float)atof(ptp->rep);
  ptp = ptp->next;
  lec_param(">>mcpima, cefficient aditif = offset (float)", ptp);
  offset = (float)atof(ptp->rep);
  ptp = ptp->next;

  /* fin: sauvegarde des parametres utilises en mode MANUEL ou FICHIER */
  param_fin(argc, argv, &par0);


/* INITIALISATION  */

  /* image d'entree */
  if( !strcmp(rea.ext, "ima") || !strcmp(rea.ext, "ras") ){
      read_imau1_init(&rea, &im0u1);  
      nc = im0u1.nc;    
      nr = im0u1.nr;    
      t0 = 1;
  }
  else if( !strcmp(rea.ext, "imw") || !strcmp(rea.ext, "IMW") ){
      read_imau2_init(&rea, &im0u2);
      nc = im0u2.nc;    
      nr = im0u2.nr;    
      t0 = 2;
  }
  else if( !strcmp(rea.ext, "iml") || !strcmp(rea.ext, "IML") ){
      read_imau4_init(&rea, &im0u4);
      nc = im0u4.nc;    
      nr = im0u4.nr;    
      t0 = 3;
  }
  else if (!strcmp(rea.ext, "imf") || !strcmp(rea.ext, "IMF") ){
      read_imafl_init(&rea, &im0fl);
      nc = im0fl.nc;    
      nr = im0fl.nr;    
      t0 = 4;
  }
  else{
      printf("\nextension %s inconnue\n", rea.ext);
      exit(0);
  }      
  write_ima_init(&wri);

  /* image de sortie  */ 
  if( !strcmp(wri.ext, "ima") || !strcmp(wri.ext, "ras")){
      imresu1.nc = nc;
      imresu1.nr = nr;
      alloc_imau1(&imresu1);
      tres = 1;
  }
  else if(!strcmp(wri.ext, "imw")){
      imresu2.nc = nc;
      imresu2.nr = nr;
      alloc_imau2(&imresu2);
      tres = 2;
  }
  else if(!strcmp(wri.ext, "iml")){
      imresu4.nc = nc;
      imresu4.nr = nr;
      alloc_imau4(&imresu4);
      tres = 3;
  }
  else if(!strcmp(wri.ext, "imf")){
      imresfl.nc = nc;
      imresfl.nr = nr;
      alloc_imafl(&imresfl);
      tres = 4;
  }
  else{
      printf("\nextension %s inconnue\n", wri.ext);
      exit(0);
  }      
  
   
/* CALCUL */
  /* prise en compte de la LUT en cas de 2 images 8 bits */
  if ( t0==1 && tres==1 ){
      imresu1.lutr = im0u1.lutr;
      imresu1.lutv = im0u1.lutv;
      imresu1.lutb = im0u1.lutb;
  }

  for(j=0; j<nr; j++)
      for(i=0; i<nc; i++){
	  /* valeur initiale */
	  switch(t0){
	  case 1 :	
	      v0 = (float)im0u1.p[j][i];
	      break;
	  case 2 :
	      v0 = (float)im0u2.p[j][i];
	      break;
	  case 3 :
	      v0 = (float)im0u4.p[j][i];
	      break;
	  case 4 :
	      v0 = im0fl.p[j][i];
	      break;
	  }
	  /* valeur resultat */
	  v0 = v0*gain + offset;
	  switch(tres){
	  case 1 :
	      v0 = v0 +0.5;
	      if( v0 <= 0.)
		  imresu1.p[j][i] = 0;
	      else if( v0 >= 256. )
		  imresu1.p[j][i] = 255;
	      else
		  imresu1.p[j][i] = (pixu1)v0;
	      break;
	  case 2 :
	      v0 = v0 +0.5;
	      if( v0 <= 0.)
		  imresu2.p[j][i] = 0;
	      else if( v0 >= USHRT_MAX+1 )
		  imresu2.p[j][i] = USHRT_MAX;
	      else
		  imresu2.p[j][i] = (pixu2)v0;
	      break;
	  case 3 :
	      v0 = v0 +0.5;
	      if( v0 <= 0.)
		  imresu4.p[j][i] = 0;
	      else if( v0 >= (float)UINT_MAX + 1.)
		  imresu4.p[j][i] = UINT_MAX;
	      else
		  imresu4.p[j][i] = (pixu4)v0;
	      break;
	  case 4 :
	      imresfl.p[j][i] = v0;
	      break;
	  }		  
      }


/* FREMETURE */
  switch(tres){
  case 1 :
      write_imau1_ferm(&wri, imresu1);
      break;
  case 2 :
      write_imau2_ferm(&wri, imresu2);
      break;
  case 3 :
      write_imau4_ferm(&wri, imresu4);
      break;
  case 4 :
      write_imafl_ferm(&wri, imresfl);
      break;
  }
}

