/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

/** Recopie d'une image dans un fichier au format RasterFile sans palette.\\
Entree : une image type u1 | u2 | fl \\
Sortie : une image type u1\\
Description : simple recopie de l'image entrante dans l'image sortante au format RasterFile sans palette. 
La palette est tout simplement supprimer, sans correction si c'etait une palette fausse couleur.
Les types sont donnes par les extesions des noms de fichier (.ima | .ras | .imw | .imf). 
Attention, les conversions en 16 ou 8 bits se font avec simple troncature 
(valeurs <0 mises a 0, > au max mises au max). 
Lecture et ecriture par les operateurs read_ima et write_ima. 
Principale utilisation: conversion d'image RasterFile avec palette en une image RasterFile sans palette a utiliser dans le logiciel ENVI.

@name mcpima_no_pal
@author L. VALET
@version 1.0 (28/02/2000)
@see lread_ima et lwrite_ima
*/
#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
#include <stdlib.h>
#include <limits.h> /* pour  USHRT_MAX ... */

int main(int argc, char *argv[]){

/* DECLARATIONS */

  /* images */ 
  imau1 im0u1, imresu1;
  imau2 im0u2;
  imafl im0fl;

  /* operateurs */
  read_ima_t rea;
  write_ima_t wri;   

  /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  int i, j, t0, tres, nr, nc;      
  pixu2 tmpu2;
  pixfl tmpfl;

  
/* LECTURE PARAMETRES */

  /* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
  param_debut(argc, argv, &par0); 
  ptp = &par0;      /* regle : ptp pointe sur la structure du parametre suivant */

  /* operateurs: ptp est passe en argument, return fournit la nouvelle position */
  ptp = read_ima_lect(&rea, ptp, ">> entree format .ima | .ras (8b) | .imw | .imf :");
  ptp = write_ima_lect(&wri, ptp, ">> sortie format .ras (8b) :");

  /* fin: sauvegarde des parametres utilises en mode MANUEL ou FICHIER */ 
  param_fin(argc, argv, &par0);


/* INITIALISATION  */

  /* operateur */
  if( !strcmp(rea.ext, "ima") || !strcmp(rea.ext, "ras")){
      read_imau1_init(&rea, &im0u1);   
      nc = im0u1.nc;    
      nr = im0u1.nr;    
      t0 = 1;
  }
  else if(!strcmp(rea.ext, "imw")){
      read_imau2_init(&rea, &im0u2);
      nc = im0u2.nc;    
      nr = im0u2.nr;    
      t0 = 2;
  }
  else if(!strcmp(rea.ext, "imf")){
      read_imafl_init(&rea, &im0fl);
      nc = im0fl.nc;    
      nr = im0fl.nr;    
      t0 = 4;
  }
  else{
      printf("\nextension d'entree %s inconnue\n", rea.ext);
      exit(0);
  }      
  write_ima_init(&wri);

  /* main : en cas de variable a allouer/initialiser dans le main  */  
  if( strcmp(wri.ext, "ras")!=0 ) {
    printf("mauvais format de sortie!!\n" );
    exit(0);
  }
  imresu1.nc = nc;
  imresu1.nr = nr;
  alloc_imau1(&imresu1);
  tres = 1;
  
     
  
   
/* CALCUL */   
  switch(t0){  
  case 1 :
      imresu1 = im0u1; 
      /*suppression purement et simplement de la palette*/
      imresu1.lutr = NULL; 
      imresu1.lutv = NULL; 
      imresu1.lutb = NULL; 
      break;  
  case 2 : 
    for(j=0; j<nr; j++)
      for(i=0; i<nc; i++)
	if( (tmpu2 = im0u2.p[j][i]) > 255)
	  imresu1.p[j][i] = 255;
	else
	  imresu1.p[j][i] = (pixu1)tmpu2;
    
    break;
  case 4 :
    for(j=0; j<nr; j++)
      for(i=0; i<nc; i++)
	if( (tmpfl = im0fl.p[j][i]+0.5) >= 256.)
	  imresu1.p[j][i] = 255;
	else if(tmpfl < 0.)
	  imresu1.p[j][i] = 0;
	else
	  imresu1.p[j][i] = (pixu1)tmpfl;      
    
    break;
  }  
  
  /* FREMETURE */
  write_imau1_ferm(&wri, imresu1);
  
  
}
