/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************* mcreate_ima.c *******************************/
/*                                                                       */
/*          CREATION DE D'IMAGE TEST 8 (u1) / 16 (u2) / 32 (fl) bits)    */
/*                                                                       */
/*  Derniere modif : E. TROUVE, 09/07/00                                 */
/* ***********************************************************************/

/** Creation d'une image de type u1, u2, u4 ou fl avec des objets carres.\\
Entree : fichier ASCII decrivant les objets \\
Sortie : une image type u1 | u2 | u4 | fl (type donne par l'extension du nom\\
\\
Contenu du fichier ASCII :\\ 
1er ligne : un entier = le nombre d'objets\\
lignes suivantes (une par objet) : 4 entiers  
(x0 y0 x1 y1 : coin superieur gauche et inferieur droit) et un reel (le niveau de l'objet)\\
Attention l'ordre dans lequel les objets sont lus determine leur profondeur :
en cas de superposition le premier objet gagne\\
@name mcreate_ima
@author E. Trouve 
@version 1.0 (09/07/2000)
@see lread_ima et lwrite_ima
*/

#include "image.h"
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
/* #include <stdlib.h> */

int main(int argc, char *argv[]){

/* DECLARATIONS */

  /* images */ 
  imau1 im0u1;          
  imau2 im0u2;          
  imau4 im0u4;          
  imafl im0fl;
          
  /* operateurs */
  write_ima_t wri;

  /* main */
  FILE *fp;
  char nomfich[200];
  int i, j, n, nbr, tmpi, *x0, *y0, *x1, *y1, dimX, dimY, t0, objet;
  float *tab, fond;
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */


  /* LECTURE PARAMETRES */

  /* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
  param_debut(argc, argv, &par0); 
  ptp = &par0;      /* regle : ptp pointe sur la structure du parametre suivant */

  lec_param(">> Nombre de colonnes de l'image", ptp);
  dimX = atoi(ptp->rep);
  ptp = ptp->next;
 
  lec_param(">> Nombre de lignes de l'image", ptp);
  dimY = atoi(ptp->rep);
  ptp = ptp->next;
 
  ptp = write_ima_lect(&wri, ptp, 
	      ">> Format : u1 (.ima|.ras), u2 (.imw), u4 (.iml) ou fl (.imf) :");

  lec_param(">> Valeur du fond", ptp);
  fond = (float)atof(ptp->rep);
  ptp = ptp->next;

  lec_param(">> Nom du fichier decrivant les objets", ptp);
  strcpy(nomfich, ptp->rep);
  ptp = ptp->next;  

  /* fin: sauvegarde des parametres utilises en mode MANUEL ou FICHIER */
  param_fin(argc, argv, &par0);

  /* INITIALISATION  */
  /* lecture fichier des objets */
  if( fp=fopen(nomfich,"r") ){
      fscanf(fp,"%d", &nbr);
      x0 = (int *)malloc(nbr*sizeof(int));
      y0 = (int *)malloc(nbr*sizeof(int));
      x1 = (int *)malloc(nbr*sizeof(int));
      y1 = (int *)malloc(nbr*sizeof(int));
      tab = (float *)malloc(nbr*sizeof(float));          
      if (x0==NULL || y0==NULL || x1==NULL || y1==NULL || tab==NULL ){
	  printf("\n>> ERREUR : allocation impossible\n");
	  exit(1);
      } 
      printf("\n>> L'image contiendra un fond a %f et %d objets : \n", fond, nbr);
      for(n=0; n<nbr; n++){
	  fscanf(fp, "%d %d %d %d %f", x0+n, y0+n, x1+n, y1+n, tab+n);
	  printf("objet %d :\t x0=%d,\t y0=%d,\t x1=%d,\t y1=%d,\t val=%f\n", 
		 n+1, x0[n], y0[n], x1[n], y1[n], tab[n]);
      }  
      fclose(fp);
  }
  else{
      printf("\n>> ERREUR : fichier %s introuvable\n", nomfich);
      exit(1);
  }
  /* image resultat */
  write_ima_init(&wri);
  if( !strcmp(wri.ext, "ima") || !strcmp(wri.ext, "ras")){
      t0 = 1;
      im0u1.nc = dimX;
      im0u1.nr = dimY;
      alloc_imau1(&im0u1);      
      for(j=0; j<dimY; j++)
	  for(i=0; i<dimX; i++)
	      im0u1.p[j][i] = (pixu1)fond;
  }
  else if(!strcmp(wri.ext, "imw")){
      t0 = 2;
      im0u2.nc = dimX;
      im0u2.nr = dimY;
      alloc_imau2(&im0u2);      
      for(j=0; j<dimY; j++)
	  for(i=0; i<dimX; i++)
	      im0u2.p[j][i] = (pixu2)fond;
  }
  else if(!strcmp(wri.ext, "iml")){
      t0 = 3;
      im0u4.nc = dimX;
      im0u4.nr = dimY;
      alloc_imau4(&im0u4);      
      for(j=0; j<dimY; j++)
	  for(i=0; i<dimX; i++)
	      im0u4.p[j][i] = (pixu4)fond;
  }
  else if(!strcmp(wri.ext, "imf")){
      t0 = 4;
      im0fl.nc = dimX;
      im0fl.nr = dimY;
      alloc_imafl(&im0fl);      
      for(j=0; j<dimY; j++)
	  for(i=0; i<dimX; i++)
	      im0fl.p[j][i] = (pixfl)fond;
  }
  else{
      printf("\n ERREUR : extension %s inconnue (.ima|.ras|.imw|.imf attendu)\n", wri.ext);
      exit(0);
  }   

  
/* CALCUL */
  for(j=0; j<dimY; j++)
      for(i=0; i<dimX; i++){
          objet = -1;
	  for(n=0; n<nbr; n++)
	      if( j>=y0[n] && j<=y1[n] && i>=x0[n] && i<=x1[n]){
		  objet = n;
		  break;
	      }
	  if( objet >= 0 )
	      switch(t0){
	      case 1:
		  im0u1.p[j][i] = (pixu1)tab[objet];
		  break;
	      case 2:
		  im0u2.p[j][i] = (pixu2)tab[objet];
		  break;
	      case 3:
		  im0u4.p[j][i] = (pixu4)tab[objet];
		  break;
	      case 4:
		  im0fl.p[j][i] = tab[objet];
		  break;
	      default:
		  printf("\n  ERREUR : type (%d) image res inconnu (1|2|3|4) attendu)\n",t0);
		  exit(0);
	      }
      }

  /* FREMETURE */
  switch(t0){
  case 1:
      write_imau1_ferm(&wri, im0u1);
      break;
  case 2:
      write_imau2_ferm(&wri, im0u2);
      break;
  case 3:
      write_imau4_ferm(&wri, im0u4);
      break;
  case 4:
      write_imafl_ferm(&wri, im0fl);
      break;
  default:
      printf("\n  ERREUR : type (%d) image res inconnu (1|2|3|4) attendu)\n",t0);
      exit(0);
  }
}



