/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************** mfiltmdate_bru.c ****************************/
/*                                                                        */
/*       FILTRAGE MULTIDATE - BRUNIQUEL  1/N                              */
/*                                                                        */
/*  Derniere modif : E. TROUVE, 27/03/2000                                */
/* ************************************************************************/
#include <math.h>
#include <limits.h>
#include "image.h"
#include "proto2D.h"
#include "radar.h"

/** executable : filtrage multidate de Bruniquel (version simplifiee en 1/N).\\
Moyenne temporelle des intensites, 
avec compensation de la moyenne locale (resultat de moy2D) 
ou compensation de la moyenne texturee (resultat du filtre de Kuan).
Le passage en intensite est realise a la lecture, 
le resultat est sauve a nouveau en amplitude\\
\\
Convention sur la numerotation des images :\\
1) les numeros se suivent!\\
2) ils sont donnes a la fin du nom de fichier, avant le .imw/.imf...\\
3) si les numero se reduisent a 1 chiffre, c'est le chiffre seul (ex: 0, 1, 2...)\\
4) si des numeros a 2 chiffres (>9) sont utilises, tous les numeros sont ecrits 
avec 2 digits (ex: 01, 02,...,10, 11...).\\
5) si les numeros depassent 99 (on en est loin!), etendre la fonction nom_image_suivante.\\
\\
Entree : suite d'images u2 (amplitude) \\
Sortie : suite d'images u2 (amplitude) filtree avec l'ensemble  \\
@name mfiltmdate_bru
@author E. Trouve
@version 1.0 (13/09/2000)
@see lmoy2D, lmoycarre2D, lkuan
*/
int main(int argc, char *argv[]){

  /* operateurs */
  read_ima_t  rea;
  moy2D_t    dmoy;
  moycarre2D_t dcarre;
  kuan_t  dkuan;
  write_ima_t *tabwri;

  /* main */
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  char nomtmp[200];
  pixu2 *pt0;
  int n, N, i, j, k, dimX, dimY, nbp, texture;
  pixfl *ptim2, *ptf2D, *ptsom, *ptcarre;  
  pixfl tab2[65536];
  double tmpdb;  

  /* images */
  imau2 im0;     /* image intermediaire u2 pour lecture et ecriture */ 
  imafl *tabim2; /* images intensite, puis filtrees (res de moy2D ou kuan), puis resultat */
  imafl imf2D;   /* image intermediaire, res de moy2D ou kuan */
  imafl imcarre; /* image intermediaire de la moyenne des carres */
  imafl imsom;   /* somme des images compensee radiometriquement */


/* LECTURE PARAMETRES */
  param_debut(argc, argv, &par0); 
  ptp = &par0; 

  lec_param("Filtre multidates de Bruniquel simplifie (1/N) : nombre d'images", ptp);
  N = atoi(ptp->rep);
  ptp = ptp->next;

  lec_param("Filtrage avec compensation des moyennes (0) ou des textures (1)", ptp);
  texture = atoi(ptp->rep);
  ptp = ptp->next;  
 
  ptp = kuan_lect(&dkuan, ptp, "Fenetre de filtrage spatial :");

  ptp = read_ima_lect(&rea, ptp, "Premiere image initiale (u2) :");

  if( (tabwri = (write_ima_t *)malloc(N*sizeof(write_ima_t))) == NULL ){
      printf ("\n allocation tableau descripteurs impossible\n");  
      exit(1);
  }
  ptp = write_ima_lect(tabwri, ptp, "Premiere image resultat (u2) :");
  for(n=1; n<N; n++){
      tabwri[n] = tabwri[0];
      nom_image_suivante( tabwri[n-1].nom, tabwri[n].nom);
  }

  param_fin(argc, argv, &par0);

/* INITIALISATION  */
  if( (tabim2 = (imafl *)malloc(N*sizeof(imafl))) == NULL ){
      printf ("\n allocation tableau image impossible\n");  
      exit(1);
  }

  /* lecture/ecriture des images */
  for(j=0; j<65536; j++)
      tab2[j] = (pixfl)(j*j);
  for(n=0; n<N; n++){
      if(n>0){
	  nom_image_suivante(rea.nom, nomtmp);
	  strcpy(rea.nom, nomtmp);
      }
      read_imau2_init(&rea, &im0);
      pt0 = im0.p[0];
      if(n==0)
	  nbp = im0.nc*im0.nr;
      else{ 
	  if (im0.nc*im0.nr != nbp ){
	      printf ("\n dimension image %s incoherente avec les precedentes\n", rea.nom);
	      exit(1);
	  }
      }
      /* passage en intensite */
      tabim2[n].nr = im0.nr; 
      tabim2[n].nc = im0.nc;
      alloc_imafl(tabim2+n);
      ptim2 = tabim2[n].p[0];
      for(k=0; k<nbp; k++)
	      ptim2[k] = tab2[pt0[k]];
      if(n<N-1)
         free_imau2(&im0); 
      write_ima_init(tabwri+n);
  }

  /* init du filtrage pour estimer la radiometrie moy2D ou kuan */
  dmoy.dimX = dkuan.dimX;
  dmoy.dimY = dkuan.dimY;
  moy2Dfl_init(&dmoy, tabim2[0], &imf2D);
  ptf2D = imf2D.p[0];
  if(texture){
      dcarre.dimX = dkuan.dimX;
      dcarre.dimY = dkuan.dimY;
      moycarre2Dfl_init(&dcarre, tabim2[0], &imcarre);
      ptcarre = imcarre.p[0];
      /* pas besoin d'init du kuan, on utilise l'image imf2D pour le resultat */
  }

  /* init de l'image somme */
  imsom.nc = imf2D.nc;  
  imsom.nr = imf2D.nr; 
  alloc_imafl(&imsom);
  ptsom = imsom.p[0];
  for(k=0; k<nbp; k++)
      ptsom[k] = 0.;  

/* CALCUL */
  for(n=0; n<N; n++){
      ptim2 = tabim2[n].p[0];
      /* filtrage spatial */
      moy2Dfl_calc(&dmoy, tabim2[n], &imf2D);
      if(texture){
	  moycarre2Dfl_calc(&dcarre, tabim2[n], &imcarre);
	  kuanfl_calc(&dkuan, tabim2[n], imf2D, imcarre, &imf2D);
      }
      for(k=0; k<nbp; k++){
	  ptsom[k] += ptim2[k]/ptf2D[k];
	  ptim2[k] = ptf2D[k];
      }
  }  
  for(k=0; k<nbp; k++)
      ptsom[k] = ptsom[k]/N;

  for(n=0; n<N; n++){ 
      ptim2 = tabim2[n].p[0];
      for(k=0; k<nbp; k++){  
	  /* resultat remis en amplitude */
	  tmpdb = sqrt((double)(ptim2[k]*ptsom[k])) + 0.5;
	  if( tmpdb < 65536. )
	      pt0[k] = (pixu2)tmpdb;
	  else
	      pt0[k] = 65535;	  
      }
      /* FREMETURE */
      write_imau2_ferm(tabwri+n, im0);
  }
}



