/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************** mfiltmdate_moyt.c ***************************/
/*                                                                        */
/*       FILTRAGE MULTIDATE - MOYENNE TEMPORELLE                          */
/*                                                                        */
/*  Derniere modif : E. TROUVE, 10/09/2000                                */
/* ************************************************************************/
#include <math.h>
#include <limits.h>
#include "image.h"
#include "proto2D.h"
#include "radar.h"

/** executable :  moyenne temporelle d'une serie d'images RSO multidates.\\
Moyenne temporelle des intensites pour augmenter le nombre de vues 
(si il n'y a pas de changement temporel).\\
\\
Convention sur la numerotation des images :\\
1) les numeros se suivent!\\
2) ils sont donnes a la fin du nom de fichier, avant le .imw/.imf...\\
3) si les numero se reduisent a 1 chiffre, c'est le chiffre seul (ex: 0, 1, 2...)\\
4) si des numeros a 2 chiffres (>9) sont utilises, tous les numeros sont ecrits 
avec 2 digits (ex: 01, 02,...,10, 11...).\\
5) si les numeros depassent 99 (on en est loin!), etendre la fonction nom_image_suivante.\\
\\
Entree : suite d'images u2 (amplitude) \\
Sortie : 1 image u2 (amplitude) : racine de la moyenne des intensites\\
@name mfiltmdate_moyt
@author E. Trouve
@version 1.0 (10/07/2000)
@see lread, lwrite
*/
int main(int argc, char *argv[]){

  /* operateurs */
  read_ima_t  rea;
  write_ima_t wri;

  /* main */
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  char nomtmp[200];
  int n, N, n0, i, nbp;
  pixu2 *ptim0;
  pixfl *ptsom;  
  float carre[65536];

  /* images */
  imau2 im0;  /* images initiales, puis resultat */ 
  imafl som;   /* somme des images elevees au carre */


/* LECTURE PARAMETRES */
  param_debut(argc, argv, &par0); 
  ptp = &par0; 

  lec_param("Moyenne temporelle d'une serie d'images : nombre d'images", ptp);
  N = atoi(ptp->rep);
  ptp = ptp->next;

  ptp = read_ima_lect(&rea, ptp, "Premiere image initiale (u2) :");

  ptp = write_ima_lect(&wri, ptp, "Images resultat (u2) :");

  param_fin(argc, argv, &par0);


/* INITIALISATION - CALCUL */
  for(i=0; i<65536; i++)
      carre[i] = (float)i*(float)i;

  write_ima_init(&wri);

  /* lecture, moyennage  des images */
  for(n=0; n<N; n++){
      if(n>0){
	  nom_image_suivante(rea.nom, nomtmp);
	  strcpy(rea.nom, nomtmp);
      }
      read_imau2_init(&rea, &im0);

      ptim0 = im0.p[0];
      if(n==0){
	  som.nr = im0.nr;
	  som.nc = im0.nc;
	  alloc_imafl(&som);
	  ptsom = som.p[0];
	  nbp = im0.nr*im0.nc;
	  for( i=0; i<nbp; i++)
	      ptsom[i] = carre[ptim0[i]];
      }
      else{
	  for( i=0; i<nbp; i++)
	      ptsom[i] += carre[ptim0[i]];
      }

      if( n < N-1){
	  free_imau2(&im0);
      }
  }
  for( i=0; i<nbp; i++)
      ptim0[i] = (pixu2)sqrt((double)(ptsom[i]/N));
      
/* FREMETURE */
  write_imau2_ferm(&wri, im0);
}



