/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** executable, calcul d'histogramme, image 8 bit (u1), 16 bit (u2) ou float (fl).\\
Entree : une image type u1 | u2 | fl [+ masque]\\
Sortie : un fichier ascii avec donnee pour representer la histograme\\
Description : programme pour calculer l'histograme d'une image de type 8 bit (u1), 
16 bits (u2) ou float.\\
NB : toute l'image est chargee en memoire. Pour une lecture bloc par bloc 
pour les grandes images de type u1 (resp.  u2), utiliser plustot mhistobpb_u1.c 
(resp.  mhistobpb_u2.c).\\
@name mhisto
@author F. Bujor E. Trouve
@version 1.0 (09/07/00)
@see lread_ima, lhisto 
*/

#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
#include "histo.h"    /* complement pour les operateurs en cours de developpt */ 

int main(int argc, char *argv[]){

/* DECLARATIONS */
  /* images */ 
  imau1 im0u1;
  imau2 im0u2;
  imafl im0fl;
  imau1 mas;

  /* operateurs */
  read_ima_t rea, reamas;
  histo_t hist; 

  /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  char nommas[200];
  int l, t0; 

/* LECTURE PARAMETRES */

  /* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
  param_debut(argc, argv, &par0); 
  ptp = &par0;
  ptp = read_ima_lect(&rea, ptp, ">> image initiale :");
  ptp = histo_lect(&hist, ptp, ">> histograme :");

  /* masque facultatif => lecture du nom par read_ima impossible si reponse sans extension */
  lec_param(">> nom image masque type u1 (.ima|.ras), 0 si aucun masque :", ptp);
  strcpy(nommas, ptp->rep);
  ptp = ptp->next;
  if( hist.ngm >= 0){
      l = strlen(nommas);
      do
	  l--;
      while(nommas[l] != '.'  && l>0);
      if( l>0 ){
	  strncpy(reamas.nom, nommas, l);
	  reamas.nom[l] = '\0';
	  strcpy(reamas.ext, nommas+l+1);
      }
      else{
	  printf("\n>>mhisto ERREUR : nom image masque incomplet (extension?)\n");
	  exit(1);
      }
  }
  param_fin(argc, argv, &par0);


/* INITIALISATION  */

  /* operateur */
  if( hist.ngm >= 0)
      read_imau1_init(&reamas, &mas);

  if( !strcmp(rea.ext, "ima") || !strcmp(rea.ext, "ras")){
      t0 = 1;
      read_imau1_init(&rea, &im0u1);
      histou1_init(&hist, im0u1, mas);
  }
  else if(!strcmp(rea.ext, "imw")){
      t0 = 2;
      read_imau2_init(&rea, &im0u2);
      histou2_init(&hist, im0u2, mas);
  }
  else if(!strcmp(rea.ext, "imf")){
      t0 = 4;
      read_imafl_init(&rea, &im0fl);
      histofl_init(&hist, im0fl, mas);
  }
  else{
      printf("\n >>mhisto ERREUR : extension %s inconnue\n", rea.ext);
      exit(0);
  }    

/* CALCUL */
  switch(t0){
  case 1 :	
      histou1_calc(&hist, im0u1, mas); 
      break;
  case 2 :
      histou2_calc(&hist, im0u2, mas); 
      break;
  case 4 :
      histofl_calc(&hist, im0fl, mas); 
      break;
  }  


 
/* FREMETURE */
  histo_ferm(&hist);
}

 
