/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** XX = u1 | u2 : calcul d'histogramme avec lecture bloc par bloc de l'image.\\
Entree : image type u1 | u2  [+ masque de type u1]\\
Sortie : un fichier ascii avec les donnees pour representer l'histogramme\\
Description : programme pour calculer l'histogramme d'une image complete ou sur une 
region donnee par un niveau de gris dans un masque.\\
NB. La lecture bloc par bloc n'est pas compatible avec les images float 
en raison de la recherche de min/max en mode auto. Utiliser mhisto pour une lecture d'une coup 
des images de type u1|u2|fl.\\
Doc complete avec l'operateur (cf. lhisto).\\
@name mhistobpb_XX
@author F. Bujor, E. trouve
@version 1.0 (09/07/00)
@see lselect_ima, lhisto et lread_ima, mhisto
*/

#include "image.h"    /* types des donnees manipulees dans ima2D */
#include "proto2D.h"  /* descripteurs et prototypes des fonctions de ima2D */
#include "histo.h"    /* complement pour les operateurs en cours de developpt */ 

int main(int argc, char *argv[]){

/* DECLARATIONS */
  /* images */ 
  imau1 bl0;
  imau1 mas;

  /* operateurs */
  select_ima_t sel, selmas;
  histo_t hist; 

  /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  char nommas[200];
  int l; 

/* LECTURE PARAMETRES */

  /* debut: OBLIGATOIRE pour compatibilite avec les 3 modes de lecture de param */
  param_debut(argc, argv, &par0); 
  ptp = &par0;
  ptp = select_ima_lect(&sel, ptp, ">> image initiale :");
  ptp = histo_lect(&hist, ptp, ">> histograme :");
  lec_param(">> nom image masque type u1 (.ima|.ras), 0 si aucun masque :", ptp);
  strcpy(nommas, ptp->rep);
  ptp = ptp->next;
  if( hist.ngm >= 0){
      selmas = sel;
      l = strlen(nommas);
      do
	  l--;
      while(nommas[l] != '.'  && l>0);
      if( l>0 ){
	  strncpy(selmas.nom, nommas, l);
	  selmas.nom[l] = '\0';
	  strcpy(selmas.ext, nommas+l+1);
      }
      else{
	  printf("\n>>mhistobpb_u1 ERREUR : nom image masque incomplet (extension?)\n");
	  exit(1);
      }
  }
  param_fin(argc, argv, &par0);

/* INITIALISATION  */

  /* operateur */
  select_imau1_init(&sel, &bl0);
  if( hist.ngm >= 0){
      select_imau1_init(&selmas, &mas);
  }
  histou1_init(&hist, bl0, mas);


/* CALCUL */
  while( select_imau1_calc(&sel, &bl0) == 0){
      if( hist.ngm >= 0){
	  if(select_imau1_calc(&selmas, &mas) != 0 ){
	      printf("\n>>mhistobpb_u1 ERREUR : fin du masque avant fin image\n");
	      break;
	  }	
      }
      histou1_calc(&hist, bl0, mas); 
  }

/* FREMETURE */
  histo_ferm(&hist);
}

