/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************* midctexp_u2.c ********************************/
/*                                                                        */
/*       FILTRAGE MULTIDATE - DERNIERE ETAPE DU FILTRE DE DANIELA         */
/*             DCT-1 SUR AXE TEMPOREL, EXP ET GAIN                        */
/*                                                                        */
/*  Derniere modif : E. TROUVE, 10/09/2000                                */
/* ************************************************************************/
#include <math.h>
#include <limits.h>
#include "image.h"
#include "proto2D.h"
#include "radar.h"
/* #include "radar_comp.h" */

/** executable : transfo dct inverse puis exp sur axe frequentiel d'une serie d'images 
correspondant aux coef de DCT d'une serie RSO multi-dates.\\
Lecture / ecriture bloc par bloc pour limiter le volume en memoire.\\
Convention sur la numerotation des images :\\
1) les numeros se suivent!\\
2) ils sont donnes a la fin du nom de fichier, avant le .imw/.imf...\\
3) si les numero se reduisent a 1 chiffre, c'est le chiffre seul (ex: 0, 1, 2...)\\
4) si des numeros a 2 chiffres (>9) sont utilises, tous les numeros sont ecrits 
avec 2 digits (ex: 01, 02,...,10, 11...).\\
5) si les numeros depassent 99 (on en est loin!), etendre la fonction nom_image_suivante.\\
\\
Entree : suite d'images fl (les coef de DCT dans l'ordre 0, 1...) \\
Sortie : suite d'images u2 correspondant aux images RSO initiales \\
@name midctexp_u2
@author E. Trouve
@version 1.0 (10/09/2000)
@see lselect, lsave, lidctexp
*/
int main(int argc, char *argv[]){

  /* operateurs  */
  select_ima_t *pt_sel;
  save_ima_t *pt_sav;
  idctexp_t idc;

  /* main  */
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  int n, N, n0, j, lectOK;
  
  /* images */
  imafl *pt_imdct;   /* bloc images des coef de DCT */
  imau2 *pt_imres;   /* bloc images finales */ 


/* LECTURE PARAMETRES */
  param_debut(argc, argv, &par0); 
  ptp = &par0; 

  ptp = idctexp_lect(&idc, ptp, "Retour en temporel (DCT-1) et multiplicatif (exp) : ");
  N = idc.N;

  /* allocation des  tableaux de descripteurs */
  if( (pt_sel = (select_ima_t *)malloc(N*sizeof(select_ima_t))) == NULL ){
      printf("\n>>midctexp_u2: ERREUR allocation tableau descripteurs impossible\n");
      exit(1);
  }
  if( (pt_sav = (save_ima_t *)malloc(N*sizeof(save_ima_t))) == NULL ){
      printf("\n>>midctexp_u2: ERREUR allocation tableau descripteurs impossible\n");
      exit(1);
  }

  ptp = select_ima_lect(pt_sel, ptp, "premiere image coeff 0 de DCT (fl) :");
  ptp = save_ima_lect(pt_sav, ptp, "premiere image resultat (u2) :");

  param_fin(argc, argv, &par0);

  for(n=1; n<N; n++){
      pt_sel[n] = pt_sel[0];
      nom_image_suivante( pt_sel[n-1].nom, pt_sel[n].nom);
      pt_sav[n] = pt_sav[0];
      nom_image_suivante( pt_sav[n-1].nom, pt_sav[n].nom);
  }


/* INITIALISATION  */
  /* allocation des tableaux de bloc images  */
  if( (pt_imres = (imau2 *)malloc(N*sizeof(imau2))) == NULL ){
      printf ("\n>>midctexp_u2: ERREUR allocation tableau image impossible\n");  
      exit(1);
  }  
  if( (pt_imdct = (imafl *)malloc(N*sizeof(imafl))) == NULL ){
      printf ("\n>>midctexp_u2: ERREUR allocation tableau image impossible\n");  
      exit(1);
  }  

  /* init lecture/ecriture */
  for(n=0; n<N; n++){
      select_imafl_init(pt_sel+n , pt_imdct+n);
      save_ima_init(pt_sav+n);
      pt_imres[n].p = NULL;    /* pour que l'allocation ait bien lieu */
  }
  idctexp_u2_init(&idc, pt_imdct, pt_imres);


/* CALCUL */
  lectOK = 1;
  while( (select_imafl_calc(pt_sel, pt_imdct)==0) && (lectOK==1)){
      for(n=1; n<N; n++)
	  if(select_imafl_calc(pt_sel+n, pt_imdct+n)){
	      lectOK = 0;
	      break;
  	  }
      if( lectOK == 0 ){
	      printf("\n Pb lecture image %s : fin prematuree\n", pt_sel[n].nom);
	      break;
      }
      idctexp_u2_calc(&idc, pt_imdct, pt_imres);
      for(n=0; n<N; n++)
	  save_imau2_calc(pt_sav+n, pt_imres[n]);
  }


      
/* FREMETURE */
  for(n=0; n<N; n++)
      save_ima_ferm(pt_sel[N-1], pt_sav[n]);

}



