/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** XX = u1 | u2 | fl : executable filtre de Kuan.\\
Moyenne ponderee entre la valeur de l'intensite du pixel et celle de la moyenne local en fonction 
de la variance locale et de celle du bruit multiplicatif.\\
ATTENTION : le filtrage est effectue sur les images ROS en intensite (module carre). 
Les donnees lues sont supposees etre de l'amplitude (ex : PRI-ESA, fichiers .imw). 
On commence donc par les elever au carre, on filtre et on reprend la racine carree.\\
Entree : image de type XX = u1 | u2 | fl\\
Sortie : image de meme type\\
@name mkuan_XX
@author E. Trouve
@version 1.0 (21/03/2000)
@see lkuan
*/

#include <math.h>

#include "image.h"
#include "proto2D.h"
#include "radar.h"

int main(int argc, char *argv[]){
  /* images */ 
  imau1 im0;
  imau2 im2, moyint;
  imafl moycarre;
  pixu2 tab2[256];

  /* operateurs */
  read_ima_t  rea;
  moy2D_t     moy;
  moycarre2D_t  moycar;
  kuan_t      kuan;
  write_ima_t wri;
 
 /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  int i, j;
  double tmpdb;
 
/* LECTURE PARAMETRES */
  param_debut(argc, argv, &par0); 
  ptp = &par0;
  ptp = read_ima_lect(&rea, ptp, ">> image initiale (u1) :");
  ptp = kuan_lect(&kuan, ptp, "Kuan :");
  moy.dimX = kuan.dimX; moycar.dimX = kuan.dimX;
  moy.dimY = kuan.dimY; moycar.dimY = kuan.dimY;
  ptp = write_ima_lect(&wri, ptp, ">> image resultat (u1) :");
  param_fin(argc, argv, &par0);

/* INITIALISATION  */
  read_imau1_init(&rea, &im0);
  /* passage en intensite */
  im2.nc = im0.nc;
  im2.nr = im0.nr;
  alloc_imau2(&im2);
  for(j=0; j<256; j++)
      tab2[j] = (pixu2)j*(pixu2)j;
  for(j=0; j<im0.nr; j++)
      for(i=0; i<im0.nc; i++)
	  im2.p[j][i] = tab2[im0.p[j][i]];
  moy2Du2_init(&moy, im2, &moyint);
  moycarre2Du2_init(&moycar, im2, &moycarre);
  /* pas de kuanu2_init, calcul sur place */; 
  write_ima_init(&wri);
 
/* CALCUL */
  moy2Du2_calc(&moy, im2, &moyint);
  moycarre2Du2_calc(&moycar, im2, &moycarre);
  kuanu2_calc(&kuan, im2, moyint, moycarre, &im2);
  /* retour en amplitude */
  for(j=0; j<im0.nr; j++)
      for(i=0; i<im0.nc; i++){
	  tmpdb = sqrt((double)im2.p[j][i]) +0.5;
	  if( tmpdb < 256. )
	      im0.p[j][i] = (pixu1)tmpdb;
	  else
	      im0.p[j][i] = 255;
      }
/* FREMETURE */
  write_imau1_ferm(&wri, im0);
}



