/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** executable du filtre de Lee_Add, images u1 (8 bits), u2 (16 bits) ou float.\\
Filtre statistique de Lee pour bruit additif (cf. operateur llee_add)\\
\\
Filtrage realise a partir de la valeur du pixel (v0), de la moyenne mu et de la 
variance locale sigma_im connaissant la variance du bruit additif sigma_br selon :\\ 
v_res = mu + (v0 - mu)*(sigma_im - sigma_br)/sigma_im\\
\\
Entree : image type u1 | u2 | fl\\
Sortie : le type de l'image entrante\\
@name mlee_add_fl
@author E. Trouve
@version 1.0 (11/09/2000)
@see llee_add
*/

#include "image.h"
#include "proto2D.h"
#include "radar.h"
/* #include "radar_comp.h" */

int main(int argc, char *argv[]){
  /* images */ 
  imau1 im0u1, moyampu1;
  imau2 im0u2, moyampu2;
  imafl im0fl, moyampfl, moycarre;
          
  /* operateurs */
  read_ima_t  rea;
  moy2D_t     moy;
  moycarre2D_t  moycar;
  lee_add_t      lee_add;
  write_ima_t wri;

 /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  int t0;


/* LECTURE PARAMETRES */
  param_debut(argc, argv, &par0); 
  ptp = &par0;
  ptp = read_ima_lect(&rea, ptp, "Image initiale (u1|u2|fl) :");
  ptp = write_ima_lect(&wri, ptp, "Image resultat (meme type) :");
  ptp = lee_add_lect(&lee_add, ptp, "Filtre spatial, bruit additif (Lee_Add) :");
  moy.dimX = lee_add.dimX; moy.dimY = lee_add.dimY;       /* operateur moy2D      */
  moycar.dimX = lee_add.dimX; moycar.dimY = lee_add.dimY; /* operateur moycarre2D */

  param_fin(argc, argv, &par0);


/* INITIALISATION  */
  if( !strcmp(rea.ext, "ima") || !strcmp(rea.ext, "ras")){
      t0 = 1;
      read_imau1_init(&rea, &im0u1);  
      moy2Du1_init(&moy, im0u1, &moyampu1);
      moycarre2Du1_init(&moycar, im0u1, &moycarre);
  }
  else if(!strcmp(rea.ext, "imw")){
      t0 = 2;
      read_imau2_init(&rea, &im0u2);
      moy2Du2_init(&moy, im0u2, &moyampu2);
      moycarre2Du2_init(&moycar, im0u2, &moycarre);
  }
  else if(!strcmp(rea.ext, "imf")){
      t0 = 4;
      read_imafl_init(&rea, &im0fl);
      moy2Dfl_init(&moy, im0fl, &moyampfl);
      moycarre2Dfl_init(&moycar, im0fl, &moycarre);
  }
  else{
      printf("\nextension %s inconnue\n", rea.ext);
      exit(0);
  }      
  /*   lee_addfl_init(&lee_add, im0, &imres);  inutil, resultat dans image initiale */
  write_ima_init(&wri);


/* CALCUL */
  switch(t0){
  case 1 :	
      moy2Du1_calc(&moy, im0u1, &moyampu1);
      moycarre2Du1_calc(&moycar, im0u1, &moycarre);
      lee_addu1_calc(&lee_add, im0u1, moyampu1, moycarre, &im0u1);
      break;
  case 2 :
      moy2Du2_calc(&moy, im0u2, &moyampu2);
      moycarre2Du2_calc(&moycar, im0u2, &moycarre);
      lee_addu2_calc(&lee_add, im0u2, moyampu2, moycarre, &im0u2);
      break;
  case 4 :
      moy2Dfl_calc(&moy, im0fl, &moyampfl);
      moycarre2Dfl_calc(&moycar, im0fl, &moycarre);
      lee_addfl_calc(&lee_add, im0fl, moyampfl, moycarre, &im0fl);
      break;
  }


/* FREMETURE */
  switch(t0){
  case 1 :	
      write_imau1_ferm(&wri, im0u1);
      break;
  case 2 :	
      write_imau2_ferm(&wri, im0u2);
      break;
  case 4 :	
      write_imafl_ferm(&wri, im0fl);
      break;
  }
}



