/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** executable moyenne des carres 2D glissante, image 8 bit (u1), 16 bit (u2) ou float (fl).\\
ATT: le resultat de l'operateur lmoycarre2D est la moyenne de carres (utile pour un calcul de 
la variance, reste a retirer le carre de la moyenne), mais dans cet executable on reprend 
la racine pour sauver le resultat au format initial.\\
En RSO, ca revient a chercher a augmenter le nombre de vues par un moyennage spatial des intensites.\\ 
Entree : image 8 bit (u1), 16 bit (u2) ou float (fl).\\
Sortie : image moyennee du meme type, \\
@name mmoycarre2D
@author E. Trouve
@version 1.0 (14/09/2000)
@see lmoycarre2D
*/

#include "image.h"
#include "proto2D.h"
#include "math.h"
#include <limits.h> /* pour  USHRT_MAX ... */

int main(int argc, char *argv[]){
  /* images */ 
  imau1 im0u1;          
  imau2 im0u2;          
  imafl im0fl, moycarre;    

  /* operateurs */
  read_ima_t  rea;
  moycarre2D_t moy;
  write_ima_t wri;

 /* main : variables et parametres propres au main*/
  param par0, *ptp;      /* tete et pointeur pour la chaine de parametres */
  int t0;
  register int n;
  register double tmpdb;
  pixu1 *ptu1;
  pixu2 *ptu2;
  pixfl *ptfl, *ptmoy;
 

/* LECTURE PARAMETRES */
  param_debut(argc, argv, &par0); 
  ptp = &par0;
  ptp = read_ima_lect(&rea, ptp, ">> image initiale (u1|u2|fl) :");
  ptp = write_ima_lect(&wri, ptp, ">> image resultat (meme type) :");
  ptp = moycarre2D_lect(&moy, ptp, ">> moyenne des carres :");
  param_fin(argc, argv, &par0);

/* INITIALISATION  */
 if( !strcmp(rea.ext, "ima") || !strcmp(rea.ext, "ras")){
      t0 = 1;
      read_imau1_init(&rea, &im0u1);  
      moycarre2Du1_init(&moy, im0u1, &moycarre);
  }
  else if(!strcmp(rea.ext, "imw")){
      t0 = 2;
      read_imau2_init(&rea, &im0u2);
      moycarre2Du2_init(&moy, im0u2, &moycarre);
  }
  else if(!strcmp(rea.ext, "imf")){
      t0 = 4;
      read_imafl_init(&rea, &im0fl);
      moycarre2Dfl_init(&moy, im0fl, &moycarre);
  }
  else{
      printf("\n>>moycarre2D ERREUR : extension %s inconnue\n", rea.ext);
      exit(0);
  }
  write_ima_init(&wri);
 
/* CALCUL : moyenne des carres (operateur) et racine pour revenir au format initial*/
  ptmoy = moycarre.p[0];
  switch(t0){
  case 1 :	
      moycarre2Du1_calc(&moy, im0u1, &moycarre);
      ptu1 = im0u1.p[0];
      for(n=moycarre.nr*moycarre.nc-1; n>=0; n--){
	  tmpdb = sqrt((double)ptmoy[n]) + 0.5;
	  ptu1[n] = (tmpdb < 256.) ? (pixu1)tmpdb : 255; 
      } 
      break;
  case 2 :
      moycarre2Du2_calc(&moy, im0u2, &moycarre);
      ptu2 = im0u2.p[0];
      for(n=moycarre.nr*moycarre.nc-1; n>=0; n--){
	  tmpdb = sqrt((double)ptmoy[n]) + 0.5;
	  ptu2[n] = (tmpdb < USHRT_MAX+1.) ? (pixu2)tmpdb : USHRT_MAX; 
      } 
      break;
  case 4 :
      moycarre2Dfl_calc(&moy, im0fl, &moycarre);
      ptfl = im0fl.p[0];
      for(n=moycarre.nr*moycarre.nc-1; n>=0; n--)
	  ptfl[n] = (float)sqrt((double)ptmoy[n]);
      break;
  }
  
/* FREMETURE */
  switch(t0){
  case 1 :	
      write_imau1_ferm(&wri, im0u1);
      break;
  case 2 :	
      write_imau2_ferm(&wri, im0u2);
      break;
  case 4 :	
      write_imafl_ferm(&wri, im0fl);
      break;
  }
}



