/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* *************************M_coocur.h  ********************************/
/*                                                                   */
/*descripteurs et prototypes associes a l'operateur de la matrice de coocurence*/
/*                                                                   */
/*  Derniere modif : G. Selaru, 29.04.05                          */
/* *******************************************************************/
/*pas de code avant cette ligne */

#ifndef __Mcoocur
#define __Mcoocur

#include "volume.h"   /* descripteurs et prototypes des fonctions de ima3D */
#include "image.h"

enum attribute_index {a_contrast, a_correlation, a_directivity, a_entropy, a_homogenity, a_local_homogenity, a_sum_average, a_sum_entropy, a_uniformity};

#define total_attributes 9

typedef struct
{
    int Matcoocur[256][256];
    int nonZeros;
//     double moyenne_x, moyenne_y, variance_x, variance_y;
    
    unsigned int contrast; // will overflow for a window larger than 40x40x40, 1 color
    unsigned int x_y_m; // for correlation; will overflow for a window larger than 40x40x40, 1 color
    unsigned int x_m; // for correlation; will overflow for a window larger than 40x40x40, 1 color
    unsigned int y_m; // for correlation
    unsigned int directivity; // will overflow for a window larger than 1625x1625x1625
    double m_log_m; // for entropy
    unsigned int sum; // for entropy and correlation; will overflow for a window larger than 1625x1625x1625
    unsigned int homogeneity; // will overflow for a window larger than 40x40x40, 1 color
    double localHomogeneity;
    unsigned int sumAverage; // will overflow for a window larger than 203x203x203, 1 color
    unsigned int anti_diagonals_sums[256 * 2 - 1]; // for sum entropy; will overflow for a window larger than 1625x1625x1625
    unsigned int uniformity; // will overflow for a window larger than 40x40x40, 1 color
} GlcmData;

typedef struct
{
    char name[20];
    char short_name[3];
    void (*update)(int row, int col, int oldValue, int NewValue, GlcmData *glcmData);
    double (*finalize)(GlcmData *glcmData);
} GlcmAttribute;

typedef struct
{
  int winx,winy,winz;//dimensions du voisinage
  int dx,dy,dz;//vecteur de direction
  int do_output[total_attributes];
} Mcoocur_t;

param* Mcoocur_lect(Mcoocur_t* des, param* ptp, char* debq);
int Mcoocur_init(Mcoocur_t* des, ima3Du1* im0, ima3Dd *imres[]);
int Mcoocur_calc(Mcoocur_t *des, ima3Du1* im0, ima3Dd *imres[]);

/*pas de code apr� cette ligne*/
#endif
