/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ****************************M_coocur.c**************************/
/*                                                                    */
/* ********************************************************************/

/** operateur : \\
Type entree : imau1 \\
Type sortie : imau1
@name M_coocur
@author Mihai ILINCA
@version 1 (01.04.03); Include : image.h
@see messai_XX,  XX = u1 | ... .
*/
/*@{ */  /* doc suivantes en sous-partie */

// #define _ISOC99_SOURCE
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include "volume.h"
#include <curses.h>

#include "M_coocur.h"

//#define with_curses

void updateContrast(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    glcmData->contrast += (row - col) * (row - col) * (newValue - oldValue);
}

double finalizeContrast(GlcmData *glcmData)
{
    return glcmData->contrast / (double) glcmData->nonZeros;
}

// double computeCorrelation(GlcmData *glcmData)
// {
//     int i, j;
//     unsigned long tmp = 0;
// 
//     if (glcmData->variance_x == 0 || glcmData->variance_y == 0)
//         return 0;
// 
//     for(j = 0; j < 256; j++)
//         for(i = 0; i < 256; i++)
//             tmp += (j - glcmData->moyenne_y) * (i - glcmData->moyenne_x) * glcmData->Matcoocur[j][i];
//     return ((double)tmp) / (glcmData->valnon * glcmData->variance_x * glcmData->variance_y);
// }

void updateCorrelation(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
//     glcmData->
}

double finalizeCorrelation(GlcmData *glcmData)
{
}

void updateDirectivity(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    if (row == col)
        glcmData->directivity += (newValue - oldValue);
}

double finalizeDirectivity(GlcmData *glcmData)
{
    return glcmData->directivity / (double) glcmData->nonZeros;
}

void updateEntropy(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    if (newValue != 0) {
        glcmData->m_log_m += newValue * log10(newValue);
        glcmData->sum += newValue;
    }
    if (oldValue != 0) {
        glcmData->m_log_m -= oldValue * log10(oldValue);
        glcmData->sum -= oldValue;
    }
}

double finalizeEntropy(GlcmData *glcmData)
{
    return 1 - (glcmData->m_log_m - log10(glcmData->nonZeros) * glcmData->sum) / (glcmData->nonZeros * log10(glcmData->nonZeros));
}

void updateHomogeneity(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    glcmData->homogeneity += (newValue * newValue - oldValue * oldValue);
}

double finalizeHomogeneity(GlcmData *glcmData)
{
    return glcmData->homogeneity / (double) glcmData->nonZeros;
}

void updateLocalHomogeneity(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    glcmData->localHomogeneity += (newValue - oldValue) / (1 + (row - col) * (row - col));
}

double finalizeLocalHomogeneity(GlcmData *glcmData)
{
    return glcmData->localHomogeneity / (double) glcmData->nonZeros;
}

void updateSumAverage(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    glcmData->sumAverage += (row + col) * (newValue - oldValue);
}

double finalizeSumAverage(GlcmData *glcmData)
{
    return glcmData->sumAverage / (double) glcmData->nonZeros;
}

void updateSumEntropy(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    glcmData->anti_diagonals_sums[row + col] += (newValue - oldValue);
}

double finalizeSumEntropy(GlcmData *glcmData)
{
    double result = 0;
    int i;
    for (i = 0; i < 256 * 2 - 1; i ++)
        if (glcmData->anti_diagonals_sums[i] != 0)
            result += glcmData->anti_diagonals_sums[i] * log10(glcmData->anti_diagonals_sums[i]);
    return result / (double) glcmData->nonZeros;
}

void updateUniformity(int row, int col, int oldValue, int newValue, GlcmData *glcmData)
{
    if (row == col)
        glcmData->uniformity += (newValue * newValue - oldValue * oldValue);   
}

double finalizeUniformity(GlcmData *glcmData)
{
    return glcmData->uniformity / (double) (glcmData->nonZeros * glcmData->nonZeros);
}

GlcmAttribute attributes[total_attributes] =
{
    { .name = "Constrast", .short_name = "Con", .update = updateContrast, .finalize = finalizeContrast},
    { .name = "Correlation", .short_name = "Cor", .update = updateCorrelation, .finalize = finalizeCorrelation},
    { .name = "Directivity", .short_name = "Dir", .update = updateDirectivity, .finalize = finalizeDirectivity},
    { .name = "Entropy", .short_name = "Ent", .update = updateEntropy, .finalize = finalizeEntropy},
    { .name = "Homogeneity", .short_name = "Hom", .update = updateHomogeneity, .finalize = finalizeHomogeneity},
    { .name = "Local Homogeneity", .short_name = "LHo", .update = updateLocalHomogeneity, .finalize = finalizeLocalHomogeneity},
    { .name = "Sum average", .short_name = "SAv", .update = updateSumAverage, .finalize = finalizeSumAverage},
    { .name = "Sum entropy", .short_name = "SEn", .update = updateSumEntropy, .finalize = finalizeSumEntropy},
    { .name = "Uniformity", .short_name = "Uni", .update = updateUniformity, .finalize = finalizeUniformity},
};

void initGlcmData(GlcmData *glcmData)
{
    int row, col;
    for (row = 0; row < 256; row ++)
        for(col = 0; col < 256; col ++)
            glcmData->Matcoocur[row][col] = 0;
    glcmData->nonZeros = 0;
    glcmData->contrast = 0;
    glcmData->directivity = 0;
    glcmData->m_log_m = 0;
    glcmData->sum = 0;
    glcmData->homogeneity = 0;
    glcmData->localHomogeneity = 0;
    glcmData->sumAverage = 0;
    glcmData->uniformity = 0;
    for (row = 0; row < 256; row ++)
        glcmData->anti_diagonals_sums[row] = 0;
}

void valueUpdateGlcmData(int row, int col, int oldValue, int newValue, GlcmData *glcmData, Mcoocur_t *des)
{
    int i;
    for (i = 0; i < total_attributes; i++)
        if (des->do_output[i])
            attributes[i].update(row, col, oldValue, newValue, glcmData);
    
    if (oldValue != 0 && newValue == 0)
        glcmData->nonZeros --;
    else if (oldValue == 0 && newValue != 0)
        glcmData->nonZeros ++;
}

void pixelUpdateGlcmData(int x, int y, int z, int increment, GlcmData *glcmData, ima3Du1* im0, Mcoocur_t *des)
{
    int row = im0->data[z][y][x];
    int col = im0->data[z + des->dz][y + des->dy][x + des->dx];
    
    int oldValue = glcmData->Matcoocur[row][col];
    int newValue = oldValue + increment;
    glcmData->Matcoocur[row][col] = newValue;

    valueUpdateGlcmData(row, col, oldValue, newValue, glcmData, des);
}

/* *************************  INITIALISATION  ***************************/
/** XX = u1 | etc : initialisation de l'operateur M_coocur.\\
    allocation image resultat,
    calcul des coeff du masque
    @name median3d_init
    @param des  pointeur descripteur
    @param im0  image initiale (taille connue)
    @param imres  pointeur image resultat
    @author Mihai ILINCA
    @version 1 (01.04.03); Include : M_coocur.h
*/
int Mcoocur_init(Mcoocur_t* des, ima3Du1* im0, ima3Dd* imres[])
{
    int i;
    /* image resultat */

    for (i = 0; i < total_attributes; i++)
    {
        if (des->do_output[i])
        {
            imres[i] = (ima3Dd *) malloc(sizeof(ima3Dd));
            imres[i]->dimx=im0->dimx;
            imres[i]->dimy=im0->dimy;
            imres[i]->dimz=im0->dimz;
            allouer_ima3Dd (imres[i]);
            imres[i]->lgtete= im0->lgtete;
            imres[i]->dept = 64;
        }
        else
            imres[i] = 0;
    }
    return 0;
}

/* *************************  LECTURE  *******************************/
/** lecture des parametres de l'operateur M_coocur. \\
    @name
    @param des  pointeur vers descripteur
    @param ptp pointeur sur parametre courant deja alloue,
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite)
    @author Mihai ILINCA
    @version 1 (15.05.05)
*/
param* Mcoocur_lect(Mcoocur_t *des, param *ptp, char *debq)
{
    char question[500];
    int i;
    //Taille de la fenetre sur X
    sprintf(question, "%s winx", debq);
    lec_param(question, ptp);
    des->winx = atoi(ptp->rep);
    ptp = ptp->next;
    //Taille de la fenetre sur Y
    sprintf(question, "%s winy", debq);
    lec_param(question, ptp);
    des->winy= atoi(ptp->rep);
    ptp = ptp->next;

    //Taille de la fenetre sur Z
    sprintf(question, "%s winz", debq);
    lec_param(question, ptp);
    des->winz = atoi(ptp->rep);
    ptp = ptp->next;

    //Vecteur de calcule de la distance
    sprintf(question, "%s dx ", debq);
    lec_param(question, ptp);
    des->dx = atoi(ptp->rep);
    ptp = ptp->next;

    //Vecteur de calcule de la distance
    sprintf(question, "%s dy", debq);
    lec_param(question, ptp);
    des->dy = atoi(ptp->rep);
    ptp = ptp->next;

    //Vecteur de calcule de la distance
    sprintf(question, "%s dz ", debq);
    lec_param(question, ptp);
    des->dz = atoi(ptp->rep);
    ptp = ptp->next;

    //Choix de l'atribut
    for (i = 0; i < total_attributes; i++)
    {
        sprintf(question, "%s %s (0/1)", debq, attributes[i].name);
        lec_param(question, ptp);
        des->do_output[i] = (ptp->rep[0] == '1');
        ptp = ptp->next;
    }
    return(ptp);
}

/* *************************  CALCUL  ***************************/
/** XX = u1 | etc : calcul de l'operateur M_coocur.\\
    Calcul du resultat de l'operateur M_coocur\\
    @name Mcoocur_calc
    @param du pointeur descripteur
    @param im0 image initiale
    @param imres pointeur image resultat
    @author Mihai ILINCA
    @version 1 (01.04.03); Include : M_coocur .h
*/
int Mcoocur_calc(Mcoocur_t *des, ima3Du1* im0, ima3Dd *imres[])
{
    // For attribute computation
    GlcmData glcmData;

    // For evaluating speed performance
    unsigned long total_iterations;
    unsigned long iterations_per_tick;
    unsigned long total_ticks, tick;
    double tick_percent, percent;
    struct timeval tv;
    double tick_start_time, tick_end_time;
    unsigned long estimated_time;
    int hours, minutes, seconds;

    // Calculate the limits for x, y and z depending on the size of the direction vector
    int xMin, xMax, yMin, yMax, zMin, zMax;
    if (des->winx > 0) {
        xMin = 0; // min limit is given by the image
        xMax = im0->dimx - des->winx - des->dx; // max limit is given by the direction vector
    } else {
        xMin = des->dx; // min limit is given by the direction vector
        xMax = im0->dimx - des->winx; // max limit is given by the image
    }
    if (des->winy > 0) {
        yMin = 0; // min limit is given by the image
        yMax = im0->dimy - des->winy - des->dy; // max limit is given by the direction vector
    } else {
        yMin = des->dy; // min limit is given by the direction vector
        yMax = im0->dimy - des->winy;
    }
    if (des->winz > 0) {
        zMin = 0; // min limit is given by the image
        zMax = im0->dimz - des->winz - des->dz; // max limit is given by the direction vector
    } else {
        zMin = des->dz; // min limit is given by the direction vector
        zMax = im0->dimz - des->winz; // max limit is given by the image
    }

    //Calculate the relative position of the output image
    int xPos, yPos, zPos;
    xPos = (des->winx + des->dx) / 2;
    yPos = (des->winy + des->dy) / 2;
    zPos = (des->winz + des->dz) / 2;
    
    //Attributes Computation

    // Speed performance measuring
    // There are 6 loops: dimx, dimy, dimz
    // One tick is one iteration of the loop dimy, so there will be a total of aprox dimx * dimy ticks

    // Speed performance measuring variables
    iterations_per_tick = (zMax - zMin); // dimz
    total_ticks = (xMax - xMin) * (yMax - yMin); // dimx * dimy
    
    total_iterations = iterations_per_tick * total_ticks;
    tick_percent = 100 / (double)total_ticks; // =aprox 100/(dimx * dimy)
    percent = 0;
    tick = 0;
    gettimeofday(&tv, 0);
    tick_start_time = tv.tv_sec + tv.tv_usec/1000000.;

    // Actual attributes computation
#ifdef with_curses
    initscr();
    mvprintw(2, 0, "Attributes Computation started on %s", ctime(&tv.tv_sec));
    refresh();
#endif
 
    int row, col;
    int x, y, z;
    

    // First step init
    initGlcmData(&glcmData);
    for(z = zMin; z < zMin + des->winz; z ++)
        for(y = yMin; y < yMin + des->winy; y ++)
            for(x = xMin; x < xMin + des->winx; x ++) {
                row = im0->data[z][y][x];
                col = im0->data[z + des->dz][y + des->dz][x + des->dz];
                glcmData.Matcoocur[row][col]++;
            }

    for(row = 0; row < 256; row++)
        for(col = 0; col < 256; col++)
            valueUpdateGlcmData(row, col, 0, glcmData.Matcoocur[row][col], &glcmData, des);
            
    int xIncrement = -1, yIncrement = -1;
    int xStart, xEnd;
    int yStart, yEnd;
    int xAddStart, xAddEnd, xSubStart, xSubEnd;
    int yAddStart, yAddEnd, ySubStart, ySubEnd;
    int zAddStart, zAddEnd, zSubStart, zSubEnd;
    
    int wx, wy, wz;
            
    xAddStart = 0;
    xSubStart = 0;
    xAddEnd = -1;
    xSubEnd = -1;
    yAddStart = 0;
    ySubStart = 0;
    yAddEnd = -1;
    ySubEnd = -1;
    zAddStart = 0;
    zAddEnd = 0;
    zSubStart = - 1;
    zSubEnd = -1;
    
                FILE *f;
                f = fopen("nonZeros.dump", "w");
    z = zMin;
    while(1) { // Loop exit is made trough a break statement
        // Togle y axis advance direction (snake-like iteration trough the image)
        if (yIncrement == -1) {
            yIncrement = +1;
            yStart = yMin;
            yEnd = yMax;
        } else {
            yIncrement = -1;
            yStart = yMax - 1;
            yEnd = yMin - 1;
        }

        y = yStart;
        while(1) { // Loop exit is made trough a break statement
            // Togle x axis advance direction (snake-like iteration trough the image)
            if (xIncrement == -1) {
                xIncrement = +1;
                xStart = xMin;
                xEnd = xMax;
            } else {
                xIncrement = -1;
                xStart = xMax - 1;
                xEnd = xMin -1;
            }

            x = xStart;
            while(1) { // Loop exit is made trough a break statement
                // Substract the contribution of pixels not anymore part of the window
                for (wz = zSubStart; wz < zSubEnd; wz++)
                    for (wy = ySubStart; wy < ySubEnd; wy++)
                        for (wx = xSubStart; wx < xSubEnd; wx++)
                            pixelUpdateGlcmData(wx, wy, wz, -1, &glcmData, im0, des);
                
                // Add the contribution of those pixels new to the window
                for (wz = zAddStart; wz < zAddEnd; wz++)
                    for (wy = yAddStart; wy < yAddEnd; wy++)
                        for (wx = xAddStart; wx < xAddEnd; wx++)
                            pixelUpdateGlcmData(wx, wy, wz, +1, &glcmData, im0, des);
                
//                 fprintf(f, "%d\n", glcmData.nonZeros);
                int i;
                for (i = 0; i < total_attributes; i++)
                    if (des->do_output[i])
                        imres[i]->data[z + zPos][y + yPos][x + xPos] = attributes[i].finalize(&glcmData);
                        
                if (x + xIncrement == xEnd) // will there be a next iteration of this loop?
                    break;
                else {
                    if (xIncrement == +1) {
                        xAddStart = x + des->winx;
                        xAddEnd = x + des->winx + 1;
                        xSubStart = x;
                        xSubEnd = x + 1;
                    } else {
                        xAddStart = x - 1;
                        xAddEnd = x;
                        xSubStart = x + des->winx - 1;
                        xSubEnd = x + des->winx;
                    }
                    yAddStart = ySubStart = y;
                    yAddEnd = ySubEnd = y + des->winy;
                    zAddStart = zSubStart = z;
                    zAddEnd = zSubEnd = z + des->winz;
                    x += xIncrement;
                }
            }
            // Speed performance measuring logic
            gettimeofday(&tv, 0);
            tick ++;
            tick_end_time = tv.tv_sec + tv.tv_usec/1000000.;
            percent += tick_percent;
            estimated_time = (total_ticks - tick) * (tick_end_time - tick_start_time);
            hours = estimated_time / (60 * 60);
            estimated_time %= (60 * 60);
            minutes = estimated_time / 60;
            seconds = estimated_time % 60;
#ifdef with_curses
            mvprintw(3, 0,
                    "Attributes Computation: %06.2f %%, %012.2f iterations/sec, %03d:%02d:%02d estimated time remaning",
                    (float)percent,
                    iterations_per_tick / (tick_end_time - tick_start_time),
                    hours,
                    minutes,
                    seconds
                );
            refresh();
#endif
            tick_start_time = tick_end_time;
            
            if (y + yIncrement == yEnd) // will there be a next iteration of this loop?
                break;
            else {
                if (yIncrement == +1) {
                    yAddStart = y + des->winy;
                    yAddEnd = y + des->winy + 1;
                    ySubStart = y;
                    ySubEnd = y + 1;
                } else {
                    yAddStart = y - 1;
                    yAddEnd = y;
                    ySubStart = y + des->winy - 1;
                    ySubEnd = y + des->winy;
                }
                xAddStart = xSubStart = x;
                xAddEnd = xSubEnd = x + des->winx;
                zAddStart = zSubStart = z;
                zAddEnd = zSubEnd = z + des->winz;
                y += yIncrement;
            }
        }
        
        if (z + 1 >= zMax) // will there be a next iteration of this loop?
            break;
        else {
            zAddStart = z + des->winy;
            zAddEnd = z + des->winy + 1;
            zSubStart = z;
            zSubEnd = z + 1;
            xAddStart = xSubStart = x;
            xAddEnd = xSubEnd = x + des->winx;
            yAddStart = ySubStart = y;
            yAddEnd = ySubEnd = y + des->winy;
            z++;
        }
    }

    gettimeofday(&tv, 0);
#ifdef with_curses
    mvprintw(10, 0, "Exit on %s", ctime(&tv.tv_sec));
    refresh();
    endwin();
#endif
fclose(f);
                
    return(0);
}


/*@} */  /* fin doc en sous partie */
