/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \defgroup carte_distance3d
    operateurs carte_distance3D: ensemble d'oprateurs pour le calcul
    de cartes de distance 3D

    Type entree : ima3du1 \n
    Type sortie : ima3du1 \n
    @name carte_distance3d
    @author L. Valet, D. Coquin
    @version 1 (25.05.07)
    @see 
*/

#include "distance3d.h"
#include <stdio.h>
#include <math.h>


#define INFINI  4294999999;/*2^32-1 car distance calculer sur 32bits maxi*/

/** \ingroup carte_distance3d
    lecture des parametres de l'operateur carte_distance3d. \\
    
    @param des  pointeur vers le descripteur de cet oprateur  
    @param ptp pointeur sur parametre courant deja alloue,     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author L.Valet, D. Coquin
    @version 1 (25.05.07); Include : distance3d.h
*/
param* carte_distance3d_lect(carte_distance3d_t* des, param* ptp, char* debq){
  des->L=1.0;
  des->H=1.0;
  des->P=1.0;
  
  return(ptp);
}



/* *******************************************************************/
float square(x)
float x;
{
	return(x * x);
}



/* *******************************************************************/
void calculer_masque(carte_distance3d_t* des)
{
        int N;
	int	i,j,k;
	float	T111, T110, T101, T011, lambda1, lambda2;
	float   T120, T121, T012, T112;
	float	d100, d010, d001, d110, d101, d011, d111;
	float	d210, d201, d120, d102, d021, d012, d211;
	float	d121, d112, d221, d212, d122;
	unsigned int	D100, D010, D001, D110, D101, D011, D111;
	unsigned int	D210, D201, D120, D102, D021, D012, D211;
	unsigned int	D121, D112, D221, D212, D122;


	float L=des->L;
	float H=des->H;
	float P=des->P;

/* dans cette partie les indices: i,j,k sont pris suivant les axes ****/


	T111 = sqrt(L*L + H*H + P*P); T110 = sqrt(L*L + H*H);
	T012 = sqrt(H*H + 4*P*P); T112 = sqrt(L*L + H*H + 4*P*P);
	T120 = sqrt(L*L + 4*H*H); T121 = sqrt(L*L + 4*H*H + P*P);
	T101 = sqrt(L*L + P*P); T011 = sqrt(H*H + P*P);

	if(L<= P)
	{
		lambda1 = (1/(L*L))*square(T120-2.0*H) + (1/(P*P))*square(T121-T120);
		d010 = (-2*H + 2*H*sqrt(1+lambda1))/lambda1;
		d100 = d010 * (L/H);
	}
	else
	{
		lambda2 = (1/(L*L))*square(T112-T012) + (1/(H*H))*square(T012-2*P);
		d001 = (-2*P + 2*P*sqrt(1+lambda2))/lambda2;
		d010 = d001 * (L/P);
	}

	d001 = d010 * (P/H);
	d110 = d010 * (T110/H);
	d101 = d010 * (T101/H);
	d011 = d010 * (T011/H);
	d111 = d010 * (T111/H);
	d210 = d010 * (sqrt(4*L*L + H*H)/H);
	d201 = d010 * (sqrt(4*L*L + P*P)/H);
	d120 = d010 * (sqrt(L*L + 4*H*H)/H);
	d021 = d010 * (sqrt(4*H*H + P*P)/H);
	d012 = d010 * (sqrt(H*H + 4*P*P)/H);
	d102 = d010 * (sqrt(L*L + 4*P*P)/H);
	d211 = d010 * (sqrt(4*L*L + H*H + P*P)/H);
	d121 = d010 * (sqrt(L*L + 4*H*H + P*P)/H);
	d112 = d010 * (sqrt(L*L + H*H + 4*P*P)/H);
	d112 = d010 * (sqrt(L*L + H*H + 4*P*P)/H);
	d221 = d010 * (sqrt(4*L*L + 4*H*H + P*P)/H);
	d212 = d010 * (sqrt(4*L*L + H*H + 4*P*P)/H);
	d122 = d010 * (sqrt(L*L + 4*H*H + 4*P*P)/H);

	
	/* calcul des coefficients entiers */
	//printf("Meilleure valeur de N pour (L=H=P=1) : 67\n");
	//implique 6% d'erreur avec un masque 3*3*3
	//printf("Entrer le facteur multiplicatif N (entier): ");
	//scanf("%d",&N);
	N = 67;
	D100 = rint(d100*N); D010 = rint(d010*N);
	D110 = rint(d110*N); D001 = rint(d001*N); 
	D011 = rint(d011*N); D101 = rint(d101*N);
	D111 = rint(d111*N);
	D210 = rint(d210*N); D201 = rint(d201*N);
	D120 = rint(d120*N); D012 = rint(d012*N); 
	D021 = rint(d021*N); D211 = rint(d211*N);
	D121 = rint(d121*N); D102 = rint(d102*N);
	D112 = rint(d112*N); D221 = rint(d221*N);
	D212 = rint(d212*N); D122 = rint(d122*N); 

	printf("L=%f,H=%f,P=%f\n",L,H,P);

	printf("d100=%f,d110=%f,d111=%f\n",d100,d110,d111);
	printf("d210=%f,d211=%f,d221=%f\n",d210,d211,d221);
	printf("N=%d\n",N);
	printf("D100=%d,D110=%d,D111=%d\n",D100,D110,D111);
	printf("D210=%d,D211=%d,D221=%d\n",D210,D211,D221);
	printf("L=%f,H=%f,P=%f\n",L,H,P);
	
	/* calcul du masque avant */
	/*   prondeur Z = 0 */
	des->mask_av[0][0][0] = 2*D111;
	des->mask_av[0][0][1] = D122;
	des->mask_av[0][0][2] = 2*D011;
	des->mask_av[0][0][3] = D122;
	des->mask_av[0][0][4] = 2*D111;
	des->mask_av[0][1][0] = D212;
	des->mask_av[0][1][1] = D112;
	des->mask_av[0][1][2] = D012;
	des->mask_av[0][1][3] = D112;
	des->mask_av[0][1][4] = D212;
	des->mask_av[0][2][0] = 2*D101;
	des->mask_av[0][2][1] = D102;
	des->mask_av[0][2][2] = 2*D001;
 	des->mask_av[0][2][3] = D102;
	des->mask_av[0][2][4] = 2*D101;
	des->mask_av[0][3][0] = D212;
	des->mask_av[0][3][1] = D112;
	des->mask_av[0][3][2] = D012;
	des->mask_av[0][3][3] = D112;
	des->mask_av[0][3][4] = D212;
	des->mask_av[0][4][0] = 2*D111;
	des->mask_av[0][4][1] = D122;
	des->mask_av[0][4][2] = 2*D011;
	des->mask_av[0][4][3] = D122;
	des->mask_av[0][4][4] = 2*D111;
	/* profondeur Z = 1 */
	des->mask_av[1][0][0] = D221;
	des->mask_av[1][0][1] = D121;
	des->mask_av[1][0][2] = D021;
	des->mask_av[1][0][3] = D121;
	des->mask_av[1][0][4] = D221;
	des->mask_av[1][1][0] = D211;
	des->mask_av[1][1][1] = D111;
	des->mask_av[1][1][2] = D011;
	des->mask_av[1][1][3] = D111;
	des->mask_av[1][1][4] = D211;
	des->mask_av[1][2][0] = D201;
	des->mask_av[1][2][1] = D101;
	des->mask_av[1][2][2] = D001;
 	des->mask_av[1][2][3] = D101;
	des->mask_av[1][2][4] = D201;
	des->mask_av[1][3][0] = D211;
	des->mask_av[1][3][1] = D111;
	des->mask_av[1][3][2] = D011;
	des->mask_av[1][3][3] = D111;
	des->mask_av[1][3][4] = D211;
	des->mask_av[1][4][0] = D221;
	des->mask_av[1][4][1] = D121;
	des->mask_av[1][4][2] = D021;
	des->mask_av[1][4][3] = D121;
	des->mask_av[1][4][4] = D221;
	/*    profondeur Z = 2 */
	des->mask_av[2][0][0] = 2*D110;
	des->mask_av[2][0][1] = D120;
	des->mask_av[2][0][2] = 2*D010;
	des->mask_av[2][0][3] = D120;
	des->mask_av[2][0][4] = 2*D110;
	des->mask_av[2][1][0] = D210;
	des->mask_av[2][1][1] = D110;
	des->mask_av[2][1][2] = D010;
	des->mask_av[2][1][3] = D110;
	des->mask_av[2][1][4] = D210;
	des->mask_av[2][2][0] = 2*D100;
	des->mask_av[2][2][1] = D100;
	des->mask_av[2][2][2] = 0;

	/* calcul du masque retour */
	/*    profondeur Z = 2 */
	des->mask_av[2][4][4] = 2*D110;
	des->mask_av[2][4][3] = D120;
	des->mask_av[2][4][2] = 2*D010;
	des->mask_av[2][4][1] = D120;
	des->mask_av[2][4][0] = 2*D110;
	des->mask_av[2][3][4] = D210;
	des->mask_av[2][3][3] = D110;
	des->mask_av[2][3][2] = D010;
	des->mask_av[2][3][1] = D110;
	des->mask_av[2][3][0] = D210;
	des->mask_av[2][2][4] = 2*D100;
	des->mask_av[2][2][3] = D100;
	des->mask_av[2][2][2] = 0;
	/* profondeur Z = 3 */
	des->mask_av[3][0][0] = D221;
	des->mask_av[3][0][1] = D121;
	des->mask_av[3][0][2] = D021;
	des->mask_av[3][0][3] = D121;
	des->mask_av[3][0][4] = D221;
	des->mask_av[3][1][0] = D211;
	des->mask_av[3][1][1] = D111;
	des->mask_av[3][1][2] = D011;
	des->mask_av[3][1][3] = D111;
	des->mask_av[3][1][4] = D211;
	des->mask_av[3][2][0] = D201;
	des->mask_av[3][2][1] = D101;
	des->mask_av[3][2][2] = D001;
 	des->mask_av[3][2][3] = D101;
	des->mask_av[3][2][4] = D201;
	des->mask_av[3][3][0] = D211;
	des->mask_av[3][3][1] = D111;
	des->mask_av[3][3][2] = D011;
	des->mask_av[3][3][3] = D111;
	des->mask_av[3][3][4] = D211;
	des->mask_av[3][4][0] = D221;
	des->mask_av[3][4][1] = D121;
	des->mask_av[3][4][2] = D021;
	des->mask_av[3][4][3] = D121;
	des->mask_av[3][4][4] = D221;	
	/*   prondeur Z = 4 */
	des->mask_av[4][0][0] = 2*D111;
	des->mask_av[4][0][1] = D122;
	des->mask_av[4][0][2] = 2*D011;
	des->mask_av[4][0][3] = D122;
	des->mask_av[4][0][4] = 2*D111;
	des->mask_av[4][1][0] = D212;
	des->mask_av[4][1][1] = D112;
	des->mask_av[4][1][2] = D012;
	des->mask_av[4][1][3] = D112;
	des->mask_av[4][1][4] = D212;
	des->mask_av[4][2][0] = 2*D101;
	des->mask_av[4][2][1] = D102;
	des->mask_av[4][2][2] = 2*D001;
 	des->mask_av[4][2][3] = D102;
	des->mask_av[4][2][4] = 2*D101;
	des->mask_av[4][3][0] = D212;
	des->mask_av[4][3][1] = D112;
	des->mask_av[4][3][2] = D012;
	des->mask_av[4][3][3] = D112;
	des->mask_av[4][3][4] = D212;
	des->mask_av[4][4][0] = 2*D111;
	des->mask_av[4][4][1] = D122;
	des->mask_av[4][4][2] = 2*D011;
	des->mask_av[4][4][3] = D122;
	des->mask_av[4][4][4] = 2*D111;
	printf("des->mask_av[0][0][0]=%d\n", des->mask_av[0][0][0]);

} /* fin calculer_masque */
/* ************************************************************/







/* *******************************************************************/
void im_distance_555(carte_distance3d_t* des, ima3Du4 * ima)
{

	int i,j,k;
	int valint;
	unsigned int maxdist;
	char	afficher;
	unsigned long int	s,min;
	int	N,M,res;
	//unsigned int	***mask_av;
	//unsigned int	mask_av[5][5][5];
	unsigned int	****im;

	im=&(ima->data);
	res = ima->dimz - 5;
	N = ima->dimy - 5;
	M = ima->dimx -5;
	//mask_av = des->mask_av;


	/* Parcour avant */
	printf("im_distance: parcours avant des->mask_av[0][0][0]=%d\n", des->mask_av[0][0][0]); 
	for(i=2;i<res+2;i++)
	  for(j=2;j<N+2;j++)
	    for(k=2;k<M+2;k++)
	    {
	      //printf("i=%d, j=%d, k%d val=%d \n", i, j, k, des->mask_av[0][0][0]);
			min = INFINI; 
			s = 0;
			/*** profondeur Z = 0 ****/
			s = (*im)[i-2][j-2][k-2] + des->mask_av[0][0][0];
			if(s<min)		min = s;
			s = (*im)[i-2][j-2][k-1] + des->mask_av[0][0][1];
			if(s<min)		min = s;
			s = (*im)[i-2][j-2][k] + des->mask_av[0][0][2];
			if(s<min)		min = s;
			s = (*im)[i-2][j-2][k+1] + des->mask_av[0][0][3];
			if(s<min)		min = s;
			s = (*im)[i-2][j-2][k+2] + des->mask_av[0][0][4];

			s = (*im)[i-2][j-1][k-2] + des->mask_av[0][1][0];
			if(s<min)		min = s;
			s = (*im)[i-2][j-1][k-1] + des->mask_av[0][1][1];
			if(s<min)		min = s;
			s = (*im)[i-2][j-1][k] + des->mask_av[0][1][2];
			if(s<min)		min = s;
			s = (*im)[i-2][j-1][k+1] + des->mask_av[0][1][3];
			if(s<min)		min = s;
			s = (*im)[i-2][j-1][k+2] + des->mask_av[0][1][4];

			s = (*im)[i-2][j][k-2] + des->mask_av[0][2][0];
			if(s<min)		min = s;
			s = (*im)[i-2][j][k-1] + des->mask_av[0][2][1];
			if(s<min)		min = s;
			s = (*im)[i-2][j][k] + des->mask_av[0][2][2];
			if(s<min)		min = s;
			s = (*im)[i-2][j][k+1] + des->mask_av[0][2][3];
			if(s<min)		min = s;
			s = (*im)[i-2][j][k+2] + des->mask_av[0][2][4];

			s = (*im)[i-2][j+1][k-2] + des->mask_av[0][3][0];
			if(s<min)		min = s;
			s = (*im)[i-2][j+1][k-1] + des->mask_av[0][3][1];
			if(s<min)		min = s;
			s = (*im)[i-2][j+1][k] + des->mask_av[0][3][2];
			if(s<min)		min = s;
			s = (*im)[i-2][j+1][k+1] + des->mask_av[0][3][3];
			if(s<min)		min = s;
			s = (*im)[i-2][j+1][k+2] + des->mask_av[0][3][4];

			s = (*im)[i-2][j+2][k-2] + des->mask_av[0][4][0];
			if(s<min)		min = s;
			s = (*im)[i-2][j+2][k-1] + des->mask_av[0][4][1];
			if(s<min)		min = s;
			s = (*im)[i-2][j+2][k] + des->mask_av[0][4][2];
			if(s<min)		min = s;
			s = (*im)[i-2][j+2][k+1] + des->mask_av[0][4][3];
			if(s<min)		min = s;
			s = (*im)[i-2][j+2][k+2] + des->mask_av[0][4][4];

			/*** profondeur Z = 1 ****/
			s = (*im)[i-1][j-2][k-2] + des->mask_av[1][0][0];
			if(s<min)		min = s;
			s = (*im)[i-1][j-2][k-1] + des->mask_av[1][0][1];
			if(s<min)		min = s;
			s = (*im)[i-1][j-2][k] + des->mask_av[1][0][2];
			if(s<min)		min = s;
			s = (*im)[i-1][j-2][k+1] + des->mask_av[1][0][3];
			if(s<min)		min = s;
			s = (*im)[i-1][j-2][k+2] + des->mask_av[1][0][4];

			s = (*im)[i-1][j-1][k-2] + des->mask_av[1][1][0];
			if(s<min)		min = s;
			s = (*im)[i-1][j-1][k-1] + des->mask_av[1][1][1];
			if(s<min)		min = s;
			s = (*im)[i-1][j-1][k] + des->mask_av[1][1][2];
			if(s<min)		min = s;
			s = (*im)[i-1][j-1][k+1] + des->mask_av[1][1][3];
			if(s<min)		min = s;
			s = (*im)[i-1][j-1][k+2] + des->mask_av[1][1][4];

			s = (*im)[i-1][j][k-2] + des->mask_av[1][2][0];
			if(s<min)		min = s;
			s = (*im)[i-1][j][k-1] + des->mask_av[1][2][1];
			if(s<min)		min = s;
			s = (*im)[i-1][j][k] + des->mask_av[1][2][2];
			if(s<min)		min = s;
			s = (*im)[i-1][j][k+1] + des->mask_av[1][2][3];
			if(s<min)		min = s;
			s = (*im)[i-1][j][k+2] + des->mask_av[1][2][4];

			s = (*im)[i-1][j+1][k-2] + des->mask_av[1][3][0];
			if(s<min)		min = s;
			s = (*im)[i-1][j+1][k-1] + des->mask_av[1][3][1];
			if(s<min)		min = s;
			s = (*im)[i-1][j+1][k] + des->mask_av[1][3][2];
			if(s<min)		min = s;
			s = (*im)[i-1][j+1][k+1] + des->mask_av[1][3][3];
			if(s<min)		min = s;
			s = (*im)[i-1][j+1][k+2] + des->mask_av[1][3][4];

			s = (*im)[i-1][j+2][k-2] + des->mask_av[1][4][0];
			if(s<min)		min = s;
			s = (*im)[i-1][j+2][k-1] + des->mask_av[1][4][1];
			if(s<min)		min = s;
			s = (*im)[i-1][j+2][k] + des->mask_av[1][4][2];
			if(s<min)		min = s;
			s = (*im)[i-1][j+2][k+1] + des->mask_av[1][4][3];
			if(s<min)		min = s;
			s = (*im)[i-1][j+2][k+2] + des->mask_av[1][4][4];

			/*** profondeur Z = 2 ****/
			s = (*im)[i][j-2][k-2] + des->mask_av[2][0][0];
			if(s<min)		min = s;
			s = (*im)[i][j-2][k-1] + des->mask_av[2][0][1];
			if(s<min)		min = s;
			s = (*im)[i][j-2][k] + des->mask_av[2][0][2];
			if(s<min)		min = s;
			s = (*im)[i][j-2][k+1] + des->mask_av[2][0][3];
			if(s<min)		min = s;
			s = (*im)[i][j-2][k+2] + des->mask_av[2][0][4];

			s = (*im)[i][j-1][k-2] + des->mask_av[2][1][0];
			if(s<min)		min = s;
			s = (*im)[i][j-1][k-1] + des->mask_av[2][1][1];
			if(s<min)		min = s;
			s = (*im)[i][j-1][k] + des->mask_av[2][1][2];
			if(s<min)		min = s;
			s = (*im)[i][j-1][k+1] + des->mask_av[2][1][3];
			if(s<min)		min = s;
			s = (*im)[i][j-1][k+2] + des->mask_av[2][1][4];

			s = (*im)[i][j][k-2] + des->mask_av[2][2][0];
			if(s<min)		min = s;
			s = (*im)[i][j][k-1] + des->mask_av[2][2][1];
			if(s<min)		min = s;
			s = (*im)[i][j][k] + des->mask_av[2][2][2];
			if(s<min)		min = s;

			(*im)[i][j][k] = min;
		}
	printf("fin parcour avant\n");    
	/* Parcours arriere */
/*	printf("im_distance: parcours arriere \n"); */
	maxdist = 0;
	for(i= res+1; i> 1;i--)
	  for(j= N+1; j> 1;j--)
	    for(k= M+1; k> 1;k--)
	    {
			min = INFINI; s = 0;

			/*** profondeur Z = 2 ****/
			s = (*im)[i][j][k] + des->mask_av[2][2][2];
			if(s<min)		min = s;
			s = (*im)[i][j][k+1] + des->mask_av[2][2][3];
			if(s<min)		min = s;
			s = (*im)[i][j][k+2] + des->mask_av[2][2][4];

			s = (*im)[i][j+1][k-2] + des->mask_av[2][3][0];
			if(s<min)		min = s;
			s = (*im)[i][j+1][k-1] + des->mask_av[2][3][1];
			if(s<min)		min = s;
			s = (*im)[i][j+1][k] + des->mask_av[2][3][2];
			if(s<min)		min = s;
			s = (*im)[i][j+1][k+1] + des->mask_av[2][3][3];
			if(s<min)		min = s;
			s = (*im)[i][j+1][k+2] + des->mask_av[2][3][4];

			s = (*im)[i][j+2][k-2] + des->mask_av[2][4][0];
			if(s<min)		min = s;
			s = (*im)[i][j+2][k-1] + des->mask_av[2][4][1];
			if(s<min)		min = s;
			s = (*im)[i][j+2][k] + des->mask_av[2][4][2];
			if(s<min)		min = s;
			s = (*im)[i][j+2][k+1] + des->mask_av[2][4][3];
			if(s<min)		min = s;
			s = (*im)[i][j+2][k+2] + des->mask_av[2][4][4];

			/*** profondeur Z = 3 ****/
			s = (*im)[i+1][j-2][k-2] + des->mask_av[3][0][0];
			if(s<min)		min = s;
			s = (*im)[i+1][j-2][k-1] + des->mask_av[3][0][1];
			if(s<min)		min = s;
			s = (*im)[i+1][j-2][k] + des->mask_av[3][0][2];
			if(s<min)		min = s;
			s = (*im)[i+1][j-2][k+1] + des->mask_av[3][0][3];
			if(s<min)		min = s;
			s = (*im)[i+1][j-2][k+2] + des->mask_av[3][0][4];

			s = (*im)[i+1][j-1][k-2] + des->mask_av[3][1][0];
			if(s<min)		min = s;
			s = (*im)[i+1][j-1][k-1] + des->mask_av[3][1][1];
			if(s<min)		min = s;
			s = (*im)[i+1][j-1][k] + des->mask_av[3][1][2];
			if(s<min)		min = s;
			s = (*im)[i+1][j-1][k+1] + des->mask_av[3][1][3];
			if(s<min)		min = s;
			s = (*im)[i+1][j-1][k+2] + des->mask_av[3][1][4];

			s = (*im)[i+1][j][k-2] + des->mask_av[3][2][0];
			if(s<min)		min = s;
			s = (*im)[i+1][j][k-1] + des->mask_av[3][2][1];
			if(s<min)		min = s;
			s = (*im)[i+1][j][k] + des->mask_av[1][3][2];
			if(s<min)		min = s;
			s = (*im)[i+1][j][k+1] + des->mask_av[3][2][3];
			if(s<min)		min = s;
			s = (*im)[i+1][j][k+2] + des->mask_av[3][2][4];

			s = (*im)[i+1][j+1][k-2] + des->mask_av[3][3][0];
			if(s<min)		min = s;
			s = (*im)[i+1][j+1][k-1] + des->mask_av[3][3][1];
			if(s<min)		min = s;
			s = (*im)[i+1][j+1][k] + des->mask_av[3][3][2];
			if(s<min)		min = s;
			s = (*im)[i+1][j+1][k+1] + des->mask_av[3][3][3];
			if(s<min)		min = s;
			s = (*im)[i+1][j+1][k+2] + des->mask_av[3][3][4];

			s = (*im)[i+1][j+2][k-2] + des->mask_av[3][4][0];
			if(s<min)		min = s;
			s = (*im)[i+1][j+2][k-1] + des->mask_av[3][4][1];
			if(s<min)		min = s;
			s = (*im)[i+1][j+2][k] + des->mask_av[3][4][2];
			if(s<min)		min = s;
			s = (*im)[i+1][j+2][k+1] + des->mask_av[3][4][3];
			if(s<min)		min = s;
			s = (*im)[i+1][j+2][k+2] + des->mask_av[3][4][4];

			/* ** profondeur Z = 4 ****/
			s = (*im)[i+2][j-2][k-2] + des->mask_av[4][0][0];
			if(s<min)		min = s;
			s = (*im)[i+2][j-2][k-1] + des->mask_av[4][0][1];
			if(s<min)		min = s;
			s = (*im)[i+2][j-2][k] + des->mask_av[4][0][2];
			if(s<min)		min = s;
			s = (*im)[i+2][j-2][k+1] + des->mask_av[4][0][3];
			if(s<min)		min = s;
			s = (*im)[i+2][j-2][k+2] + des->mask_av[4][0][4];

			s = (*im)[i+2][j-1][k-2] + des->mask_av[4][1][0];
			if(s<min)		min = s;
			s = (*im)[i+2][j-1][k-1] + des->mask_av[4][1][1];
			if(s<min)		min = s;
			s = (*im)[i+2][j-1][k] + des->mask_av[4][1][2];
			if(s<min)		min = s;
			s = (*im)[i+2][j-1][k+1] + des->mask_av[4][1][3];
			if(s<min)		min = s;
			s = (*im)[i+2][j-1][k+2] + des->mask_av[4][1][4];

			s = (*im)[i+2][j][k-2] + des->mask_av[4][2][0];
			if(s<min)		min = s;
			s = (*im)[i+2][j][k-1] + des->mask_av[4][2][1];
			if(s<min)		min = s;
			s = (*im)[i+2][j][k] + des->mask_av[4][2][2];
			if(s<min)		min = s;
			s = (*im)[i+2][j][k+1] + des->mask_av[4][2][3];
			if(s<min)		min = s;
			s = (*im)[i+2][j][k+2] + des->mask_av[4][2][4];

			s = (*im)[i+2][j+1][k-2] + des->mask_av[4][3][0];
			if(s<min)		min = s;
			s = (*im)[i+2][j+1][k-1] + des->mask_av[4][3][1];
			if(s<min)		min = s;
			s = (*im)[i+2][j+1][k] + des->mask_av[4][3][2];
			if(s<min)		min = s;
			s = (*im)[i+2][j+1][k+1] + des->mask_av[4][3][3];
			if(s<min)		min = s;
			s = (*im)[i+2][j+1][k+2] + des->mask_av[4][3][4];

			s = (*im)[i+2][j+2][k-2] + des->mask_av[4][4][0];
			if(s<min)		min = s;
			s = (*im)[i+2][j+2][k-1] + des->mask_av[4][4][1];
			if(s<min)		min = s;
			s = (*im)[i+2][j+2][k] + des->mask_av[4][4][2];
			if(s<min)		min = s;
			s = (*im)[i+2][j+2][k+1] + des->mask_av[4][4][3];
			if(s<min)		min = s;
			s = (*im)[i+2][j+2][k+2] + des->mask_av[4][4][4];

			(*im)[i][j][k] = min;
			if(min > (maxdist))		maxdist=min;
		}
			
} /* fin im_distance_555 */














/** \ingroup carte_distance3d
    Initialisation de l'operateur carte_distance3d. 

    @param des  pointeur vers le descripteur de cet oprateur  
    @return 0 si tout va bien 
    @author L.Valet, D. Coquin
    @version 1 (25.05.07); Include : distance3d.h
*/
int carte_distance3d_init(carte_distance3d_t* des){

  calculer_masque( des);

}







/** \ingroup carte_distance3d
    Calcul de l'operateur carte_distance3d.

    @param des  pointeur vers le descripteur de cet oprateur  
    @param ima image 3d de type u1 contenant l'objet de base  avec la valeur 0 et le fond avec la valeur INFINI,     
    @return 0 si tout va bien
    @author L.Valet, D. Coquin
    @version 1 (25.05.07); Include : distance3d.h
*/
int carte_distance3d_calc(carte_distance3d_t* des,  ima3Du4* ima){


  im_distance_555( des, ima);

}




