/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* **************************** lessai.c ******************************/
/*                                                                    */
/* ********************************************************************/

/** \defgroup essai

operateur essai : exemple d'operateur type.

Illustration du decoupage d'un operateur en 3 fonctions (lect, init, calc).
Ici simple recopie de l'image entrante dans l'image sortant


Type entree : imau1 | ... (selon les types)\n
Type sortie : imau1 | ... (respectivement)

\author Lionel Valet
\version 1.0 (25/11/2006); Include : volume.h, essai.h
\see messai_XX,  XX = u1 | ... .
*/



#include "volume.h"
#include "proto2D.h"
#include "essai.h"
#include <stdlib.h>
#include <string.h>

/* *************************  LECTURE  *******************************/
/** \ingroup essai

    lecture des parametres de l'operateur essai. \n
    dimX, dimY : exemple d'entiers : nombre de colonnes, de lignes d'un masque\n
    gain : exemple de reel\n
    forme : exemple de chaine de charactere : rectangle/triangle
    
    @param des pointeur descripteur     
    @param ptp pointeur sur parametre courant deja alloue (a lire),     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h, essai.h
*/
param *essai_lect( essai_t *des, param *ptp, char *debq){
  char question[500];

  sprintf(question, "%s dimX (entier)", debq); 
  lec_param(question, ptp);
  des->dimX = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s dimY (entier)", debq); 
  lec_param(question, ptp);
  des->dimY = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s gain (reel)", debq); 
  lec_param(question, ptp);
  des->gain = (float)atof(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s forme (nom)", debq); 
  lec_param(question, ptp);
  strcpy(des->forme, ptp->rep);
  ptp = ptp->next;

  return(ptp);
}

/* *************************  INITIALISATION  ***************************/
/** \ingroup essai

    XX = u1 | etc : initialisation de l'operateur essai.

    allocation image resultat et images de travail, 
    calcul des coeff du masque
    
    @param des  pointeur descripteur
    @param imaIn  image initiale (taille connue)
    @param imaOut  pointeur image resultat
    @author Lionel Valet
    @version 1.0 (25/11/2006); Include : proto2D.h, essai.h
*/
int essaiu1_init(essai_t *des, ima3Du1 imaIn, ima3Du1 *imaOut){
  int i, j;
 
  /* Copie des dimensions de l'image */ 
  imaOut->dimx = imaIn.dimx;
  imaOut->dimy = imaIn.dimy;
  imaOut->dimz = imaIn.dimz;
  allouer_ima3Du1 (imaOut);

  /* calcul du masque selon la forme */
  des->msq = des->dimX *des->dimY + des->gain;
}


/* *************************  CALCUL  ***************************/
/** \ingroup essai
    
    XX = u1 | etc : calcul de l'operateur essai.

    Calcul du resultat de l'operateur essai (ici simple recopie!)

    @param des  pointeur descripteur
    @param imaIn  image initiale 
    @param imaOut  pointeur image resultat
    @author E. Trouve
    @version 1.0 (19/10/99); Include : proto2D.h, essai.h
*/
int essaiu1_calc(essai_t *des, ima3Du1 imaIn, ima3Du1 *imaOut){
  register int x, y, z;
      

  /*traitement simple pour l'exemple*/
  for(z=0; z<imaIn.dimz; z++)
    for(y=0; y<imaIn.dimy; y++)
      for(x=0; x<imaIn.dimx; x++)
    	  imaOut->data[z][y][x] = imaIn.data[z][y][x] + des->msq;

}

