/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

/** \defgroup eval3d
    operateur eval_volume : oprateur pour l'valuation des objets contenu dans un volume
    Descripteur : 
    Doc a completer...
    Type entree : ima3du1
    Type sortie : ima3du1 
    @name eval_volume
    @author L. Valet
    @version 1 (22.11.06)
    @see 
*/


#include "eval_volume.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#define ETMAX 17000000 

#define ABSOL(x,y)  (((x) > (y))? abs(x-y) : abs(y-x))
#define MAX(x,y) (((x)>(y))? (x):(y))
#define MIN(x,y) (((x)<(y))? (x):(y))
#define MIN3(x,y,z) ( ( ((x)<(y))&&((x)<(z))   )?(x):( ( ((x)>(y))&&((y)<(z))  )?y:z  )  )
#define VRAI 1
#define FAUX 0

typedef char booleen;

 

/** \ingroup eval3d
    lecture des parametres de l'operateur eval_volume. \\
    @name eval_volume_lect
    @param des  pointeur vers le descripteur de cet oprateur  
    @param ptp pointeur sur parametre courant deja alloue,     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author L.Valet
    @version 1 (22.11.06); Include : eval_volume.h
*/
param* eval_volume_lect(eval_volume_t* des, param* ptp, char* debq){
  char question[500];

  sprintf(question, "%s Classe a etudier (niveau de gris)", debq); 
  lec_param(question, ptp);
  des->classe = atoi(ptp->rep);
  ptp = ptp->next;
  
  return(ptp);
}





/** \ingroup eval3d
    Initialisation de l'operateur eval_volume .\\
    Cette fonction va allouer l'image des tiquettes qui servira a stocker
    les tiquettes entre l'tape d'tiquetage et celle de rtiquetage.

    L'image des tiquettes est une image de type ima3Du8. Cela signifie que
    les voxels de l'image sont cods dans unsigned long int c'est a dire 8octets
    sur une machine avec une architecture 64bits et 4 octets sur une machine 32bits
    @name eval_volume_init

    @param des  pointeur descripteur
    @param imaIn  image initiale (taille connue)
    @param imres  pointeur sur une structure deja alloue pour l'image des tiquettes
    @author L. Valet
    @version 1 (22.11.06); Include : eval_volume.h
*/
int eval_volume_init(eval_volume_t* des, ima3Du1* imaIn){

  /* Allocation de l'image rsultat dans le descripteur*/
  des->ima_etiq = (ima3Du8 *)malloc(sizeof(ima3Du8));
  des->ima_etiq->dimx = imaIn->dimx;
  des->ima_etiq->dimy = imaIn->dimy;
  des->ima_etiq->dimz = imaIn->dimz;
  des->ima_etiq->lgtete = imaIn->lgtete;
  des->ima_etiq->dept = imaIn->dept;
  allouer_ima3Du8 (des->ima_etiq);

}



/*
  Gestion de l'quivalence entre deux tiquettes (a et b) dans la liste de chainage.
  Principe:
  On recherche l'tiquette minimum entre a et b. C'est cette tiquette qui sera
  utilis pour le pixel courant ainsi que pour matrialiser l'quivalence.
  Puis les deux tiquettes sont traites de la meme facon: si l'quivalence pour 
  l'tiquette a est diffrente du minimum alors sauvegarder l'quivalence existante 
  et ecrire le minimum. Ce traitement est ensuite reappliqu sur l'tiquette de 
  l'quivalence sauvgarde jusqu'a retrouver l'tiquette minimum dans la liste 
  de chainage.
    @param pt_etiq  pointeur sur la liste de chainage
    @param a  premiere etiquette
    @param b  deuxime etiquette


  Le but est de conserver la plus petite des 2 tiquettes comme l'tiquette qui
  reprsentera l'quivalence. Cette tiquette va etre remplacee dans toute la chaine
  descendante (vers la gauche du tableau).

  exemple: 
  indice:   1 2 3 4 5 6 7 8 9
  chainage: 1 2 3 2 5 2 7 8 1
  
  Le rsultat de l'quivalence entre 8 et 6 donnera:
  indice:   1 2 3 4 5 6 7 8 9
  chainage: 1 6 3 2 5 6 7 6 1


*/
unsigned long int eval_equivalence_2_label( unsigned  long int *pt_etiq, 
					    unsigned  long int  a, 
					    unsigned  long int  b){
  
  unsigned long  int minimum;
  unsigned long  int beta;   /*tiquette en cours de traitement*/
  unsigned long  int tmp;    /*sauvegarde des quivalences existants*/
  
  minimum = MIN(a,b);

  beta = a;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  beta = b;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  return( minimum);

}





/*
  Meme traitement que pour l'quivalence de deux tiquettes,
  mais pour trois.
*/
unsigned long  int eval_equivalence_3_label( unsigned long  int *pt_etiq, 
					unsigned long  int a, 
					unsigned long  int b,
					unsigned long  int c){
  
  unsigned long  int minimum;
  unsigned long  int beta;
  unsigned long  int tmp;
  
  minimum = MIN3( a, b, c);

  beta = a;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  beta = b;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  beta = c;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }
  return( minimum);

}





/*
Rappel sur la composition de la variable cas:
 __________________________________________
| 0 | 0 | g-h | a-g | a-h | g-c | h-c | a-c|
--------------------------------------------

Les bits 0  5 sont a 1 si la diffrence est infrieure au seuil


Action a raliser en fonction des diffrents cas possibles: 4 actions possibles
- nouvelle etiquette                   c <-- new_etiq()
- tiquette du voisin sans quivalence c <-- etiq[a]
- equivalence entre deux tiquettes    c <-- equiv(a,g)
- quivalence entre trois tiquettes   c <-- equiv(a,g,h)
Les cas non marqu ci-dessous sont des quivalence entre trois tiquettes.

-- 0--0000 0000---                  c <-- new_etiq()
--------------------
-- 1--0000 0001--a,c--              c <-- etiq[a]
-- 2--0000 0010--h,c--              c <-- etiq[h]
-- 3--0000 0011--h,c-a,c--          c <-- equiv(a,h)
-- 4--0000 0100--g,c--              c <-- etiq[g]
-- 5--0000 0101--g,c-a,c--          c <-- equiv(a,g)
-- 6--0000 0110--g,c-h,c--          c <-- equiv(g,h)
-- 7--0000 0111--g,c-h,c-a,c--
-- 8--0000 1000--a,h--              c <-- new_etiq()
-- 9--0000 1001--a,h-a,c--          c <-- equiv(a,h)
--10--0000 1010--a,h-h,c--          c <-- equiv(a,h)
--------------------
--11--0000 1011--a,h-h,c-a,c--      c <-- equiv(a,h)
--12--0000 1100--a,h-g,c--          c <-- etiq[g]
--13--0000 1101--a,h-g,c-a,c--
--14--0000 1110--a,h-g,c-h,c--
--15--0000 1111--a,h-g,c-h,c-a,c--
--16--0001 0000--a,g--              c <-- new_etiq()
--17--0001 0001--a,g-a,c--          c <-- equiv(a,g)
--18--0001 0010--a,g-h,c--          c <-- etiq[h]
--19--0001 0011--a,g-h,c-a,c--
--20--0001 0100--a,g-g,c--          c <-- equiv(a,g)
--------------------
--21--0001 0101--a,g-g,c-a,c--      c <-- equiv(a,g)
--22--0001 0110--a,g-g,c-h,c--
--23--0001 0111--a,g-g,c-h,c-a,c--
--24--0001 1000--a,g-a,h--          c <-- new_etiq()
--25--0001 1001--a,g-a,h-a,c--
--26--0001 1010--a,g-a,h-h,c--
--27--0001 1011--a,g-a,h-h,c-a,c--
--28--0001 1100--a,g-a,h-g,c--
--29--0001 1101--a,g-a,h-g,c-a,c--
--30--0001 1110--a,g-a,h-g,c-h,c--
--------------------
--31--0001 1111--a,g-a,h-g,c-h,c-a,c--
--32--0010 0000--g,h--              c <-- new_etiq()
--33--0010 0001--g,h-a,c--          c <-- etiq[a]
--34--0010 0010--g,h-h,c--          c <-- equiv(g,h)
--35--0010 0011--g,h-h,c-a,c--
--36--0010 0100--g,h-g,c--          c <-- equiv(g,h)
--37--0010 0101--g,h-g,c-a,c--
--38--0010 0110--g,h-g,c-h,c--      c <-- equiv(g,h)
--39--0010 0111--g,h-g,c-h,c-a,c--
--40--0010 1000--g,h-a,h--          c <-- new_etiq()
--------------------
--41--0010 1001--g,h-a,h-a,c--
--42--0010 1010--g,h-a,h-h,c--
--43--0010 1011--g,h-a,h-h,c-a,c--
--44--0010 1100--g,h-a,h-g,c--
--45--0010 1101--g,h-a,h-g,c-a,c--
--46--0010 1110--g,h-a,h-g,c-h,c--
--47--0010 1111--g,h-a,h-g,c-h,c-a,c--
--48--0011 0000--g,h-a,g--                c <-- new_etiq()
--49--0011 0001--g,h-a,g-a,c--
--50--0011 0010--g,h-a,g-h,c--
--------------------
--51--0011 0011--g,h-a,g-h,c-a,c--
--52--0011 0100--g,h-a,g-g,c--
--53--0011 0101--g,h-a,g-g,c-a,c--
--54--0011 0110--g,h-a,g-g,c-h,c--
--55--0011 0111--g,h-a,g-g,c-h,c-a,c--
--56--0011 1000--g,h-a,g-a,h--            c <-- new_etiq()
--57--0011 1001--g,h-a,g-a,h-a,c--
--58--0011 1010--g,h-a,g-a,h-h,c--
--59--0011 1011--g,h-a,g-a,h-h,c-a,c--
--60--0011 1100--g,h-a,g-a,h-g,c--
--------------------
--61--0011 1101--g,h-a,g-a,h-g,c-a,c--
--62--0011 1110--g,h-a,g-a,h-g,c-h,c--
--63--0011 1111--g,h-a,g-a,h-g,c-h,c-a,c--

*/

void eval_label_management( unsigned char cas,           /*le cas a traiter*/
		       unsigned long  int haut,     /*l'tiquette du haut si elle existe*/
		       unsigned long  int gauche,   /*l'tiquette de la gauche si elle existe*/
		       unsigned long  int avant,    /*l'tiquette de la section d'avant si elle existe*/
		       unsigned long  int *courant, /*pointeur sur la case contenant l'tiquette 
						      courante a dterminer (dans l'image des tiquette)*/
		       unsigned long  int *pt_etiq, /*pointeur sur le tableau permettant le 
						      chainage des tiquettes*/
		       unsigned long  int *nb_etiq  /*nombre d'tiquettes deja utilises*/
	    ) 
{
  
  
  if(cas>=64) printf("PROBLEME\n");

  switch (cas) {

  case 0: 
  case 8:
  case 16:
  case 24:
  case 32:
  case 40:
  case 48:
  case 56:
    /* on attribue une nouvelle etiquette*/
    {
      *nb_etiq +=1;
      if(*nb_etiq>ETMAX-1) *nb_etiq=ETMAX-1;
      *courant = *nb_etiq;
      return;
    }

  case 1:
  case 33:
    /*  propagation de l'etiquette avant*/
    {
      *courant = pt_etiq[avant];
      return;
    }

  case 2: 
  case 18:
    /* propagation de l'etiquette du haut*/
    {
      *courant = pt_etiq[haut];
      return;
    }

  case 4 : 
  case 12:
    /* propagation de l'etiquette de gauche*/
    {
      *courant = pt_etiq[gauche];
      return;
    }
    
  case 3:
  case 9:
  case 10:
  case 11:
    /*equivalence entre avant et haut*/
    {
      *courant = eval_equivalence_2_label( pt_etiq, avant, haut);
      return;
    }


  case 5:
  case 17:
  case 20:
  case 21:
    /*equivalence entre avant et gauche*/
    {
      *courant = eval_equivalence_2_label(pt_etiq, avant, gauche);
      return;
    }

  case 6:
  case 34:
  case 36:
  case 38:
    /*equivalence entre gauche et haut*/
    {
      *courant = eval_equivalence_2_label(pt_etiq, haut, gauche);
      return;
    }

  default:
    /*quivalence entre gauche, haut et avant */
    {
      *courant = eval_equivalence_3_label(pt_etiq, haut, gauche, avant);  

      return;
    }    
  }/*fin du switch*/

}



/*
  Fonction pour la mise  jour des labels  l'aide du tableau de chainage
*/

void 
eval_reetiquetage( ima3Du1* ima_ori,
	      ima3Du8* ima_etiq,   /* adresse de l'image destination (etiquettes) */
	      unsigned long  int *pt_etiq,
	      unsigned long  int *pt_nb_etiq,     /* pointeur sur le nombre de label utilis moins un*/
	      unsigned long  int *pt_nb_real_etiq)/* Nombre ral de label apres mise a jour  */
{
  register unsigned int x,y,z;
  unsigned long int i;
  //unsigned long  int label_effectif_max =0;
  //unsigned long  int nb_label_effectif =0;
  unsigned long  int tmp;
  //unsigned long int *pt_nbel; 
  //float *pt_moy;
  unsigned long int etiq_reelle; /*compteur pour transformer les etiquettes
				   issues de la premiere analyse en tiquettes conscutives*/


  /*--Mise a jour du tableau de chainage--
    On verifie que tous les indices du tableau contiennent un numro de chainage qui
    est directe, c'est a dire vers un indice sans chainage. Dans le cas contraire, 
    on parcourt tous le chainage afin d'en trouver le bout que l'on place dans l'indice
    en cours d'etude.
    Ces cas peuvent arriver car la gestion des quivalences ne s'intresse qu'au chainage
    lis aux tiquettes pour lesquelles on veut mmoriser une quivalence.
    
    Reprenons l'exemple utilise prcdemment:
    exemple: 
    indice:   1 2 3 4 5 6 7 8 9
    chainage: 1 2 3 2 5 2 7 8 1
  
    Le rsultat de l'quivalence entre 8 et 6 donnera avait donn:
    indice:   1 2 3 4 5 6 7 8 9
    chainage: 1 6 3 2 5 6 7 6 1

    On voit que l'etiquette 4 est quivalente a 2 mais l'tiquette 2 a t mmorise comme
    quivalente a 6 lors de la gestion de l'quivalence entre 6 et 8.
    Il faut maintenant mmoriser que l'tiquette 4 est chaine avec l'tiquette 6:

    Rsultat:
    indice:   1 2 3 4 5 6 7 8 9
    chainage: 1 6 3 6 5 6 7 6 1
  */
  for (i = 0; i <= (*pt_nb_etiq); i++) {        
    tmp = i;
    while (pt_etiq[tmp]!=tmp){
      tmp = pt_etiq[tmp];
    }
    pt_etiq[i] = tmp;
  }

  /*--comptage du nombre de label effectif et re-numrotation des classes de sortie --
    Afin d'avoir une numrotation croissante des labels de sortie et de connaitre le nombre
    exacte d'ojets, il est ncesaire de les renumroter tout en conservant les correspondance
    dans le tableau de chainage.

    Le nombre d'objet est gale au nombre d'tiquette sans chainage (case du tableau
    ou indice=chainage)

    Dans un premier temps la renumrotation ne peut se faire que pour les indices sans chainages
    et pour les indices avec un chainage infrieur a l'indice. Donc pour lequel, on a deja gr
    la re-numration.
    
    Pour les indices ayant un numro de chainage suprieur  l'indice il faudra refaire un passage
    apres.
    
    exemple:
    indice:   1 2 3 4 5 6 7 8 9
    chainage: 1 6 3 6 5 6 7 6 1
    
    resultat du 1er passage:
    indice:   1 2 3 4 5 6 7 8 9
    chainage: 1 6 2 6 3 4 5 4 1
                ?   ? 
  */ 
  etiq_reelle = 0;
  for (i = 0; i <= (*pt_nb_etiq); i++) {
    if(pt_etiq[i] == i){
      pt_etiq[i] = etiq_reelle;
      etiq_reelle++;
    }else if(pt_etiq[i] < i){
      pt_etiq[i] = pt_etiq[pt_etiq[i]];
    }
  }

  /*dernier passage pour grer les tiquettes dont le numro de chainage etait suprieur lors
    du 1er passage.

    resultat du 1er passage:
    indice:   1 2 3 4 5 6 7 8 9
    chainage: 1 6 2 6 3 4 5 4 1

    resultat du 2eme passage:
    indice:   1 2 3 4 5 6 7 8 9
    chainage: 1 4 2 4 3 4 5 4 1
  */
  for (i = 0; i <= (*pt_nb_etiq); i++) {
    if( pt_etiq[i]>i )
      pt_etiq[i] = pt_etiq[pt_etiq[i]];
  }



  /*Mise  jour des labels dans l'image des tiquettes*/ 
  for( z=0; z<ima_etiq->dimz; z++){
    for( x=0; x<ima_etiq->dimx; x++){
      for( y=0; y<ima_etiq->dimy; y++){
	ima_etiq->data[z][y][x] = pt_etiq[ ima_etiq->data[z][y][x] ];
      }
    }
  }

  /*comme on incrmente apres utilisation le compteur du nombre des tiquettes relles, il faut
    enlever 1 pour avoir le nombre d'objets rel*/
  *pt_nb_real_etiq = etiq_reelle-1;

}













/*
  Fonction pour la recherche des tiquettes et la gestion du chainage.
  Cette fonction suppose etre applique sur une image du type "classification"
  C'est  dire que tous niveaux de gris diffrents correspondent  objet
  diffrents

*/
void 
eval_etiquetage_strict(ima3Du1* ima_in,     /* adr de l'image origine (initiale ou segmentee) */
	   ima3Du8* ima_etiq,   /* adresse de l'image destination (etiquettes) */
	   unsigned char classe,      /* numro de la classe  tiqueter (valeur ds ima_in) */
	   unsigned long int *pt_nb_etiq,     /* nombre de label utilis*/
	   unsigned long int *nb_real_etiq)   /* nombre de label apres simplification */
{
  int cas;  
  register  int x,y,z;
  unsigned long int i;
  unsigned long  int *pt_etiq;		/* pointeur tableau d'etiquettes */  
  unsigned long  int nb_etiq;          /* compteur des etiquettes */ 
 

  //printf("size au float %d\n", sizeof(double));
  /*Allocation du tableau de chainage*/
  pt_etiq = (unsigned long  int *)calloc(ETMAX, sizeof(unsigned long  int)); 
  if(pt_etiq == NULL){
    printf("Probleme d'allocation mmoire (pt_etiq)\n");
    exit(1);
  }


  /*initialisation du tableau*/
  for( i=0; i<ETMAX ; i++)
    pt_etiq[i] = i; 
  /*initialisation de nombre d'tiquette dj utilise*/
  nb_etiq = 0;
  

  /*traitement du premier point du bloc*/
  if( ima_in->data[0][0][0] != classe){
    ima_etiq->data[0][0][0] = 0;
  }
  else{
    /*attribution d'une nouvelle tiquette*/
    nb_etiq +=1;
    if(nb_etiq>ETMAX-1) nb_etiq=ETMAX-1;
    ima_etiq->data[0][0][0] = nb_etiq;
  }

  /*traitement de la premiere ligne du bloc*/
  x = 0;
  z = 0;
  for( y=1; y<ima_in->dimy; y++){
    /*Si le voxel n'appartient pas  la classe alors mettre
      directement l'tiquette 0 du fond quelque soit sa classe
      de dpart. Puis passer directement au point suivant*/
    if(ima_in->data[z][y][x] != classe){
      ima_etiq->data[z][y][x] = 0;
      continue;
    }
    /*Etiquetage du point*/
    cas = 0;
    if( ima_in->data[z][y][x] == ima_in->data[z][y-1][x] ) 
      cas += 4; /*g et c similaire*/
    eval_label_management( cas,
		      0, /*haut*/
		      ima_etiq->data[z][y-1][x],/*gauche*/ 
		      0,/*avant*/
		      &ima_etiq->data[z][y][x], /*courant*/
		      pt_etiq,
		      &nb_etiq
	  );
  }
  //printf("\nFin Traitement premiere ligne\n");
  //for( x=0; x<=nb_etiq; x++)
  //printf("%d --> %d\n", x, pt_etiq[x]);


  /*traitement des autres lignes de la premiere section*/ 
  z = 0;
  for( x=1; x<ima_in->dimx; x++){
    for( y=0; y<ima_in->dimy; y++){

      /*Si le voxel n'appartient pas  la classe alors mettre
	directement l'tiquette 0 du fond quelque soit sa classe
	de dpart. Puis passer directement au point suivant*/
      if(ima_in->data[z][y][x] != classe){
	ima_etiq->data[z][y][x] = 0;
	continue;
      }

      /*Etiquetage du point*/
      cas = 0;

      /*on peut toujours comparer avec le haut*/
      if( ima_in->data[z][y][x] == ima_in->data[z][y][x-1] ) 
	cas += 2; /*h et c similaire*/

      /* En plus, si on n'est pas sur la premiere colonne
	 on peut faire les comparaisons avec la gauche*/
      if( y!=0){ 
	/*comparaison avec celui de gauche*/
	if( ima_in->data[z][y][x] == ima_in->data[z][y-1][x] ) 
	  cas += 4; /*g et c similaire*/
	if( ima_in->data[z][y-1][x] == ima_in->data[z][y][x-1] ) 
	  cas += 32; /*g et h similaire*/	 	
	eval_label_management( cas,
			  ima_etiq->data[z][y][x-1] /*haut*/,
			  ima_etiq->data[z][y-1][x]/*gauche*/, 
			  0, /*avant*/
			  &ima_etiq->data[z][y][x], /*courant*/
			  pt_etiq,
			  &nb_etiq
			  );
      }
      else{
	eval_label_management( cas,
			  ima_etiq->data[z][y][x-1] /*haut*/,
			  0, /*gauche*/ 
			  0, /*avant*/
			  &ima_etiq->data[z][y][x], /*courant*/
			  pt_etiq,
			  &nb_etiq
			  );
      }
    }
  }

  
  /*traitement des autres sections du bloc*/
  for( z=1; z<ima_in->dimz; z++){
    for( x=0; x<ima_in->dimx; x++){
      for( y=0; y<ima_in->dimy; y++){
	
	/*Si le voxel n'appartient pas  la classe alors mettre
	  directement l'tiquette 0 du fond quelque soit sa classe
	  de dpart. Puis passer directement au point suivant*/
	if(ima_in->data[z][y][x] != classe){
	  ima_etiq->data[z][y][x] = 0;
	  continue;
	}
	
	/*Etiquetage du point*/
	cas = 0;
	
	/*comparaison avec le plan d'avant toujours possible*/
	if( ima_in->data[z][y][x] == ima_in->data[z-1][y][x] ) 
	  cas += 1; /*a et c similaire*/
	
	
	/*gestion de la premiere ligne*/
	if(x==0){ 
	  if(y!=0){ /*on ne se trouve pas sur le premier point de la ligne*/
	    if( ima_in->data[z][y][x] == ima_in->data[z][y-1][x] ) 
	      cas += 4; /*g et c similaire*/	    
	    if( ima_in->data[z][y-1][x] == ima_in->data[z-1][y][x] ) 
	      cas += 16; /*g et a similaire*/
	    eval_label_management( cas,
			      0 /*haut*/,
			      ima_etiq->data[z][y-1][x] /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );
	    
	  }
	  else{
	    eval_label_management( cas,
			      0 /*haut*/,
			      0 /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );
	    
	  }
	}
	else{ /*gestion des lignes suivantes*/

	  /*comparaison avec le haut toujours possible*/
	  if(  ima_in->data[z][y][x] == ima_in->data[z][y][x-1] ) 
	    cas += 2; /*h et c similaire*/	    
	  if(  ima_in->data[z][y][x-1] == ima_in->data[z-1][y][x] ) 
	    cas += 8; /*h et a similaire*/	
	    
	  if(y!=0){ /*on ne se trouve pas sur le premier point de la ligne*/
	    if( ima_in->data[z][y][x] == ima_in->data[z][y-1][x] ) 
	      cas += 4; /*g et c similaire*/	    
	    if( ima_in->data[z][y-1][x] == ima_in->data[z-1][y][x] ) 
	      cas += 16; /*g et a similaire*/	    
	    if( ima_in->data[z][y-1][x] ==  ima_in->data[z][y][x-1] ) 
	      cas += 32; /*g et h similaire*/
	    eval_label_management( cas,
			      ima_etiq->data[z][y][x-1] /*haut*/,
			      ima_etiq->data[z][y-1][x] /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );
	  }
	  else{
	    eval_label_management( cas,
			      ima_etiq->data[z][y][x-1] /*haut*/,
			      0 /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );	    
	  }
	} /*fin test sur les lignes*/
	
	
      } /*fin sur y*/
    } /*fin sur x*/
  } /*fin sur z*/
  
  
  printf("nb_etiq = %d (max %d)\n", nb_etiq+1, ETMAX);
  if( nb_etiq+1 == ETMAX) {
    printf("-----DANGER Pas assez d'etiquettes !!!----\n");
  }

  *pt_nb_etiq = nb_etiq+1;

  
  eval_reetiquetage( ima_in,
  		ima_etiq,   /* adresse de l'image destination (etiquettes) */
		pt_etiq,
		&nb_etiq,        /* nombre de label utilis*/
		nb_real_etiq);   /*Nombre ral de label apres mise a jour  */


  /*liberation de la liste de chainage*/
  free( pt_etiq);
}



/* 
   La fonction calcul_vol_objets calcul le volume (nombre de pixel) de chacun
   des objets prsents dans un bloc 3D. Ce bloc doit etre avant tout tiquet avec
   des tiquettes conscutives. 
   L'tiquette 0 reprsente le fond de la piece c'est a dire l'ensemble des points
   qui n'appartiennent pas  la classe tudie.
   Le parametre nb_objet doit contenir le nombre objets connexes prsents dans le bloc
   (sans compter le fond)
   Les rsultats sont sauvegards dans un fichier (paramtre filename) suivant deux
   colonnes: La 1ere contient le numro de l'tiquette de l'objet et la 2eme le nombre 
   de voxel appartenant  l'objet. 
*/

int calcul_vol_objets(eval_volume_t* des, const char *filename, unsigned long int nb_objet){

  unsigned int *vol;
  unsigned int repartition_vol[5]={0,0,0,0,0}; /*pour la rpartition des volumes*/
  unsigned int vol_min, vol_max;
  double vol_moy, vol_ecart_type;
  register unsigned int x, y, z;
  FILE *f;


  vol = calloc( nb_objet+1,sizeof(unsigned int));
  if( vol==NULL) {
    printf("Probleme allocation memoire\n");
    exit(1);
  }

  /*cumul des points*/
  for( z=0; z<des->ima_etiq->dimz; z++)
    for( y=0; y<des->ima_etiq->dimy; y++)
      for( x=0; x<des->ima_etiq->dimx; x++){
	vol[des->ima_etiq->data[z][y][x]]++;
     }
  
  printf("un\n");
  /*sauvegarde dans un fichier*/  
  f=fopen( filename, "w");
  if(f==NULL){
    perror("Dans calcul_vol_objets");
    printf("Erreur ouverture de %s\n", filename);
  }
  vol_min = vol[1];
  vol_max = vol[1];
  vol_moy = 0.0;
  for( x=1; x<=nb_objet; x++){
    fprintf( f, "%d %d\n", x, vol[x] );
    if (vol[x] > vol_max) vol_max = vol[x];
    if (vol[x] < vol_min) vol_min = vol[x];
    vol_moy +=  (double)vol[x]/(double)nb_objet;
    /*mise a jour de la rpartition des volumes*/
    if( vol[x]<10 ) repartition_vol[0]++;
    else if ( vol[x]<100 ) repartition_vol[1]++;
    else if ( vol[x]<1000 ) repartition_vol[2]++;
    else if ( vol[x]<10000 ) repartition_vol[3]++;
    else repartition_vol[4]++;
  }
  //fclose(f);  /*fclose provoque une grave erreur!!! probleme dans la glibc!!!*/
  
  /*calcul de l'ecart type des volumes*/
  vol_ecart_type = 0.0;
  for( x=1; x<=nb_objet; x++){
    vol_ecart_type += pow( (double)vol[x]-vol_moy, 2.0) / (double)nb_objet;
  }
  vol_ecart_type = sqrt( vol_ecart_type);

  /*affichage des rsultats*/
  printf("Volume minimal: %d\n", vol_min);
  printf("Volume maximal: %d\n", vol_max);
  printf("Volume moyen: %lf\n", vol_moy);
  printf("Ecart type: %lf\n", vol_ecart_type);

  printf("Rpartion des volumes:\n");
  printf("0->9:\t %d\n", repartition_vol[0]);
  printf("10->99:\t %d\n", repartition_vol[1]);
  printf("100->999:\t %d\n", repartition_vol[2]);
  printf("1000->9999:\t %d\n", repartition_vol[3]);
  printf("10000-> + :\t %d\n", repartition_vol[4]);


  /*liberation de la memoire*/
  free( vol);

}



/*
  Cette fonction calcul la compacit 3D de chacun des objets.
  La compacit 3D est dfinie par: V/( (4/3) * Pi * (S/ (4*Pi))^(3/2) )

*/

int calcul_attributs_objets(eval_volume_t* des, const char *filename, unsigned long int nb_objet){

  unsigned int *vol, *surf;
  register unsigned int x, y, z;
  register char l, m, n;
  FILE *f;
  booleen env;
  double compacite;
  double comp_moy;

  comp_moy = 0.0;

  vol = calloc( nb_objet+1,sizeof(unsigned int));
  if( vol==NULL) {
    printf("Probleme allocation memoire\n");
    exit(1);
  }

  surf = calloc( nb_objet+1,sizeof(unsigned int));
  if( surf==NULL) {
    printf("Probleme allocation memoire\n");
    exit(1);
  }

  
  /*cumul des points pour le volume et l'enveloppe*/
  printf("Calcul des attributs en cours\n");
  for( z=0; z<des->ima_etiq->dimz; z++)
    for( y=0; y<des->ima_etiq->dimy; y++)
      for( x=0; x<des->ima_etiq->dimx; x++){
	/*mise a jour du volume pour la region*/
	vol[des->ima_etiq->data[z][y][x]]++;
	/*etude de l'enveloppe*/
	env=FAUX;
	if( z==0 ||  /*gestion des bords de l'image*/
	    y==0 || 
	    x==0 ||
	    z==des->ima_etiq->dimz-1 ||
	    y==des->ima_etiq->dimy-1 ||
	    x==des->ima_etiq->dimx-1){ 
	  env=VRAI;
	}else{ /*sinon etude des etiquettes dans le voisinage*/
	  for( l=-1; l<2; l++ )
	    for( m=-1; m<2; m++ )
	      for( n=-1; n<2; n++ ){
		//printf("%d,%d,%d\n,", z+l, y+m, x+n);
		//printf("%d\n",des->ima_etiq->data[z+l][y+m][x+n]);
		if( des->ima_etiq->data[z][y][x] != des->ima_etiq->data[z+l][y+m][x+n]){
		  env = VRAI;
		  continue;
		}
	      }
	}
	/*mise a jour de compteur des voxels appartenant a une enveloppes*/
	if(env==VRAI) surf[des->ima_etiq->data[z][y][x]]++;
     }
  

  /*sauvegarde dans un fichier*/
  printf("Sauvagarde des rsultats dans %s\n",filename );
  f=fopen( filename, "w");
  if(f==NULL){
    perror("Dans calcul_attributs_objets");
    printf("Erreur ouverture de %s\n", filename);
  }

  for( x=1; x<=nb_objet; x++){
    compacite = (double)vol[x] / ( (4.0/3.0) * M_PI * pow( surf[x]/(4.0*M_PI) ,1.5) );
    fprintf( f, "%d %d %d %.4lf\n", x, vol[x], surf[x], compacite );
    comp_moy +=  compacite/(double)nb_objet;
  }

  printf("Compacitee moyenne: %lf\n", comp_moy);

  /*liberation de la memoire*/
  free( vol);
  free( surf);
}






/* *************************  CALCUL  ***************************/
/** \ingroup eval3d
    Calcul de l'operateur eval_volume.\\
    Calcul du resultat de l'operateur eval_volume\\
    www.ulib.org/webRoor/Books/Numerical_Recipes/
    @name median3d_calc  
    @param du pointeur descripteur
    @param im0 image initiale 
    @param imres pointeur image resultat
    @author G. Marinescu
    @version 1 (01.04.03); Include : median3d.h
*/
int eval_volume_calc(eval_volume_t* des, ima3Du1* imaIn){

  unsigned long int nb_objet_reel, nb_etiq_utilise;

  eval_etiquetage_strict(imaIn, des->ima_etiq, des->classe, &nb_etiq_utilise, &nb_objet_reel);
	   
  //printf("nb etiquettes utilisees: %d\n", nb_etiq_utilise );
  printf("Nombre d'objets: %d\n", nb_objet_reel);
  
  //calcul_vol_objets(des, "eval.txt", nb_objet_reel);
  printf("Calcul compacite ...\n");
  calcul_attributs_objets(des, "attr.txt", nb_objet_reel);

}




int eval_volume_ferm(eval_volume_t* des){

  /*libration de l'image des tiquettes*/
  liberer_ima3Du8( des->ima_etiq);
  /*libration de la structure pour stocker ima_etiq*/
  free(des->ima_etiq);


}
