/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \defgroup ogi3d
    operateur gradient organisation : oprateur pour l'valuation des objets contenu dans un volume
    Descripteur : 
    Doc a completer...
    Type entree : ima3du1
    Type sortie : ima3du1 
    @name gradient_organisation
    @author L. Valet
    @version 1 (13.06.07)
    @see 
*/

#include <stdio.h>
#include "gradient_organisation.h"
#include <math.h>

#define MAX(x,y) (((x)>(y))? (x):(y))
#define PI_DOUBLE 6.2832













/******************************************************************/
float CalculAlpha(int top, int nb, float eps)
{
    float alpha,alpha1,alpha2,val,c;
    int i;

alpha1=0; alpha2=10.0;
alpha=5.0;
c=(1.0-exp(-alpha))*(1.0-exp(-alpha))/(1.0+2.0*alpha*exp(-alpha)-exp(-2.0*alpha));
for(i=0;i<nb;i++)
 {
    val=c*(1.0+alpha*top)*exp(-alpha*top)-eps;
    if (val<0)  alpha2=alpha;
    else alpha1=alpha;
    alpha=(alpha1+alpha2)*0.5;
 }
return(alpha);
}





/******************************************************************/
void Equation3D(double *g,double *lambda,double vect[3][3],int flag)
{
    double b,c,d,p,q,delta,teta,module,x,y,z,lambda_temp,val;
    int i;

b= -g[0]-g[3]-g[5];
c= -g[1]*g[1]-g[2]*g[2]-g[4]*g[4]+g[0]*g[3]+g[0]*g[5]+g[3]*g[5];
d= -g[0]*g[3]*g[5]+g[0]*g[4]*g[4]+g[3]*g[2]*g[2]+g[5]*g[1]*g[1]-
	2.0*g[1]*g[2]*g[4];					
p= c-b*b/3.0;
q= d-b*c/3.0+2.0*b*b*b/27.0;
delta=q*q+4*p*p*p/27;
if(delta > 0) { 
	teta = 0;
	module = 1.0 ;
	lambda[0] = lambda[1] = lambda[2] = 0.0;
} else {
	if (delta==0.0 && q==0.0) teta = 0.0;
	else teta = atan2(sqrt(-delta), -q);
	module = 2.0*sqrt(-p/3.0);
	lambda[0] = module*cos(teta/3.0)-b/3.0;	
	lambda[1] = module*cos((teta+PI_DOUBLE)/3.0)-b/3.0;	
	lambda[2] = module*cos((teta+2.0*PI_DOUBLE)/3.0)-b/3.0;
}
if ( fabs(lambda[1]) > fabs(lambda[0]) ) 
 { 
    lambda_temp = lambda[0]; 
    lambda[0] = lambda[1]; 
    lambda[1] = lambda_temp; 
 }
if ( fabs(lambda[2]) > fabs(lambda[0]) ) 
 {  
    lambda_temp = lambda[0]; 
    lambda[0] = lambda[2]; 
    lambda[2] = lambda_temp; 
 }
if ( fabs(lambda[2]) > fabs(lambda[1]) ) 
 { 
    lambda_temp = lambda[1]; 
    lambda[1] = lambda[2]; 
    lambda[2] = lambda_temp; 
 }
x = y = z = 0.0;
for(i=0;i<3;i++)
 {	
    val=lambda[i];

    module=(g[3]-val)*(g[5]-val)-g[4]*g[4];
    if ( fabs(module) > 0.01 )
    {
        x=1.0;
        y=( -(g[5]-val)*g[1]+g[4]*g[2] ) / module;
        z=( -(g[3]-val)*g[2]+g[4]*g[1] ) / module;
    }
    else
    {
    	module=(g[0]-val)*(g[3]-val)-g[1]*g[1];
    	if ( fabs(module) > 0.01 )
    	{
    	    x=( -(g[3]-val)*g[2]+g[4]*g[1] ) / module;
    	    y=( -(g[0]-val)*g[4]+g[1]*g[2] ) / module;
	    z=1.0;
    	}
    	else
    	{
	    module = (g[0]-val)*(g[5]-val)-g[2]*g[2];
    	    if ( module > 0.01 )
    	    {
        	y = 1.0;
        	x = ((val-g[5])*g[1]+g[2]*g[4])/module;
        	z = ((val-g[0])*g[4]+g[1]*g[2])/module;
    	    }
	    else { x=0; y=0; z=0; }
    	}
    }
    module = sqrt(x*x+y*y+z*z);
    if ( fabs(module) > 0.01 )
    {
    	vect[i][0]=x/module;
    	vect[i][1]=y/module;
    	vect[i][2]=z/module;
    }
    else
    {
        vect[i][0]=1.0;
        vect[i][1]=0.0;
        vect[i][2]=0.0;
    }
 }
if (flag==1) 
 {
    lambda[0]=fabs(b);
    lambda[1]=fabs(c);
    lambda[2]=fabs(d);
 }
}



/******************************************************************/
void Acp3D_NS(short **gradx, short **grady, short **gradz,
                        int largeur, int hauteur,
                        float *acp_x, float *acp_y, float *acp_z,
                        float *conf1, float *conf2, float *conf3,
                        int tax, int tay, int taz, int flag)
{
    long k, k_inf, k_sup, l, n, p, m, limx = tax/2, limy=tay/2;
    long add, add_sup, add_inf, taille = largeur*hauteur;
    double * somme_colonne_xx, * somme_colonne_yy, * somme_colonne_zz,
                 * somme_colonne_xy, * somme_colonne_xz, * somme_colonne_yz;
    double somme_xx, somme_yy, somme_zz, somme_xy, somme_xz, somme_yz,
                gx, gy, gz, gx_sup, gx_inf, gy_sup, gy_inf, gz_sup, gz_inf;
    double tf;
    double g[6], vect[3][3], lambda[3] , val;

for (add=0; add<taille; add++)
{ acp_x[add] = acp_y[add] = acp_z[add] = (float)0.0; }
somme_colonne_xx = (double *) malloc (largeur*sizeof(double));
somme_colonne_yy = (double *) malloc (largeur*sizeof(double));
somme_colonne_zz = (double *) malloc (largeur*sizeof(double));
somme_colonne_xy = (double *) malloc (largeur*sizeof(double));
somme_colonne_xz = (double *) malloc (largeur*sizeof(double));
somme_colonne_yz = (double *) malloc (largeur*sizeof(double));
tf=(double)tax*tay*taz;
for (k=0; k<largeur; k++)
 {
    somme_colonne_xx[k] = somme_colonne_yy[k] = somme_colonne_zz[k]
                        = somme_colonne_xy[k] = somme_colonne_xz[k]
                        = somme_colonne_yz[k] = 0.0;
    for (l=0; l<tay; l++)
    {
        add = l*largeur+k;
        for (m=0; m<taz; m++)
        {
            gx=(double)gradx[m][add];
            gy=(double)grady[m][add];
            gz=(double)gradz[m][add];
            somme_colonne_xx[k] += gx*gx;
            somme_colonne_yy[k] += gy*gy;
            somme_colonne_zz[k] += gz*gz;
            somme_colonne_xy[k] += gx*gy;
            somme_colonne_xz[k] += gx*gz;
            somme_colonne_yz[k] += gy*gz;
        }
    }
 }
for (p=limy; p<hauteur-limy; p++)
 {
    somme_xx = somme_yy = somme_zz = somme_xy = somme_xz = somme_yz = 0.0;
    for (k=0; k<tax; k++)
    {
        somme_xx += somme_colonne_xx[k];
        somme_yy += somme_colonne_yy[k];
        somme_zz += somme_colonne_zz[k];
        somme_xy += somme_colonne_xy[k];
        somme_xz += somme_colonne_xz[k];
        somme_yz += somme_colonne_yz[k];
    }
    for (n=limx; n<largeur-limx; n++)
    {
        add = p*largeur+n;
        g[0] = somme_xx/tf;
        g[1] = somme_xy/tf;
        g[2] = somme_xz/tf;
        g[3] = somme_yy/tf;
        g[4] = somme_yz/tf;
        g[5] = somme_zz/tf;
        Equation3D(g,lambda,vect,flag);
        acp_x[add] = (float)vect[0][0];
        acp_y[add] = (float)vect[0][1];
        acp_z[add] = (float)vect[0][2];
        if (acp_x[add]<0)
        {
            acp_x[add] = -acp_x[add];
            acp_y[add] = -acp_y[add];
            acp_z[add] = -acp_z[add];
        }
        conf1[add] = (float)(lambda[0]);
        conf2[add] = (float)(lambda[1]);
        conf3[add] = (float)(lambda[2]);

        if ( n < (largeur-limx-1) )
        {
            k_sup = n+limx+1; k_inf = n-limx;
            somme_xx += somme_colonne_xx[k_sup]-somme_colonne_xx[k_inf];
            somme_yy += somme_colonne_yy[k_sup]-somme_colonne_yy[k_inf];
            somme_zz += somme_colonne_zz[k_sup]-somme_colonne_zz[k_inf];
            somme_xy += somme_colonne_xy[k_sup]-somme_colonne_xy[k_inf];
            somme_xz += somme_colonne_xz[k_sup]-somme_colonne_xz[k_inf];
            somme_yz += somme_colonne_yz[k_sup]-somme_colonne_yz[k_inf];

        }
    }
    if ( p != (hauteur-limy-1) )
        for (k=0; k<largeur; k++)
        {
            add_sup = (p+limy+1)*largeur+k;
            add_inf = (p-limy)*largeur+k;
            for (m=0; m<taz; m++)
            {
                    gx_sup = (double)gradx[m][add_sup];
                    gx_inf = (double)gradx[m][add_inf];
                    gy_sup = (double)grady[m][add_sup];
                    gy_inf = (double)grady[m][add_inf];
                    gz_sup = (double)gradz[m][add_sup];
                    gz_inf = (double)gradz[m][add_inf];
                    somme_colonne_xx[k] += gx_sup*gx_sup-gx_inf*gx_inf;
                    somme_colonne_yy[k] += gy_sup*gy_sup-gy_inf*gy_inf;
                    somme_colonne_zz[k] += gz_sup*gz_sup-gz_inf*gz_inf;
                    somme_colonne_xy[k] += gx_sup*gy_sup-gx_inf*gy_inf;
                    somme_colonne_xz[k] += gx_sup*gz_sup-gx_inf*gz_inf;
                    somme_colonne_yz[k] += gy_sup*gz_sup-gy_inf*gz_inf;
            }
        }
 }
free(somme_colonne_xx); free(somme_colonne_xy); free(somme_colonne_yy);
free(somme_colonne_xz); free(somme_colonne_yz); free(somme_colonne_zz);
}



/********************************************************/
void LissageDeriche (float alpha,float *ima,int largeur,int hauteur)
{
    float  *imatmp;
    double *vectima,*vectaller,*vectretour,*tmpvect;
    float *tmpdeb,*tmpfin,*tmp;
    int m, n;
    double a1, a2,a3,a4, b, b1, b2;
    long adresse, add, taille;

taille=(long)largeur*(long)hauteur;
imatmp=(float  *)calloc(taille,sizeof(float));

b=(1-exp(-alpha))*(1-exp(-alpha))/(1+2*alpha*exp(-alpha)-exp(-2*alpha));
a1=b;
a2=b*exp(-alpha)*(alpha-1);
a3=b*exp(-alpha)*(alpha+1);
a4=-b*exp(-2*alpha);
b1=2*exp(-alpha);
b2=-exp(-2*alpha);
/********************************************************/
/************** Filtrage PB ligne************************/
/********************************************************/
vectima=(double *)calloc(largeur,sizeof(double));
vectaller=(double *)calloc(largeur,sizeof(double));
vectretour=(double *)calloc(largeur,sizeof(double));
for (n=0; n<hauteur; n++)
 {
    adresse=(long)n*largeur;
    tmpdeb=ima+adresse;
    tmpfin=tmpdeb+largeur;
    tmpvect=vectima;
    for(tmp=tmpdeb;tmp<tmpfin;*(tmpvect++)=(double)( *(tmp++)) );
	/************************************************/
	/********** Filtrage PB ligne aller**************/
	/************************************************/
    vectaller[0]=(a1+a2)/(1-b1-b2)*vectima[0];
    vectaller[1]=a1*vectima[1]+a2*vectima[0]+
		(b1+b2)*vectaller[0];
    for (add=2; add<largeur; add++)
	vectaller[add]=a1*vectima[add]+a2*vectima[add-1]
			+b1*vectaller[add-1]+b2*vectaller[add-2];

	/************************************************/
	/********** Filtrage PB ligne retour*************/
	/************************************************/
    vectretour[largeur-1]=(a3+a4)/(1-b1-b2)*vectima[largeur-1];
    vectretour[largeur-2]=(a3+a4)*vectima[largeur-1] +
			(b1+b2)*vectretour[largeur-1];
    for (add=largeur-3; add>-1; add--)
	vectretour[add]=a3*vectima[add+1]+a4*vectima[add+2]
			+b1*vectretour[add+1]+b2*vectretour[add+2];

	/************************************************/
	/********** Filtrage PB ligne********************/
	/************************************************/
    for (add=0; add<largeur; add++)
	vectima[add]=vectaller[add]+vectretour[add];

    tmpdeb=imatmp+adresse;
    tmpfin=tmpdeb+largeur;
    tmpvect=vectima;
    for(tmp=tmpdeb;tmp<tmpfin;*(tmp++)=(float)( *(tmpvect++)) );
 }
free(vectima); free(vectaller); free(vectretour);

/********************************************************/
/************** Filtrage PB colonne**********************/
/********************************************************/
vectima=(double *)calloc(hauteur,sizeof(double));
vectaller=(double *)calloc(hauteur,sizeof(double));
vectretour=(double *)calloc(hauteur,sizeof(double));
for (m=0; m<largeur; m++)
 {
    adresse=(long)m;
    tmpdeb=imatmp+adresse;
    tmpfin=tmpdeb+(long)largeur*(hauteur-1);
    tmpvect=vectima;
    for(tmp=tmpdeb;tmp<=tmpfin;tmp+=largeur)
	*(tmpvect++)=(double)( *tmp );
	/************************************************/
	/********** Filtrage PB colonne aller************/
	/************************************************/
    vectaller[0]=(a1+a2)/(1-b1-b2)*vectima[0];
    vectaller[1]=a1*vectima[1]+a2*vectima[0]+
		(b1+b2)*vectaller[0];
    for (add=2; add<hauteur; add++)
	vectaller[add]=a1*vectima[add]+a2*vectima[add-1]
			+b1*vectaller[add-1]+b2*vectaller[add-2];

	/************************************************/
	/********** Filtrage PB colonne retour***********/
	/************************************************/
    vectretour[hauteur-1]=(a3+a4)/(1-b1-b2)*vectima[hauteur-1];
    vectretour[hauteur-2]=(a3+a4)*vectima[hauteur-1] +
			(b1+b2)*vectretour[hauteur-1];
    for (add=hauteur-3; add>-1; add--)
	vectretour[add]=a3*vectima[add+1]+a4*vectima[add+2]
			+b1*vectretour[add+1]+b2*vectretour[add+2];

	/************************************************/
	/********** Filtrage PB colonne******************/
	/************************************************/
    for (add=0; add<hauteur; add++)
	vectima[add]=vectaller[add]+vectretour[add];

    tmpdeb=ima+adresse;
    tmpfin=tmpdeb+(long)largeur*(hauteur-1);
    tmpvect=vectima;
    for(tmp=tmpdeb;tmp<=tmpfin;tmp+=largeur)
	*tmp=(float)( *(tmpvect++) );
 }
free(vectima); free(vectaller); free(vectretour);
free(imatmp);
}









void gradient3D(unsigned char ***ima,short *gradx,short *grady,short *gradz,int dimx,int dimy,float alpha,int n)
{
    long add,taille;
    int i,j,k;
    float *tmp1, *tmp2, val;
    double *a, *b, soma, somb;
    unsigned int x,y; /*ajout lionel*/

a=(double *)calloc((n+1),sizeof(double));
b=(double *)calloc((n+1),sizeof(double));
somb=0.0;
soma=0.0;
for(i=0;i<=n;i++)
 {
    b[i]=(1.0+alpha*i)*exp(-alpha*i);
    a[i]=(double)i*exp(-alpha*i);
    if (i==0) { soma+=a[i]; somb+=b[i]; }
    else { soma+=2.0*a[i]; somb+=2*b[i]; }
 }
for(i=0;i<=n;i++)
 {
    a[i] /= soma;
    b[i] /= somb;
 }
taille=(long)dimy*dimx;
tmp1=(float *)calloc(taille,sizeof(float));
tmp2=(float *)calloc(taille,sizeof(float));

/****************** GRADIENT X *********************/

//for (add=0;add<taille;add++)
for ( y=0; y<dimy; y++)
  for (x=0; x<dimx; x++)
 {
   //val=b[0]*(float)ima[n][add];
    val=b[0]*(float)ima[n][y][x];
    for(k=1;k<=n;k++)
      //val += b[k]*((float)ima[n-k][add]+ima[n+k][add]);
	val += b[k]*((float)ima[n-k][y][x]+ima[n+k][y][x]);
    //tmp1[add]=val;
    tmp1[y*dimx+x]=val;
 }
for (i=0;i<dimy;i++)
    for(j=n;j<dimx-n;j++)
    {
        add=i*dimx+j;
	val=0.0;
	for (k=1;k<=n;k++)
	    val += a[k]*(tmp1[add+k]-tmp1[add-k]);
        tmp2[add]=val;
    }
for (i=n;i<dimy-n;i++)
    for(j=0;j<dimx;j++)
    {
        add=i*dimx+j;
	val=b[0]*tmp2[add];
	for (k=1;k<=n;k++)
            val += b[k]*(tmp2[add-k*dimx]+tmp2[add+k*dimx]);
        gradx[add]=(short)val;
    }
for (add=0;add<taille;add++) tmp2[add]=0;

/****************** GRADIENT Y *********************/

for (i=n;i<dimy-n;i++)
    for(j=0;j<dimx;j++)
    {
        add=i*dimx+j;
        val=0.0;
        for (k=1;k<=n;k++)
            val += a[k]*(tmp1[add+k*dimx]-tmp1[add-k*dimx]);
        tmp2[add]=val;
    }
for (i=0;i<dimy;i++)
    for(j=n;j<dimx-n;j++)
    {
        add=i*dimx+j;
        val=b[0]*tmp2[add];
        for (k=1;k<=n;k++)
            val += b[k]*(tmp2[add-k]+tmp2[add+k]);
        grady[add]=(short)val;
    }
for (add=0;add<taille;add++) tmp2[add]=0;

/****************** GRADIENT Z *********************/

//for (add=0;add<taille;add++)
for ( y=0; y<dimy; y++)
  for (x=0; x<dimx; x++)
 {
    val = 0.0;
    for(k=1;k<=n;k++)
      //val += a[k]*((float)ima[n+k][add]-ima[n-k][add]);
        val += a[k]*((float)ima[n+k][y][x]-ima[n-k][y][x]);
    //tmp1[add]=val;
    tmp1[y*dimx+x]=val;
 }
for (i=0;i<dimy;i++)
    for(j=n;j<dimx-n;j++)
    {
        add=i*dimx+j;
        val=b[0]*tmp1[add];
        for (k=1;k<=n;k++)
            val += b[k]*(tmp1[add-k]+tmp1[add+k]);
        tmp2[add]=val;
    }
for (i=n;i<dimy-n;i++)
    for(j=0;j<dimx;j++)
    {
        add=i*dimx+j;
        val=b[0]*tmp2[add];
        for (k=1;k<=n;k++)
            val += b[k]*(tmp2[add-k*dimx]+tmp2[add+k*dimx]);
        gradz[add]=(short)val;
    }
free(tmp1); free(tmp2);
free(a); free(b);
}













/** \ingroup ogi3d
    lecture des parametres de l'operateur gradient_organisation. \\
    @name gradient_organisation_lect
    @param des  pointeur vers le descripteur de cet oprateur  
    @param ptp pointeur sur parametre courant deja alloue,     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author L.Valet
    @version 1 (22.11.06); Include : eval_volume.h
*/
param* gradient_organisation_lect(ogi_t* des, param* ptp, char* debq){
  char question[500];

  sprintf(question, "Alpha (filtre de Deriche):", debq); 
  lec_param(question, ptp);
  des->alpha = atof(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "Taille ACP x:", debq); 
  lec_param(question, ptp);
  des->ax = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "Taille ACP y:", debq); 
  lec_param(question, ptp);
  des->ay = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "Taille ACP z:", debq); 
  lec_param(question, ptp);
  des->az = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "Taille gradient:", debq); 
  lec_param(question, ptp);
  des->tgrad = atoi(ptp->rep);
  ptp = ptp->next;

  printf("\nListe des attributs");
  printf("\n\t         1 : v1");
  printf("\n\t         2 : v2");
  printf("\n\t         3 : v3");
  printf("\n\t         4 : v1+v2+v3");
  printf("\n\t         5 : v2-v3");
  printf("\n\t         6 : (v2-v3)/(v1+v2+v3)");
  printf("\n\t         7 : v2+v3");
  printf("\n\t         8 : (v2+v3)/(v1+v2+v3)");
  printf("\n\t         9 : erreur de flux");
  printf("\n\t        10 : v3/(v1+v2+v3)");
  printf("\n\t        11 : v2*v3");
  printf("\n\t        12 : (v2*v3)/(v1+v2+v3)'2");
  printf("\n\t        13 : v1*v2*v3");
  printf("\n\t        14 : (v1*v2*v3)/(v1+v2+v3)'3");
  printf("\n\t        15 : (v2+v3)'n/(v1+v2+v3)'p");
  printf("\n\t        16 : v2/(v1+v3)");
  printf("\n\t        17 : (v2-v3)/(v2+v3)");
  

  sprintf(question, "Type d'attribut:", debq); 
  lec_param(question, ptp);
  des->type = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "Coefficient de normalisation:", debq); 
  lec_param(question, ptp);
  des->normalisation = atof(ptp->rep);
  ptp = ptp->next;

  return(ptp);
}







int gradient_organisation_init(ogi_t* des, ima3Du1 * ima){

  int taille = ima->dimx*ima->dimy; 
  int i;
  
  /*allocation mmoire pour le calcul des gradients*/
  des->gradx=(short **)calloc(des->az,sizeof(short *));
  des->grady=(short **)calloc(des->az,sizeof(short *));
  des->gradz=(short **)calloc(des->az,sizeof(short *));
  for(i=0;i<des->az;i++)
    {
      des->gradx[i]=(short *)calloc(taille,sizeof(short));
      des->grady[i]=(short *)calloc(taille,sizeof(short));
      des->gradz[i]=(short *)calloc(taille,sizeof(short));
    }

  /*Allocation mmoire pour le calcul de l'ACP*/
  des->acp_x=(float *)calloc(taille,sizeof(float));
  des->acp_y=(float *)calloc(taille,sizeof(float));
  des->acp_z=(float *)calloc(taille,sizeof(float));
  des->conf1=(float *)calloc(taille,sizeof(float));
  des->conf2=(float *)calloc(taille,sizeof(float));
  des->conf3=(float *)calloc(taille,sizeof(float));
  

  des->alp=CalculAlpha(des->tgrad,50,0.02);


}







int gradient_organisation_calc(ogi_t* des,  ima3Du1 * imaIn,  ima3Du1 * imaOut){

  int i;
  int num;
  float conf[20];
  int add;
  int x, y;

  short **gradx = des->gradx;
  short **grady = des->grady;
  short **gradz = des->gradz;
  float *acp_x  = des->acp_x;
  float *acp_y  = des->acp_y;
  float *acp_z  = des->acp_z;
  float *conf1  = des->conf1;  
  float *conf2  = des->conf2;
  float *conf3  = des->conf3;

  short *tmp_s1, *tmp_s2, *tmp_s3;
  float v1, v2, v3, val;

  /*calcul des gradients sur les AZ premiere sections*/
  gradient3D( imaIn->data, 
	      gradx[0], grady[0], gradz[0],
	      imaIn->dimx, imaIn->dimy,
	      des->alp, des->tgrad/2);
  for(i=1;i<des->az;i++){
    gradient3D( imaIn->data, 
		gradx[i], grady[i], gradz[i],
		imaIn->dimx, imaIn->dimy,
		des->alp, 2);
    lect_volume_u1_sequence_next(imaIn);
  }

  /*calcul de l'ACP sur toutes les sections*/  
  for( num=0; num<imaIn->dimz; num++){
    printf("Section %d/%d...\n",num+1,imaIn->dimz);
    Acp3D_NS( gradx, grady, gradz,
	      imaIn->dimx,imaIn->dimy,
	      acp_x, acp_y, acp_z,
	      conf1,conf2,conf3,des->ax,des->ay,des->az,0);

    if (des->alpha!=0.0)
     {
        LissageDeriche( des->alpha, conf1, imaIn->dimx, imaIn->dimy);
        LissageDeriche( des->alpha, conf2, imaIn->dimx, imaIn->dimy);
        LissageDeriche( des->alpha, conf3, imaIn->dimx, imaIn->dimy);
     }

    for( add=0; add<(imaIn->dimx*imaIn->dimy); add++) {
	v1 = conf1[add];
	v2 = conf2[add];
	v3 = conf3[add];
	
	conf[1]=v1;
	conf[2]=v2;
	conf[3]=v3;
	conf[4]=v1+v2+v3;
	conf[5]=v2-v3;
	conf[6] = (conf[4]>0.01) ? (conf[2]-conf[3])/conf[4] : 0.0;
	conf[7]=(conf[2]+conf[3]);
	conf[8] = (conf[4]>0.01) ? (conf[2]+conf[3])/conf[4] : 0.0;
	if (v2>v3 && v2!=0.0){
	  val=sqrt((v2-v3)/v2);
	  conf[9] = v3/val*log( (1+val)/(1-val) );
	}
	else conf[9] = v2;
	conf[10] = (conf[4]>0.01) ? conf[3]/conf[4] : 0.0;
	conf[11]=conf[2]*conf[3];
	conf[12] = (conf[4]>0.01) ? conf[2]*conf[3]/(conf[4]*conf[4]) : 0.0;
	conf[13]=conf[1]*conf[2]*conf[3];
	conf[14] = (conf[4]>0.01) ? conf[1]*conf[2]*conf[3]/(conf[4]*conf[4]*conf[4]) : 0.0;
	//conf[15] = (conf[4]>0.5) ? pow(conf[7],n)/pow(conf[4],p) : 0.0;

	conf[16] = (v1>0.01) ? v2/(v1+v3) : 0.0;
	conf[17] = (v1-v2>0.01) ? (v2-v3)/(v1-v2) : 0.0;
	
	conf1[add]=conf[des->type];
    }
    

    for ( y=0; y<imaIn->dimy; y++)
      for ( x=0; x<imaIn->dimx; x++){
	add = y*imaIn->dimx+x;
        val=fabs(conf1[add]*des->normalisation); 
	val = (val>255) ? 255 : val; 
	val = (val<0) ? 0 : val; 
	imaOut->data[0][y][x] = (unsigned char)val;
      }

    write_volume_u1_sequence_next( imaOut);
    
    tmp_s1=gradx[0]; tmp_s2=grady[0]; tmp_s3=gradz[0];
    for( i=0; i<des->az-1; i++){
      gradx[i]=gradx[i+1];
      grady[i]=grady[i+1];
      gradz[i]=gradz[i+1];
    }
    gradx[des->az-1]=tmp_s1; grady[des->az-1]=tmp_s2; gradz[des->az-1]=tmp_s3;
    
    lect_volume_u1_sequence_next(imaIn);
    gradient3D( imaIn->data,
		gradx[des->az-1],grady[des->az-1],gradz[des->az-1],
		imaIn->dimx,imaIn->dimy,des->alp,des->tgrad/2);
    
  }/*fin for sur les sections*/

 
}





int gradient_organisation_ferm(ogi_t* des){
  int i;
  
  for(i=0;i<des->az;i++) { 
    free(des->gradx[i]); 
    free(des->grady[i]); 
    free(des->gradz[i]); 
  }
  free(des->gradx); 
  free(des->grady); 
  free(des->gradz);

  free(des->acp_x);
  free(des->acp_y);
  free(des->acp_z);
  free(des->conf1);
  free(des->conf2);
  free(des->conf3);

}




