/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \defgroup haar_wavelet
    operateurs haar_wavelet : ensemble d'op�ateurs pour le calcul
    des coefficients d'ondelette de Haar et la cr�tion d'attributs
    �partir de ces coefficients
    Type entree : ima3du1
    Type sortie : ima3du1 
    @name haar_wavelet
    @author L. Valet
    @version 1 (23.12.06)
    @see 
*/

#include "haar_wavelet.h"
#include <stdio.h>
#include <math.h>
#include <stdlib.h>


#define MIN3(x,y,z) ( ( ((x)<(y))&&((x)<(z))   )?(x):( ( ((x)>(y))&&((y)<(z))  )?y:z  )  )


/** \ingroup haar_wavelet
    lecture des parametres de l'operateur hw_coef_entire_image. \\
    @name haar_wavelet_coef_lect
    @param des  pointeur vers le descripteur de cet op�ateur  
    @param ptp pointeur sur parametre courant deja alloue,     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author L.Valet
    @version 1 (22.11.06); Include : haar_wavelet.h
*/
param* hw_coef_entire_image_lect(hw_coef_entire_image_t* des, param* ptp, char* debq){
  char question[500];

  sprintf(question, "%s Scale:", debq); 
  lec_param(question, ptp);
  des->scale = atoi(ptp->rep);
  ptp = ptp->next;
  
  return(ptp);
}


/** \ingroup haar_wavelet
    Initialisation pour le calcul des coeeficients d'ondelette de Haar sur l'ensemble
    d'un bloc 3D.
    Allocation de l'image de sortie aux meme dimension que l'image d'entree
    @name hw_coef_entire_image_init
    @param des pointeur ves un descripteur
    @param im0 pointeur image initiale 
    @param imres pointeur image �allouer
    @author L. Valet
    @version 1 (23.12.06); Include : haar_wavelet.h
*/
int hw_coef_entire_image_init(hw_coef_entire_image_t* des, ima3Du1* imaIn, ima3Du1* imaOut){

  imaOut->dimx = imaIn->dimx;
  imaOut->dimy = imaIn->dimy;
  imaOut->dimz = imaIn->dimz;
  imaOut->dept = imaIn->dept;
  allouer_ima3Du1(imaOut);

}


/* *************************  CALCUL  ***************************/
/** \ingroup haar_wavelet
    Calcul de l'operateur hw_coef_entire_image.\\
    @name hw_coef_entire_image_calc  
    @param des pointeur ves un descripteur
    @param im0 pointeur image initiale 
    @param imres pointeur image resultat
    @author L. Valet
    @version 1 (23.12.06); Include : haar_wavelet.h
*/
int hw_coef_entire_image_calc(hw_coef_entire_image_t* des, ima3Du1* imaIn, ima3Du1* imaOut){

  register unsigned int x,y,z,s;
  unsigned int  depl_x, depl_y, depl_z;
  unsigned int  dim_opt_x, dim_opt_y, dim_opt_z;

  /*calcul des dimensions utilisables de l'image en fonction de la profondeur d'analyse*/
  dim_opt_x = imaIn->dimx-(imaIn->dimx%(unsigned int)(pow(2,des->scale)));
  dim_opt_y = imaIn->dimy-(imaIn->dimy%(unsigned int)(pow(2,des->scale)));
  dim_opt_z = imaIn->dimz-(imaIn->dimz%(unsigned int)(pow(2,des->scale)));

  printf("dim opt x:%d y:%d z:%d\n", dim_opt_x, dim_opt_y, dim_opt_z);

  depl_x = dim_opt_x/2;
  depl_y = dim_opt_y/2;
  depl_z = dim_opt_z/2;

  /*calcul pour les differentes echelles*/
  for( s=0; s<des->scale; s++){
    printf("Calcul echelle %d\n", s+1);

    for(z=0; z<dim_opt_z-1; z+=2){
      for(y=0; y<dim_opt_y-1; y+=2){
        for(x=0; x<dim_opt_x-1; x+=2){
	
	
	  imaOut->data[z/2+depl_z][y/2][x/2]=(unsigned char)(128 +(imaIn->data[z][y][x]
								   +imaIn->data[z][y][x+1]
								   +imaIn->data[z][y+1][x]
								   +imaIn->data[z][y+1][x+1]
								   -imaIn->data[z+1][y][x]
								   -imaIn->data[z+1][y][x+1]
								   -imaIn->data[z+1][y+1][x]
								   -imaIn->data[z+1][y+1][x+1] )/8);
	
	  imaOut->data[z/2][y/2+depl_y][x/2]=(unsigned char)(128 +( imaIn->data[z][y][x]
							       +imaIn->data[z][y][x+1]
							       -imaIn->data[z][y+1][x]
							       -imaIn->data[z][y+1][x+1]
							       +imaIn->data[z+1][y][x]
							       +imaIn->data[z+1][y][x+1]
							       -imaIn->data[z+1][y+1][x]
							       -imaIn->data[z+1][y+1][x+1] )/8);
	
	  imaOut->data[z/2+depl_z][y/2+depl_y][x/2]=(unsigned char)(128 +( imaIn->data[z][y][x]
									-imaIn->data[z][y][x+1]
									+imaIn->data[z][y+1][x]
									-imaIn->data[z][y+1][x+1]
									-imaIn->data[z+1][y][x]
									-imaIn->data[z+1][y][x+1]
									+imaIn->data[z+1][y+1][x]
									+imaIn->data[z+1][y+1][x+1] )/8);
	
	  imaOut->data[z/2][y/2][x/2+depl_x]= (unsigned char)(128+
							      (imaIn->data[z][y][x]
							       -imaIn->data[z][y][x+1]
							       +imaIn->data[z][y+1][x]
							       -imaIn->data[z][y+1][x+1]
							       +imaIn->data[z+1][y][x]
							       -imaIn->data[z+1][y][x+1]
							       +imaIn->data[z+1][y+1][x]
							       -imaIn->data[z+1][y+1][x+1] )/8);
							      
	
	  imaOut->data[z/2+depl_z][y/2][x/2+depl_x]=(unsigned char)(128 +( imaIn->data[z][y][x]
									-imaIn->data[z][y][x+1]
									+imaIn->data[z][y+1][x]
									-imaIn->data[z][y+1][x+1]
									-imaIn->data[z+1][y][x]
									-imaIn->data[z+1][y][x+1]
									+imaIn->data[z+1][y+1][x]
									+imaIn->data[z+1][y+1][x+1] )/8);
	
	  imaOut->data[z/2][y/2+depl_y][x/2+depl_x]=(unsigned char)(128 +( imaIn->data[z][y][x]
									-imaIn->data[z][y][x+1]
									-imaIn->data[z][y+1][x]
									+imaIn->data[z][y+1][x+1]
									+imaIn->data[z+1][y][x]
									-imaIn->data[z+1][y][x+1]
									-imaIn->data[z+1][y+1][x]
									+imaIn->data[z+1][y+1][x+1] )/8);
	
	  imaOut->data[z/2+depl_z][y/2+depl_y][x/2+depl_x]=(unsigned char)(128 +( imaIn->data[z][y][x]
										 -imaIn->data[z][y][x+1]
										 -imaIn->data[z][y+1][x]
										 +imaIn->data[z][y+1][x+1]
										 -imaIn->data[z+1][y][x]
										 +imaIn->data[z+1][y][x+1]
										 +imaIn->data[z+1][y+1][x]
										 -imaIn->data[z+1][y+1][x+1] )/8);
	  if(s==des->scale-1){
	    imaOut->data[z/2][y/2][x/2]=(unsigned char)(( imaIn->data[z][y][x]
							 +imaIn->data[z][y][x+1]
							 +imaIn->data[z][y+1][x]
							 +imaIn->data[z][y+1][x+1]
							 +imaIn->data[z+1][y][x]
							 +imaIn->data[z+1][y][x+1]
							 +imaIn->data[z+1][y+1][x]
							 +imaIn->data[z+1][y+1][x+1] )/8);
	  }else{
	    imaIn->data[z/2][y/2][x/2]=(unsigned char)(( imaIn->data[z][y][x]
							 +imaIn->data[z][y][x+1]
							 +imaIn->data[z][y+1][x]
							 +imaIn->data[z][y+1][x+1]
							 +imaIn->data[z+1][y][x]
							 +imaIn->data[z+1][y][x+1]
							 +imaIn->data[z+1][y+1][x]
							 +imaIn->data[z+1][y+1][x+1] )/8);
	  }	    
	  
	
	}//fin boucle x
      }//fin boucle y
    }//fin boucle z
  
    /*modification des dimensions optimales pour l'echelle suivante*/
    dim_opt_x = dim_opt_x/2;
    dim_opt_y = dim_opt_y/2;
    dim_opt_z = dim_opt_z/2;
    /*mise a jour des deplacements*/
    depl_x = dim_opt_x/2;
    depl_y = dim_opt_y/2;
    depl_z = dim_opt_z/2;
    
  }//fin boucle s


  /*mettre la valeur 255 dans les voxels de l'image de sortie
    que ne peuvent etre trait�du a une taille non multiple de 2^n*/
/*   for(z=dim_min; z<imaIn->dimz-1; z+=2){ */
/*     for(y=dim_min; y<imaIn->dimy-1; y+=2){ */
/*       for(x=dim_min; x<imaIn->dimx-1; x+=2){ */
/* 	imaOut->data[z/2][y/2][x/2] = 255; */
/* 	imaOut->data[z/2+window/2][y/2][x/2] = 255; */
/* 	imaOut->data[z/2][y/2+window/2][x/2] = 255; */
/* 	imaOut->data[z/2+window/2][y/2+window/2][x/2] = 255; */
/* 	imaOut->data[z/2][y/2][x/2+window/2] = 255; */
/* 	imaOut->data[z/2+window/2][y/2][x/2+window/2] = 255; */
/* 	imaOut->data[z/2][y/2+window/2][x/2+window/2] = 255; */
/* 	imaOut->data[z/2+window/2][y/2+window/2][x/2+window/2] = 255; */
  //}//fin boucle x
  // }//fin boucle y
  //}//fin boucle z

}//fin fonction calc

















/** \ingroup haar_wavelet
    lecture des parametres de l'operateur hw_coef_on_window. \\
    @name hw_coef_on_window_lect
    @param des  pointeur vers le descripteur de cet op351rateur  
    @param ptp pointeur sur parametre courant deja alloue,     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author L.Valet
    @version 1 (03.05.07); Include : haar_wavelet.h
*/
param* hw_coef_on_window_lect(hw_coef_on_window_t* des, param* ptp, char* debq){
  char question[500];

  sprintf(question, "%s Scale:", debq); 
  lec_param(question, ptp);
  des->scale = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "%s Window size X:", debq); 
  lec_param(question, ptp);
  des->win_x = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "%s Window size Y:", debq); 
  lec_param(question, ptp);
  des->win_y = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "%s Window size Z:", debq); 
  lec_param(question, ptp);
  des->win_z = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "%s Direction:", debq); 
  lec_param(question, ptp);
  des->direction = atoi(ptp->rep);
  ptp = ptp->next;
  
  sprintf(question, "%s Feature:", debq); 
  lec_param(question, ptp);
  des->feature = atoi(ptp->rep);
  ptp = ptp->next;
   
  return(ptp);
}








/** \ingroup haar_wavelet
    Initialisation pour le calcul des coeeficient d'ondelette de Haar sur une fenetre d'analyse
    Allocation de l'image de sortie aux memes dimensions que l'image d'entree
    @name hw_coef_on_window_init
    @param des pointeur ves un descripteur
    @param im0 pointeur image initiale 
    @param imres pointeur image 340 allouer
    @author L. Valet
    @version 1 (23.12.06); Include : haar_wavelet.h
*/
int hw_coef_on_window_init(hw_coef_on_window_t* des, ima3Du1* imaIn, ima3Du1* imaOut){

  imaOut->lgtete = imaIn->lgtete;
  imaOut->dimx = imaIn->dimx;
  imaOut->dimy = imaIn->dimy;
  imaOut->dimz = imaIn->dimz;
  imaOut->dept = imaIn->dept;
  allouer_ima3Du1(imaOut);

  /*allocation pour le calcul de la densite d'appartion des intensites*/
  des->densite = (float *)calloc( 256, sizeof(float));

  /*calcul des dimensions utilisables de la fenetre d'analyse en fonction de la profondeur d'analyse*/
  des->win_opt_x = des->win_x-(des->win_x%(unsigned int)(pow(2,des->scale)));
  des->win_opt_y = des->win_y-(des->win_y%(unsigned int)(pow(2,des->scale)));
  des->win_opt_z = des->win_z-(des->win_z%(unsigned int)(pow(2,des->scale)));
  printf("Dim win optimal: %d %d %d\n", des->win_opt_x, des->win_opt_y, des->win_opt_z);
  
  des->imaCoef = (ima3Du1* )malloc( sizeof(ima3Du1));
  des->imaCoef->dimx = des->win_opt_x;
  des->imaCoef->dimy = des->win_opt_y;
  des->imaCoef->dimz = des->win_opt_z;
  allouer_ima3Du1( des->imaCoef);

  des->imaVal = (ima3Du1* )malloc( sizeof(ima3Du1));
  des->imaVal->dimx = des->win_opt_x;
  des->imaVal->dimy = des->win_opt_y;
  des->imaVal->dimz = des->win_opt_z;
  allouer_ima3Du1( des->imaVal);
}




/** \ingroup haar_wavelet
    Désallocation de la mémoire allouée dans le descripteur
    @name hw_coef_on_window_ferm
    @param des pointeur ves un descripteur
    @author L. Valet
    @version 1 (10.12.07); Include : haar_wavelet.h
 */
int hw_coef_on_window_ferm( hw_coef_on_window_t* des){
  
  free( des->densite);
  
  liberer_ima3Du1( des->imaCoef);
  free( des->imaCoef);
  
  liberer_ima3Du1( des->imaVal);
  free( des->imaVal);
}






/* *************************  CALCUL  ***************************/
/** \ingroup haar_wavelet
    Calcul de l'operateur hw_coef_on_window.\\
    @name hw_coef_on_window_calc  
    @param des pointeur ves un descripteur
    @param im0 pointeur image initiale 
    @param imres pointeur image resultat
    @author L. Valet
    @version 1 (03.05.07); Include : haar_wavelet.h
*/
int hw_coef_on_window_calc(hw_coef_on_window_t* des, ima3Du1* imaIn, ima3Du1* imaOut){

  register unsigned int x,y,z; /*position relative dans la win d'analyse*/
  register unsigned int xo, yo, zo; /*position dans l'image*/
  register unsigned int s;
  unsigned char  depl_x, depl_y, depl_z;  /*verif utilisation et a virer*/
  unsigned char  win_opt_x, win_opt_y, win_opt_z; /*dim optimal de la fenetre d'analyse*/
  ima3Du1 * imaVal = des->imaVal;
  ima3Du1 * imaCoef = des->imaCoef;
  int somme, nb, fac; /*utilisé dans le calcul de l'attribut*/
  float entropy, max=0.0;
  

  /*Pour tous les pixels traitables de l'images
   C'est a dire ceux situe dans une fenetre d'analyse*/
  for( zo=0; zo<imaIn->dimz-des->win_opt_z-1; zo+=1){
    printf("section %u/%u\n", zo+1, imaIn->dimz);
    for( yo=0; yo<imaIn->dimy-des->win_opt_y-1; yo+=1){
      for( xo=0; xo<imaIn->dimx-des->win_opt_x-1; xo+=1){
          //printf("section %d-%d-%d\n", zo, yo, xo);
        
        win_opt_x = des->win_opt_x;
        win_opt_y = des->win_opt_y;
        win_opt_z = des->win_opt_z;
  
        depl_x = win_opt_x/2;
        depl_y = win_opt_y/2;
        depl_z = win_opt_z/2;

	  /*recopie des valeurs de l'image sur la fenetre dans l'image de travail*/
	  for(z=0; z<win_opt_z; z++)
	    for(y=0; y<win_opt_y; y++)
	      for(x=0; x<win_opt_x; x++){
		des->imaVal->data[z][y][x] = imaIn->data[zo+z][yo+y][xo+x];
	      }

           /*calcul des coefs pour les differentes echelles*/
          for( s=0; s<des->scale; s++){
            //printf("Calcul echelle %d\n", s+1);

            /*Parcours de la fenetre d'analyse*/
            for(z=0; z<win_opt_z; z+=2){
              for(y=0; y<win_opt_y; y+=2){
                for(x=0; x<win_opt_x; x+=2){
                  //printf("x=%u/%u, y=%u/%u, z=%u/%u\n", x, win_opt_x, y, win_opt_y, z, win_opt_z);

                  /*Calcul pour toutes les directions possibles*/
                  /*Une optilisation possible peut consisté a ne calculer que les coefficients 
                    pour une direction donnée*/
		  imaCoef->data[z/2+depl_z][y/2][x/2]=(unsigned char)(128 +(imaVal->data[z][y][x]
                                                                         +imaVal->data[z][y][x+1]
                                                                         +imaVal->data[z][y+1][x]
                                                                         +imaVal->data[z][y+1][x+1]
                                                                         -imaVal->data[z+1][y][x]
                                                                         -imaVal->data[z+1][y][x+1]
                                                                         -imaVal->data[z+1][y+1][x]
                                                                         -imaVal->data[z+1][y+1][x+1] )/8);

                  imaCoef->data[z/2][y/2+depl_y][x/2]=(unsigned char)(128 +( imaVal->data[z][y][x]
                                                               +imaVal->data[z][y][x+1]
                                                               -imaVal->data[z][y+1][x]
                                                               -imaVal->data[z][y+1][x+1]
                                                               +imaVal->data[z+1][y][x]
                                                               +imaVal->data[z+1][y][x+1]
                                                               -imaVal->data[z+1][y+1][x]
                                                               -imaVal->data[z+1][y+1][x+1] )/8);
        
                  imaCoef->data[z/2+depl_z][y/2+depl_y][x/2]=(unsigned char)(128 +( imaVal->data[z][y][x]
                                                                        -imaVal->data[z][y][x+1]
                                                                        +imaVal->data[z][y+1][x]
                                                                        -imaVal->data[z][y+1][x+1]
                                                                        -imaVal->data[z+1][y][x]
                                                                        -imaVal->data[z+1][y][x+1]
                                                                        +imaVal->data[z+1][y+1][x]
                                                                        +imaVal->data[z+1][y+1][x+1] )/8);

     
                  imaCoef->data[z/2][y/2][x/2+depl_x]= (unsigned char)(128+
                                                              (imaVal->data[z][y][x]
                                                               -imaVal->data[z][y][x+1]
                                                               +imaVal->data[z][y+1][x]
                                                               -imaVal->data[z][y+1][x+1]
                                                               +imaVal->data[z+1][y][x]
                                                               -imaVal->data[z+1][y][x+1]
                                                               +imaVal->data[z+1][y+1][x]
                                                               -imaVal->data[z+1][y+1][x+1] )/8);

                  imaCoef->data[z/2+depl_z][y/2][x/2+depl_x]=(unsigned char)(128 +( imaVal->data[z][y][x]
                                                                        -imaVal->data[z][y][x+1]
                                                                        +imaVal->data[z][y+1][x]
                                                                        -imaVal->data[z][y+1][x+1]
                                                                        -imaVal->data[z+1][y][x]
                                                                        -imaVal->data[z+1][y][x+1]
                                                                        +imaVal->data[z+1][y+1][x]
                                                                        +imaVal->data[z+1][y+1][x+1] )/8);
 
                  imaCoef->data[z/2][y/2+depl_y][x/2+depl_x]=(unsigned char)(128 +( imaVal->data[z][y][x]
                                                                        -imaVal->data[z][y][x+1]
                                                                        -imaVal->data[z][y+1][x]
                                                                        +imaVal->data[z][y+1][x+1]
                                                                        +imaVal->data[z+1][y][x]
                                                                        -imaVal->data[z+1][y][x+1]
                                                                        -imaVal->data[z+1][y+1][x]
                                                                        +imaVal->data[z+1][y+1][x+1] )/8);

                  imaCoef->data[z/2+depl_z][y/2+depl_y][x/2+depl_x]=(unsigned char)(128 +( imaVal->data[z][y][x]
                                                                                 -imaVal->data[z][y][x+1]
                                                                                 -imaVal->data[z][y+1][x]
                                                                                 +imaVal->data[z][y+1][x+1]
                                                                                 -imaVal->data[z+1][y][x]
                                                                                 +imaVal->data[z+1][y][x+1]
                                                                                 +imaVal->data[z+1][y+1][x]
                                                                                 -imaVal->data[z+1][y+1][x+1] )/8);
 
                  /*Mise a jour des données pour le calcul des échelles suivantes*/
                  if(s==des->scale-1){
                    imaCoef->data[z/2][y/2][x/2]=(unsigned char)(( imaVal->data[z][y][x]
                                                         +imaVal->data[z][y][x+1]
                                                         +imaVal->data[z][y+1][x]
                                                         +imaVal->data[z][y+1][x+1]
                                                         +imaVal->data[z+1][y][x]
                                                         +imaVal->data[z+1][y][x+1]
                                                         +imaVal->data[z+1][y+1][x]
                                                         +imaVal->data[z+1][y+1][x+1] )/8);
                  }else{  /*sinon on place directement la valeur moyenne en sortie*/
                    imaVal->data[z/2][y/2][x/2]=(unsigned char)(( imaVal->data[z][y][x]
                                                         +imaVal->data[z][y][x+1]
                                                         +imaVal->data[z][y+1][x]
                                                         +imaVal->data[z][y+1][x+1]
                                                         +imaVal->data[z+1][y][x]
                                                         +imaVal->data[z+1][y][x+1]
                                                         +imaVal->data[z+1][y+1][x]
                                                         +imaVal->data[z+1][y+1][x+1] )/8);
                  }

              }//fin boucle x
            }//fin boucle y
          }//fin boucle z
          
          /*modification des dimensions optimales pour l'echelle suivante*/
          win_opt_x = win_opt_x/2;
          win_opt_y = win_opt_y/2;
          win_opt_z = win_opt_z/2;
          /*mise a jour des deplacements*/
          depl_x = win_opt_x/2;
          depl_y = win_opt_y/2;
          depl_z = win_opt_z/2;
    
        }//fin boucle s

        /*calcul de l'attribut*/
        /*Idée d'optimisation: voir si le calcul ne peut pas se faire a chaque échelle (dans la boucle s)*/
        fac = (int)pow(2,des->scale);
        nb = 0;
        switch( des->feature){
          case 1:
            /*Essai de la valeur moyenne sur la composante basse fréquence*/
            somme = 0;
            for(z=0; z< des->win_opt_z/fac; z++)
              for(y=0; y<des->win_opt_y/fac; y++)
                for(x=0; x<des->win_opt_x/fac; x++){
                  somme +=  imaVal->data[z][y][x];
                  nb++;
                }
            imaOut->data[zo+des->win_opt_z/2][yo+des->win_opt_y/2][xo+des->win_opt_x/2]= somme/nb;
            break;
          case 2:
            /*Entropy sur la distribution dans le premier cadran*/
            entropy = 0.0;
            for( s=0; s<256; s++) des->densite[s]=0.0; /*initialisation de la distribution*/
            for(z=0; z< des->win_opt_z/fac; z++)
              for(y=0; y<des->win_opt_y/fac; y++)
                for(x=0; x<des->win_opt_x/fac; x++){
                  des->densite[ imaVal->data[z][y][x] ] += 1.0;
                  nb++;
                }
            for( s=0; s<256; s++) des->densite[s]= des->densite[s]/nb; /*normalisation*/
            for( s=0; s<256; s++) {
              if (des->densite[s]!=0.0) entropy = entropy - des->densite[s]*log10f( des->densite[s]);
              if( entropy>max) max = entropy;
              //printf("test %f -- %f\n", des->densite[s], des->densite[s]*log10f( des->densite[s]));
            }
            //printf("Max entropy: %f sur %f\n", max, log10f(256));
            imaOut->data[zo+des->win_opt_z/2][yo+des->win_opt_y/2][xo+des->win_opt_x/2]= (unsigned char)( entropy/log10(256)*255.0);
            break;
          default:
            imaOut->data[zo+des->win_opt_z/2][yo+des->win_opt_y/2][xo+des->win_opt_x/2]= 255;
        }

        /*sauvegarde du resultat dans le voxel central de la fenetre d'analyse sachant que les
        position dans la fenetre d'analyse sont toutes exprimées par rapport au coin supérieur gauche
        (1er coin par rapport a l'origine)*/
        //imaOut->data[zo+des->win_opt_z/2][yo+des->win_opt_y/2][xo+des->win_opt_x/2]= somme/nb;

      }//fin boucle sur xo
    }//fin boucle sur yo
  }//fin boucle sur zo


}//fin fonction calc
