/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

/** \defgroup median3d 
operateur median3d
oprateur ralisant un filtrage mdian

Descripteur : \n
    - int winX  : dimension selon X de l'lment structurant
    - int winY  : dimension selon Y de l'lment structurant
    - int winZ  : dimension selon Z de l'lment structurant

Type entree : imau1 
Type sortie : imau1 

@name median3d
@author G. Marinescu, L.Valet
@version 1 (01.04.03); Include :median3d.h
@see 
*/
  

#include "image.h"
#include "median3d.h"
#include <stdlib.h>


/* *************************  LECTURE  *******************************/
/** \ingroup median3d
    lecture des parametres de l'operateur median. 
    winX, winY, winZ parametre du masque \\

    @param tMasque  pointeur vers descripteur     
    @param ptp pointeur sur parametre courant deja alloue,     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author G.Marinescu , L.Valet
    @version 1 (01.04.03); Include : median3d.h
*/
param* median3d_lect(median3d_t* tMasque, param* ptp, char* debq)
{
  char question[500];


  sprintf(question, "%s winX (entier impair positif)", debq); 
  lec_param(question, ptp);
  tMasque->winX = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s winY (entier impair positif)", debq); 
  lec_param(question, ptp);
  tMasque->winY = atoi(ptp->rep);
  ptp = ptp->next;

  sprintf(question, "%s winZ (entier impair positif)", debq); 
  lec_param(question, ptp);
  tMasque->winZ = atoi(ptp->rep);
  ptp = ptp->next;

  return(ptp);
}

/* *************************  INITIALISATION  ***************************/
/** \ingroup median3d
    XX = u1 | etc : initialisation de l'operateur median .\\
    allocation image resultat, 
    calcul des coeff du masque

    @param tMasque  pointeur descripteur
    @param im0  image initiale (taille connue)
    @param imres  pointeur image resultat
    @author G. Marinescu
    @version 1 (01.04.03); Include : median3d.h
*/
int median3d_init(median3d_t* tMasque, ima3Du1 im0, ima3Du1* imres)
{
  /* image resultat */ 
  imres->dimx= im0.dimx; 
  imres->dimy= im0.dimy;
  imres->dimz= im0.dimz;


  //printf("\n masque :%d x %d x %d \n",tMasque->winX,tMasque->winY,tMasque->winZ); 
  allouer_ima3Du1 (imres);
  imres->lgtete= im0.lgtete;
  imres->dept= im0.dept;
}


/* *************************  CALCUL  ***************************/
/** \ingroup median3d
    XX = u1 | etc : calcul de l'operateur median.\\
    Calcul du resultat de l'operateur median\\
    www.ulib.org/webRoor/Books/Numerical_Recipes/
 
    @param tMasque pointeur descripteur
    @param im0 image initiale 
    @param imres pointeur image resultat
    @author G. Marinescu
    @version 1 (01.04.03); Include : median3d.h
*/
int median3d_calc(median3d_t *tMasque, ima3Du1 im0, ima3Du1 *imres)
{
 int x=0;
 int y=0;
 int z=0;
 int elx=0;
 int ely=0;
 int elz=0;
 //nb. de elements dans le masque
 int nb_case=tMasque->winX*tMasque->winY*tMasque->winZ;
 int pixel[nb_case+1];
 int q=1;
 unsigned long i,j,inc;
 int v;
 inc=1;
 //parcour de l'image
   for(x=(tMasque->winX-1)/2;x<im0.dimx-(tMasque->winX-1)/2;x++){
     for(y=(tMasque->winY-1)/2;y<im0.dimy-(tMasque->winY-1)/2;y++){
       for(z=(tMasque->winZ-1)/2;z<im0.dimz-(tMasque->winZ-1)/2;z++){
	 // selection des pixels x,y,z de l'image permettant d'appliquer completement le filtre
	 for(elx=-(tMasque->winX-1)/2;elx<=(tMasque->winX-1)/2;elx++){
	   for(ely=-(tMasque->winY-1)/2;ely<=(tMasque->winY-1)/2;ely++){
	     for(elz=-(tMasque->winZ-1)/2;elz<=(tMasque->winZ-1)/2;elz++){
	       //vecteur 1D  avec les elements de la masque
	       pixel[q]=im0.data[z+elz][y+ely][x+elx];
/* 	       printf("%d %d ",q,pixel[q]); */
	       q++;
	     }
	   }
	 }
	 //triage de le vecteur pixel par la methode Shell
	 // www.ulib.org/webRoor/Books/Numerical_Recipes/
	 do {
	   inc*=3;
	   inc++;
	 }while (inc<=nb_case);
	 do {
	   inc/=3;
	   for (i=inc+1;i<=nb_case;i++) {
	     v=pixel[i];
	     j=i;
	     while (pixel[j-inc]>v) {
	       pixel[j]=pixel[j-inc];
	       j-=inc;
	       if (j<=inc) break;
	     }
	     pixel[j]=v;
	   }
	 } while (inc>1);
/* 	 for (q=1;q<nb_case+1;q++) printf("%d %d ",q,pixel[q]); */
	 q=1;
	 imres->data[z][y][x]=pixel[(nb_case+1)/2];
/* 	 printf("%d ",pixel[(nb_case+1)/2]); */
       }
     }
   }
 return(0);
}


