/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include "volume.h"



int lecture_entete_lvf(char * nomfich, ima3Du1 * pt_image)
{
  FILE * f;
  unsigned int val, nb_element_lu;
  unsigned int tab_val[5];
  struct stat info;


  /*test de l'extension du fichier*/
  //afaire

  /*ouverture du fichier*/
  f = fopen(nomfich,"r");
  if( f==NULL ){
    printf("Dans %s, ligne %d, ", __FILE__,__LINE__);
    fflush(stdout);
    perror("lect_volume_u1");
    exit(-1);
  }

  /*lecture des attributs du fichier*/
  if( stat( nomfich, &info) ){
    perror("stat");
    fclose(f);
    return(-1);
  }

  /*lecture du numro magique*/
  nb_element_lu = fread( &val, sizeof(int), 1, f);
  if( nb_element_lu != 1){
    printf("lecture impossible du nombre magique\n");
    fclose(f);
    return(-1);
  }

  /*analyse et controle du nombre magique*/
  if( val != LVF_MAGIC_NUMBER){ 
    if( val == LVF_MAGIC_NUMBER_INV)
      printf("Attention image codee dans l'ordre inverse de la lecture\n");
    printf("Nombre magique incorrecte\n");
    fclose(f);
    return(-1);
  }

  /*lecture des paramtres dimensionnels*/
  nb_element_lu = fread( tab_val, sizeof(int), 5, f);
  if( nb_element_lu != 5 ){
    printf("probleme lecture paramtres dimensionnels de l'image\n");
    fclose(f);
    return(-1);
  }

  /*controle du codage de l'image*/
  if( tab_val[4]!=8 ){
    printf("Image non codee sur 8bits\n");
    fclose(f);
    return(-1);
  }

  /*controle de la taille du fichier*/
  if( info.st_size != tab_val[0] + tab_val[1]*tab_val[2]*tab_val[3]){
    printf("fichier de %d octets alors que %d octets attendus!\n", 
	   info.st_size,
	   tab_val[0] + tab_val[1]*tab_val[2]*tab_val[3]);
    fclose(f);
    return(-1);
  }


  /*affectation des dimensions*/
  pt_image->lgtete = tab_val[0];
  pt_image->dimx = tab_val[1];
  pt_image->dimy = tab_val[2];
  pt_image->dimz = tab_val[3];
  pt_image->dept = tab_val[4];
  pt_image->f = f;

  /*deplacement pour le lecture des donnees*/
  fseek( f, tab_val[0], SEEK_SET);

}





/*
pt_image pointeur vers une struture ima3Du1 prsente en mmoire

format des fichiers .lvf
int1: nombre magique
int2: longueur de l'entete en octet (nombre magique inclu)
int3: dimx
int4: dimy
int5: dimz
int6: nombre de bit
entete minimale de 24octets
option:
code |      nom        |  champs
1    |   rsolution    |  un flottant + 2 caractres (unit)
2    |  sous block     |  chaine + dx,dy,dz
3    |  valeur stat    |  min(nb_bit) + max(nb_bit) + moyenne ...
*/
int lect_volume_u1(char * nomfich, ima3Du1 * pt_image){

  FILE * f;
  unsigned int val, nb_element_lu;
  unsigned int tab_val[5];
  register unsigned int x,y,z;
  struct stat info;


  /*test de l'extension du fichier*/
  //afaire

  /*ouverture du fichier*/
  f = fopen(nomfich,"r");
  if( f==NULL ){
    printf("Dans %s, ligne %d, ", __FILE__,__LINE__);
    fflush(stdout);
    perror("lect_volume_u1");
    exit(-1);
  }

  /*lecture des attributs du fichier*/
  if( stat( nomfich, &info) ){
    perror("stat");
    fclose(f);
    return(-1);
  }

  /*lecture du numro magique*/
  nb_element_lu = fread( &val, sizeof(int), 1, f);
  if( nb_element_lu != 1){
    printf("lecture impossible du nombre magique\n");
    fclose(f);
    return(-1);
  }

  /*analyse et controle du nombre magique*/
  if( val != LVF_MAGIC_NUMBER){ 
    if( val == LVF_MAGIC_NUMBER_INV)
      printf("Attention image codee dans l'ordre inverse de la lecture\n");
    printf("Nombre magique incorrecte\n");
    fclose(f);
    return(-1);
  }

  /*lecture des paramtres dimensionnels*/
  nb_element_lu = fread( tab_val, sizeof(int), 5, f);
  if( nb_element_lu != 5 ){
    printf("probleme lecture paramtres dimensionnels de l'image\n");
    fclose(f);
    return(-1);
  }

  /*controle du codage de l'image*/
  if( tab_val[4]!=8 ){
    printf("Image non codee sur 8bits\n");
    fclose(f);
    return(-1);
  }

  /*controle de la taille du fichier*/
  if( info.st_size != tab_val[0] + tab_val[1]*tab_val[2]*tab_val[3]){
    printf("fichier de %d octets alors que %d octets attendus!\n", 
	   info.st_size,
	   tab_val[0] + tab_val[1]*tab_val[2]*tab_val[3]);
    fclose(f);
    return(-1);
  }


  /*affectation des dimensions*/
  pt_image->lgtete = tab_val[0];
  pt_image->dimx = tab_val[1];
  pt_image->dimy = tab_val[2];
  pt_image->dimz = tab_val[3];
  pt_image->dept = tab_val[4];

  /*allocation de l'image en mmoire*/  
  pt_image->data = (unsigned char ***)malloc( tab_val[3] * sizeof(unsigned char **));
  if( pt_image->data == NULL){
    perror("Probleme d'allocation");
    fclose(f);
    return(-1);
  }
  for(z=0; z<tab_val[3]; z++){
    pt_image->data[z] = (unsigned char **)malloc(pt_image->dimy * sizeof(unsigned char *));
    if( pt_image->data[z] == NULL){
      perror("Probleme d'allocation");
      fclose(f);
      return(-1);
    }
    for(y=0; y<pt_image->dimy; y++){
      pt_image->data[z][y] = (unsigned char *)malloc(pt_image->dimx * sizeof(unsigned char));
      if( pt_image->data[z][y] == NULL){
	perror("Probleme d'allocation");
	fclose(f);
	return(-1);
      }
    }
  }
  
  /*lecture des donnees*/
  fseek( f, tab_val[0], SEEK_SET);
  for(z=0; z<tab_val[3]; z++)
    for(y=0; y<tab_val[2]; y++)
      for(x=0; x<tab_val[1]; x++){
	fread( &(pt_image->data[z][y][x]), sizeof(unsigned char), 1, f);
	//printf("x=%d, y=%d, z=%d, val=%d\n",x,y,z,pt_image->data[z][y][x]);
      }
  /*fermeture du fichier*/
  fclose(f);
  return(0);
}




/*
sw=sequence_width
*/
int lect_volume_u1_sequence_first(char * nomfich, ima3Du1 * pt_image, int sw){
  int z,y;

  lecture_entete_lvf( nomfich,  pt_image);

  /*control de la parit de la largeur de la fentre*/
  if( sw%2==0){
    printf("Sequence_width must be even: %d used instead of %d\n", sw+1, sw);
    sw=sw+1;
  }

  /*allocation de la mmoire pour la squence*/
  pt_image->sequence_width = sw;
  allouer_ima3Du1_sequence(pt_image);

  /*lecture des sections concernees par la squence*/
  for(z=0; z<sw-1; z++)
    for(y=0; y<pt_image->dimy; y++)
      fread( pt_image->data[z][y], sizeof(unsigned char), pt_image->dimx, pt_image->f);

  /*positionnement de la section en cours d'tude*/
  pt_image->sequence_number = sw/2;

}





/*
Lecture de la squence suivante
*/
int lect_volume_u1_sequence_next(ima3Du1 * pt_image){
  unsigned char **tampon;
  int i;
  int z,y;
  
  /*sequence_width/2 reprsente le nombre de sections autour de la section courante*/
  /*vrification de la possibilit de lire la section suivante*/
  if( pt_image->sequence_width/2+pt_image->sequence_number >= pt_image->dimz ){
    printf("Lecture sequence suivante impossible: fin du bloc\n");
    return( -1);
  }
  
  /*tourniquet sur les pointeurs des sections*/
  tampon = pt_image->data[0];  
  for( z=0; z<pt_image->sequence_width-1; z++)
    pt_image->data[z] = pt_image->data[z+1];
  pt_image->data[ pt_image->sequence_width-1 ] = tampon;

  /*lecture de la nouvelle section*/
  for(y=0; y<pt_image->dimy; y++)
    fread( pt_image->data[pt_image->sequence_width-1][y], sizeof(unsigned char), pt_image->dimx, pt_image->f);
  

  /*avancement de la section en cours*/
  pt_image->sequence_number++;

  /*pour un repositionnement absolu dans le fichier faire:
    fseek( pt_image->f, pt_image->lgtete+(pt_image->sequence_number*pt_image->dimy*dimx) + (pt_image->sequence_width/2)  , SEEK_SET);
  si necessaire*/
  
}




/*
lecture de la squence prcdente
*/
int lect_volume_u1_sequence_previous(ima3Du1 * pt_image){
}



/*
lecture d'une squence particuliere
*/
int lect_volume_u1_sequence_number(ima3Du1 * pt_image){
}




/*

*/
int ecriture_entete_lvf( char * nomfich, ima3Du1 * pt_image){

  FILE * f;
  unsigned int val, nb_element_ecrit;
  register unsigned int x,y,z;
  

  /*ouverture du fichier en ecriture*/
  f = fopen(nomfich,"w");
  if( f==NULL ){
    printf("Dans %s, ligne %d, ", __FILE__,__LINE__);
    fflush(stdout);
    perror("write_volume_u1");
    return(-1);
  }

  /*criture du numro magique*/
  val = LVF_MAGIC_NUMBER;
  nb_element_ecrit = fwrite( &val, sizeof(int), 1, f);
  if( nb_element_ecrit != 1){
    printf("Ecriture impossible du nombre magique\n");
    fclose(f);
    return(-1);    
  }
  

  /*criture des paramtres dimensionnels*/
  val = 6*sizeof(int); 
  fwrite( &pt_image->lgtete, sizeof(int), 1, f);
  fwrite( &pt_image->dimx, sizeof(int), 1, f);
  fwrite( &pt_image->dimy, sizeof(int), 1, f);
  fwrite( &pt_image->dimz, sizeof(int), 1, f);
  fwrite( &pt_image->dept, sizeof(int), 1, f);

  /*conservation du descripteur de fichier*/
  pt_image->f = f;

}





int write_volume_u1(char * nomfich, ima3Du1 * pt_image){

  FILE * f;
  unsigned int val, nb_element_ecrit;
  register unsigned int x,y,z;
  

  /*ouverture du fichier en ecriture*/
  f = fopen(nomfich,"w");
  if( f==NULL ){
    printf("Dans %s, ligne %d, ", __FILE__,__LINE__);
    fflush(stdout);
    perror("write_volume_u1");
    return(-1);
  }

  /*criture du numro magique*/
  val = LVF_MAGIC_NUMBER;
  nb_element_ecrit = fwrite( &val, sizeof(int), 1, f);
  if( nb_element_ecrit != 1){
    printf("Ecriture impossible du nombre magique\n");
    fclose(f);
    return(-1);    
  }
  
  /*verification de la dimension de l'entete*/
  if( pt_image->lgtete!=24){
    printf("ATTENTION ecriture d'image impossible: entete=%d\n", pt_image->lgtete);
    fclose(f);
    return 1;
  }


  /*verification de a profondeur de l'image*/
  if( pt_image->dept!=8){
    printf("ATTENTION ecriture d'image impossible: profondeur=%d\n", pt_image->dept);
    fclose(f);
    return 1;
  }


  /*criture des paramtres dimensionnels*/ 
  fwrite( &pt_image->lgtete, sizeof(int), 1, f);
  fwrite( &pt_image->dimx, sizeof(int), 1, f);
  fwrite( &pt_image->dimy, sizeof(int), 1, f);
  fwrite( &pt_image->dimz, sizeof(int), 1, f);
  fwrite( &pt_image->dept, sizeof(int), 1, f);

  /*criture de l'images*/
  for(z=0; z< pt_image->dimz; z++)
    for(y=0; y<pt_image->dimy; y++)
      fwrite( &pt_image->data[z][y][0], sizeof(unsigned char), pt_image->dimx, f);

  fclose(f);
}




int write_part_of_volume_u1(char * nomfich, ima3Du1 * pt_image, 
			    unsigned int x1, unsigned int y1, unsigned int z1,
			    unsigned int x2, unsigned int y2, unsigned int z2){


  FILE * f;
  unsigned int val, nb_element_ecrit;
  register unsigned int x,y,z;
  unsigned dimx,dimy,dimz;
  

  /*ouverture du fichier en ecriture*/
  f = fopen(nomfich,"w");
  if( f==NULL ){
    printf("Dans %s, ligne %d, ", __FILE__,__LINE__);
    fflush(stdout);
    perror("write_volume_u1");
    return(-1);
  }

  /*criture du numro magique*/
  val = LVF_MAGIC_NUMBER;
  nb_element_ecrit = fwrite( &val, sizeof(int), 1, f);
  if( nb_element_ecrit != 1){
    printf("Ecriture impossible du nombre magique\n");
    fclose(f);
    return(-1);    
  }
  
  dimx = x2-x1+1;
  dimy = y2-y1+1;
  dimz = z2-z1+1;

  /*criture des paramtres dimensionnels*/
  //fwrite( &pt_image->lgtete, sizeof(int), 1, f);
  val = 6*sizeof(int);
  fwrite( &val, sizeof(int), 1, f);
  fwrite( &dimx, sizeof(int), 1, f);
  fwrite( &dimy, sizeof(int), 1, f);
  fwrite( &dimz, sizeof(int), 1, f);
  fwrite( &pt_image->dept, sizeof(int), 1, f);

  /*criture de l'images*/
  for(z=z1; z<=z2; z++)
    for(y=y1; y<=y2; y++)
      fwrite( &pt_image->data[z][y][x1], sizeof(unsigned char), dimx, f);

}


/*

*/
int copie_entete_ima3du1(ima3Du1 * ptImaSrc, ima3Du1 * ptImaDst){

  ptImaDst->dimx = ptImaSrc->dimx;
  ptImaDst->dimy = ptImaSrc->dimy;
  ptImaDst->dimz = ptImaSrc->dimz;
  ptImaDst->lgtete = ptImaSrc->lgtete;
  ptImaDst->dept = ptImaSrc->dept;
  ptImaDst->sequence_number = ptImaSrc->sequence_number;
  ptImaDst->sequence_width = ptImaSrc->sequence_width;

}




/*
Sauvegarde de sections vides sur au debut du bloc pour compenser la taille
des eventuelles fenetre d'anayse
A utiliser dans tous les cas!
*/
int write_volume_u1_sequence_preparation( char * nomfich, ima3Du1 * pt_image, int nb_section_avant){
  int x,y,z;
  unsigned char val=0;

  ecriture_entete_lvf( nomfich, pt_image);

  /*criture des premires sections de l'images*/
  for( z=0; z<nb_section_avant; z++)
    for( y=0; y<pt_image->dimy; y++)
      for( x=0; x<pt_image->dimx; x++)
	fwrite( &val, sizeof(unsigned char), 1, pt_image->f);

}





/*
Sauvegarde de la section en cours d'analyse  
*/
int write_volume_u1_sequence_next(ima3Du1 * pt_image){
  int y;

  /*criture de l'images*/
    for(y=0; y<pt_image->dimy; y++)
      fwrite( pt_image->data[pt_image->sequence_number][y], sizeof(unsigned char), pt_image->dimx, pt_image->f);

    //pt_image->sequence_number++;
}




/*
Sauvegarde de sections vides si ncessaire  la fin du bloc pour compenser
la taille des ventuelles fenetres de traitement  
A utiliser dans tous les cas! 
*/
int write_volume_u1_sequence_fermeture(ima3Du1 * pt_image, int nb_section_apres){
  int x,y,z;
  unsigned char val=0;

    /*criture des dernieres sections de l'images*/
    for( z=0; z<nb_section_apres; z++)
      for( y=0; y<pt_image->dimy; y++)
	for( x=0; x<pt_image->dimx; x++)
	  fwrite( &val, sizeof(unsigned char), 1, pt_image->f);
  

  /*fermeture du fichier*/
  fclose( pt_image->f);

}
