/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/

/** operateur reggr3d : Croissance itrative de rgions  applique  des images 3D\\
Descripteur : \\
Doc a completer...
\\
Type entree : imau1 \\
Type sortie : imau1 
@name median
@author L. Valet
@version 1 (13.12.04)
@see 
*/
/*@{ */  /* doc suivantes en sous-partie */  



#include "image.h"
#include "reggr3d.h"
#include <stdlib.h>

/*
  ETMAX correspond au nombre d'tiquette qu'il est possible de grer.
  Cette valeur implique une rservation de 4*ETMAX octets pour la liste de chainage (long int).
  Puis il y aura 4*ETMAX pour les sommes par tiquette (float) et 4*ETMAX pour le nombre de points
  par tiquette (unsigned long int)
  ex: si ETMAX = 50 000 000 
  il faut 3*4*50M = 600Moctets 
*/
#define ETMAX 17000000 
//#define ETMAX 9000 

#define ABSOL(x,y)  (((x) > (y))? abs(x-y) : abs(y-x))
#define MAX(x,y) (((x)>(y))? (x):(y))
#define MIN(x,y) (((x)<(y))? (x):(y))
#define MIN3(x,y,z) ( ( ((x)<(y))&&((x)<(z))   )?(x):( ( ((x)>(y))&&((y)<(z))  )?y:z  )  )







/* *************************  LECTURE  *******************************/
/** lecture des parametres de l'operateur reggr3d. \\
    @name reggr3d_lect
    @param des  pointeur vers le descripteur de cet oprateur  
    @param ptp pointeur sur parametre courant deja alloue,     
    @param debq pointeur de chaine de caracteres placee en debut de question
    @return renvoie nouveau pointeur de parametre deja alloue (pour la suite) 
    @author L.Valet
    @version 1 (01.04.03); Include : reggr3d.h
*/
param* reggr3d_lect(reggr3d_t* des, param* ptp, char* debq)
{
  char question[500];
  
  sprintf(question, "%s nb d'iteration ", debq); 
  lec_param(question, ptp);
  des->nb_loop = atoi(ptp->rep);
  ptp = ptp->next; 

  sprintf(question, "%s seuil de depart aggreg de 2 regions ", debq); 
  lec_param(question, ptp);
  des->seuil0 = atoi(ptp->rep);
  ptp = ptp->next;
 
  sprintf(question, "%s incrementation du seuil ", debq); 
  lec_param(question, ptp);
  des->step = atoi(ptp->rep);
  ptp = ptp->next; 

/*   sprintf(question, "%s nombre des labels ", debq);  */
/*   lec_param(question, ptp); */
/*   des->lab_nr = atoi(ptp->rep); */
/*   ptp = ptp->next; */

  return(ptp);
}









/* *************************  INITIALISATION  ***************************/
/** Initialisation de l'operateur reggr3d .\\
    A partir de l'image initiale a tiqueter, l'image de travail de type 
    ima3Du4 (long int) est alloue pour le stockage des tiquettes. 
    @name median3d_init
    @param des  pointeur descripteur
    @param im0  image initiale (taille connue)
    @param imres  pointeur sur une structure deja alloue pour l'image des tiquettes
    @author L. Valet
    @version 1 (01.04.03); Include : reggr3d.h
*/
int reggr3d_init(reggr3d_t* des, ima3Du1* im0, ima3Du4* imres)
{

  /* Allocation de l'image rsultat */ 
  imres->dimx=im0->dimx;
  imres->dimy=im0->dimy;
  imres->dimz=im0->dimz;
  imres->lgtete= im0->lgtete;
  imres->dept= im0->dept;
  allouer_ima3Du4 (imres);


}



/*
  Gestion de l'quivalence entre deux tiquettes (a et b) dans la liste de chainage.
  Principe:
  On recherche l'tiquette minimum entre a et b. C'est cette tiquette qui sera
  utilis pour le pixel courant ainsi que pour matrialiser l'quivalence.
  Puis les deux tiquettes sont traites de la meme facon: si l'quivalence pour 
  l'tiquette a est diffrente du minimum alors sauvegarder l'quivalence existante 
  et ecrire le minimum. Ce traitement est ensuite reappliqu sur l'tiquette de 
  l'quivalence sauvgarde jusqu'a retrouver l'tiquette minimum dans la liste 
  de chainage.
    @param pt_etiq  pointeur sur la liste de chainage
    @param a  premiere etiquette
    @param b  deuxime etiquette
*/
unsigned long int equivalence_2_label( unsigned  long int *pt_etiq, 
				       unsigned  long int  a, 
				       unsigned  long int  b){
  
  unsigned long  int minimum;
  unsigned long  int beta;   /*tiquette en cours de traitement*/
  unsigned long  int tmp;    /*sauvegarde des quivalences existants*/
  
  minimum = MIN(a,b);
  //minimum = MIN( pt_etiq[a], pt_etiq[b]);

  beta = a;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  beta = b;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  return( minimum);

}





/*
  Meme traitement que pour l'quivalence de deux tiquettes,
  mais pour trois.
*/
unsigned long  int equivalence_3_label( unsigned long  int *pt_etiq, 
					unsigned long  int a, 
					unsigned long  int b,
					unsigned long  int c){
  
  unsigned long  int minimum;
  unsigned long  int beta;
  unsigned long  int tmp;
  
  //minimum = MIN3( pt_etiq[a], pt_etiq[b], pt_etiq[c]);
  minimum = MIN3( a, b, c);

  beta = a;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  beta = b;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }

  beta = c;
  while( pt_etiq[beta]!= minimum){
    tmp = pt_etiq[beta];
    pt_etiq[beta] = minimum;
    beta = tmp;
  }
  return( minimum);

}





/*
Rappel sur la composition de la variable cas:
 __________________________________________
| 0 | 0 | g-h | a-g | a-h | g-c | h-c | a-c|
--------------------------------------------

Les bits 0  5 sont a 1 si la diffrence est infrieure au seuil


Action a raliser en fonction des diffrents cas possibles: 4 actions possibles
- nouvelle etiquette                   c <-- new_etiq()
- tiquette du voisin sans quivalence c <-- etiq[a]
- equivalence entre deux tiquettes    c <-- equiv(a,g)
- quivalence entre trois tiquettes   c <-- equiv(a,g,h)
Les cas non marqu ci-dessous sont des quivalence entre trois tiquettes.

-- 0--0000 0000---                  c <-- new_etiq()
--------------------
-- 1--0000 0001--a,c--              c <-- etiq[a]
-- 2--0000 0010--h,c--              c <-- etiq[h]
-- 3--0000 0011--h,c-a,c--          c <-- equiv(a,h)
-- 4--0000 0100--g,c--              c <-- etiq[g]
-- 5--0000 0101--g,c-a,c--          c <-- equiv(a,g)
-- 6--0000 0110--g,c-h,c--          c <-- equiv(g,h)
-- 7--0000 0111--g,c-h,c-a,c--
-- 8--0000 1000--a,h--              c <-- new_etiq()
-- 9--0000 1001--a,h-a,c--          c <-- equiv(a,h)
--10--0000 1010--a,h-h,c--          c <-- equiv(a,h)
--------------------
--11--0000 1011--a,h-h,c-a,c--      c <-- equiv(a,h)
--12--0000 1100--a,h-g,c--          c <-- etiq[g]
--13--0000 1101--a,h-g,c-a,c--
--14--0000 1110--a,h-g,c-h,c--
--15--0000 1111--a,h-g,c-h,c-a,c--
--16--0001 0000--a,g--              c <-- new_etiq()
--17--0001 0001--a,g-a,c--          c <-- equiv(a,g)
--18--0001 0010--a,g-h,c--          c <-- etiq[h]
--19--0001 0011--a,g-h,c-a,c--
--20--0001 0100--a,g-g,c--          c <-- equiv(a,g)
--------------------
--21--0001 0101--a,g-g,c-a,c--      c <-- equiv(a,g)
--22--0001 0110--a,g-g,c-h,c--
--23--0001 0111--a,g-g,c-h,c-a,c--
--24--0001 1000--a,g-a,h--          c <-- new_etiq()
--25--0001 1001--a,g-a,h-a,c--
--26--0001 1010--a,g-a,h-h,c--
--27--0001 1011--a,g-a,h-h,c-a,c--
--28--0001 1100--a,g-a,h-g,c--
--29--0001 1101--a,g-a,h-g,c-a,c--
--30--0001 1110--a,g-a,h-g,c-h,c--
--------------------
--31--0001 1111--a,g-a,h-g,c-h,c-a,c--
--32--0010 0000--g,h--              c <-- new_etiq()
--33--0010 0001--g,h-a,c--          c <-- etiq[a]
--34--0010 0010--g,h-h,c--          c <-- equiv(g,h)
--35--0010 0011--g,h-h,c-a,c--
--36--0010 0100--g,h-g,c--          c <-- equiv(g,h)
--37--0010 0101--g,h-g,c-a,c--
--38--0010 0110--g,h-g,c-h,c--      c <-- equiv(g,h)
--39--0010 0111--g,h-g,c-h,c-a,c--
--40--0010 1000--g,h-a,h--          c <-- new_etiq()
--------------------
--41--0010 1001--g,h-a,h-a,c--
--42--0010 1010--g,h-a,h-h,c--
--43--0010 1011--g,h-a,h-h,c-a,c--
--44--0010 1100--g,h-a,h-g,c--
--45--0010 1101--g,h-a,h-g,c-a,c--
--46--0010 1110--g,h-a,h-g,c-h,c--
--47--0010 1111--g,h-a,h-g,c-h,c-a,c--
--48--0011 0000--g,h-a,g--                c <-- new_etiq()
--49--0011 0001--g,h-a,g-a,c--
--50--0011 0010--g,h-a,g-h,c--
--------------------
--51--0011 0011--g,h-a,g-h,c-a,c--
--52--0011 0100--g,h-a,g-g,c--
--53--0011 0101--g,h-a,g-g,c-a,c--
--54--0011 0110--g,h-a,g-g,c-h,c--
--55--0011 0111--g,h-a,g-g,c-h,c-a,c--
--56--0011 1000--g,h-a,g-a,h--            c <-- new_etiq()
--57--0011 1001--g,h-a,g-a,h-a,c--
--58--0011 1010--g,h-a,g-a,h-h,c--
--59--0011 1011--g,h-a,g-a,h-h,c-a,c--
--60--0011 1100--g,h-a,g-a,h-g,c--
--------------------
--61--0011 1101--g,h-a,g-a,h-g,c-a,c--
--62--0011 1110--g,h-a,g-a,h-g,c-h,c--
--63--0011 1111--g,h-a,g-a,h-g,c-h,c-a,c--

*/

void label_management( unsigned char cas,           /*le cas a traiter*/
		       unsigned long  int haut,     /*l'tiquette du haut si elle existe*/
		       unsigned long  int gauche,   /*l'tiquette de la gauche si elle existe*/
		       unsigned long  int avant,    /*l'tiquette de la section d'avant si elle existe*/
		       unsigned int *courant, /*pointeur sur la case contenant l'taiquette 
						      courante a dterminer*/
		       unsigned long  int *pt_etiq, /*pointeur sur le tableau permettant le 
						      chainage des tiquettes*/
		       unsigned long  int *nb_etiq  /*nombre d'tiquettes deja utilises*/
	    ) 
{
  
  
  if(cas>=64) printf("PROBLEME\n");

  switch (cas) {

  case 0: 
  case 8:
  case 16:
  case 24:
  case 32:
  case 40:
  case 48:
  case 56:
    /* on attribue une nouvelle etiquette*/
    {
      *nb_etiq +=1;
      if(*nb_etiq>ETMAX-1) *nb_etiq=ETMAX-1;
      *courant = *nb_etiq;
      return;
    }

  case 1:
  case 33:
    /*  propagation de l'etiquette avant*/
    {
      *courant = pt_etiq[avant];
      return;
    }

  case 2: 
  case 18:
    /* propagation de l'etiquette du haut*/
    {
      *courant = pt_etiq[haut];
      return;
    }

  case 4 : 
  case 12:
    /* propagation de l'etiquette de gauche*/
    {
      *courant = pt_etiq[gauche];
      return;
    }
    
  case 3:
  case 9:
  case 10:
  case 11:
    /*equivalence entre avant et haut*/
    {
      *courant = equivalence_2_label( pt_etiq, avant, haut);
      return;
    }


  case 5:
  case 17:
  case 20:
  case 21:
    /*equivalence entre avant et gauche*/
    {
      *courant = equivalence_2_label(pt_etiq, avant, gauche);
      return;
    }

  case 6:
  case 34:
  case 36:
  case 38:
    /*equivalence entre gauche et haut*/
    {
      *courant = equivalence_2_label(pt_etiq, haut, gauche);
      return;
    }

  default:
    /*quivalence entre gauche, haut et avant */
    {
      *courant = equivalence_3_label(pt_etiq, haut, gauche, avant);  

      return;
    }    
  }/*fin du switch*/

}



/*
  Fonction pour la mise  jour des labels  l'aide du tableau de chainage
*/

void 
reetiquetage( ima3Du1* ima_ori,
	      ima3Du4* ima_etiq,   /* adresse de l'image destination (etiquettes) */
	      unsigned long  int *pt_etiq,
	      unsigned long  int *pt_nb_etiq,        /* pointeur sur le nombre de label utilis moins un*/
	      unsigned long  int *pt_nb_real_etiq)   /* Nombre ral de label apres mise a jour  */
{
  register unsigned int x,y,z;
  unsigned long int i;
  unsigned long  int label_effectif_max =0;
  unsigned long  int nb_label_effectif  =1;
  unsigned long  int nb_label_effectif2 =0;
  unsigned long  int tmp;
  unsigned long int *pt_nbel; 
  float *pt_moy;

  /*mise a jour du tableau de chainage*/
  for (i = 0; i <= (*pt_nb_etiq); i++) {
    tmp = i;
    while (pt_etiq[tmp]!=tmp){
      tmp = pt_etiq[tmp];
    }
    pt_etiq[i] = tmp;
  }
  

  /*Allocation du tableau pour compter le nombre de points pour chaque label*/
  pt_nbel = (unsigned long int *)calloc( (*pt_nb_etiq)+1, sizeof(*pt_nbel));  
  /*Allocation du tableau pour cumuler les niveau de gris*/
  pt_moy = (float *)calloc( (*pt_nb_etiq)+1, sizeof(*pt_moy)); 
  if( pt_nbel == NULL || pt_moy == NULL){
    printf("Probleme d'allocation memoire\n");
    exit(1);
  }

  /*mise a zero des tableaux*/
  for(i=0; i<=(*pt_nb_etiq); i++){
    pt_nbel[i] = 0;
    pt_moy[i]  = 0.0;
  }


  /*Mise  jour des labels et comptage du nombre de points par rgion*/ 
  for( z=0; z<ima_etiq->dimz; z++){
    for( x=0; x<ima_etiq->dimx; x++){
      for( y=0; y<ima_etiq->dimy; y++){
	ima_etiq->data[z][y][x] = pt_etiq[ ima_etiq->data[z][y][x] ];
	pt_nbel[ima_etiq->data[z][y][x]] = pt_nbel[ima_etiq->data[z][y][x]] + 1.0;
      }
    }
  }
  
  /*Calcul de l'intensit moyenne pour chaque regions*/ 
  /*possibilit de faire du filtrage*/
  for( z=0; z<ima_etiq->dimz; z++)
    for( x=0; x<ima_etiq->dimx; x++)
      for( y=0; y<ima_etiq->dimy; y++)
	pt_moy[ima_etiq->data[z][y][x]] +=  
	  (float)(ima_ori->data[z][y][x]) / (float)(pt_nbel[ima_etiq->data[z][y][x]]);
 

  /*filtrage sur les regions en fonction de leur volume*/
  /*
  for( i=0; i<= (*pt_nb_etiq); i++){
    if( pt_nbel[i] < 1000) pt_moy[i]=255.0;
  }
  */

  /*comptage du nombre de label effectif*/
  for( i=0; i<= (*pt_nb_etiq); i++){
    if( pt_nbel[i]!= 0.0){
      ++nb_label_effectif2;
    }
  }
  //printf("Autre facon de compter les elements effectifs!!! %d\n", nb_label_effectif2);

  /*dernier passage pour mettre la moyenne dans chaque region*/  
  for( z=0; z<ima_etiq->dimz; z++)
    for( x=0; x<ima_etiq->dimx; x++)
      for( y=0; y<ima_etiq->dimy; y++)
	ima_ori->data[z][y][x] = (unsigned char)(pt_moy[ ima_etiq->data[z][y][x] ]); 
  
  *pt_nb_real_etiq = nb_label_effectif2;
  printf("nb_label_effectif: %d\n", nb_label_effectif2);


  /*desallocation*/
  free(pt_nbel);
  free(pt_moy);
}













/*
  Fonction pour la recherche des tiquettes et la gestion du chainage

*/
void 
etiquetage(ima3Du1* ima_in,     /* adr de l'image origine (initiale ou segmentee) */
	   ima3Du4* ima_etiq,   /* adresse de l'image destination (etiquettes) */
	   unsigned char seuil,      /* seuil pour la comparaison des pixels */
	   unsigned long int *pt_nb_etiq,     /* nombre de label utilis*/
	   unsigned long int *nb_real_etiq)   /* nombre de label apres simplification */
{
  int cas;  
  register  int x,y,z;
  unsigned long int i;
  unsigned long  int *pt_etiq;		/* pointeur tableau d'etiquettes */  
  unsigned long  int nb_etiq;          /* compteur des etiquettes */ 
 

  //printf("size au float %d\n", sizeof(double));
  /*Allocation du tableau de chainage*/
  pt_etiq = (unsigned long  int *)calloc(ETMAX, sizeof(unsigned long  int)); 
  if(pt_etiq == NULL){
    printf("Probleme d'allocation mmoire (pt_etiq)\n");
    exit(1);
  }


  /*initialisation du tableau*/
  for( i=0; i<ETMAX ; i++)
    pt_etiq[i] = i;  
  nb_etiq = 0;
  

  /*mise a zero de la premiere tiquette*/
  ima_etiq->data[0][0][0] = 0;


  /*traitement de la premiere ligne du bloc*/
  x = 0;
  z = 0;
  for( y=1; y<ima_in->dimy; y++){
    cas = 0;
    //printf("diff=%d\n", ABSOL( ima_in->data[z][y][x], ima_in->data[z][y-1][x]) );
    if( ABSOL( ima_in->data[z][y][x], ima_in->data[z][y-1][x]) <= seuil) 
      cas += 4; /*g et c similaire*/
    label_management( cas,
		      0, /*haut*/
		      ima_etiq->data[z][y-1][x],/*gauche*/ 
		      0,/*avant*/
		      &ima_etiq->data[z][y][x], /*courant*/
		      pt_etiq,
		      &nb_etiq
	  );
  }
  //printf("\nFin Traitement premiere ligne\n");
  //for( x=0; x<=nb_etiq; x++)
  //printf("%d --> %d\n", x, pt_etiq[x]);


  /*traitement des autres lignes de la premiere section*/ 
  z = 0;
  for( x=1; x<ima_in->dimx; x++){
    for( y=0; y<ima_in->dimy; y++){
      cas = 0;

      /*on peut toujours comparer avec le haut*/
      if( ABSOL( ima_in->data[z][y][x], ima_in->data[z][y][x-1]) <= seuil) 
	cas += 2; /*h et c similaire*/

      /* En plus, si on n'est pas sur la premiere colonne
	 on peut faire les comparaisons avec la gauche*/
      if( y!=0){ 
	/*comparaison avec celui de gauche*/
	if( ABSOL( ima_in->data[z][y][x], ima_in->data[z][y-1][x]) <= seuil) 
	  cas += 4; /*g et c similaire*/
	if( ABSOL( ima_in->data[z][y-1][x], ima_in->data[z][y][x-1]) <= seuil) 
	  cas += 32; /*g et h similaire*/	 	
	label_management( cas,
			  ima_etiq->data[z][y][x-1] /*haut*/,
			  ima_etiq->data[z][y-1][x]/*gauche*/, 
			  0, /*avant*/
			  &ima_etiq->data[z][y][x], /*courant*/
			  pt_etiq,
			  &nb_etiq
			  );
      }
      else{
	label_management( cas,
			  ima_etiq->data[z][y][x-1] /*haut*/,
			  0, /*gauche*/ 
			  0, /*avant*/
			  &ima_etiq->data[z][y][x], /*courant*/
			  pt_etiq,
			  &nb_etiq
			  );
      }
    }
  }

  
  /*traitement des autres sections du bloc*/
  for( z=1; z<ima_in->dimz; z++){
    for( x=0; x<ima_in->dimx; x++){
      for( y=0; y<ima_in->dimy; y++){
	cas = 0;
	
	/*comparaison avec le plan d'avant toujours possible*/
	if( ABSOL( ima_in->data[z][y][x], ima_in->data[z-1][y][x]) <= seuil) 
	  cas += 1; /*a et c similaire*/
	
	
	/*gestion de la premiere ligne*/
	if(x==0){ 
	  if(y!=0){ /*on ne se trouve pas sur le premier point de la ligne*/
	    if( ABSOL( ima_in->data[z][y][x], ima_in->data[z][y-1][x]) <= seuil) 
	      cas += 4; /*g et c similaire*/	    
	    if( ABSOL( ima_in->data[z][y-1][x], ima_in->data[z-1][y][x]) <= seuil) 
	      cas += 16; /*g et a similaire*/
	    label_management( cas,
			      0 /*haut*/,
			      ima_etiq->data[z][y-1][x] /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );
	    
	  }
	  else{
	    label_management( cas,
			      0 /*haut*/,
			      0 /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );
	    
	  }
	}
	else{ /*gestion des lignes suivantes*/

	  /*comparaison avec le haut toujours possible*/
	  if( ABSOL( ima_in->data[z][y][x], ima_in->data[z][y][x-1]) <= seuil) 
	    cas += 2; /*h et c similaire*/	    
	  if( ABSOL( ima_in->data[z][y][x-1], ima_in->data[z-1][y][x]) <= seuil) 
	    cas += 8; /*h et a similaire*/	
	    
	  if(y!=0){ /*on ne se trouve pas sur le premier point de la ligne*/
	    if( ABSOL( ima_in->data[z][y][x], ima_in->data[z][y-1][x]) <= seuil) 
	      cas += 4; /*g et c similaire*/	    
	    if( ABSOL( ima_in->data[z][y-1][x], ima_in->data[z-1][y][x]) <= seuil) 
	      cas += 16; /*g et a similaire*/	    
	    if( ABSOL( ima_in->data[z][y-1][x], ima_in->data[z][y][x-1]) <= seuil) 
	      cas += 32; /*g et h similaire*/
	    label_management( cas,
			      ima_etiq->data[z][y][x-1] /*haut*/,
			      ima_etiq->data[z][y-1][x] /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );
	  }
	  else{
	    label_management( cas,
			      ima_etiq->data[z][y][x-1] /*haut*/,
			      0 /*gauche*/, 
			      ima_etiq->data[z-1][y][x] /*avant*/,
			      &ima_etiq->data[z][y][x], /*courant*/
			      pt_etiq,
			      &nb_etiq
			      );	    
	  }
	} /*fin test sur les lignes*/
	
	
      } /*fin sur y*/
    } /*fin sur x*/
  } /*fin sur z*/
  
  
  printf("nb_etiq = %d (max %d)\n", nb_etiq+1, ETMAX);
  if( nb_etiq+1 == ETMAX) {
    printf("-----DANGER Pas assez d'etiquettes !!!----\n");
  }

  *pt_nb_etiq = nb_etiq+1;

  
  reetiquetage( ima_in,
  		ima_etiq,   /* adresse de l'image destination (etiquettes) */
		pt_etiq,
		&nb_etiq,        /* nombre de label utilis*/
		nb_real_etiq);   /*Nombre ral de label apres mise a jour  */

  /*liberation de la liste de chainage*/
  free( pt_etiq);
}






/* *************************  CALCUL  ***************************/
/** XX = u1 | etc : calcul de l'operateur median.\\
    Calcul du resultat de l'operateur median\\
    www.ulib.org/webRoor/Books/Numerical_Recipes/
    @name median3d_calc  
    @param du pointeur descripteur
    @param im0 image initiale 
    @param imres pointeur image resultat
    @author G. Marinescu
    @version 1 (01.04.03); Include : median3d.h
*/
int reggr3d_calc(reggr3d_t *des, ima3Du1* im0, ima3Du4 *imres)
{
  
  unsigned char i;

  for( i=0; i<des->nb_loop; i++){
    printf("-------Etiquettage %d --- seuil %d -----\n", i, des->seuil0 + (i*des->step));
    etiquetage( im0,   /* adr de l'image origine (initiale ou segmentee) */
		imres, /* adresse de l'image destination (etiquettes) */
		des->seuil0 + (i*des->step), /* seuil pour la comparaison des pixels */
		&(des->nb_etiq),             /* nombre de label utilis*/
		&(des->nb_label)             /* nombre de label restant apres simplfication  */
		);           
  }
    
  

}
