/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdio.h>
#include <string.h>

#include "volume.h" 
#include "image.h"
#include "proto2D.h"


/** Sauvegarde d'une section d'un bloc au format rasterfile.\\
Cet executable cree une image RasterFile  partir d'un bloc sismique.
La direction (xy, xz, yz) et le numro de section peuvent tre passs en paramtre. Dans le cas o aucun paramtre n'est prcis, les valeurs par dfauts sont le plan xy et la premire section.\\
Utilisation: sistoras <fichier image ELF> <fichier destination RasterFile>\\
option:
 -s numero de section\\
 -t direction\\
exemple: sistoras toto.xt toto15.ras -s 15 -t yz


@name sistoras
@author Lionel VALET
@version Mai 1999
*/

#define XY 1
#define XZ 2
#define YZ 3






/*programme principal*/
main(argc,argv)
    int argc;
    char *argv[];
{

  ima3Du1  ima;
  imau1 out;
  int x, y, z, s=1, t=XY;
  write_ima_t wri;   
  param par0, *ptp;
  char nomloc[200];
  

  /*lecture des paramtres*/
  param_debut(argc, argv, &par0); 
  ptp = &par0;      

  lec_param("fichier volume xt:", ptp);
  strcpy(nomloc, ptp->rep);
  ptp = ptp->next;  

  lec_param("Orientation [1=XY,2=XZ,3=YZ]:", ptp);
  t = atoi(ptp->rep);
  ptp = ptp->next;

  lec_param("Numero de la section [0..dim-1]:", ptp);
  s = atoi(ptp->rep);
  ptp = ptp->next;

  ptp = write_ima_lect(&wri, ptp, "Fichier rasterfile:");
  param_fin(argc, argv, &par0);


  /*traitement*/
  switch(t){
  case XY:
    {
      printf("XY\n");
      lect_ima3Du1( nomloc, &ima, ONE, s);
      out.nr=ima.dimx;
      out.nc=ima.dimy;
      alloc_imau1(&out);
      for(x=0; x<out.nr; x++)
	for(y=0; y<out.nc; y++)
	  out.p[x][y] = ima.data[0][y][x];
      break;
    }
  case XZ:
    {
      lect_ima3Du1(argv[1], &ima, ALL);
      out.nr=ima.dimx;
      out.nc=ima.dimz;
      alloc_imau1(&out);
      for(x=0; x<out.nr; x++)
	for(z=0; z<out.nc; z++)
	  out.p[x][z] = ima.data[z][s][x];
      break;
	}
  case YZ:
    {
      lect_ima3Du1(argv[1], &ima, ALL);
      out.nr=ima.dimy;
      out.nc=ima.dimz;
      alloc_imau1(&out);
      for(y=0; y<out.nr; y++)
	for(z=0; z<out.nc; z++)
	  out.p[y][z] = ima.data[z][y][s];
      break;
	}
  }
  
  /*sauvegarde de l'image*/
  write_ima_init( &wri);

  /*libration mmoire*/
  write_imau1_ferm( &wri, out);
  liberer_ima3Du1(&ima);
 
}
