/*
*    Copyright (c) 2007. The BATI team. All right reserved.
*
*    This file is part of BATI library.
*
*    BATI library is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    BATI library  is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with BATI library.  If not, see <http://www.gnu.org/licenses/>.
*/
/* ************************************************************* */
/*                           fuzzy_c_means.h                               */
/* ************************************************************* */
/*                                                               */
/*      Descripteurs et prototypes associs  l'oprateur fcm    */
/*                                                               */
/* ************************************************************* */
/*                                                               */
/*  Dernire modif : Ramasso E. & Jullien S.  08/12/03           */ 
/*                                                               */
/* ************************************************************* */


#ifndef __fuzzy_c_means
#define __fuzzy_c_means

/*! 
\ingroup fcm
\brief descripteur de l'oprateur fuzzy C-mean.

Il contient toutes les informations utiles pour classifier des donnes 
grce  la mthode du fuzzy C-mean.
 */

typedef struct {
  int nb_class;               /*!< nombre de classes  dterminer*/
  int nb_it;                  /*!< nombre maximal d'iterations  */
  float fuzzy_coeff;          /*!< indice de flou (>1) */
  float threshold;            /*!< le seuil d'arret */
  float variation;            /*!< variation apres chaque iteration */
  unsigned short **centre;    /*!< centre[i][j] = le centrode de la 
				classe i dans le j-eme attribut */
  char extension[200];        /*!< format de fichiers qui contiendront 
				les coefficients d'appartenance  */
  char init_type[50];         /*!< type d'initialisation [ 0 | 1 | nom_fichier ] 
				-  0 = initialisation tableaux avec les coefficients 
				d'appartenance directement
				-  1 = initialisation centre des classes RANDOM, 
				et apres calcul les tableaux avec coef. d'appartenance 
				directement 
				-  nom_fichier = initialisation centre des classes 
				avec informations lues dans un fichier ".txt" */
} fuzzy_c_means_t;




param *fuzzy_c_means_lect(fuzzy_c_means_t *desc, 
			  param *ptp, 
			  char *debq);


int fuzzy_c_means_init(fuzzy_c_means_t *fcm,
		       data_input dataI , 
		       data_output *dataO);


int fuzzy_c_means_calc(fuzzy_c_means_t *fcm,
		       data_input dataI , 
		       data_output *dataO );     


char *Calloc(unsigned nelem,unsigned elsize);


void calcul_centroides(fuzzy_c_means_t *fcm, 
		       data_input dataI , 
		       data_output *dataO);


void calcul_coeff_appartenance(fuzzy_c_means_t *fcm,
			       data_input dataI , 
			       data_output *dataO);


void init_coeff_appartenance(fuzzy_c_means_t *fcm,
			     data_input dataI , 
			     data_output *dataO );


#endif

